@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include <stdlib.h>
 #include <string.h>
@@ -15,8 +15,7 @@
 
 CTX::CTX()
 {
-  // initialize everything that has no default value in
-  // DefaultOptions.h
+  // initialize everything that has no default value in DefaultOptions.h
   short int word = 0x0001;
   char *byte = (char*)&word;
   bigEndian = (byte[0] ? 0 : 1);
@@ -24,8 +23,13 @@ CTX::CTX()
   const char *tmp;
   if((tmp = GetEnvironmentVar("GMSH_HOME")))
     homeDir = tmp;
+#if defined(WIN32)
+  else if((tmp = GetEnvironmentVar("APPDATA")))
+    homeDir = tmp;
+#else
   else if((tmp = GetEnvironmentVar("HOME")))
     homeDir = tmp;
+#endif
   else if((tmp = GetEnvironmentVar("TMP")))
     homeDir = tmp;
   else if((tmp = GetEnvironmentVar("TEMP")))
@@ -40,6 +44,7 @@ CTX::CTX()
   outputFileName = "";
   bgmFileName = "";
   createAppendMeshStatReport = 0;
+  launchSolverAtStartup = -1;
   lc = 1.;
   min[0] = min[1] = min[2] = max[2] = 0.;
   max[0] = max[1] = 1.; // for nice view when adding point in new model
@@ -87,6 +92,10 @@ CTX::CTX()
   color.mesh.tangents = color.mesh.line = color.mesh.quadrangle = 0;
   for (int i = 0; i < 20; i++)
     color.mesh.carousel[i] = 0;
+  // added by Gauthier Becker (these are not initialized by default 
+  // leading to valgrind error) Feel free to change the default values)
+  mesh.switchElementTags=0;
+  terminal=0;
 }
 
 CTX *CTX::_instance = NULL;
