@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #ifndef _CONTEXT_H_
 #define _CONTEXT_H_
@@ -19,7 +19,7 @@ struct contextMeshOptions {
   int points, lines, triangles, quadrangles, tetrahedra, hexahedra, prisms, pyramids;
   int surfacesEdges, surfacesFaces, volumesEdges, volumesFaces, numSubEdges;
   int pointsNum, linesNum, surfacesNum, volumesNum, qualityType, labelType;
-  int optimize, optimizeNetgen, optimizeLloyd, refineSteps, remove4triangles;
+  int optimize, optimizeNetgen, optimizeLloyd, smoothCrossField, refineSteps, remove4triangles;
   double normals, tangents, explode, angleSmoothNormals, allowSwapEdgeAngle;
   double mshFileVersion, mshFilePartitioned, pointSize, lineWidth;
   double qualityInf, qualitySup, radiusInf, radiusSup;
@@ -28,7 +28,7 @@ struct contextMeshOptions {
   int lcFromPoints, lcFromCurvature, lcExtendFromBoundary;
   int dual, voronoi, drawSkinOnly, colorCarousel, labelSampling;
   int fileFormat, nbSmoothing, algo2d, algo3d, algoSubdivide;
-  int algoRecombine, recombineAll;
+  int algoRecombine, recombineAll, recombine3DAll;
   int remeshParam, remeshAlgo;
   int order, secondOrderLinear, secondOrderIncomplete;
   int secondOrderExperimental, meshOnlyVisible;
@@ -49,10 +49,11 @@ struct contextMeshOptions {
 
 struct contextGeometryOptions {
   int draw, light, lightTwoSide, points, lines, surfaces, volumes;
-  int pointsNum, linesNum, surfacesNum, volumesNum;
+  int pointsNum, linesNum, surfacesNum, volumesNum, labelType;
   double pointSize, lineWidth, selectedPointSize, selectedLineWidth;
   int pointType, lineType, surfaceType, numSubEdges;
-  int oldCircle, extrudeSplinePoints, extrudeReturnLateral, oldNewreg;
+  int oldCircle, oldNewreg, oldRuledSurface;
+  int extrudeSplinePoints, extrudeReturnLateral;
   double normals, tangents, scalingFactor;
   int autoCoherence, highlightOrphans, clip, useTransform;
   double tolerance, snap[3], transform[3][3], offset[3];
@@ -60,7 +61,7 @@ struct contextGeometryOptions {
   int occSewFaces, occConnectFaces;
   int copyMeshingMethod, exactExtrusion;
   int matchGeomAndMesh;
-  int hideCompounds;
+  int hideCompounds, orientedPhysicals;
 };
 
 class CTX {
@@ -82,6 +83,8 @@ class CTX {
   std::vector<std::string> recentFiles;
   // create mesh statistics report (0: do nothing, 1: create, 2: append)
   int createAppendMeshStatReport;
+  // should we launch a solver at startup?
+  int launchSolverAtStartup ;
   // save session/option file on exit?
   int sessionSave, optionsSave;
   // ask confirmation when overwriting files?
@@ -94,18 +97,18 @@ class CTX {
   int terminal;
   // number of graphical windows/tiles
   int numWindows, numTiles;
-  // text editor and web browser command (with included '%s')
-  std::string editor, webBrowser;
+  // text editor command (with included '%s')
+  std::string editor;
   // pattern of files to watch out for
   std::string watchFilePattern;
   // show tootips in the GUI?
   int tooltips;
   // position and size of various windows in the GUI
-  int menuPosition[2], glPosition[2], glSize[2], msgSize;
+  int glPosition[2], glSize[2], msgSize, menuPosition[2], menuSize[2], detachedMenu;
   int optPosition[2], visPosition[2], hotPosition[2], clipPosition[2], manipPosition[2];
-  int statPosition[2], ctxPosition[2], solverPosition[2], solverSize[2];
+  int statPosition[2], ctxPosition[2];
   int pluginPosition[2], pluginSize[2], fieldPosition[2], fieldSize[2];
-  int fileChooserPosition[2];
+  int fileChooserPosition[2], extraPosition[2], extraSize[2];
   // use the system menu bar on Mac OS X?
   int systemMenuBar;
   // batch mode (-4: lua session, -3: server daemon, -2: check coherence, -1: write
@@ -149,8 +152,8 @@ class CTX {
   // small axes options
   int smallAxes, smallAxesSize, smallAxesPos[2];
   // large axes options
-  int axes, axesAutoPosition, axesTics[3], axesMikado;
-  double axesPosition[6];
+  int axes, axesAutoPosition, axesMikado, axesForceValue;
+  double axesPosition[6], axesValue[6], axesTics[3];
   std::string axesLabel[3], axesFormat[3];
   // simple dynamic lock (should be a mutex)
   int lock;
@@ -222,6 +225,8 @@ class CTX {
     double timeout;
     std::string socketName;
     std::string name[5], executable[5], remoteLogin[5];
+    int autoSaveDatabase, autoArchiveOutputFiles, autoMesh, autoMergeFile;
+    int autoHideNewViews, autoShowLastStep, autoCheck;
   }solver;
   // print options
   struct{
@@ -233,6 +238,7 @@ class CTX {
     int gifDither, gifSort, gifInterlace, gifTransparent;
     int posElementary, posElement, posGamma, posEta, posRho, posDisto;
     int compositeWindows, deleteTmpFiles, background;
+    int width, height;
   } print;
   // color options
   struct{
@@ -252,15 +258,13 @@ class CTX {
   } color;
   // is the machine big-endian?
   int bigEndian;
-  // how RGBA values are packed and unpacked into/from an unsigned
-  // integer to be fed to glColor4ubv (depends on machine byte
-  // ordering!):
+  // how RGBA values are packed and unpacked into/from an unsigned integer to be
+  // fed to glColor4ubv (depends on machine byte ordering!):
   unsigned int packColor(int R, int G, int B, int A);
   int unpackRed(unsigned int X);
   int unpackGreen(unsigned int X);
   int unpackBlue(unsigned int X);
   int unpackAlpha(unsigned int X);
-
 };
 
 #endif
