@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include "GmshConfig.h"
 #include "GmshMessage.h"
@@ -27,9 +27,8 @@
 #include "gl2yuv.h"
 #endif
 
-int GuessFileFormatFromFileName(std::string fileName)
+int GetFileFormatFromExtension(const std::string &ext)
 {
-  std::string ext = SplitFileName(fileName)[2];
   if     (ext == ".geo")  return FORMAT_GEO;
   else if(ext == ".msh")  return FORMAT_MSH;
   else if(ext == ".pos")  return FORMAT_POS;
@@ -73,6 +72,12 @@ int GuessFileFormatFromFileName(std::string fileName)
   else                           return -1;
 }
 
+int GuessFileFormatFromFileName(const std::string &fileName)
+{
+  std::string ext = SplitFileName(fileName)[2];
+  return GetFileFormatFromExtension(ext);
+}
+
 std::string GetDefaultFileName(int format)
 {
   std::vector<std::string> split = SplitFileName(GModel::current()->getFileName());
@@ -119,8 +124,38 @@ std::string GetDefaultFileName(int format)
 #if defined(HAVE_FLTK)
 static PixelBuffer *GetCompositePixelBuffer(GLenum format, GLenum type)
 {
+  openglWindow *newg = 0;
+
+  if(CTX::instance()->print.width > 0 || CTX::instance()->print.height > 0){
+    GLint width = FlGui::instance()->getCurrentOpenglWindow()->w();
+    GLint height = FlGui::instance()->getCurrentOpenglWindow()->h();
+    if(CTX::instance()->print.width <= 0){
+      double w = width * CTX::instance()->print.height / (double)height;
+      width = (int)w;
+      height = CTX::instance()->print.height;
+    }
+    else if(CTX::instance()->print.height <= 0){
+      double h = height * CTX::instance()->print.width / (double)width;
+      height = (int)h;
+      width = CTX::instance()->print.width;
+    }
+    else{
+      width = CTX::instance()->print.width;
+      height = CTX::instance()->print.height;
+    }
+    newg = new openglWindow(100, 100, width, height);
+    int mode = FL_RGB | FL_DEPTH | (CTX::instance()->db ? FL_DOUBLE : FL_SINGLE);
+    if(CTX::instance()->antialiasing) mode |= FL_MULTISAMPLE;
+    newg->mode(mode);
+    newg->end();
+    newg->getDrawContext()->copyViewAttributes
+      (FlGui::instance()->getCurrentOpenglWindow()->getDrawContext());
+    newg->show();
+    openglWindow::setLastHandled(newg);
+  }
+
   PixelBuffer *buffer;
-  if(!CTX::instance()->print.compositeWindows){
+  if(newg || !CTX::instance()->print.compositeWindows){
     GLint width = FlGui::instance()->getCurrentOpenglWindow()->w();
     GLint height = FlGui::instance()->getCurrentOpenglWindow()->h();
     buffer = new PixelBuffer(width, height, format, type);
@@ -154,14 +189,21 @@ static PixelBuffer *GetCompositePixelBuffer(GLenum format, GLenum type)
       delete buffers[i];
     }
   }
+
+  if(newg){
+    openglWindow::setLastHandled(0);
+    newg->hide();
+    delete newg;
+  }
+
   return buffer;
 }
 #endif
 
-void CreateOutputFile(std::string fileName, int format, bool redraw)
+void CreateOutputFile(const std::string &fileName, int format, bool redraw)
 {
-  if(fileName.empty())
-    fileName = GetDefaultFileName(format);
+  std::string name = fileName;
+  if(name.empty()) name = GetDefaultFileName(format);
 
   int oldFormat = CTX::instance()->print.fileFormat;
   CTX::instance()->print.fileFormat = format;
@@ -169,120 +211,120 @@ void CreateOutputFile(std::string fileName, int format, bool redraw)
   bool error = false;
 
   if(redraw)
-    Msg::StatusBar(2, true, "Writing '%s'...", fileName.c_str());
+    Msg::StatusBar(true, "Writing '%s'...", name.c_str());
 
   switch (format) {
 
   case FORMAT_AUTO:
-    CreateOutputFile(fileName, GuessFileFormatFromFileName(fileName), false);
+    CreateOutputFile(name, GuessFileFormatFromFileName(name), false);
     break;
 
   case FORMAT_OPT:
-    PrintOptions(0, GMSH_FULLRC, 1, 1, fileName.c_str());
+    PrintOptions(0, GMSH_FULLRC, 1, 1, name.c_str());
     break;
 
   case FORMAT_MSH:
     if(GModel::current()->getMeshPartitions().size() &&
        CTX::instance()->mesh.mshFilePartitioned == 1)
       GModel::current()->writePartitionedMSH
-        (fileName, CTX::instance()->mesh.binary, CTX::instance()->mesh.saveAll,
+        (name, CTX::instance()->mesh.binary, CTX::instance()->mesh.saveAll,
          CTX::instance()->mesh.saveParametric, CTX::instance()->mesh.scalingFactor);
     else if(GModel::current()->getMeshPartitions().size() &&
             CTX::instance()->mesh.mshFilePartitioned == 2)
       GModel::current()->writeMSH
-        (fileName, CTX::instance()->mesh.mshFileVersion,
+        (name, CTX::instance()->mesh.mshFileVersion,
          CTX::instance()->mesh.binary, CTX::instance()->mesh.saveAll,
          CTX::instance()->mesh.saveParametric, CTX::instance()->mesh.scalingFactor,
          0, -1000);
     else
       GModel::current()->writeMSH
-        (fileName, CTX::instance()->mesh.mshFileVersion,
+        (name, CTX::instance()->mesh.mshFileVersion,
          CTX::instance()->mesh.binary, CTX::instance()->mesh.saveAll,
          CTX::instance()->mesh.saveParametric, CTX::instance()->mesh.scalingFactor);
     break;
 
   case FORMAT_STL:
     GModel::current()->writeSTL
-      (fileName, CTX::instance()->mesh.binary, CTX::instance()->mesh.saveAll,
+      (name, CTX::instance()->mesh.binary, CTX::instance()->mesh.saveAll,
        CTX::instance()->mesh.scalingFactor);
     break;
 
   case FORMAT_VRML:
     GModel::current()->writeVRML
-      (fileName, CTX::instance()->mesh.saveAll, CTX::instance()->mesh.scalingFactor);
+      (name, CTX::instance()->mesh.saveAll, CTX::instance()->mesh.scalingFactor);
     break;
 
   case FORMAT_PLY2:
-    GModel::current()->writePLY2(fileName);
+    GModel::current()->writePLY2(name);
     break;
 
   case FORMAT_UNV:
     GModel::current()->writeUNV
-      (fileName, CTX::instance()->mesh.saveAll, CTX::instance()->mesh.saveGroupsOfNodes,
+      (name, CTX::instance()->mesh.saveAll, CTX::instance()->mesh.saveGroupsOfNodes,
        CTX::instance()->mesh.scalingFactor);
     break;
 
   case FORMAT_VTK:
     GModel::current()->writeVTK
-      (fileName, CTX::instance()->mesh.binary, CTX::instance()->mesh.saveAll,
+      (name, CTX::instance()->mesh.binary, CTX::instance()->mesh.saveAll,
        CTX::instance()->mesh.scalingFactor,
        CTX::instance()->bigEndian);
     break;
 
   case FORMAT_MESH:
     GModel::current()->writeMESH
-      (fileName, CTX::instance()->mesh.saveElementTagType,
+      (name, CTX::instance()->mesh.saveElementTagType,
        CTX::instance()->mesh.saveAll, CTX::instance()->mesh.scalingFactor);
     break;
 
   case FORMAT_MAIL:
     GModel::current()->writeMAIL
-      (fileName, CTX::instance()->mesh.saveAll, CTX::instance()->mesh.scalingFactor);
+      (name, CTX::instance()->mesh.saveAll, CTX::instance()->mesh.scalingFactor);
     break;
 
   case FORMAT_IR3:
     GModel::current()->writeIR3
-      (fileName, CTX::instance()->mesh.saveElementTagType,
+      (name, CTX::instance()->mesh.saveElementTagType,
        CTX::instance()->mesh.saveAll, CTX::instance()->mesh.scalingFactor);
     break;
 
   case FORMAT_BDF:
     GModel::current()->writeBDF
-      (fileName, CTX::instance()->mesh.bdfFieldFormat,
+      (name, CTX::instance()->mesh.bdfFieldFormat,
        CTX::instance()->mesh.saveElementTagType, CTX::instance()->mesh.saveAll,
        CTX::instance()->mesh.scalingFactor);
     break;
 
   case FORMAT_DIFF:
     GModel::current()->writeDIFF
-      (fileName, CTX::instance()->mesh.binary, CTX::instance()->mesh.saveAll,
+      (name, CTX::instance()->mesh.binary, CTX::instance()->mesh.saveAll,
        CTX::instance()->mesh.scalingFactor);
     break;
 
   case FORMAT_INP:
     GModel::current()->writeINP
-      (fileName, CTX::instance()->mesh.saveAll, CTX::instance()->mesh.scalingFactor);
+      (name, CTX::instance()->mesh.saveAll, CTX::instance()->mesh.scalingFactor);
     break;
 
   case FORMAT_P3D:
     GModel::current()->writeP3D
-      (fileName, CTX::instance()->mesh.saveAll, CTX::instance()->mesh.scalingFactor);
+      (name, CTX::instance()->mesh.saveAll, CTX::instance()->mesh.scalingFactor);
     break;
 
   case FORMAT_CGNS:
     GModel::current()->writeCGNS
-      (fileName, CTX::instance()->mesh.zoneDefinition, CTX::instance()->cgnsOptions,
+      (name, CTX::instance()->mesh.zoneDefinition, CTX::instance()->cgnsOptions,
        CTX::instance()->mesh.scalingFactor);
     break;
 
   case FORMAT_MED:
     GModel::current()->writeMED
-      (fileName, CTX::instance()->mesh.saveAll, CTX::instance()->mesh.scalingFactor);
+      (name, CTX::instance()->mesh.saveAll, CTX::instance()->mesh.scalingFactor);
     break;
 
   case FORMAT_POS:
     GModel::current()->writePOS
-      (fileName, CTX::instance()->print.posElementary,
+      (name, CTX::instance()->print.posElementary,
        CTX::instance()->print.posElement, CTX::instance()->print.posGamma,
        CTX::instance()->print.posEta, CTX::instance()->print.posRho,
        CTX::instance()->print.posDisto, CTX::instance()->mesh.saveAll,
@@ -290,16 +332,16 @@ void CreateOutputFile(std::string fileName, int format, bool redraw)
     break;
 
   case FORMAT_GEO:
-    GModel::current()->writeGEO(fileName, CTX::instance()->print.geoLabels,
+    GModel::current()->writeGEO(name, CTX::instance()->print.geoLabels,
                                 CTX::instance()->print.geoOnlyPhysicals);
     break;
 
   case FORMAT_BREP:
-    GModel::current()->writeOCCBREP(fileName);
+    GModel::current()->writeOCCBREP(name);
     break;
 
   case FORMAT_STEP:
-    GModel::current()->writeOCCSTEP(fileName);
+    GModel::current()->writeOCCSTEP(name);
     break;
 
 #if defined(HAVE_FLTK)
@@ -311,9 +353,9 @@ void CreateOutputFile(std::string fileName, int format, bool redraw)
     {
       if(!FlGui::available()) break;
 
-      FILE *fp = fopen(fileName.c_str(), "wb");
+      FILE *fp = fopen(name.c_str(), "wb");
       if(!fp){
-        Msg::Error("Unable to open file '%s'", fileName.c_str());
+        Msg::Error("Unable to open file '%s'", name.c_str());
         error = true;
         break;
       }
@@ -348,13 +390,13 @@ void CreateOutputFile(std::string fileName, int format, bool redraw)
     {
       if(!FlGui::available()) break;
 
-      FILE *fp = fopen(fileName.c_str(), "wb");
+      FILE *fp = fopen(name.c_str(), "wb");
       if(!fp){
-        Msg::Error("Unable to open file '%s'", fileName.c_str());
+        Msg::Error("Unable to open file '%s'", name.c_str());
         error = true;
         break;
       }
-      std::string base = SplitFileName(fileName)[1];
+      std::string base = SplitFileName(name)[1];
       GLint width = FlGui::instance()->getCurrentOpenglWindow()->w();
       GLint height = FlGui::instance()->getCurrentOpenglWindow()->h();
       GLint viewport[4] = {0, 0, width, height};
@@ -419,13 +461,13 @@ void CreateOutputFile(std::string fileName, int format, bool redraw)
     {
       if(!FlGui::available()) break;
 
-      FILE *fp = fopen(fileName.c_str(), "w");
+      FILE *fp = fopen(name.c_str(), "w");
       if(!fp){
-        Msg::Error("Unable to open file '%s'", fileName.c_str());
+        Msg::Error("Unable to open file '%s'", name.c_str());
         error = true;
         break;
       }
-      std::string base = SplitFileName(fileName)[1];
+      std::string base = SplitFileName(name)[1];
       GLint width = FlGui::instance()->getCurrentOpenglWindow()->w();
       GLint height = FlGui::instance()->getCurrentOpenglWindow()->h();
       GLint viewport[4] = {0, 0, width, height};
@@ -479,15 +521,15 @@ void CreateOutputFile(std::string fileName, int format, bool redraw)
       int repeat = (int)(CTX::instance()->post.animDelay * 24);
       if(repeat < 1) repeat = 1;
       std::string pattern("I");
-      // including P frames would lead to smaller files, but the
-      // quality degradation is perceptible:
+      // including P frames would lead to smaller files, but the quality
+      // degradation is perceptible:
       // for(int i = 1; i < repeat; i++) pattern += "P";
       fprintf(fp, "PATTERN %s\nBASE_FILE_FORMAT PPM\nGOP_SIZE %d\n"
               "SLICES_PER_FRAME 1\nPIXEL FULL\nRANGE 10\n"
               "PSEARCH_ALG EXHAUSTIVE\nBSEARCH_ALG CROSS2\n"
               "IQSCALE 1\nPQSCALE 1\nBQSCALE 25\nREFERENCE_FRAME DECODED\n"
               "OUTPUT %s\nINPUT_CONVERT *\nINPUT_DIR %s\nINPUT\n",
-              pattern.c_str(), repeat, fileName.c_str(),
+              pattern.c_str(), repeat, name.c_str(),
               CTX::instance()->homeDir.c_str());
       for(unsigned int i = 0; i < frames.size(); i++){
         fprintf(fp, "%s", frames[i].c_str());
@@ -526,7 +568,7 @@ void CreateOutputFile(std::string fileName, int format, bool redraw)
   CTX::instance()->printing = 0;
 
   if(redraw && !error)
-    Msg::StatusBar(2, true, "Done writing '%s'", fileName.c_str());
+    Msg::StatusBar(true, "Done writing '%s'", name.c_str());
 
 #if defined(HAVE_OPENGL)
   if(redraw) drawContext::global()->draw();
