@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #ifndef _DEFAULT_OPTIONS_H_
 #define _DEFAULT_OPTIONS_H_
@@ -46,7 +46,12 @@ StringXString GeneralOptions_String[] = {
   { F,   "Display" , opt_general_display , "" ,
     "X server to use (only for Unix versions)" },
 
-  { F|O, "ErrorFileName" , opt_general_error_filename , ".gmsh-errors" ,
+  { F|O, "ErrorFileName" , opt_general_error_filename ,
+#if defined(WIN32)
+    "gmsh-errors" ,
+#else
+    ".gmsh-errors" ,
+#endif
     "File into which the log is saved if a fatal error occurs" },
 
   { F,   "FileName" , opt_general_filename , "" ,
@@ -59,7 +64,12 @@ StringXString GeneralOptions_String[] = {
   { F|O, "GraphicsFontTitle" , opt_general_graphics_font_title , "Helvetica" ,
     "Font used in the graphic window for titles" },
 
-  { F|S, "OptionsFileName" , opt_general_options_filename , ".gmsh-options" ,
+  { F|S, "OptionsFileName" , opt_general_options_filename ,
+#if defined(WIN32)
+    "gmsh-options" ,
+#else
+    ".gmsh-options" ,
+#endif
     "Option file created with `Tools->Options->Save'; automatically read on startup" },
 
   { F|S, "RecentFile0", opt_general_recent_file0 , "untitled.geo" ,
@@ -73,7 +83,12 @@ StringXString GeneralOptions_String[] = {
   { F|S, "RecentFile4", opt_general_recent_file4 , "untitled.geo" ,
     "5th most recent opened file"},
 
-  { 0,   "SessionFileName" , opt_general_session_filename , ".gmshrc" ,
+  { 0,   "SessionFileName" , opt_general_session_filename ,
+#if defined(WIN32)
+    "gmshrc" ,
+#else
+    ".gmshrc" ,
+#endif
     "Option file into which session specific information is saved; automatically "
     "read on startup" },
 
@@ -86,18 +101,13 @@ StringXString GeneralOptions_String[] = {
     "sensible-editor '%s'" ,
 #endif
     "System command to launch a text editor" },
-  { F|S, "TmpFileName" , opt_general_tmp_filename , ".gmsh-tmp" ,
-    "Temporary file used by the geometry module" },
-
-  { F|O, "WebBrowser" , opt_general_web_browser ,
+  { F|S, "TmpFileName" , opt_general_tmp_filename ,
 #if defined(WIN32)
-    "explorer.exe %s" ,
-#elif defined(__APPLE__)
-    "open %s" ,
+    "gmsh-tmp" ,
 #else
-    "sensible-browser %s" ,
+    ".gmsh-tmp" ,
 #endif
-    "System command to launch a web browser" },
+    "Temporary file used by the geometry module" },
 
   { F, "WatchFilePattern", opt_general_watch_file_pattern , "" ,
      "Pattern of files to merge as they become available"},
@@ -163,6 +173,9 @@ StringXString PostProcessingOptions_String[] = {
 } ;
 
 StringXString ViewOptions_String[] = {
+  { F|O, "Attributes" , opt_view_attributes , "" ,
+    "Optional string attributes" },
+
   { F|O, "AxesFormatX" , opt_view_axes_format0 , "%.3g" ,
     "Number format for X-axis (in standard C form)" },
   { F|O, "AxesFormatY" , opt_view_axes_format1 , "%.3g" ,
@@ -238,6 +251,8 @@ StringXNumber GeneralOptions_Number[] = {
     "Mikado axes style" },
   { F|O, "AxesAutoPosition" , opt_general_axes_auto_position , 1. ,
     "Position the axes automatically" },
+  { F|O, "AxesForceValue" , opt_general_axes_force_value , 0. ,
+    "Force values on axes (otherwise use natural coordinates)" },
   { F|O, "AxesMaxX" , opt_general_axes_xmax , 1. ,
     "Maximum X-axis coordinate" },
   { F|O, "AxesMaxY" , opt_general_axes_ymax , 1. ,
@@ -256,6 +271,18 @@ StringXNumber GeneralOptions_Number[] = {
     "Number of tics on the Y-axis" },
   { F|O, "AxesTicsZ" , opt_general_axes_tics2 , 5. ,
     "Number of tics on the Z-axis" },
+  { F|O, "AxesValueMaxX" , opt_general_axes_value_xmax , 1. ,
+    "Maximum X-axis forced value" },
+  { F|O, "AxesValueMaxY" , opt_general_axes_value_ymax , 1. ,
+    "Maximum Y-axis forced value" },
+  { F|O, "AxesValueMaxZ" , opt_general_axes_value_zmax , 1. ,
+    "Maximum Z-axis forced value" },
+  { F|O, "AxesValueMinX" , opt_general_axes_value_xmin , 0. ,
+    "Minimum X-axis forced value" },
+  { F|O, "AxesValueMinY" , opt_general_axes_value_ymin , 0. ,
+    "Minimum Y-axis forced value" },
+  { F|O, "AxesValueMinZ" , opt_general_axes_value_zmin , 0. ,
+    "Minimum Z-axis forced value" },
 
   { F|O, "BackgroundGradient" , opt_general_background_gradient , 1. ,
     "Draw background gradient (0=none, 1=vertical, 2=horizontal, 3=radial)" },
@@ -349,6 +376,8 @@ StringXNumber GeneralOptions_Number[] = {
     "Vertical position (in pixels) of the upper left corner of the contextual "
     "windows" },
 
+  { F|S, "DetachedMenu" , opt_general_detached_menu , 0. ,
+    "Should the menu window be detached from the graphic window?" },
   { F|O, "DisplayBorderFactor" , opt_general_display_border_factor , 0.2 ,
     "Border factor for model display (0: model fits window size exactly)" },
   { F|O, "DoubleBuffer" , opt_general_double_buffer , 1. ,
@@ -360,6 +389,14 @@ StringXNumber GeneralOptions_Number[] = {
   { F|O, "ExpertMode" , opt_general_expert_mode, 0. ,
     "Enable expert mode (to disable all the messages meant for inexperienced "
     "users)" },
+  { F|S, "ExtraPositionX" , opt_general_extra_position0 , 650. ,
+    "Horizontal position (in pixels) of the upper left corner of the generic extra window" },
+  { F|S, "ExtraPositionY" , opt_general_extra_position1 , 350. ,
+    "Vertical position (in pixels) of the upper left corner of the generic extra window" },
+  { F|S, "ExtraHeight" , opt_general_extra_size1 , 100. ,
+    "Height (in pixels) of the generic extra window" },
+  { F|S, "ExtraWidth" , opt_general_extra_size0 , 100. ,
+    "Width (in pixels) of the generic extra window" },
 
  { F|O, "FastRedraw" , opt_general_fast_redraw, 0. ,
     "Draw simplified model while rotating, panning and zooming" },
@@ -367,9 +404,9 @@ StringXNumber GeneralOptions_Number[] = {
     "Horizontal position (in pixels) of the upper left corner of the field window" },
   { F|S, "FieldPositionY" , opt_general_field_position1 , 550. ,
     "Vertical position (in pixels) of the upper left corner of the field window" },
-  { F|S, "FieldHeight" , opt_general_field_size1 , 300. ,
+  { F|S, "FieldHeight" , opt_general_field_size1 , 320. ,
     "Height (in pixels) of the field window" },
-  { F|S, "FieldWidth" , opt_general_field_size0 , 300. ,
+  { F|S, "FieldWidth" , opt_general_field_size0 , 420. ,
     "Width (in pixels) of the field window" },
   { F|S, "FileChooserPositionX" , opt_general_file_chooser_position0 , 200. ,
     "Horizontal position (in pixels) of the upper left corner of the file chooser "
@@ -380,9 +417,9 @@ StringXNumber GeneralOptions_Number[] = {
   { F|O, "FontSize" , opt_general_fontsize , -1. ,
     "Size of the font in the user interface (-1=automatic)" },
 
-  { F|O, "GraphicsFontSize" , opt_general_graphics_fontsize , 17. ,
+  { F|O, "GraphicsFontSize" , opt_general_graphics_fontsize , 15. ,
     "Size of the font in the graphic window" },
-  { F|O, "GraphicsFontSizeTitle" , opt_general_graphics_fontsize_title , 19. ,
+  { F|O, "GraphicsFontSizeTitle" , opt_general_graphics_fontsize_title , 18. ,
     "Size of the font in the graphic window for titles" },
   { F|S, "GraphicsHeight" , opt_general_graphics_size1 , 600. ,
     "Height (in pixels) of the graphic window" },
@@ -392,7 +429,7 @@ StringXNumber GeneralOptions_Number[] = {
   { F|S, "GraphicsPositionY" , opt_general_graphics_position1 , 50. ,
     "Vertical position (in pixels) of the upper left corner of the graphic "
     "window" },
-  { F|S, "GraphicsWidth" , opt_general_graphics_size0 , 600. ,
+  { F|S, "GraphicsWidth" , opt_general_graphics_size0 , 800. ,
     "Width (in pixels) of the graphic window" },
 
   { F|O, "InitialModule", opt_general_initial_context, 0. ,
@@ -480,12 +517,16 @@ StringXNumber GeneralOptions_Number[] = {
     "Maximum model coordinate along the Y-axis (read-only)" },
   { F,   "MaxZ" , opt_general_zmax , 0. ,
     "Maximum model coordinate along the Z-axis (read-only)" },
-  { F|S, "MenuPositionX" , opt_general_menu_position0 , 800. ,
-    "Horizontal position (in pixels) of the upper left corner of the menu window" },
-  { F|S, "MenuPositionY" , opt_general_menu_position1 , 50. ,
-    "Vertical position (in pixels) of the upper left corner of the menu window" },
+  { F|S, "MenuWidth" , opt_general_menu_size0 , 200. ,
+    "Width (in pixels) of the menu tree" },
+  { F|S, "MenuHeight" , opt_general_menu_size1 , 200. ,
+    "Height (in pixels) of the (detached) menu tree" },
+  { F|S, "MenuPositionX" , opt_general_menu_position0 , 400. ,
+    "Horizontal position (in pixels) of the (detached) menu tree" },
+  { F|S, "MenuPositionY" , opt_general_menu_position1 , 400. ,
+    "Vertical position (in pixels) of the (detached) menu tree" },
   { F|S, "MessageHeight" , opt_general_message_size , 300. ,
-    "Height (in pixels) of the message console" },
+    "Height (in pixels) of the message console when it is visible (should be > 0)" },
   { F,   "MinX" , opt_general_xmin , 0. ,
     "Minimum model coordinate along the X-axis (read-only)" },
   { F,   "MinY" , opt_general_ymin , 0. ,
@@ -517,9 +558,9 @@ StringXNumber GeneralOptions_Number[] = {
     "window" },
   { F|S, "PluginPositionY" , opt_general_plugin_position1 , 550. ,
     "Vertical position (in pixels) of the upper left corner of the plugin window" },
-  { F|S, "PluginHeight" , opt_general_plugin_size1 , 300. ,
+  { F|S, "PluginHeight" , opt_general_plugin_size1 , 320. ,
     "Height (in pixels) of the plugin window" },
-  { F|S, "PluginWidth" , opt_general_plugin_size0 , 300. ,
+  { F|S, "PluginWidth" , opt_general_plugin_size0 , 420. ,
     "Width (in pixels) of the plugin window" },
   { F|O, "PointSize" , opt_general_point_size , 3. ,
     "Display size of points (in pixels)" },
@@ -534,6 +575,8 @@ StringXNumber GeneralOptions_Number[] = {
     "Polygon offset factor (offset = factor * DZ + r * units)" },
   { F|O, "PolygonOffsetUnits" , opt_general_polygon_offset_units , 1. ,
     "Polygon offset units (offset = factor * DZ + r * units)" },
+  { F|O, "ProgressMeterStep" , opt_general_progress_meter_step , 20. ,
+    "Increment (in percent) of the progress meter bar" },
 
   { F|O, "QuadricSubdivisions" , opt_general_quadric_subdivisions, 6. ,
     "Number of subdivisions used to draw points or lines as spheres or cylinders" },
@@ -580,15 +623,6 @@ StringXNumber GeneralOptions_Number[] = {
     " centered)" },
   { F|O, "SmallAxesSize" , opt_general_small_axes_size , 30. ,
     "Size (in pixels) of small axes" },
-  { F|S, "SolverPositionX" , opt_general_solver_position0 , 650. ,
-    "Horizontal position (in pixels) of the upper left corner of the solver "
-    "window" },
-  { F|S, "SolverPositionY" , opt_general_solver_position1 , 150. ,
-    "Vertical position (in pixels) of the upper left corner of the solver window" },
-  { F|S, "SolverHeight" , opt_general_solver_size1 , 400. ,
-    "Height (in pixels) of the solver window" },
-  { F|S, "SolverWidth" , opt_general_solver_size0 , 300. ,
-    "Width (in pixels) of the solver window" },
   { F|S, "StatisticsPositionX" , opt_general_statistics_position0 , 650. ,
     "Horizontal position (in pixels) of the upper left corner of the statistic "
     "window" },
@@ -647,7 +681,7 @@ StringXNumber GeometryOptions_Number[] = {
   { F,   "Clip" , opt_geometry_clip , 0.,
     "Enable clipping planes? (Plane[i]=2^i, i=0,...,5)" },
   { F|O, "CopyMeshingMethod" , opt_geometry_copy_meshing_method, 0. ,
-    "Copy meshing method when duplicating geometrical entities?" },
+    "Copy meshing method (unstructured or transfinite) when duplicating geometrical entities?" },
 
   { F|O, "ExactExtrusion" , opt_geometry_exact_extrusion, 1. ,
     "Use exact extrusion formula in interpolations (set to 0 to allow "
@@ -662,6 +696,8 @@ StringXNumber GeometryOptions_Number[] = {
   { F|O, "HighlightOrphans" , opt_geometry_highlight_orphans, 0. ,
     "Highlight orphan entities (lines connected to a single surface, etc.)?" },
 
+  { F|O, "LabelType" , opt_geometry_label_type , 1. ,
+    "Type of entity label (1=elementary number, 2=physical number)" },
   { F|O, "Light" , opt_geometry_light , 1. ,
     "Enable lighting for the geometry" },
   { F|O, "LightTwoSide" , opt_geometry_light_two_side , 1. ,
@@ -704,6 +740,8 @@ StringXNumber GeometryOptions_Number[] = {
     "Model display offset along Z-axis (in model coordinates)" },
   { F|O, "OldCircle" , opt_geometry_old_circle , 0. ,
     "Use old circle description (compatibility option for old Gmsh geometries)" },
+  { F|O, "OldRuledSurface" , opt_geometry_old_ruled_surface , 0. ,
+    "Use old 3-sided ruled surface interpolation (compatibility option for old Gmsh geometries)" },
   { F|O, "OldNewReg" , opt_geometry_old_newreg , 1. ,
     "Use old newreg definition for geometrical transformations (compatibility "
     "option for old Gmsh geometries)" },
@@ -721,6 +759,8 @@ StringXNumber GeometryOptions_Number[] = {
 
   { F|O, "ScalingFactor" , opt_geometry_scaling_factor , 1.0 ,
     "Global geometry scaling factor" },
+  { F|O, "OrientedPhysicals" , opt_geometry_oriented_physicals, 1. ,
+    "Use sign of elementary entity in physical definition as orientation indicator" },
   { F|O, "SnapX" , opt_geometry_snap0 , 0.1 ,
     "Snapping grid spacing along the X-axis" },
   { F|O, "SnapY" , opt_geometry_snap1 , 0.1 ,
@@ -791,6 +831,10 @@ StringXNumber MeshOptions_Number[] = {
     "Write mesh files in binary format (if possible)" },
   { F|O, "Bunin" , opt_mesh_bunin , 0. ,
     "Apply Bunin optimization on quad meshes (the parameter is the maximal size of a cavity that may be remeshed)" },
+  { F|O, "Lloyd" , opt_mesh_lloyd , 0. ,
+    "Apply lloyd optimization on surface meshes" },
+  { F|O, "SmoothCrossField" , opt_mesh_smooth_cross_field , 0. ,
+    "Apply n barycentric smoothing passes to the cross field" },
 
   { F|O, "CgnsImportOrder" , opt_mesh_cgns_import_order , 1. ,
    "Enable the creation of high-order mesh from CGNS structured meshes."
@@ -1006,7 +1050,8 @@ StringXNumber MeshOptions_Number[] = {
     "Mesh recombination algorithm (0=standard, 1=blossom)" },
   { F|O, "RecombineAll" , opt_mesh_recombine_all , 0 ,
     "Apply recombination algorithm to all surfaces, ignoring per-surface spec" },
-
+  { F|O, "Recombine3DAll" , opt_mesh_recombine3d_all , 0 ,
+    "Apply recombination3D algorithm to all volumes, ignoring per-volume spec" },
   { F|O, "RemeshAlgorithm" , opt_mesh_remesh_algo , 0 ,
     "Remeshing algorithm (0=no split, 1=automatic, 2=automatic only with metis)" },
   { F|O, "RemeshParametrization" , opt_mesh_remesh_param , 4 ,
@@ -1082,12 +1127,26 @@ StringXNumber MeshOptions_Number[] = {
 StringXNumber SolverOptions_Number[] = {
   { F|O, "AlwaysListen" , opt_solver_listen , 0. ,
     "Always listen to incoming connection requests?" },
+  { F|O, "AutoArchiveOutputFiles" , opt_solver_auto_archive_output_files , 0. ,
+    "Automatically archive output files after each computation" },
+  { F|O, "AutoCheck" , opt_solver_auto_check , 1. ,
+    "Automatically check model every time a parameter is changed" },
+  { F|O, "AutoSaveDatabase" , opt_solver_auto_save_database , 0. ,
+    "Automatically save database after each computation" },
+  { F|O, "AutoMesh" , opt_solver_auto_mesh , 1. ,
+    "Automatically mesh if necesssary" },
+  { F|O, "AutoMergeFile" , opt_solver_auto_merge_file , 1. ,
+    "Automatically merge result files" },
+  { F|O, "AutoHideNewViews" , opt_solver_auto_hide_new_views , 0. ,
+    "Automcatically hide newly merged results" },
+  { F|O, "AutoShowLastStep" , opt_solver_auto_show_last_step , 1. ,
+    "Automatically show the last time step in newly merged results" },
 
   { F|O, "Plugins" , opt_solver_plugins , 0. ,
     "Enable default solver plugins?" },
 
   { F|O, "Timeout" , opt_solver_timeout , 5. ,
-    "Time (in seconds) before closing the socket if no connection is happening." },
+    "Time (in seconds) before closing the socket if no connection is happening" },
 
   { 0, 0 , 0 , 0. , 0 }
 } ;
@@ -1128,6 +1187,8 @@ StringXNumber PostProcessingOptions_Number[] = {
 } ;
 
 StringXNumber ViewOptions_Number[] = {
+  { F|O, "AbscissaRangeType" , opt_view_abscissa_range_type , 1 ,
+    "Ascissa scale range type (1=default, 2=custom)" },
   { F|O, "AdaptVisualizationGrid" , opt_view_adapt_visualization_grid , 0. ,
     "Use adaptive visualization grid (for high-order elements)?" },
   { F|O, "AngleSmoothNormals" , opt_view_angle_smooth_normals , 30.0 ,
@@ -1138,7 +1199,8 @@ StringXNumber ViewOptions_Number[] = {
     "Minimum display size of arrows (in pixels)" },
   { F|O, "AutoPosition" , opt_view_auto_position , 1. ,
     "Position the scale or 2D plot automatically (0: manual, 1: automatic, 2: top left, "
-    "3: top right, 4: bottom left, 5: bottom right, 6: top, 7: bottom, 8: left, 9: right)" },
+    "3: top right, 4: bottom left, 5: bottom right, 6: top, 7: bottom, 8: left, 9: right, "
+    "10: full, 11: top third)" },
   { F|O, "Axes" , opt_view_axes , 0 ,
     "Axes (0=none, 1=simple axes, 2=box, 3=full grid, 4=open grid, 5=ruler)" },
   { F|O, "AxesMikado" , opt_view_axes_mikado , 0. ,
@@ -1208,6 +1270,10 @@ StringXNumber ViewOptions_Number[] = {
     "Forced component 7 (if View.ForceComponents > 0)" },
   { F|O, "ComponentMap8" , opt_view_component_map8 , 8,
     "Forced component 8 (if View.ForceComponents > 0)" },
+  { F,   "CustomAbscissaMax" , opt_view_custom_abscissa_max , 0. ,
+    "User-defined maximum abscissa value" },
+  { F,   "CustomAbscissaMin" , opt_view_custom_abscissa_min , 0. ,
+    "User-defined minimum abscissa value" },
   { F,   "CustomMax" , opt_view_custom_max , 0. ,
     "User-defined maximum value to be displayed" },
   { F,   "CustomMin" , opt_view_custom_min , 0. ,
@@ -1425,7 +1491,7 @@ StringXNumber PrintOptions_Number[] = {
 
   { F|O, "GeoLabels" , opt_print_geo_labels , 1. ,
     "Save labels in unrolled Gmsh geometries" },
-  { F|O, "GeoOnlyPhysicals" , opt_print_geo_only_physicals , 1. ,
+  { F|O, "GeoOnlyPhysicals" , opt_print_geo_only_physicals , 0. ,
     "Only save entities that belong to physical groups" },
   { F|O, "GifDither" , opt_print_gif_dither , 0. ,
     "Apply dithering to GIF output" },
@@ -1436,6 +1502,9 @@ StringXNumber PrintOptions_Number[] = {
   { F|O, "GifTransparent" , opt_print_gif_transparent , 0. ,
     "Output transparent GIF image" },
 
+  { F|O, "Height" , opt_print_height , -1. ,
+    "Height of printed image; use (possibly scaled) current height if < 0" },
+
   { F|O, "JpegQuality" , opt_print_jpeg_quality , 100. ,
     "JPEG quality (between 1 and 100)" },
   { F|O, "JpegSmoothing" , opt_print_jpeg_smoothing , 0. ,
@@ -1464,6 +1533,9 @@ StringXNumber PrintOptions_Number[] = {
   { F|O, "Text" , opt_print_text , 1. ,
     "Print text strings?" },
 
+  { F|O, "Width" , opt_print_width , -1. ,
+    "Width of printed image; use (possibly scaled) current width if < 0)" },
+
   { 0, 0 , 0 , 0. }
 } ;
 
