@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #ifndef _GMSH_DEFINES_H_
 #define _GMSH_DEFINES_H_
@@ -161,7 +161,7 @@
 #define MSH_HEX_56   99
 #define MSH_HEX_98  100
 #define MSH_HEX_152 101
-#define MSH_HEX_222 102
+#define MSH_HEX_218 102
 #define MSH_HEX_296 103
 #define MSH_HEX_386 104
 #define MSH_HEX_488 105
@@ -197,8 +197,12 @@
 #define MSH_PYR_245 131
 // Additional types
 #define MSH_PYR_1   132
+#define MSH_PNT_SUB 133
+#define MSH_LIN_SUB 134
+#define MSH_TRI_SUB 135
+#define MSH_TET_SUB 136
 
-#define MSH_NUM_TYPE 132
+#define MSH_NUM_TYPE 136
 
 // Geometric entities
 #define ENT_NONE     0
@@ -216,6 +220,7 @@
 #define ALGO_2D_FRONTAL        6
 #define ALGO_2D_BAMG           7
 #define ALGO_2D_FRONTAL_QUAD   8
+#define ALGO_2D_PACK_PRLGRMS   9
 
 // 3D meshing algorithms (numbers should not be changed)
 #define ALGO_3D_DELAUNAY       1
