@@ -1,12 +1,14 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <math.h>
 #include <string.h>
 #include <time.h>
+#include <sys/stat.h>
 #include "GmshConfig.h"
 #include "GmshMessage.h"
 #include "GmshSocket.h"
@@ -14,6 +16,7 @@
 #include "GModel.h"
 #include "Options.h"
 #include "Context.h"
+#include "OpenFile.h"
 #include "OS.h"
 
 #if defined(HAVE_ONELAB)
@@ -41,15 +44,18 @@
 int Msg::_commRank = 0;
 int Msg::_commSize = 1;
 int Msg::_verbosity = 5;
-int Msg::_progressMeterStep = 10;
+int Msg::_progressMeterStep = 20;
 int Msg::_progressMeterCurrent = 0;
 std::map<std::string, double> Msg::_timers;
 int Msg::_warningCount = 0;
 int Msg::_errorCount = 0;
+std::string Msg::_firstWarning;
+std::string Msg::_firstError;
 GmshMessage *Msg::_callback = 0;
 std::string Msg::_commandLine;
 std::string Msg::_launchDate;
 GmshClient *Msg::_client = 0;
+std::string Msg::_execName;
 #if defined(HAVE_ONELAB)
 onelab::client *Msg::_onelabClient = 0;
 onelab::server *onelab::server::_server = 0;
@@ -82,11 +88,20 @@ void Msg::Init(int argc, char **argv)
   MPI_Errhandler_set(MPI_COMM_WORLD, MPI_ERRORS_RETURN);
 #endif
 #if defined(HAVE_PETSC)
-  PetscInitialize(&argc, &argv, PETSC_NULL, PETSC_NULL);
+  int sargc = 0;
+  char **sargv = new char*[argc + 1];
+  // prune argv from gmsh-specific options that make PETSc verbose
+  for(int i = 0; i < argc; i++){
+    std::string val(argv[i]);
+    if(val != "-info" && val != "-help" && val != "-v")
+      sargv[sargc++] = argv[i];
+  }
+  PetscInitialize(&sargc, &sargv, PETSC_NULL, PETSC_NULL);
   PetscPopSignalHandler();
-#endif
 #if defined(HAVE_SLEPC)
-  SlepcInitialize(&argc, &argv, PETSC_NULL, PETSC_NULL);
+  SlepcInitialize(&sargc, &sargv, PETSC_NULL, PETSC_NULL);
+#endif
+  delete [] sargv;
 #endif
   time_t now;
   time(&now);
@@ -155,11 +170,47 @@ void Msg::Exit(int level)
   exit(_errorCount);
 }
 
+static int streamIsFile(FILE* stream)
+{
+  // the given stream is definitely not interactive if it is a regular file
+  struct stat stream_stat;
+  if(fstat(fileno(stream), &stream_stat) == 0){
+    if(stream_stat.st_mode & S_IFREG) return 1;
+  }
+  return 0;
+}
+
+static int streamIsVT100(FILE* stream)
+{
+  // if running inside emacs the terminal is not VT100
+  const char* emacs = getenv("EMACS");
+  if(emacs && *emacs == 't') return 0;
+
+  // list of known terminal names (from cmake)
+  static const char* names[] =
+    {"Eterm", "ansi", "color-xterm", "con132x25", "con132x30", "con132x43",
+     "con132x60", "con80x25",  "con80x28", "con80x30", "con80x43", "con80x50",
+     "con80x60",  "cons25", "console", "cygwin", "dtterm", "eterm-color", "gnome",
+     "gnome-256color", "konsole", "konsole-256color", "kterm", "linux", "msys",
+     "linux-c", "mach-color", "mlterm", "putty", "rxvt", "rxvt-256color",
+     "rxvt-cygwin", "rxvt-cygwin-native", "rxvt-unicode", "rxvt-unicode-256color",
+     "screen", "screen-256color", "screen-256color-bce", "screen-bce", "screen-w",
+     "screen.linux", "vt100", "xterm", "xterm-16color", "xterm-256color",
+     "xterm-88color", "xterm-color", "xterm-debian", 0};
+  const char** t = 0;
+  const char* term = getenv("TERM");
+  if(term){
+    for(t = names; *t && strcmp(term, *t) != 0; ++t) {}
+  }
+  if(!(t && *t)) return 0;
+  return 1;
+}
+
 void Msg::Fatal(const char *fmt, ...)
 {
   _errorCount++;
 
-  char str[1024];
+  char str[5000];
   va_list args;
   va_start(args, fmt);
   vsnprintf(str, sizeof(str), fmt, args);
@@ -173,7 +224,8 @@ void Msg::Fatal(const char *fmt, ...)
     FlGui::instance()->check();
     std::string tmp = std::string("@C1@.") + "Fatal   : " + str;
     FlGui::instance()->addMessage(tmp.c_str());
-    FlGui::instance()->showMessages();
+    if(_firstError.empty()) _firstError = str;
+    FlGui::instance()->setLastStatus(FL_RED);
     FlGui::instance()->saveMessages
       ((CTX::instance()->homeDir + CTX::instance()->errorFileName).c_str());
     fl_alert("A fatal error has occurred which will force Gmsh to abort.\n"
@@ -183,10 +235,14 @@ void Msg::Fatal(const char *fmt, ...)
 #endif
 
   if(CTX::instance()->terminal){
+    const char *c0 = "", *c1 = "";
+    if(!streamIsFile(stderr) && streamIsVT100(stderr)){
+      c0 = "\33[1m\33[31m"; c1 = "\33[0m";  // bold red
+    }
     if(_commSize > 1)
-      fprintf(stderr, "Fatal   : [On processor %d] %s\n", _commRank, str);
+      fprintf(stderr, "%sFatal   : [On processor %d] %s%s\n", c0, _commRank, str, c1);
     else
-      fprintf(stderr, "Fatal   : %s\n", str);
+      fprintf(stderr, "%sFatal   : %s%s\n", c0, str, c1);
     fflush(stderr);
   }
 
@@ -200,7 +256,7 @@ void Msg::Error(const char *fmt, ...)
 
   if(_verbosity < 1) return;
 
-  char str[1024];
+  char str[5000];
   va_list args;
   va_start(args, fmt);
   vsnprintf(str, sizeof(str), fmt, args);
@@ -214,15 +270,20 @@ void Msg::Error(const char *fmt, ...)
     FlGui::instance()->check();
     std::string tmp = std::string("@C1@.") + "Error   : " + str;
     FlGui::instance()->addMessage(tmp.c_str());
-    FlGui::instance()->showMessages();
+    if(_firstError.empty()) _firstError = str;
+    FlGui::instance()->setLastStatus(FL_RED);
   }
 #endif
 
   if(CTX::instance()->terminal){
+    const char *c0 = "", *c1 = "";
+    if(!streamIsFile(stderr) && streamIsVT100(stderr)){
+      c0 = "\33[1m\33[31m"; c1 = "\33[0m";  // bold red
+    }
     if(_commSize > 1)
-      fprintf(stderr, "Error   : [On processor %d] %s\n", _commRank, str);
+      fprintf(stderr, "%sError   : [On processor %d] %s%s\n", c0, _commRank, str, c1);
     else
-      fprintf(stderr, "Error   : %s\n", str);
+      fprintf(stderr, "%sError   : %s%s\n", c0, str, c1);
     fflush(stderr);
   }
 }
@@ -233,7 +294,7 @@ void Msg::Warning(const char *fmt, ...)
 
   if(_commRank || _verbosity < 2) return;
 
-  char str[1024];
+  char str[5000];
   va_list args;
   va_start(args, fmt);
   vsnprintf(str, sizeof(str), fmt, args);
@@ -247,11 +308,17 @@ void Msg::Warning(const char *fmt, ...)
     FlGui::instance()->check();
     std::string tmp = std::string("@C5@.") + "Warning : " + str;
     FlGui::instance()->addMessage(tmp.c_str());
+    if(_firstWarning.empty()) _firstWarning = str;
+    FlGui::instance()->setLastStatus();
   }
 #endif
 
   if(CTX::instance()->terminal){
-    fprintf(stderr, "Warning : %s\n", str);
+    const char *c0 = "", *c1 = "";
+    if(!streamIsFile(stderr) && streamIsVT100(stderr)){
+      c0 = "\33[35m"; c1 = "\33[0m";  // magenta
+    }
+    fprintf(stderr, "%sWarning : %s%s\n", c0, str, c1);
     fflush(stderr);
   }
 }
@@ -260,7 +327,7 @@ void Msg::Info(const char *fmt, ...)
 {
   if(_commRank || _verbosity < 4) return;
 
-  char str[1024];
+  char str[5000];
   va_list args;
   va_start(args, fmt);
   vsnprintf(str, sizeof(str), fmt, args);
@@ -270,10 +337,13 @@ void Msg::Info(const char *fmt, ...)
   if(_client) _client->Info(str);
 
 #if defined(HAVE_FLTK)
-  if(FlGui::available()){
-    FlGui::instance()->check();
-    std::string tmp = std::string("Info    : ") + str;
-    FlGui::instance()->addMessage(tmp.c_str());
+  #pragma omp critical
+  {
+    if(FlGui::available()){
+      FlGui::instance()->check();
+      std::string tmp = std::string("Info    : ") + str;
+      FlGui::instance()->addMessage(tmp.c_str());
+    }
   }
 #endif
 
@@ -287,20 +357,7 @@ void Msg::Direct(const char *fmt, ...)
 {
   if(_commRank || _verbosity < 3) return;
 
-  char str[1024];
-  va_list args;
-  va_start(args, fmt);
-  vsnprintf(str, sizeof(str), fmt, args);
-  va_end(args);
-
-  Direct(3, str);
-}
-
-void Msg::Direct(int level, const char *fmt, ...)
-{
-  if(_commRank || _verbosity < level) return;
-
-  char str[1024];
+  char str[5000];
   va_list args;
   va_start(args, fmt);
   vsnprintf(str, sizeof(str), fmt, args);
@@ -310,33 +367,31 @@ void Msg::Direct(int level, const char *fmt, ...)
   if(_client) _client->Info(str);
 
 #if defined(HAVE_FLTK)
-  if(FlGui::available()){
-    FlGui::instance()->check();
-    std::string tmp;
-    if(level < 2)
-      tmp = std::string("@C1@.") + str;
-    else if(level < 3)
-      tmp = std::string("@C5@.") + str;
-    else
-      tmp = std::string("@C4@.") + str;
-    FlGui::instance()->addMessage(tmp.c_str());
-    if(level == 1)
-      FlGui::instance()->showMessages();
+#pragma omp master
+  {
+    if(FlGui::available()){
+      FlGui::instance()->check();
+      std::string tmp = std::string("@C4@.") + str;
+      FlGui::instance()->addMessage(tmp.c_str());
+    }
   }
 #endif
 
   if(CTX::instance()->terminal){
-    fprintf(stdout, "%s\n", str);
+    const char *c0 = "", *c1 = "";
+    if(!streamIsFile(stdout) && streamIsVT100(stdout)){
+      c0 = "\33[34m"; c1 = "\33[0m";  // blue
+    }
+    fprintf(stdout, "%s%s%s\n", c0, str, c1);
     fflush(stdout);
   }
 }
 
-void Msg::StatusBar(int num, bool log, const char *fmt, ...)
+void Msg::StatusBar(bool log, const char *fmt, ...)
 {
   if(_commRank || _verbosity < 4) return;
-  if(num < 1 || num > 3) return;
 
-  char str[1024];
+  char str[5000];
   va_list args;
   va_start(args, fmt);
   vsnprintf(str, sizeof(str), fmt, args);
@@ -346,13 +401,16 @@ void Msg::StatusBar(int num, bool log, const char *fmt, ...)
   if(_client && log) _client->Info(str);
 
 #if defined(HAVE_FLTK)
-  if(FlGui::available()){
-    if(log) FlGui::instance()->check();
-    if(!log || num != 2 || _verbosity > 4)
-      FlGui::instance()->setStatus(str, num - 1);
-    if(log){
-      std::string tmp = std::string("Info    : ") + str;
-      FlGui::instance()->addMessage(tmp.c_str());
+#pragma omp master
+  {
+    if(FlGui::available()){
+      if(log) FlGui::instance()->check();
+      if(!log || _verbosity > 4)
+	FlGui::instance()->setStatus(str);
+      if(log){
+	std::string tmp = std::string("Info    : ") + str;
+	FlGui::instance()->addMessage(tmp.c_str());
+      }
     }
   }
 #endif
@@ -363,11 +421,25 @@ void Msg::StatusBar(int num, bool log, const char *fmt, ...)
   }
 }
 
+void Msg::StatusGl(const char *fmt, ...)
+{
+#if defined(HAVE_FLTK)
+  if(_commRank) return;
+  char str[5000];
+  va_list args;
+  va_start(args, fmt);
+  vsnprintf(str, sizeof(str), fmt, args);
+  va_end(args);
+  if(FlGui::available())
+    FlGui::instance()->setStatus(str, true);
+#endif
+}
+
 void Msg::Debug(const char *fmt, ...)
 {
   if(_verbosity < 99) return;
 
-  char str[1024];
+  char str[5000];
   va_list args;
   va_start(args, fmt);
   vsnprintf(str, sizeof(str), fmt, args);
@@ -392,57 +464,39 @@ void Msg::Debug(const char *fmt, ...)
   }
 }
 
-void Msg::ProgressMeter(int n, int N, const char *fmt, ...)
+void Msg::ProgressMeter(int n, int N, bool log, const char *fmt, ...)
 {
-  if(_commRank || _verbosity < 4) return;
+  if(_commRank || _verbosity < 4 ||
+     _progressMeterStep <= 0 || _progressMeterStep >= 100) return;
 
   double percent = 100. * (double)n/(double)N;
 
-  if(percent >= _progressMeterCurrent){
-    char str[1024];
+  if(percent >= _progressMeterCurrent || n > N - 1){
+    char str[5000], str2[5000];
     va_list args;
     va_start(args, fmt);
     vsnprintf(str, sizeof(str), fmt, args);
     va_end(args);
+    sprintf(str2, "%3d%%    : %s", _progressMeterCurrent, str);
 
-    if(strlen(fmt)) strcat(str, " ");
-
-    char str2[1024];
-    sprintf(str2, "(%d %%)", _progressMeterCurrent);
-    strcat(str, str2);
-
-    if(_client) _client->Progress(str);
+    if(_client) _client->Progress(str2);
 
 #if defined(HAVE_FLTK)
-    if(FlGui::available()){
-      if(_verbosity > 4) FlGui::instance()->setStatus(str, 1);
+    if(FlGui::available() && _verbosity > 4){
       FlGui::instance()->check();
+      FlGui::instance()->setProgress(str, (n > N - 1) ? 0 : n, 0, N);
     }
 #endif
 
-    if(CTX::instance()->terminal){
-      fprintf(stdout, "%s                     \r", str);
+    if(!streamIsFile(stdout) && log && CTX::instance()->terminal){
+      fprintf(stdout, "%s                                          \r",
+              (n > N - 1) ? "" : str2);
       fflush(stdout);
     }
 
     while(_progressMeterCurrent < percent)
       _progressMeterCurrent += _progressMeterStep;
   }
-
-  if(n > N - 1){
-    if(_client) _client->Progress("Done!");
-
-#if defined(HAVE_FLTK)
-    if(FlGui::available()){
-      if(_verbosity > 4) FlGui::instance()->setStatus("", 1);
-    }
-#endif
-
-    if(CTX::instance()->terminal){
-      fprintf(stdout, "Done!                                              \r");
-      fflush(stdout);
-    }
-  }
 }
 
 void Msg::PrintTimers()
@@ -467,6 +521,15 @@ void Msg::PrintTimers()
   }
 }
 
+void Msg::ResetErrorCounter()
+{
+  _warningCount = 0; _errorCount = 0;
+  _firstWarning.clear(); _firstError.clear();
+#if defined(HAVE_FLTK)
+  if(FlGui::available()) FlGui::instance()->setLastStatus();
+#endif
+}
+
 void Msg::PrintErrorCounter(const char *title)
 {
   if(_commRank || _verbosity < 1) return;
@@ -481,25 +544,27 @@ void Msg::PrintErrorCounter(const char *title)
 
 #if defined(HAVE_FLTK)
   if(FlGui::available()){
-    std::string red("@C1@.");
-    FlGui::instance()->addMessage((red + prefix + line).c_str());
-    FlGui::instance()->addMessage((red + prefix + title).c_str());
-    FlGui::instance()->addMessage((red + prefix + warn).c_str());
-    FlGui::instance()->addMessage((red + prefix + err).c_str());
-    FlGui::instance()->addMessage((red + prefix + help).c_str());
-    FlGui::instance()->addMessage((red + prefix + line).c_str());
-    if(_errorCount){
-      FlGui::instance()->showMessages();
-      fl_beep();
-    }
+    std::string col = _errorCount ? "@C1@." : "@C5@.";
+    FlGui::instance()->addMessage((col + prefix + line).c_str());
+    FlGui::instance()->addMessage((col + prefix + title).c_str());
+    FlGui::instance()->addMessage((col + prefix + warn).c_str());
+    FlGui::instance()->addMessage((col + prefix + err).c_str());
+    FlGui::instance()->addMessage((col + prefix + help).c_str());
+    FlGui::instance()->addMessage((col + prefix + line).c_str());
+    if(_errorCount) fl_beep();
   }
 #endif
 
   if(CTX::instance()->terminal){
-    fprintf(stderr, "%s\n%s\n%s\n%s\n%s\n%s\n", (prefix + line).c_str(),
+    const char *c0 = "", *c1 = "";
+    if(!streamIsFile(stderr) && streamIsVT100(stderr)){
+      c0 = _errorCount ? "\33[1m\33[31m" : "\33[35m"; // bold red or magenta
+      c1 = "\33[0m";
+    }
+    fprintf(stderr, "%s%s\n%s\n%s\n%s\n%s\n%s%s\n", c0, (prefix + line).c_str(),
             (prefix + title).c_str(), (prefix + warn).c_str(),
             (prefix + err).c_str(), (prefix + help).c_str(),
-            (prefix + line).c_str());
+            (prefix + line).c_str(), c1);
     fflush(stderr);
   }
 }
@@ -591,29 +656,88 @@ bool Msg::UseOnelab()
 #endif
 }
 
+void Msg::SetOnelabNumber(std::string name, double val, bool visible)
+{
+#if defined(HAVE_ONELAB)
+  if(_onelabClient){
+    std::vector<onelab::number> numbers;
+    _onelabClient->get(numbers, name);
+    if(numbers.empty()){
+      numbers.resize(1);
+      numbers[0].setName(name);
+    }
+    numbers[0].setValue(val);
+    numbers[0].setVisible(visible);
+    _onelabClient->set(numbers[0]);
+  }
+#endif
+}
+
+void Msg::SetOnelabString(std::string name, std::string val, bool visible)
+{
+#if defined(HAVE_ONELAB)
+  if(_onelabClient){
+    std::vector<onelab::string> strings;
+    _onelabClient->get(strings, name);
+    if(strings.empty()){
+      strings.resize(1);
+      strings[0].setName(name);
+    }
+    strings[0].setValue(val);
+    strings[0].setVisible(visible);
+    _onelabClient->set(strings[0]);
+  }
+#endif
+}
+
+#if defined(HAVE_ONELAB)
+class localGmsh : public onelab::localClient {
+public:
+  localGmsh() : onelab::localClient("Gmsh") {}
+  // redefinition of virtual onelab_client::sendMergeFileRequest
+  void sendMergeFileRequest(const std::string &name)
+  {
+    if(name.find(".geo") != std::string::npos){
+      MergePostProcessingFile(name, CTX::instance()->solver.autoShowLastStep,
+			      CTX::instance()->solver.autoHideNewViews, true);
+      GModel::current()->setFileName(name);
+    }
+    else if((name.find(".opt") != std::string::npos)){
+      MergeFile(name);
+    }
+    else if((name.find(".macro") != std::string::npos)){
+      MergeFile(name);
+    }
+    else
+      MergePostProcessingFile(name, CTX::instance()->solver.autoShowLastStep,
+			      CTX::instance()->solver.autoHideNewViews, true);
+  }
+  void sendInfo(const std::string &msg){ Msg::Info("%s", msg.c_str()); }
+  void sendWarning(const std::string &msg){ Msg::Warning("%s", msg.c_str()); }
+  void sendError(const std::string &msg){ Msg::Error("%s", msg.c_str()); }
+};
+#endif
+
 void Msg::InitializeOnelab(const std::string &name, const std::string &sockname)
 {
 #if defined(HAVE_ONELAB)
   if(_onelabClient) delete _onelabClient;
-  if(sockname.empty())
-    _onelabClient = new onelab::localClient(name);
+  if(sockname.empty()){
+    _onelabClient = new localGmsh();
+    if(name != "Gmsh"){ // load db from file:
+      if(!_onelabClient->fromFile(name))
+        Error("Error loading onelab database '%s'", name.c_str());
+    }
+  }
   else{
     onelab::remoteNetworkClient *c = new onelab::remoteNetworkClient(name, sockname);
     _onelabClient = c;
     _client = c->getGmshClient();
 
-    onelab::number o5(name + "/UseCommandLine",1);
-    o5.setVisible(false);
-    _onelabClient->set(o5);
-    onelab::string o(name + "/FileExtension", ".geo");
-    o.setVisible(false);
-    _onelabClient->set(o);
-    onelab::string o3(name + "/9CheckCommand", "-");
-    o3.setVisible(false);
-    _onelabClient->set(o3);
-    onelab::string o4(name + "/9ComputeCommand", "-3");
-    o4.setVisible(false);
-    _onelabClient->set(o4);
+    SetOnelabNumber(name + "/UseCommandLine", 1, false);
+    SetOnelabString(name + "/FileExtension", ".geo", false);
+    SetOnelabString(name + "/9CheckCommand", "-", false);
+    SetOnelabString(name + "/9ComputeCommand", "-3", false);
 
     std::vector<onelab::string> ps;
     _onelabClient->get(ps, name + "/Action");
@@ -621,19 +745,74 @@ void Msg::InitializeOnelab(const std::string &name, const std::string &sockname)
       //Info("Performing OneLab '%s'", ps[0].getValue().c_str());
       if(ps[0].getValue() == "initialize") Exit(0);
     }
-
   }
 #endif
 }
 
-void Msg::ExchangeOnelabParameter(const std::string &key,
-                                  std::vector<double> &val,
-                                  std::map<std::string, std::vector<double> > &fopt,
-                                  std::map<std::string, std::vector<std::string> > &copt)
+void Msg::LoadOnelabClient(const std::string &clientName, const std::string &sockName)
 {
 #if defined(HAVE_ONELAB)
-  if(!_onelabClient || val.empty()) return;
+  onelab::remoteNetworkClient *client = 0;
+  client = new onelab::remoteNetworkClient(clientName,sockName);
+  if(client){
+    std::string action, cmd;
+    std::vector<onelab::string> ps;
+    client->get(ps,clientName+"/Action");
+    if(ps.size() && ps[0].getValue().size())
+      action.assign(ps[0].getValue());
 
+    //cmd.assign("");
+    if(!action.compare("compute")){
+      std::vector<onelab::string> ps;
+      client->get(ps,clientName+"/FullCmdLine");
+      if(ps.size() && ps[0].getValue().size())
+	cmd.assign(ps[0].getValue());
+
+      if(cmd.size()){
+	Msg::Info("Loader calls <%s>",cmd.c_str());
+	//client->sendInfo(strcat("Loader calls",cmd.c_str()));
+	std::cout << "Loader calls " << cmd << std::endl;
+	SystemCall(cmd.c_str(),true); //true->blocking
+      }
+      else
+	Msg::Info("No full command line found for <%s>",
+		    clientName.c_str());
+    }
+    Msg::Info("Stopping client <%s>", clientName.c_str());
+    delete client;
+  }
+  exit(1);
+#endif
+}
+
+#if defined(HAVE_ONELAB)
+static void _setStandardOptions(onelab::parameter *p,
+                                std::map<std::string, std::vector<double> > &fopt,
+                                std::map<std::string, std::vector<std::string> > &copt)
+{
+  // strings
+  if(copt.count("Label")) p->setLabel(copt["Label"][0]);
+  if(copt.count("ShortHelp")) // for backward compatibility
+    p->setLabel(copt["ShortHelp"][0]);
+  if(copt.count("Help")) p->setHelp(copt["Help"][0]);
+  if(copt.count("Highlight")) p->setAttribute("Highlight", copt["Highlight"][0]);
+  if(copt.count("Macro")) p->setAttribute("Macro", copt["Macro"][0]);
+  if(copt.count("GmshOption")) p->setAttribute("GmshOption", copt["GmshOption"][0]);
+  if(copt.count("AutoCheck")) // for backward compatibility
+    p->setAttribute("AutoCheck", copt["AutoCheck"][0]);
+
+  // numbers
+  if(fopt.count("Visible")) p->setVisible(fopt["Visible"][0] ? true : false);
+  if(fopt.count("ReadOnly")) p->setReadOnly(fopt["ReadOnly"][0] ? true : false);
+  if(fopt.count("ReadOnlyRange"))
+    p->setAttribute("ReadOnlyRange", fopt["ReadOnlyRange"][0] ? "1" : "0");
+  if(fopt.count("AutoCheck"))
+    p->setAttribute("AutoCheck", fopt["AutoCheck"][0] ? "1" : "0");
+}
+
+static std::string _getParameterName(const std::string &key,
+                                     std::map<std::string, std::vector<std::string> > &copt)
+{
   std::string name(key);
   if(copt.count("Path")){
     std::string path = copt["Path"][0];
@@ -645,22 +824,41 @@ void Msg::ExchangeOnelabParameter(const std::string &key,
     else
       name = path + "/" + name;
   }
+  return name;
+}
+#endif
+
+void Msg::ExchangeOnelabParameter(const std::string &key,
+                                  std::vector<double> &val,
+                                  std::map<std::string, std::vector<double> > &fopt,
+                                  std::map<std::string, std::vector<std::string> > &copt)
+{
+#if defined(HAVE_ONELAB)
+  if(!_onelabClient || val.empty()) return;
+
+  std::string name = _getParameterName(key, copt);
 
   std::vector<onelab::number> ps;
   _onelabClient->get(ps, name);
-  bool noRange = true, noChoices = true, noLoop = true, noGraph = true;
+  bool noRange = true, noChoices = true, noLoop = true;
+  bool noGraph = true, noClosed = true;
   if(ps.size()){
-    if(ps[0].getReadOnly())
-      ps[0].setValue(val[0]); // use value from gmsh (so it is updated if necessary)
+    if(fopt.count("ReadOnly") && fopt["ReadOnly"][0])
+      ps[0].setValue(val[0]); // use local value
     else
       val[0] = ps[0].getValue(); // use value from server
-    // keep track of these attributes, which can be changed server-side
-    if(ps[0].getMin() != -onelab::parameter::maxNumber() ||
-       ps[0].getMax() != onelab::parameter::maxNumber() ||
-       ps[0].getStep() != 0.) noRange = false;
-    if(ps[0].getChoices().size()) noChoices = false;
+    // keep track of these attributes, which can be changed server-side (unless,
+    // for the range/choices, when explicitely setting these attributes as
+    // ReadOnly)
+    if(!(fopt.count("ReadOnlyRange") && fopt["ReadOnlyRange"][0])){
+      if(ps[0].getMin() != -onelab::parameter::maxNumber() ||
+         ps[0].getMax() != onelab::parameter::maxNumber() ||
+         ps[0].getStep() != 0.) noRange = false;
+      if(ps[0].getChoices().size()) noChoices = false;
+    }
     if(ps[0].getAttribute("Loop").size()) noLoop = false;
     if(ps[0].getAttribute("Graph").size()) noGraph = false;
+    if(ps[0].getAttribute("Closed").size()) noClosed = false;
   }
   else{
     ps.resize(1);
@@ -681,18 +879,80 @@ void Msg::ExchangeOnelabParameter(const std::string &key,
     ps[0].setMax(fopt["Max"][0]); ps[0].setMin(-onelab::parameter::maxNumber());
   }
   if(noRange && fopt.count("Step")) ps[0].setStep(fopt["Step"][0]);
+  // if no range/min/max/step info is provided, try to compute a reasonnable
+  // range and step (this makes the gui much nicer to use)
+  if(noRange && !fopt.count("Range") && !fopt.count("Step") &&
+     !fopt.count("Min") && !fopt.count("Max")){
+    bool isInteger = (floor(val[0]) == val[0]);
+    double fact = isInteger ? 5. : 20.;
+    if(val[0] > 0){
+      ps[0].setMin(val[0] / fact);
+      ps[0].setMax(val[0] * fact);
+      ps[0].setStep((ps[0].getMax() - ps[0].getMin()) / 100.);
+    }
+    else if(val[0] < 0){
+      ps[0].setMin(val[0] * fact);
+      ps[0].setMax(val[0] / fact);
+      ps[0].setStep((ps[0].getMax() - ps[0].getMin()) / 100.);
+    }
+    if(val[0] && isInteger){
+      ps[0].setMin((int)ps[0].getMin());
+      ps[0].setMax((int)ps[0].getMax());
+      ps[0].setStep((int)ps[0].getStep());
+    }
+  }
   if(noChoices && fopt.count("Choices")){
     ps[0].setChoices(fopt["Choices"]);
     if(copt.count("Choices")) ps[0].setChoiceLabels(copt["Choices"]);
   }
-  if(fopt.count("Visible")) ps[0].setVisible(fopt["Visible"][0] ? true : false);
-  if(fopt.count("ReadOnly")) ps[0].setReadOnly(fopt["ReadOnly"][0] ? true : false);
-  if(copt.count("Help")) ps[0].setHelp(copt["Help"][0]);
-  if(copt.count("Label")) ps[0].setLabel(copt["Label"][0]);
-  if(copt.count("ShortHelp")) ps[0].setLabel(copt["ShortHelp"][0]);
   if(noLoop && copt.count("Loop")) ps[0].setAttribute("Loop", copt["Loop"][0]);
   if(noGraph && copt.count("Graph")) ps[0].setAttribute("Graph", copt["Graph"][0]);
-  if(copt.count("Highlight")) ps[0].setAttribute("Highlight", copt["Highlight"][0]);
+  if(noClosed && copt.count("Closed")) // for backward compatibility
+    ps[0].setAttribute("Closed", copt["Closed"][0]);
+  if(noClosed && fopt.count("Closed"))
+    ps[0].setAttribute("Closed", fopt["Closed"][0] ? "1" : "0");
+  _setStandardOptions(&ps[0], fopt, copt);
+  _onelabClient->set(ps[0]);
+#endif
+}
+
+void Msg::ExchangeOnelabParameter(const std::string &key,
+                                  std::string &val,
+                                  std::map<std::string, std::vector<double> > &fopt,
+                                  std::map<std::string, std::vector<std::string> > &copt)
+{
+#if defined(HAVE_ONELAB)
+  if(!_onelabClient || val.empty()) return;
+
+  std::string name = _getParameterName(key, copt);
+
+  std::vector<onelab::string> ps;
+  _onelabClient->get(ps, name);
+  bool noChoices = true, noClosed = true, noMultipleSelection = true;
+  if(ps.size()){
+    if(fopt.count("ReadOnly") && fopt["ReadOnly"][0])
+      ps[0].setValue(val); // use local value
+    else
+      val = ps[0].getValue(); // use value from server
+    // keep track of these attributes, which can be changed server-side
+    if(ps[0].getChoices().size()) noChoices = false;
+    if(ps[0].getAttribute("Closed").size()) noClosed = false;
+    if(ps[0].getAttribute("MultipleSelection").size()) noMultipleSelection = false;
+  }
+  else{
+    ps.resize(1);
+    ps[0].setName(name);
+    ps[0].setValue(val);
+  }
+  if(copt.count("Kind")) ps[0].setKind(copt["Kind"][0]);
+  if(noChoices && copt.count("Choices")) ps[0].setChoices(copt["Choices"]);
+  if(noClosed && copt.count("Closed")) // for backward compatibility
+    ps[0].setAttribute("Closed", copt["Closed"][0]);
+  if(noClosed && fopt.count("Closed"))
+    ps[0].setAttribute("Closed", fopt["Closed"][0] ? "1" : "0");
+  if(noMultipleSelection && copt.count("MultipleSelection"))
+    ps[0].setAttribute("MultipleSelection", copt["MultipleSelection"][0]);
+  _setStandardOptions(&ps[0], fopt, copt);
   _onelabClient->set(ps[0]);
 #endif
 }
@@ -703,7 +963,7 @@ void Msg::ImportPhysicalsAsOnelabRegions()
   if(_onelabClient){
     std::map<int, std::vector<GEntity*> > groups[4];
     GModel::current()->getPhysicalGroups(groups);
-    for(int dim = 0; dim < 3; dim++){
+    for(int dim = 0; dim <= 3; dim++){
       for(std::map<int, std::vector<GEntity*> >::iterator it = groups[dim].begin();
           it != groups[dim].end(); it++){
         // create "read-only" onelab region
@@ -714,10 +974,12 @@ void Msg::ImportPhysicalsAsOnelabRegions()
           name = std::string("Physical") +
             ((dim == 3) ? "Volume" : (dim == 2) ? "Surface" :
              (dim == 1) ? "Line" : "Point") + num.str();
-        name.insert(0, "Gmsh/Physical groups/");
+        name.insert(0, "Gmsh parameters/Physical groups/");
         onelab::region p(name, num.str());
         p.setDimension(dim);
         p.setReadOnly(true);
+        p.setVisible(false);
+        p.setAttribute("Closed", "1");
         _onelabClient->set(p);
       }
     }
