@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #ifndef _GMSH_MESSAGE_H_
 #define _GMSH_MESSAGE_H_
@@ -36,6 +36,7 @@ class Msg {
   static std::map<std::string, double> _timers;
   // counters
   static int _warningCount, _errorCount;
+  static std::string _firstWarning, _firstError;
   // callback
   static GmshMessage *_callback;
   // command-line and startup time
@@ -44,6 +45,8 @@ class Msg {
   static GmshClient *_client;
   // communication with onelab server
   static onelab::client *_onelabClient;
+  // executable name
+  static std::string _execName;
  public:
   Msg() {}
   static void Init(int argc, char **argv);
@@ -66,30 +69,43 @@ class Msg {
   static void Warning(const char *fmt, ...);
   static void Info(const char *fmt, ...);
   static void Direct(const char *fmt, ...);
-  static void Direct(int level, const char *fmt, ...);
-  static void StatusBar(int num, bool log, const char *fmt, ...);
+  static void StatusBar(bool log, const char *fmt, ...);
+  static void StatusGl(const char *fmt, ...);
   static void Debug(const char *fmt, ...);
-  static void ProgressMeter(int n, int N, const char *fmt, ...);
-  static void ProgressMeter(int n, int N){ ProgressMeter(n, N, ""); }
+  static void ProgressMeter(int n, int N, bool log, const char *fmt, ...);
   static void SetProgressMeterStep(int step){ _progressMeterStep = step; }
+  static int GetProgressMeterStep(){ return _progressMeterStep; }
   static void ResetProgressMeter(){ if(!_commRank) _progressMeterCurrent = 0; }
   static double &Timer(std::string str){ return _timers[str]; }
   static void PrintTimers();
-  static void ResetErrorCounter(){ _warningCount = 0; _errorCount = 0; }
-  static int GetErrorCount(){ return _errorCount; }
+  static void ResetErrorCounter();
   static void PrintErrorCounter(const char *title);
+  static int GetWarningCount(){ return _warningCount; }
+  static int GetErrorCount(){ return _errorCount; }
+  static std::string GetFirstWarning(){ return _firstWarning; }
+  static std::string GetFirstError(){ return _firstError; }
   static double GetValue(const char *text, double defaultval);
   static std::string GetString(const char *text, std::string defaultval);
   static int GetAnswer(const char *question, int defaultval, const char *zero,
                        const char *one, const char *two=0);
   static void InitializeOnelab(const std::string &name, const std::string &sockname="");
+  static void SetExecutableName(const std::string &name) { _execName.assign(name); }
+  static std::string GetExecutableName() { return _execName; }
+  static void LoadOnelabClient(const std::string &name, const std::string &sockName);
   static GmshClient *GetGmshClient(){ return _client; }
+  static onelab::client *GetOnelabClient(){ return _onelabClient; }
   static void FinalizeOnelab();
   static bool UseOnelab();
+  static void SetOnelabNumber(std::string name, double val, bool visible);
+  static void SetOnelabString(std::string name, std::string val, bool visible);
   static void ExchangeOnelabParameter(const std::string &key,
                                       std::vector<double> &val,
                                       std::map<std::string, std::vector<double> > &fopt,
                                       std::map<std::string, std::vector<std::string> > &copt);
+  static void ExchangeOnelabParameter(const std::string &key,
+                                      std::string &val,
+                                      std::map<std::string, std::vector<double> > &fopt,
+                                      std::map<std::string, std::vector<std::string> > &copt);
   static void ImportPhysicalsAsOnelabRegions();
 };
 
