@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // Permission is hereby granted, free of charge, to any person
 // obtaining a copy of this software and associated documentation
@@ -22,12 +22,15 @@
 // ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
 // OF THIS SOFTWARE.
 //
-// Please report all bugs and problems to <gmsh@geuz.org>.
+// Please report all bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #ifndef _GMSH_SOCKET_H_
 #define _GMSH_SOCKET_H_
 
+#if !defined(ONELAB_LOADER)
 #include "GmshConfig.h"
+#endif
+
 #include <string>
 #include <stdio.h>
 #include <stdlib.h>
@@ -75,6 +78,7 @@ class GmshSocket{
     GMSH_PARAMETER_QUERY = 24,
     GMSH_PARAM_QUERY_ALL = 25,
     GMSH_PARAM_QUERY_END = 26,
+    GMSH_CONNECT         = 27,
     GMSH_SPEED_TEST      = 30,
     GMSH_OPTION_1        = 100,
     GMSH_OPTION_2        = 101,
@@ -195,7 +199,7 @@ class GmshSocket{
   int ReceiveHeader(int *type, int *len, int *swap)
   {
     *swap = 0;
-    if(_ReceiveData(type, sizeof(int))){
+    if(_ReceiveData(type, sizeof(int)) > 0){
       if(*type < 0) return 0;
       if(*type > 65535){
         // the data comes from a machine with different endianness and
@@ -203,7 +207,7 @@ class GmshSocket{
         *swap = 1;
         _SwapBytes((char*)type, sizeof(int), 1);
       }
-      if(_ReceiveData(len, sizeof(int))){
+      if(_ReceiveData(len, sizeof(int)) > 0){
         if(*len < 0) return 0;
         if(*swap) _SwapBytes((char*)len, sizeof(int), 1);
         return 1;
@@ -363,7 +367,7 @@ class GmshServer : public GmshSocket{
 #if !defined(WIN32) || defined(__CYGWIN__)
       if(tmpsock < 0)
 #else
-      if(tmpsock == INVALID_SOCKET)
+	if(tmpsock == (int)INVALID_SOCKET)
 #endif
         throw "Couldn't create socket";
       // bind the socket to its name
