@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include <sstream>
 #include <string.h>
@@ -20,6 +20,10 @@
 #include "GeomMeshMatcher.h"
 #include "Field.h"
 
+#if defined(HAVE_POPPLER)
+#include "gmshPopplerWrapper.h"
+#endif
+
 #if defined(HAVE_PARSER)
 #include "Parser.h"
 #endif
@@ -37,7 +41,8 @@
 #if defined(HAVE_FLTK)
 #include <FL/fl_ask.H>
 #include "FlGui.h"
-#include "menuWindow.h"
+#include "onelabGroup.h"
+#include "graphicWindow.h"
 #include "drawContext.h"
 #endif
 
@@ -152,9 +157,25 @@ void AddToTemporaryBoundingBox(double x, double y, double z)
   for(int i = 0; i < 3; i++) CTX::instance()->cg[i] = temp_bb.center()[i];
 }
 
+static void ComputeMaxEntityNum()
+{
+  GModel::current()->getGEOInternals()->MaxPointNum =
+    std::max(GModel::current()->getGEOInternals()->MaxPointNum,
+             GModel::current()->getMaxElementaryNumber(0));
+  GModel::current()->getGEOInternals()->MaxLineNum =
+    std::max(GModel::current()->getGEOInternals()->MaxLineNum,
+             GModel::current()->getMaxElementaryNumber(1));
+  GModel::current()->getGEOInternals()->MaxSurfaceNum =
+    std::max(GModel::current()->getGEOInternals()->MaxSurfaceNum,
+             GModel::current()->getMaxElementaryNumber(2));
+  GModel::current()->getGEOInternals()->MaxVolumeNum =
+    std::max(GModel::current()->getGEOInternals()->MaxVolumeNum,
+             GModel::current()->getMaxElementaryNumber(3));
+}
+
 static std::vector<FILE*> openedFiles;
 
-int ParseFile(std::string fileName, bool close, bool warnIfMissing)
+int ParseFile(const std::string &fileName, bool close, bool warnIfMissing)
 {
 #if !defined(HAVE_PARSER)
   Msg::Error("Gmsh parser is not compiled in this version");
@@ -189,7 +210,8 @@ int ParseFile(std::string fileName, bool close, bool warnIfMissing)
   while(!feof(gmsh_yyin)){
     gmsh_yyparse();
     if(gmsh_yyerrorstate > 20){
-      Msg::Error("Too many errors: aborting...");
+      if(gmsh_yyerrorstate != 999) // 999 is a volontary exit
+        Msg::Error("Too many errors: aborting parser...");
       gmsh_yyflush();
       break;
     }
@@ -218,7 +240,7 @@ int ParseFile(std::string fileName, bool close, bool warnIfMissing)
 #endif
 }
 
-void ParseString(std::string str)
+void ParseString(const std::string &str)
 {
   if(str.empty()) return;
   std::string fileName = CTX::instance()->homeDir + CTX::instance()->tmpFileName;
@@ -230,7 +252,7 @@ void ParseString(std::string str)
   }
 }
 
-int MergeFile(std::string fileName, bool warnIfMissing)
+int MergeFile(const std::string &fileName, bool warnIfMissing)
 {
   if(GModel::current()->getName() == ""){
     GModel::current()->setFileName(fileName);
@@ -255,7 +277,7 @@ int MergeFile(std::string fileName, bool warnIfMissing)
   if(!fgets(header, sizeof(header), fp)) return 0;
   fclose(fp);
 
-  Msg::StatusBar(2, true, "Reading '%s'...", fileName.c_str());
+  Msg::StatusBar(true, "Reading '%s'...", fileName.c_str());
 
   std::vector<std::string> split = SplitFileName(fileName);
   std::string noExt = split[0] + split[1], ext = split[2];
@@ -317,6 +339,14 @@ int MergeFile(std::string fileName, bool warnIfMissing)
   else if(ext == ".diff" || ext == ".DIFF"){
     status = GModel::current()->readDIFF(fileName);
   }
+  else if(ext == ".pdf" || ext == ".PDF"){
+#if defined(HAVE_POPPLER)
+    status = gmshPopplerWrapper::instance()->load_from_file(fileName);
+#else
+    Msg::Error("Gmsh has to be compiled with POPPLER for displaying PDF documents");
+    status = 0;
+#endif
+  }
   else if(ext == ".med" || ext == ".MED" || ext == ".mmed" || ext == ".MMED" ||
           ext == ".rmed" || ext == ".RMED"){
     status = GModel::readMED(fileName);
@@ -371,6 +401,26 @@ int MergeFile(std::string fileName, bool warnIfMissing)
   else if(ext == ".csv"){
     status = readFile3M(fileName);
   }
+#endif
+#if defined(HAVE_ONELAB) && defined(HAVE_FLTK)
+  else if(ext == ".pro" && opt_solver_name(0, GMSH_GET, "") == "GetDP"){
+    std::vector<std::string> split = SplitFileName(fileName);
+    GModel::current()->setName("");
+    status = MergeFile(split[0] + split[1] + ".geo");
+    CTX::instance()->launchSolverAtStartup = 0;
+    return status;
+  }
+#endif
+#if defined(HAVE_ONELAB_METAMODEL) && defined(HAVE_FLTK)
+  else if(ext == ".ol"){
+    // FIXME: this is a hack -- think about a better way
+    status = metamodel_cb(fileName);
+  }
+  else if(ext == ".py"){
+    FlGui::instance()->onelab->addSolver("python", fileName, "", 1);
+    onelab_cb(0, (void*)"check");
+    status = 1;
+  }
 #endif
   else {
     CTX::instance()->geom.draw = 1;
@@ -410,6 +460,7 @@ int MergeFile(std::string fileName, bool warnIfMissing)
     }
   }
 
+  ComputeMaxEntityNum();
   SetBoundingBox();
 
   CTX::instance()->geom.draw = 1;
@@ -428,7 +479,7 @@ int MergeFile(std::string fileName, bool warnIfMissing)
   Msg::ImportPhysicalsAsOnelabRegions();
 
   if(!status) Msg::Error("Error loading '%s'", fileName.c_str());
-  Msg::StatusBar(2, true, "Done reading '%s'", fileName.c_str());
+  Msg::StatusBar(true, "Done reading '%s'", fileName.c_str());
 
    // merge the associated option file if there is one
   if(!StatFile(fileName + ".opt"))
@@ -437,8 +488,88 @@ int MergeFile(std::string fileName, bool warnIfMissing)
   return status;
 }
 
+int MergePostProcessingFile(const std::string &fileName, bool showLastStep,
+                            bool hideNewViews, bool warnIfMissing)
+{
+#if defined(HAVE_POST)
+  // check if there is a mesh in the file
+  FILE *fp = fopen(fileName.c_str(), "rb");
+  if(!fp){
+    if(warnIfMissing) Msg::Warning("Unable to open file '%s'", fileName.c_str());
+    return 0;
+  }
+  char header[256];
+  if(!fgets(header, sizeof(header), fp)) return 0;
+  bool haveMesh = false;
+  if(!strncmp(header, "$MeshFormat", 11)){
+    while(!feof(fp) && fgets(header, sizeof(header), fp)){
+      if(!strncmp(header, "$Nodes", 6)){
+        haveMesh = true;
+        break;
+      }
+      else if(!strncmp(header, "$NodeData", 9) ||
+              !strncmp(header, "$ElementData", 12) ||
+              !strncmp(header, "$ElementNodeData", 16)){
+        break;
+      }
+    }
+  }
+  fclose(fp);
+
+  // store old step values
+  unsigned int n = PView::list.size();
+  std::vector<int> steps(n, 0);
+  if(showLastStep){
+    for(unsigned int i = 0; i < PView::list.size(); i++)
+      steps[i] = (int)opt_view_nb_timestep(i, GMSH_GET, 0);
+  }
+
+  // if there is a mesh, create a new model to store it (don't merge elements in
+  // the current mesh!)
+  GModel *old = GModel::current();
+  if(haveMesh){
+    GModel *m = new GModel();
+    GModel::setCurrent(m);
+  }
+  int ret = MergeFile(fileName, warnIfMissing);
+  GModel::setCurrent(old);
+  old->setVisibility(1);
+
+  // hide everything except the onelab X-Y graphs
+  if(hideNewViews){
+    for(unsigned int i = 0; i < PView::list.size(); i++){
+      if(PView::list[i]->getData()->getFileName().substr(0, 6) != "OneLab")
+        PView::list[i]->getOptions()->visible = 0;
+    }
+  }
+  else if(n != PView::list.size()){
+    // if we created new views, assume we only want to see those (and the
+    // onelab X-Y graphs)
+    for(unsigned int i = 0; i < n; i++){
+      if(PView::list[i]->getData()->getFileName().substr(0, 6) != "OneLab")
+        PView::list[i]->getOptions()->visible = 0;
+    }
+  }
+
+  // if we added steps, go to the last one
+  if(showLastStep){
+    steps.resize(PView::list.size(), 0);
+    for(unsigned int i = 0; i < PView::list.size(); i++){
+      int step = (int)opt_view_nb_timestep(i, GMSH_GET, 0);
+      if(step > steps[i])
+        opt_view_timestep(i, GMSH_SET|GMSH_GUI, step - 1);
+    }
+  }
+
+  return ret;
+#else
+  return 0;
+#endif
+}
+
 void ClearProject()
 {
+  Msg::Info("Clearing all models and views...");
 #if defined(HAVE_POST)
   for(int i = PView::list.size() - 1; i >= 0; i--)
     delete PView::list[i];
@@ -455,6 +586,7 @@ void ClearProject()
       fclose(openedFiles[i]);
     openedFiles.clear();
   }
+  Msg::Info("Done clearing all models and views");
 
   new GModel();
   GModel::current()->setFileName(CTX::instance()->defaultFileName);
@@ -468,9 +600,10 @@ void ClearProject()
     GModel::current()->setSelection(0);
   }
 #endif
+  Msg::ResetErrorCounter();
 }
 
-void OpenProject(std::string fileName)
+void OpenProject(const std::string &fileName)
 {
   if(CTX::instance()->lock) {
     Msg::Info("I'm busy! Ask me that later...");
@@ -478,6 +611,8 @@ void OpenProject(std::string fileName)
   }
   CTX::instance()->lock = 1;
 
+  Msg::ResetErrorCounter();
+
   if(GModel::current()->empty()){
     // if the current model is empty, make sure it's reaaally
     // cleaned-up, and reuse it (don't clear the parser variables: if
@@ -510,7 +645,7 @@ void OpenProject(std::string fileName)
       CTX::instance()->recentFiles.resize(5);
 #if defined(HAVE_FLTK)
     if(FlGui::available())
-      FlGui::instance()->menu->fillRecentHistoryMenu();
+      FlGui::instance()->graph[0]->fillRecentHistoryMenu();
 #endif
   }
 
@@ -538,8 +673,12 @@ void OpenProject(std::string fileName)
 void OpenProjectMacFinder(const char *fileName)
 {
 #if defined(HAVE_FLTK)
-  if(FlGui::available()){
-    FlGui::instance()->setOpenedThroughMacFinder(true);
+  if(!FlGui::available()){
+    // Gmsh is not ready: will open the file later
+    FlGui::setOpenedThroughMacFinder(fileName);
+  }
+  else if(FlGui::available()){
+    // Gmsh is running
     OpenProject(fileName);
     drawContext::global()->draw();
   }
