@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #ifndef _OPTIONS_H_
 #define _OPTIONS_H_
@@ -9,9 +9,11 @@
 #include <string>
 #include "ColorTable.h"
 
-#define GMSH_SET       (1<<0)
-#define GMSH_GET       (1<<1)
-#define GMSH_GUI       (1<<2)
+#define GMSH_SET         (1<<0)
+#define GMSH_GET         (1<<1)
+#define GMSH_GUI         (1<<2)
+#define GMSH_SET_DEFAULT (1<<3)
+#define GMSH_GET_DEFAULT (1<<4)
 
 #define GMSH_SESSIONRC (1<<0)
 #define GMSH_OPTIONSRC (1<<1)
@@ -45,7 +47,6 @@ std::string opt_general_recent_file2(OPT_ARGS_STR);
 std::string opt_general_recent_file3(OPT_ARGS_STR);
 std::string opt_general_recent_file4(OPT_ARGS_STR);
 std::string opt_general_editor(OPT_ARGS_STR);
-std::string opt_general_web_browser(OPT_ARGS_STR);
 std::string opt_general_watch_file_pattern(OPT_ARGS_STR);
 std::string opt_general_gui_theme(OPT_ARGS_STR);
 std::string opt_general_graphics_font(OPT_ARGS_STR);
@@ -91,6 +92,7 @@ std::string opt_view_stipple6(OPT_ARGS_STR);
 std::string opt_view_stipple7(OPT_ARGS_STR);
 std::string opt_view_stipple8(OPT_ARGS_STR);
 std::string opt_view_stipple9(OPT_ARGS_STR);
+std::string opt_view_attributes(OPT_ARGS_STR);
 
 // NUMBERS
 
@@ -102,10 +104,6 @@ double opt_general_graphics_position0(OPT_ARGS_NUM);
 double opt_general_graphics_position1(OPT_ARGS_NUM);
 double opt_general_graphics_size0(OPT_ARGS_NUM);
 double opt_general_graphics_size1(OPT_ARGS_NUM);
-double opt_general_solver_position0(OPT_ARGS_NUM);
-double opt_general_solver_position1(OPT_ARGS_NUM);
-double opt_general_solver_size0(OPT_ARGS_NUM);
-double opt_general_solver_size1(OPT_ARGS_NUM);
 double opt_general_context_position0(OPT_ARGS_NUM);
 double opt_general_context_position1(OPT_ARGS_NUM);
 double opt_general_file_chooser_position0(OPT_ARGS_NUM);
@@ -113,10 +111,13 @@ double opt_general_file_chooser_position1(OPT_ARGS_NUM);
 double opt_general_polygon_offset_always(OPT_ARGS_NUM);
 double opt_general_polygon_offset_factor(OPT_ARGS_NUM);
 double opt_general_polygon_offset_units(OPT_ARGS_NUM);
-double opt_general_menu_position0(OPT_ARGS_NUM);
-double opt_general_menu_position1(OPT_ARGS_NUM);
 double opt_general_system_menu_bar(OPT_ARGS_NUM);
 double opt_general_message_size(OPT_ARGS_NUM);
+double opt_general_detached_menu(OPT_ARGS_NUM);
+double opt_general_menu_size0(OPT_ARGS_NUM);
+double opt_general_menu_size1(OPT_ARGS_NUM);
+double opt_general_menu_position0(OPT_ARGS_NUM);
+double opt_general_menu_position1(OPT_ARGS_NUM);
 double opt_general_option_position0(OPT_ARGS_NUM);
 double opt_general_option_position1(OPT_ARGS_NUM);
 double opt_general_plugin_position0(OPT_ARGS_NUM);
@@ -127,6 +128,10 @@ double opt_general_field_position0(OPT_ARGS_NUM);
 double opt_general_field_position1(OPT_ARGS_NUM);
 double opt_general_field_size0(OPT_ARGS_NUM);
 double opt_general_field_size1(OPT_ARGS_NUM);
+double opt_general_extra_position0(OPT_ARGS_NUM);
+double opt_general_extra_position1(OPT_ARGS_NUM);
+double opt_general_extra_size0(OPT_ARGS_NUM);
+double opt_general_extra_size1(OPT_ARGS_NUM);
 double opt_general_statistics_position0(OPT_ARGS_NUM);
 double opt_general_statistics_position1(OPT_ARGS_NUM);
 double opt_general_visibility_position0(OPT_ARGS_NUM);
@@ -164,6 +169,7 @@ double opt_general_background_gradient(OPT_ARGS_NUM);
 double opt_general_background_image_position0(OPT_ARGS_NUM);
 double opt_general_background_image_position1(OPT_ARGS_NUM);
 double opt_general_verbosity(OPT_ARGS_NUM);
+double opt_general_progress_meter_step(OPT_ARGS_NUM);
 double opt_general_nopopup(OPT_ARGS_NUM);
 double opt_general_non_modal_windows(OPT_ARGS_NUM);
 double opt_general_terminal(OPT_ARGS_NUM);
@@ -193,6 +199,13 @@ double opt_general_axes_zmax(OPT_ARGS_NUM);
 double opt_general_axes_tics0(OPT_ARGS_NUM);
 double opt_general_axes_tics1(OPT_ARGS_NUM);
 double opt_general_axes_tics2(OPT_ARGS_NUM);
+double opt_general_axes_force_value(OPT_ARGS_NUM);
+double opt_general_axes_value_xmin(OPT_ARGS_NUM);
+double opt_general_axes_value_xmax(OPT_ARGS_NUM);
+double opt_general_axes_value_ymin(OPT_ARGS_NUM);
+double opt_general_axes_value_ymax(OPT_ARGS_NUM);
+double opt_general_axes_value_zmin(OPT_ARGS_NUM);
+double opt_general_axes_value_zmax(OPT_ARGS_NUM);
 double opt_general_small_axes(OPT_ARGS_NUM);
 double opt_general_small_axes_position0(OPT_ARGS_NUM);
 double opt_general_small_axes_position1(OPT_ARGS_NUM);
@@ -287,6 +300,7 @@ double opt_geometry_offset1(OPT_ARGS_NUM);
 double opt_geometry_offset2(OPT_ARGS_NUM);
 double opt_geometry_auto_coherence(OPT_ARGS_NUM);
 double opt_geometry_hide_compounds(OPT_ARGS_NUM);
+double opt_geometry_oriented_physicals(OPT_ARGS_NUM);
 double opt_geometry_highlight_orphans(OPT_ARGS_NUM);
 double opt_geometry_tolerance(OPT_ARGS_NUM);
 double opt_geometry_normals(OPT_ARGS_NUM);
@@ -299,6 +313,7 @@ double opt_geometry_points_num(OPT_ARGS_NUM);
 double opt_geometry_lines_num(OPT_ARGS_NUM);
 double opt_geometry_surfaces_num(OPT_ARGS_NUM);
 double opt_geometry_volumes_num(OPT_ARGS_NUM);
+double opt_geometry_label_type(OPT_ARGS_NUM);
 double opt_geometry_point_size(OPT_ARGS_NUM);
 double opt_geometry_point_sel_size(OPT_ARGS_NUM);
 double opt_geometry_point_type(OPT_ARGS_NUM);
@@ -315,6 +330,7 @@ double opt_geometry_occ_sew_faces(OPT_ARGS_NUM);
 double opt_geometry_occ_connect_faces(OPT_ARGS_NUM);
 double opt_geometry_old_circle(OPT_ARGS_NUM);
 double opt_geometry_old_newreg(OPT_ARGS_NUM);
+double opt_geometry_old_ruled_surface(OPT_ARGS_NUM);
 double opt_geometry_num_sub_edges(OPT_ARGS_NUM);
 double opt_geometry_extrude_spline_points(OPT_ARGS_NUM);
 double opt_geometry_extrude_return_lateral(OPT_ARGS_NUM);
@@ -389,12 +405,15 @@ double opt_mesh_partition_tet_weight(OPT_ARGS_NUM);
 double opt_mesh_partition_tri_weight(OPT_ARGS_NUM);
 double opt_mesh_binary(OPT_ARGS_NUM);
 double opt_mesh_bunin(OPT_ARGS_NUM);
+double opt_mesh_lloyd(OPT_ARGS_NUM);
+double opt_mesh_smooth_cross_field(OPT_ARGS_NUM);
 double opt_mesh_bdf_field_format(OPT_ARGS_NUM);
 double opt_mesh_nb_smoothing(OPT_ARGS_NUM);
 double opt_mesh_algo2d(OPT_ARGS_NUM);
 double opt_mesh_algo3d(OPT_ARGS_NUM);
 double opt_mesh_algo_recombine(OPT_ARGS_NUM);
 double opt_mesh_recombine_all(OPT_ARGS_NUM);
+double opt_mesh_recombine3d_all(OPT_ARGS_NUM);
 double opt_mesh_remesh_algo(OPT_ARGS_NUM);
 double opt_mesh_remesh_param(OPT_ARGS_NUM);
 double opt_mesh_algo_subdivide(OPT_ARGS_NUM);
@@ -456,6 +475,13 @@ double opt_mesh_ignore_part_bound(OPT_ARGS_NUM);
 double opt_solver_listen(OPT_ARGS_NUM);
 double opt_solver_timeout(OPT_ARGS_NUM);
 double opt_solver_plugins(OPT_ARGS_NUM);
+double opt_solver_auto_save_database(OPT_ARGS_NUM);
+double opt_solver_auto_archive_output_files(OPT_ARGS_NUM);
+double opt_solver_auto_check(OPT_ARGS_NUM);
+double opt_solver_auto_mesh(OPT_ARGS_NUM);
+double opt_solver_auto_merge_file(OPT_ARGS_NUM);
+double opt_solver_auto_hide_new_views(OPT_ARGS_NUM);
+double opt_solver_auto_show_last_step(OPT_ARGS_NUM);
 double opt_post_horizontal_scales(OPT_ARGS_NUM);
 double opt_post_link(OPT_ARGS_NUM);
 double opt_post_smooth(OPT_ARGS_NUM);
@@ -473,6 +499,8 @@ double opt_view_min(OPT_ARGS_NUM);
 double opt_view_max(OPT_ARGS_NUM);
 double opt_view_custom_min(OPT_ARGS_NUM);
 double opt_view_custom_max(OPT_ARGS_NUM);
+double opt_view_custom_abscissa_min(OPT_ARGS_NUM);
+double opt_view_custom_abscissa_max(OPT_ARGS_NUM);
 double opt_view_xmin(OPT_ARGS_NUM);
 double opt_view_xmax(OPT_ARGS_NUM);
 double opt_view_ymin(OPT_ARGS_NUM);
@@ -567,6 +595,7 @@ double opt_view_draw_skin_only(OPT_ARGS_NUM);
 double opt_view_scale_type(OPT_ARGS_NUM);
 double opt_view_tensor_type(OPT_ARGS_NUM);
 double opt_view_range_type(OPT_ARGS_NUM);
+double opt_view_abscissa_range_type(OPT_ARGS_NUM);
 double opt_view_vector_type(OPT_ARGS_NUM);
 double opt_view_glyph_location(OPT_ARGS_NUM);
 double opt_view_center_glyphs(OPT_ARGS_NUM);
@@ -612,6 +641,8 @@ double opt_print_text(OPT_ARGS_NUM);
 double opt_print_tex_as_equation(OPT_ARGS_NUM);
 double opt_print_composite_windows(OPT_ARGS_NUM);
 double opt_print_delete_tmp_files(OPT_ARGS_NUM);
+double opt_print_height(OPT_ARGS_NUM);
+double opt_print_width(OPT_ARGS_NUM);
 
 // COLORS
 
@@ -718,7 +749,8 @@ typedef struct {
 void InitOptions(int num);
 void InitOptionsGUI(int num);
 void ReInitOptions(int num);
-void PrintOptions(int num, int level, int diff, int help, const char *filename);
+void PrintOptions(int num, int level, int diff, int help, const char *filename,
+                  std::vector<std::string> *vec=0);
 void PrintOptionsDoc();
 
 bool StringOption(int action, const char *category, int num,
