@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include "GmshConfig.h"
 #if !defined(HAVE_NO_STDINT_H)
@@ -15,11 +15,10 @@ typedef unsigned long intptr_t;
 #include <FL/Fl_Tooltip.H>
 #include <FL/Fl_Shared_Image.H>
 #include <FL/Fl_File_Icon.H>
-#include <FL/x.H>
-#include <FL/gl.h>
+#include <FL/fl_draw.H>
 #include "FlGui.h"
+#include "drawContextFltk.h"
 #include "graphicWindow.h"
-#include "menuWindow.h"
 #include "optionWindow.h"
 #include "fieldWindow.h"
 #include "pluginWindow.h"
@@ -29,8 +28,8 @@ typedef unsigned long intptr_t;
 #include "clippingWindow.h"
 #include "manipWindow.h"
 #include "contextWindow.h"
-#include "onelabWindow.h"
-#include "aboutWindow.h"
+#include "onelabGroup.h"
+#include "helpWindow.h"
 #include "colorbarWindow.h"
 #include "fileDialogs.h"
 #include "GmshDefines.h"
@@ -42,13 +41,17 @@ typedef unsigned long intptr_t;
 #include "Plugin.h"
 #include "PluginManager.h"
 #include "OpenFile.h"
-#include "Win32Icon.h"
+#include "WinIcon.h"
 #include "Options.h"
 #include "CommandLine.h"
 #include "Context.h"
 #include "StringUtils.h"
 #include "Generator.h"
 #include "gl2ps.h"
+#include "gmshPopplerWrapper.h"
+#if defined(HAVE_3M)
+#include "3M.h"
+#endif
 
 // check (now!) if there are any pending events, and process them
 void FlGui::check(){ Fl::check(); }
@@ -59,154 +62,163 @@ void FlGui::wait(){ Fl::wait(); }
 // wait (at most time seconds) for any events, then process them
 void FlGui::wait(double time){ Fl::wait(time); }
 
-class drawContextFltk : public drawContextGlobal{
- public:
-  void draw()
-  {
-    if(!FlGui::available()) return;
-    for(unsigned int i = 0; i < FlGui::instance()->graph.size(); i++){
-      for(unsigned int j = 0; j < FlGui::instance()->graph[i]->gl.size(); j++){
-        FlGui::instance()->graph[i]->gl[j]->make_current();
-        FlGui::instance()->graph[i]->gl[j]->redraw();
-	// to initialize the camera distance from model
-	drawContext * ctx = FlGui::instance()->graph[i]->gl[j]->getDrawContext();
-	ctx->camera.update();
-      }
-    }
-    FlGui::instance()->check();
-  }
-  void drawCurrentOpenglWindow(bool make_current)
-  {
-    if(!FlGui::available()) return;
-    openglWindow *gl = FlGui::instance()->getCurrentOpenglWindow();
-    if(make_current) gl->make_current();
-    gl->redraw();
-    glFlush();
-    FlGui::instance()->check();
-  }
-  int getFontIndex(const char *fontname)
-  {
-    if(fontname){
-      for(int i = 0; i < NUM_FONTS; i++)
-        if(!strcmp(menu_font_names[i].label(), fontname))
-          return i;
-    }
-    Msg::Error("Unknown font \"%s\" (using \"Helvetica\" instead)", fontname);
-    Msg::Info("Available fonts:");
-    for(int i = 0; i < NUM_FONTS; i++)
-      Msg::Info("  \"%s\"", menu_font_names[i].label());
-    return 4;
-  }
-  int getFontEnum(int index)
-  {
-    if(index >= 0 && index < NUM_FONTS)
-      return (intptr_t)menu_font_names[index].user_data();
-    return FL_HELVETICA;
-  }
-  const char *getFontName(int index)
-  {
-    if(index >= 0 && index < NUM_FONTS)
-      return menu_font_names[index].label();
-    return "Helvetica";
-  }
-  int getFontAlign(const char *alignstr)
-  {
-    if(alignstr){
-      if(!strcmp(alignstr, "BottomLeft") || !strcmp(alignstr, "Left") ||
-         !strcmp(alignstr, "left"))
-        return 0;
-      else if(!strcmp(alignstr, "BottomCenter") || !strcmp(alignstr, "Center") ||
-              !strcmp(alignstr, "center"))
-        return 1;
-      else if(!strcmp(alignstr, "BottomRight") || !strcmp(alignstr, "Right") ||
-              !strcmp(alignstr, "right"))
-        return 2;
-      else if(!strcmp(alignstr, "TopLeft"))
-        return 3;
-      else if(!strcmp(alignstr, "TopCenter"))
-        return 4;
-      else if(!strcmp(alignstr, "TopRight"))
-        return 5;
-      else if(!strcmp(alignstr, "CenterLeft"))
-        return 6;
-      else if(!strcmp(alignstr, "CenterCenter"))
-        return 7;
-      else if(!strcmp(alignstr, "CenterRight"))
-        return 8;
-    }
-    Msg::Error("Unknown font alignment \"%s\" (using \"Left\" instead)", alignstr);
-    Msg::Info("Available font alignments:");
-    Msg::Info("  \"Left\" (or \"BottomLeft\")");
-    Msg::Info("  \"Center\" (or \"BottomCenter\")");
-    Msg::Info("  \"Right\" (or \"BottomRight\")");
-    Msg::Info("  \"TopLeft\"");
-    Msg::Info("  \"TopCenter\"");
-    Msg::Info("  \"TopRight\"");
-    Msg::Info("  \"CenterLeft\"");
-    Msg::Info("  \"CenterCenter\"");
-    Msg::Info("  \"CenterRight\"");
-    return 0;
-  }
-  int getFontSize()
-  {
-    if(CTX::instance()->fontSize > 0){
-      return CTX::instance()->fontSize;
-    }
-    else{
-      int w = Fl::w();
-      if(w <= 1024)      return 11;
-      else if(w <= 1280) return 12;
-      else if(w <= 1680) return 13;
-      else if(w <= 1920) return 14;
-      else               return 15;
-    }
-  }
-  void setFont(int fontid, int fontsize)
-  {
-    gl_font(fontid, fontsize);
-  }
-  double getStringWidth(const char *str)
-  {
-    return gl_width(str);
-  }
-  int getStringHeight()
-  {
-    return gl_height();
-  }
-  int getStringDescent()
-  {
-    return gl_descent();
-  }
-  void drawString(const char *str)
-  {
-    gl_draw(str);
-  }
-  void resetFontTextures()
-  {
-#if defined(__APPLE__) && (FL_MAJOR_VERSION == 1) && (FL_MINOR_VERSION == 3)
-    gl_texture_pile_height(1); // force font texture recomputation
-#endif
-  }
-};
-
 static int globalShortcut(int event)
 {
   if(!FlGui::available()) return 0;
   return FlGui::instance()->testGlobalShortcuts(event);
 }
 
-FlGui::FlGui(int argc, char **argv) : _openedThroughMacFinder(false)
+static void simple_right_box_draw(int x, int y, int w, int h, Fl_Color c)
+{
+  fl_color(c); fl_rectf(x, y, w, h);
+  fl_color(FL_DARK2); fl_line(x + w - 1, y, x + w - 1, y + h);
+}
+
+static void simple_top_box_draw(int x, int y, int w, int h, Fl_Color c)
+{
+  fl_color(c); fl_rectf(x, y, w, h);
+  fl_color(FL_DARK2); fl_line(x, y, x + w, y);
+  //fl_color(FL_LIGHT2); fl_line(x, y+1, x + w, y+1);
+}
+
+// Icons for the satus bar
+#define vv(x,y) fl_vertex(x,y)
+#define bl fl_begin_loop()
+#define el fl_end_loop()
+
+static void gmsh_play(Fl_Color c)
+{
+  fl_color(c);
+  bl; vv(-0.3,0.8); vv(0.5,0.0); vv(-0.3,-0.8); el;
+}
+
+static void gmsh_pause(Fl_Color c)
+{
+  fl_color(c);
+  bl; vv(-0.8,-0.8); vv(-0.3,-0.8); vv(-0.3,0.8); vv(-0.8,0.8); el;
+  bl; vv(0.0,-0.8); vv(0.5,-0.8); vv(0.5,0.8); vv(0.0,0.8); el;
+}
+
+static void gmsh_rewind(Fl_Color c)
+{
+  fl_color(c);
+  bl; vv(-0.8,-0.8); vv(-0.3,-0.8); vv(-0.3,0.8); vv(-0.8,0.8); el;
+  bl; vv(-0.3,0.0); vv(0.5,-0.8); vv(0.5,0.8); el;
+}
+
+static void gmsh_forward(Fl_Color c)
+{
+  fl_color(c);
+  bl; vv(0.0,0.8); vv(0.8,0.0); vv(0.0,-0.8); el;
+  bl; vv(-0.8,0.8); vv(-0.3,0.8); vv(-0.3,-0.8); vv(-0.8,-0.8); el;
+}
+
+static void gmsh_back(Fl_Color c)
+{
+  fl_rotate(180);
+  gmsh_forward(c);
+}
+
+static void gmsh_ortho(Fl_Color c)
+{
+  fl_color(c);
+  bl; vv(-0.8,0.8); vv(0.3,0.8); vv(0.3,-0.3); vv(-0.8,-0.3); el;
+  bl; vv(-0.3,0.3); vv(0.8,0.3); vv(0.8,-0.8); vv(-0.3,-0.8); el;
+  fl_begin_line(); vv(-0.8,0.8); vv(-0.3,0.3); fl_end_line();
+  fl_begin_line(); vv(0.3,0.8); vv(0.8,0.3); fl_end_line();
+  fl_begin_line(); vv(0.3,-0.3); vv(0.8,-0.8); fl_end_line();
+  fl_begin_line(); vv(-0.8,-0.3); vv(-0.3,-0.8); fl_end_line();
+}
+
+static void gmsh_rotate(Fl_Color c)
+{
+  fl_color(c);
+  fl_begin_line(); fl_arc(0.0, -0.1, 0.7, 0.0, 270.0); fl_end_line();
+  fl_begin_polygon(); vv(0.5,0.6); vv(-0.1,0.9); vv(-0.1,0.3); fl_end_polygon();
+}
+
+static void gmsh_models(Fl_Color c)
+{
+  fl_color(c);
+  bl; vv(-0.8,-0.7); vv(0.8,-0.7); el;
+  bl; vv(-0.8,-0.2); vv(0.8,-0.2); el;
+  bl; vv(-0.8,0.3); vv(0.8,0.3); el;
+  bl; vv(-0.8,0.8); vv(0.8,0.8); el;
+}
+
+static void gmsh_clscale(Fl_Color c)
+{
+  fl_color(c);
+  bl; vv(-0.8,0.8); vv(-0.1,0.8); vv(-0.8,0.1); el;
+  bl; vv(-0.2,0.2); vv(0.9,0.2); vv(-0.2,-0.9); el;
+}
+
+static void gmsh_gear(Fl_Color c)
+{
+  fl_color(c);
+  double w = 0.12;
+  double h1 = 0.5;
+#if defined(WIN32)
+  double h2 = 1.0;
+#else
+  double h2 = 1.05;
+#endif
+  fl_line_style(FL_SOLID, 3);
+  fl_begin_line();
+  fl_circle(0, 0, 0.5);
+  fl_end_line();
+  fl_line_style(FL_SOLID);
+  for(int i = 0; i < 8; i++){
+    fl_rotate(45);
+    fl_begin_polygon();
+    fl_vertex(h1, -w);
+    fl_vertex(h2, -w);
+    fl_vertex(h2, w);
+    fl_vertex(h1, w);
+    fl_end_polygon();
+  }
+}
+
+static void gmsh_graph(Fl_Color c)
+{
+  fl_color(c);
+  fl_begin_line(); vv(-0.8,-0.8); vv(-0.8,0.8); vv(0.8,0.8); fl_end_line();
+  fl_begin_line(); vv(-0.8,0.3); vv(-0.2,-0.2); vv(0.3,0.1); vv(0.8,-0.4); fl_end_line();
+}
+
+static void gmsh_search(Fl_Color col)
+{
+  double e = 0.5;
+  fl_color(col);
+  fl_begin_polygon();
+  vv(.6-e, .33); vv(1.2-e, .93); vv(.93-e, 1.2); vv(.33-e, .6);
+  fl_end_polygon();
+  fl_line_style(FL_SOLID, 2);
+  fl_begin_loop(); fl_circle(0-e, 0, .6); fl_end_loop();
+  fl_line_style(FL_SOLID);
+}
+
+#undef vv
+#undef bl
+#undef el
+
+FlGui::FlGui(int argc, char **argv)
 {
   // set X display
   if(CTX::instance()->display.size())
     Fl::display(CTX::instance()->display.c_str());
 
 #if 0 // dark scheme... not bad, but needs work
-  Fl::background(60, 60, 60);
-  Fl::background2(120, 120, 120);
-  Fl::foreground(200, 200, 200);
+  Fl::background(110, 110, 110);
+  Fl::background2(140, 140, 140);
+  Fl::foreground(230, 230, 230);
+  //Fl::set_color(FL_SELECTION_COLOR, 50, 50, 0);
 #endif
 
+  // add new box types (dx dy dw dh)
+  Fl::set_boxtype(GMSH_SIMPLE_RIGHT_BOX, simple_right_box_draw, 0, 0, 1, 0);
+  Fl::set_boxtype(GMSH_SIMPLE_TOP_BOX, simple_top_box_draw, 0, 1, 0, 1);
+
   // add global shortcuts
   Fl::add_handler(globalShortcut);
 
@@ -224,27 +236,38 @@ FlGui::FlGui(int argc, char **argv) : _openedThroughMacFinder(false)
   // register image formats not in core fltk library (jpeg/png)
   fl_register_images();
 
+  // add our own icons
+  fl_add_symbol("gmsh_rewind", gmsh_rewind, 1);
+  fl_add_symbol("gmsh_back", gmsh_back, 1);
+  fl_add_symbol("gmsh_play", gmsh_play, 1);
+  fl_add_symbol("gmsh_pause", gmsh_pause, 1);
+  fl_add_symbol("gmsh_forward", gmsh_forward, 1);
+  fl_add_symbol("gmsh_ortho", gmsh_ortho, 1);
+  fl_add_symbol("gmsh_rotate", gmsh_rotate, 1);
+  fl_add_symbol("gmsh_models", gmsh_models, 1);
+  fl_add_symbol("gmsh_clscale", gmsh_clscale, 1);
+  fl_add_symbol("gmsh_gear", gmsh_gear, 1);
+  fl_add_symbol("gmsh_graph", gmsh_graph, 1);
+  fl_add_symbol("gmsh_search", gmsh_search, 1);
+
   // load default system icons (for file browser)
   Fl_File_Icon::load_system_icons();
 
   // add callback to respond to Mac Finder
 #if defined(__APPLE__)
+  Fl_Mac_App_Menu::print = ""; // remove this: it crashes
   fl_open_callback(OpenProjectMacFinder);
-#if (FL_MAJOR_VERSION == 1) && (FL_MINOR_VERSION == 3)
   fl_mac_set_about(help_about_cb, 0);
-#endif
 #endif
 
-  // all the windows are contructed (even if some are not displayed)
-  // since the shortcuts should be valid even for hidden windows, and
-  // we don't want to test for widget existence every time
-  menu = new menuWindow();
-  graph.push_back(new graphicWindow(true, CTX::instance()->numTiles));
-
+  // create main graphic window (note that we create all the windows even if
+  // some are not displayed, since the shortcuts should be valid even for hidden
+  // windows, and we don't want to test for widget existence every time)
+  graph.push_back(new graphicWindow(true, CTX::instance()->numTiles,
+                                    CTX::instance()->detachedMenu ? true : false));
 #if defined(WIN32)
-  graph[0]->win->icon
-    ((const char*)LoadImage(fl_display, MAKEINTRESOURCE(IDI_ICON),
-                            IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR));
+  graph[0]->getWindow()->icon
+    ((const void*)LoadIcon(fl_display, MAKEINTRESOURCE(IDI_ICON)));
 #elif defined(__APPLE__)
   // nothing to do here
 #else
@@ -261,33 +284,46 @@ FlGui::FlGui(int argc, char **argv) : _openedThroughMacFinder(false)
     0x08, 0x00, 0xff, 0x1f, 0x08, 0x00, 0xff, 0x1f, 0x04, 0x40, 0xfd, 0x3f,
     0x04, 0xa8, 0xea, 0x3f, 0x02, 0x55, 0x55, 0x7f, 0xa2, 0xaa, 0xaa, 0x7a,
     0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00};
-  graph[0]->win->icon
-    ((const char*)XCreateBitmapFromData(fl_display, DefaultRootWindow(fl_display),
-                                        gmsh32x32, 32, 32));
-  menu->win->icon
+  graph[0]->getWindow()->icon
     ((const char*)XCreateBitmapFromData(fl_display, DefaultRootWindow(fl_display),
                                         gmsh32x32, 32, 32));
 #endif
 
-  // open graphic window first for correct non-modal behaviour on
-  // Win32
-  graph[0]->win->show(1, argv);
-  menu->win->show();
+  graph[0]->getWindow()->show(1, argv);
+  if(graph[0]->getMenuWindow()) graph[0]->getMenuWindow()->show();
 
-  // graphic window should have the initial focus (so we can
-  // e.g. directly loop through time steps with the keyboard)
+  // graphic window should have the initial focus (so we can e.g. directly loop
+  // through time steps with the keyboard)
   //graph[0]->gl[0]->take_focus();
   Fl::focus(graph[0]->gl[0]);
 
+  // get onelab tree group (FIXME: should clean this up)
+  onelab = graph.back()->getMenu();
+
   // create additional graphic windows
   for(int i = 1; i < CTX::instance()->numWindows; i++){
     graphicWindow *g = new graphicWindow(false, CTX::instance()->numTiles);
-    g->win->resize(graph.back()->win->x() + 10, graph.back()->win->y() + 10,
-                   graph.back()->win->w(), graph.back()->win->h());
-    g->win->show();
+    g->getWindow()->resize(graph.back()->getWindow()->x() + 10,
+                           graph.back()->getWindow()->y() + 10,
+                           graph.back()->getWindow()->w(),
+                           graph.back()->getWindow()->h());
+    g->getWindow()->show();
     graph.push_back(g);
   }
 
+  // create fullscreen window
+  fullscreen = new openglWindow(100, 100, 100, 100);
+  int mode = FL_RGB | FL_DEPTH | (CTX::instance()->db ? FL_DOUBLE : FL_SINGLE);
+  if(CTX::instance()->antialiasing) mode |= FL_MULTISAMPLE;
+  if(CTX::instance()->stereo) {
+    mode |= FL_DOUBLE;
+    mode |= FL_STEREO;
+  }
+  fullscreen->mode(mode);
+  fullscreen->end();
+  fullscreen->fullscreen();
+
+  // create all other windows
   options = new optionWindow(CTX::instance()->deltaFontSize);
   fields = new fieldWindow(CTX::instance()->deltaFontSize);
   plugins = new pluginWindow(CTX::instance()->deltaFontSize);
@@ -298,10 +334,7 @@ FlGui::FlGui(int argc, char **argv) : _openedThroughMacFinder(false)
   manip = new manipWindow(CTX::instance()->deltaFontSize);
   geoContext = new geometryContextWindow(CTX::instance()->deltaFontSize);
   meshContext = new meshContextWindow(CTX::instance()->deltaFontSize);
-  about = new aboutWindow();
-#if defined(HAVE_ONELAB) && (FL_MAJOR_VERSION == 1) && (FL_MINOR_VERSION == 3)
-  onelab = new onelabWindow(CTX::instance()->deltaFontSize);
-#endif
+  help = new helpWindow();
 
   // init solver plugin stuff
   callForSolverPlugin(-1);
@@ -310,11 +343,10 @@ FlGui::FlGui(int argc, char **argv) : _openedThroughMacFinder(false)
   for(unsigned int i = 0; i < graph.size(); i++)
     for(unsigned int j = 0; j < graph[i]->gl.size(); j++)
       graph[i]->gl[j]->redraw();
-
-  menu->setContext(menu_geometry, 0);
 }
 
 FlGui *FlGui::_instance = 0;
+std::string FlGui::_openedThroughMacFinder = "";
 
 FlGui *FlGui::instance(int argc, char **argv)
 {
@@ -323,8 +355,7 @@ FlGui *FlGui::instance(int argc, char **argv)
     // set all options in the new GUI
     InitOptionsGUI(0);
     // say welcome!
-    Msg::StatusBar(1, false, "Geometry");
-    Msg::StatusBar(2, false, "Gmsh %s", GetGmshVersion());
+    Msg::StatusBar(false, "Gmsh %s", GetGmshVersion());
     // log the following for bug reports
     Msg::Info("-------------------------------------------------------");
     Msg::Info("Gmsh version   : %s", GetGmshVersion());
@@ -343,9 +374,9 @@ FlGui *FlGui::instance(int argc, char **argv)
 
 int FlGui::run()
 {
-  // bounding box computation necessary if we run the gui without
-  // merging any files (e.g. if we build the geometry with python and
-  // create the gui from the python script)
+  // bounding box computation necessary if we run the gui without merging any
+  // files (e.g. if we build the geometry with python and create the gui from
+  // the python script)
   SetBoundingBox();
 
   // draw the scene
@@ -362,63 +393,44 @@ int FlGui::testGlobalShortcuts(int event)
   int status = 0;
 
   if(Fl::test_shortcut('0')) {
-    // FIXME: here we should also reset all onelab variables that depend on Gmsh
     geometry_reload_cb(0, 0);
-    mod_geometry_cb(0, 0);
     status = 1;
   }
   else if(Fl::test_shortcut('1') || Fl::test_shortcut(FL_F + 1)) {
     mesh_1d_cb(0, 0);
-    mod_mesh_cb(0, 0);
     status = 1;
   }
   else if(Fl::test_shortcut('2') || Fl::test_shortcut(FL_F + 2)) {
     mesh_2d_cb(0, 0);
-    mod_mesh_cb(0, 0);
     status = 1;
   }
   else if(Fl::test_shortcut('3') || Fl::test_shortcut(FL_F + 3)) {
     mesh_3d_cb(0, 0);
-    mod_mesh_cb(0, 0);
     status = 1;
   }
-  // FIXME TEST
-  else if(Fl::test_shortcut('4') || Fl::test_shortcut(FL_F + 4)) {
-    RecombineMesh(GModel::current());
-    status = 2;
-  }
   else if(Fl::test_shortcut(FL_CTRL + 'q') || Fl::test_shortcut(FL_META + 'q')){
-    // only necessary when using the system menu bar, but hey, it
-    // cannot hurt...
+    // only necessary when using the system menu bar, but hey, it cannot hurt...
     file_quit_cb(0, 0);
     status = 1;
   }
+  else if(Fl::test_shortcut(FL_CTRL + 't') || Fl::test_shortcut(FL_META + 't')){
+    show_hide_menu_cb(0, 0);
+    status = 1;
+  }
   else if(Fl::test_shortcut('g')) {
-    mod_geometry_cb(0, 0);
-    Fl::focus(menu->scroll);
+    FlGui::instance()->openModule("Geometry");
     status = 1;
   }
   else if(Fl::test_shortcut('m')) {
-    mod_mesh_cb(0, 0);
-    Fl::focus(menu->scroll);
+    FlGui::instance()->openModule("Mesh");
     status = 1;
   }
   else if(Fl::test_shortcut('s')) {
-    mod_solver_cb(0, 0);
-    Fl::focus(menu->scroll);
+    FlGui::instance()->openModule("Solver");
     status = 1;
   }
   else if(Fl::test_shortcut('p')) {
-    mod_post_cb(0, 0);
-    Fl::focus(menu->scroll);
-    status = 1;
-  }
-  else if(Fl::test_shortcut('<')) {
-    mod_back_cb(0, 0);
-    status = 1;
-  }
-  else if(Fl::test_shortcut('>')) {
-    mod_forward_cb(0, 0);
+    FlGui::instance()->openModule("Post-processing");
     status = 1;
   }
   else if(Fl::test_shortcut('w')) {
@@ -460,19 +472,25 @@ int FlGui::testGlobalShortcuts(int event)
           Fl::test_shortcut(FL_SHIFT + FL_Escape) ||
           Fl::test_shortcut(FL_CTRL + FL_Escape) ||
           Fl::test_shortcut(FL_ALT + FL_Escape)) {
-    bool lasso = false;
-    for(unsigned int i = 0; i < graph.size(); i++)
-      for(unsigned int j = 0; j < graph[i]->gl.size(); j++)
-        if(graph[i]->gl[j]->lassoMode) lasso = true;
-    if(lasso){
-      for(unsigned int i = 0; i < graph.size(); i++)
-        for(unsigned int j = 0; j < graph[i]->gl.size(); j++)
-          graph[i]->gl[j]->lassoMode = false;
-      status = 2;
+    if(fullscreen->shown()){
+      window_cb(0, (void*)"fullscreen");
+      status = 1;
     }
     else{
-      status_options_cb(0, (void *)"S");
-      status = 1;
+      bool lasso = false;
+      for(unsigned int i = 0; i < graph.size(); i++)
+        for(unsigned int j = 0; j < graph[i]->gl.size(); j++)
+          if(graph[i]->gl[j]->lassoMode) lasso = true;
+      if(lasso){
+        for(unsigned int i = 0; i < graph.size(); i++)
+          for(unsigned int j = 0; j < graph[i]->gl.size(); j++)
+            graph[i]->gl[j]->lassoMode = false;
+        status = 2;
+      }
+      else{
+        status_options_cb(0, (void *)"S");
+        status = 1;
+      }
     }
   }
   else if(Fl::test_shortcut(FL_SHIFT + 'a')) {
@@ -714,20 +732,30 @@ int FlGui::testArrowShortcuts()
     status_play_manual(0, CTX::instance()->post.animStep);
     return 1;
   }
+#if defined(HAVE_POPPLER)
+  else if(Fl::test_shortcut(FL_ALT + FL_Up)) {
+    gmshPopplerWrapper::setCurrentPageDown();
+    drawContext::global()->draw();
+    return 1;
+  }
+  else if(Fl::test_shortcut(FL_ALT + FL_Down)) {
+    gmshPopplerWrapper::setCurrentPageUp();
+    drawContext::global()->draw();
+    return 1;
+  }
+#endif
   return 0;
 }
 
 void FlGui::setGraphicTitle(std::string title)
 {
   for(unsigned int i = 0; i < graph.size(); i++){
-    if(!i){
-      graph[i]->setTitle(title);
-    }
-    else{
-      std::ostringstream sstream;
-      sstream << title << " [" << i << "]";
-      graph[i]->setTitle(sstream.str());
-    }
+    std::ostringstream sstream;
+    if(!i)
+      sstream << "Gmsh - " << title;
+    else
+      sstream << "Gmsh - " << title << " [" << i << "]";
+    graph[i]->setTitle(sstream.str());
   }
 }
 
@@ -736,8 +764,7 @@ void FlGui::updateViews(bool numberOfViewsHasChanged)
   for(unsigned int i = 0; i < graph.size(); i++)
     graph[i]->checkAnimButtons();
   if(numberOfViewsHasChanged){
-    if(menu->module->value() == 3)
-      menu->setContext(menu_post, 0);
+    onelab->rebuildTree();
     options->resetBrowser();
     options->resetExternalViewList();
     fields->loadFieldViewList();
@@ -769,10 +796,8 @@ void FlGui::splitCurrentOpenglWindow(char how)
 {
   openglWindow *g = getCurrentOpenglWindow();
   for(unsigned int i = 0; i < graph.size(); i++){
-    if(graph[i]->tile->find(g) != graph[i]->tile->children()){
-      graph[i]->split(g, how);
+    if(graph[i]->split(g, how))
       break;
-    }
   }
 }
 
@@ -783,42 +808,92 @@ char FlGui::selectEntity(int type)
      selectedElements);
 }
 
-void FlGui::setStatus(const char *msg, int num)
+void FlGui::setStatus(const std::string &msg, bool opengl)
 {
-  if(num == 0 || num == 1){
-    static char buff[2][1024];
-    strncpy(buff[num], msg, sizeof(buff[num]) - 1);
-    buff[num][sizeof(buff[num]) - 1] = '\0';
+  if(!opengl){
+    _lastStatus = msg;
+    static char buff[1024];
+    std::string tmp = std::string(" ") + msg;
+    int ne = Msg::GetErrorCount(), nw = Msg::GetWarningCount();
+    if((ne || nw) && graph[0]->getMessageHeight() < FL_NORMAL_SIZE){
+      tmp += "  -  ";
+      char n[128]; sprintf(n, "%d", ne ? ne : nw);
+      tmp += n;
+      tmp += (ne > 1) ? " Errors" : ne ? " Error" : (nw > 1) ? " Warnings" : " Warning";
+      tmp += " : Click to show messages [ ... ";
+      tmp += (ne ? Msg::GetFirstError() : Msg::GetFirstWarning());
+      tmp += " ... ]";
+    }
+    strncpy(buff, tmp.c_str(), sizeof(buff) - 1);
+    buff[sizeof(buff) - 1] = '\0';
     for(unsigned int i = 0; i < graph.size(); i++){
-      graph[i]->label[num]->label(buff[num]);
-      graph[i]->label[num]->redraw();
+      graph[i]->getProgress()->label(buff);
+      graph[i]->getProgress()->redraw();
     }
   }
-  else if(num == 2){
+  else{
     openglWindow *gl = getCurrentOpenglWindow();
-    int n = strlen(msg);
+    int n = msg.size();
     int i = 0;
     while(i < n) if(msg[i++] == '\n') break;
-    gl->screenMessage[0] = std::string(msg);
+    gl->screenMessage[0] = msg;
     if(i)
       gl->screenMessage[0].resize(i - 1);
     if(i < n)
-      gl->screenMessage[1] = std::string(&msg[i]);
+      gl->screenMessage[1] = msg.substr(i);
     else
       gl->screenMessage[1].clear();
     drawContext::global()->draw();
   }
 }
 
+void FlGui::setLastStatus(int col)
+{
+  for(unsigned int i = 0; i < graph.size(); i++){
+    if(col >= 0 && graph[0]->getMessageHeight() < FL_NORMAL_SIZE)
+      graph[i]->getProgress()->labelcolor(col);
+    else
+      graph[i]->getProgress()->labelcolor(FL_FOREGROUND_COLOR);
+  }
+  setStatus(_lastStatus);
+}
+
+void FlGui::setProgress(const std::string &msg, double val, double min, double max)
+{
+  for(unsigned int i = 0; i < FlGui::instance()->graph.size(); i++){
+    if(FlGui::instance()->graph[i]->getProgress()->value() != val)
+      FlGui::instance()->graph[i]->getProgress()->value(val);
+    if(FlGui::instance()->graph[i]->getProgress()->minimum() != min)
+      FlGui::instance()->graph[i]->getProgress()->minimum(min);
+    if(FlGui::instance()->graph[i]->getProgress()->maximum() != max)
+      FlGui::instance()->graph[i]->getProgress()->maximum(max);
+  }
+  setStatus(msg);
+}
+
+void FlGui::setProgressColor(int col)
+{
+  for(unsigned int i = 0; i < FlGui::instance()->graph.size(); i++)
+    FlGui::instance()->graph[i]->getProgress()->labelcolor(col);
+}
+
 void FlGui::storeCurrentWindowsInfo()
 {
-  CTX::instance()->menuPosition[0] = menu->win->x();
-  CTX::instance()->menuPosition[1] = menu->win->y();
-  CTX::instance()->glPosition[0] = graph[0]->win->x();
-  CTX::instance()->glPosition[1] = graph[0]->win->y();
-  CTX::instance()->glSize[0] = graph[0]->win->w();
-  CTX::instance()->glSize[1] = (graph[0]->win->h() - graph[0]->bottom->h());
-  CTX::instance()->msgSize = graph[0]->getMessageHeight();
+  CTX::instance()->glPosition[0] = graph[0]->getWindow()->x();
+  CTX::instance()->glPosition[1] = graph[0]->getWindow()->y();
+  CTX::instance()->glSize[0] = graph[0]->getGlWidth();
+  CTX::instance()->glSize[1] = graph[0]->getGlHeight();
+  CTX::instance()->msgSize = graph[0]->getMessageHeight() ?
+    graph[0]->getMessageHeight() : CTX::instance()->msgSize;
+  CTX::instance()->menuSize[0] = graph[0]->getMenuWidth();
+  if(graph[0]->isMenuDetached()){
+    CTX::instance()->detachedMenu = 1;
+    CTX::instance()->menuSize[1] = graph[0]->getMenuHeight();
+    CTX::instance()->menuPosition[0] = graph[0]->getMenuPositionX();
+    CTX::instance()->menuPosition[1] = graph[0]->getMenuPositionY();
+  }
+  else
+    CTX::instance()->detachedMenu = 0;
   CTX::instance()->optPosition[0] = options->win->x();
   CTX::instance()->optPosition[1] = options->win->y();
   CTX::instance()->pluginPosition[0] = plugins->win->x();
@@ -841,12 +916,10 @@ void FlGui::storeCurrentWindowsInfo()
   CTX::instance()->manipPosition[1] = manip->win->y();
   CTX::instance()->ctxPosition[0] = geoContext->win->x();
   CTX::instance()->ctxPosition[1] = meshContext->win->y();
-#if defined(HAVE_ONELAB) && (FL_MAJOR_VERSION == 1) && (FL_MINOR_VERSION == 3)
-  CTX::instance()->solverPosition[0] = onelab->x();
-  CTX::instance()->solverPosition[1] = onelab->y();
-  CTX::instance()->solverSize[0] = onelab->w();
-  CTX::instance()->solverSize[1] = onelab->h();
+#if defined(HAVE_3M)
+  storeWindowPosition3M();
 #endif
+
   fileChooserGetPosition(&CTX::instance()->fileChooserPosition[0],
                          &CTX::instance()->fileChooserPosition[1]);
 }
@@ -866,13 +939,14 @@ void redraw_cb(Fl_Widget *w, void *data)
 
 void window_cb(Fl_Widget *w, void *data)
 {
-  static int oldx = 0, oldy = 0, oldw = 0, oldh = 0, zoom = 1;
+  static int oldx = 0, oldy = 0, oldw = 0, oldh = 0, zoom = 0, fullscreen = 0;
+
   std::string str((const char*)data);
 
   if(str == "minimize"){
     for(unsigned int i = 0; i < FlGui::instance()->graph.size(); i++)
-      if(FlGui::instance()->graph[i]->win->shown())
-        FlGui::instance()->graph[i]->win->iconize();
+      if(FlGui::instance()->graph[i]->getWindow()->shown())
+        FlGui::instance()->graph[i]->getWindow()->iconize();
     if(FlGui::instance()->options->win->shown())
       FlGui::instance()->options->win->iconize();
     if(FlGui::instance()->plugins->win->shown())
@@ -889,39 +963,47 @@ void window_cb(Fl_Widget *w, void *data)
       FlGui::instance()->manip->win->iconize();
     if(FlGui::instance()->stats->win->shown())
       FlGui::instance()->stats->win->iconize();
-    if(FlGui::instance()->menu->win->shown())
-      FlGui::instance()->menu->win->iconize();
   }
   else if(str == "zoom"){
-    if(zoom){
-      oldx = FlGui::instance()->graph[0]->win->x();
-      oldy = FlGui::instance()->graph[0]->win->y();
-      oldw = FlGui::instance()->graph[0]->win->w();
-      oldh = FlGui::instance()->graph[0]->win->h();
-//#define FS
-#ifndef FS
-      FlGui::instance()->graph[0]->win->resize(Fl::x(), Fl::y(), Fl::w(), Fl::h());
-      FlGui::instance()->graph[0]->hideMessages();
-      FlGui::check();
-#else
-      FlGui::instance()->graph[0]->win->fullscreen();
-#endif
-      zoom = 0;
-    }
-    else{
-#ifndef FS
-      FlGui::instance()->graph[0]->win->resize(oldx, oldy, oldw, oldh);
-#else
-      FlGui::instance()->graph[0]->win->fullscreen_off();
-#endif
+    if(!zoom){
+      oldx = FlGui::instance()->graph[0]->getWindow()->x();
+      oldy = FlGui::instance()->graph[0]->getWindow()->y();
+      oldw = FlGui::instance()->graph[0]->getWindow()->w();
+      oldh = FlGui::instance()->graph[0]->getWindow()->h();
+      FlGui::instance()->graph[0]->getWindow()->resize(Fl::x(), Fl::y(), Fl::w(), Fl::h());
       zoom = 1;
     }
-    FlGui::instance()->menu->win->show();
+    else{
+      FlGui::instance()->graph[0]->getWindow()->resize(oldx, oldy, oldw, oldh);
+      zoom = 0;
+    }
+  }
+  else if(str == "fullscreen"){
+    if(!fullscreen){
+      FlGui::instance()->fullscreen->show();
+      FlGui::instance()->fullscreen->getDrawContext()->copyViewAttributes
+        (FlGui::instance()->getCurrentOpenglWindow()->getDrawContext());
+      openglWindow::setLastHandled(FlGui::instance()->fullscreen);
+      for(unsigned int i = 0; i < FlGui::instance()->graph.size(); i++)
+        FlGui::instance()->graph[i]->getWindow()->hide();
+      drawContext::global()->draw();
+      fullscreen = 1;
+    }
+    else{
+      for(unsigned int i = 0; i < FlGui::instance()->graph.size(); i++)
+        FlGui::instance()->graph[i]->getWindow()->show();
+      FlGui::instance()->graph[0]->gl[0]->getDrawContext()->copyViewAttributes
+        (FlGui::instance()->getCurrentOpenglWindow()->getDrawContext());
+      openglWindow::setLastHandled(FlGui::instance()->graph[0]->gl[0]);
+      FlGui::instance()->fullscreen->hide();
+      drawContext::global()->draw();
+      fullscreen = 0;
+    }
   }
   else if(str == "front"){
     // the order is important!
     for(unsigned int i = 0; i < FlGui::instance()->graph.size(); i++)
-      FlGui::instance()->graph[i]->win->show();
+      FlGui::instance()->graph[i]->getWindow()->show();
     if(FlGui::instance()->options->win->shown())
       FlGui::instance()->options->win->show();
     if(FlGui::instance()->plugins->win->shown())
@@ -932,10 +1014,6 @@ void window_cb(Fl_Widget *w, void *data)
       FlGui::instance()->geoContext->win->show();
     if(FlGui::instance()->meshContext->win->shown())
       FlGui::instance()->meshContext->win->show();
-#if defined(HAVE_ONELAB) && (FL_MAJOR_VERSION == 1) && (FL_MINOR_VERSION == 3)
-    if(FlGui::instance()->onelab->shown())
-      FlGui::instance()->onelab->show();
-#endif
     if(FlGui::instance()->visibility->win->shown())
       FlGui::instance()->visibility->win->show();
     if(FlGui::instance()->highordertools->win->shown())
@@ -946,7 +1024,6 @@ void window_cb(Fl_Widget *w, void *data)
       FlGui::instance()->manip->win->show();
     if(FlGui::instance()->stats->win->shown())
       FlGui::instance()->stats->win->show();
-    FlGui::instance()->menu->win->show();
   }
 }
 
@@ -956,13 +1033,18 @@ void FlGui::addMessage(const char *msg)
     FlGui::instance()->graph[i]->addMessage(msg);
 }
 
-void FlGui::showMessages()
-{
-  for(unsigned int i = 0; i < FlGui::instance()->graph.size(); i++)
-    FlGui::instance()->graph[i]->showMessages();
-}
-
 void FlGui::saveMessages(const char *fileName)
 {
   FlGui::instance()->graph[0]->saveMessages(fileName);
 }
+
+void FlGui::rebuildTree()
+{
+  onelab->rebuildTree();
+}
+
+void FlGui::openModule(const std::string &name)
+{
+  if(!onelab->isManuallyClosed("0Modules/" + name))
+    onelab->openTreeItem("0Modules/" + name);
+}
