@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #ifndef _FL_GUI_H_
 #define _FL_GUI_H_
@@ -10,17 +10,16 @@
 #include <vector>
 
 #define GMSH_WINDOW_BOX FL_FLAT_BOX
-#define NB_BUTT_SCROLL 25
-#define NB_HISTORY_MAX 1000
-#define NB_SOLVER_MAX 5
+#define GMSH_SIMPLE_RIGHT_BOX (Fl_Boxtype)(FL_FREE_BOXTYPE+1)
+#define GMSH_SIMPLE_TOP_BOX (Fl_Boxtype)(FL_FREE_BOXTYPE+2)
+
 #define IW (10 * FL_NORMAL_SIZE)    // input field width
 #define BB (7 * FL_NORMAL_SIZE)     // width of a button with internal label
 #define BH (2 * FL_NORMAL_SIZE + 1) // button height
-#define WB (7)                      // window border
+#define WB (5)                      // window border
 
 class graphicWindow;
 class openglWindow;
-class menuWindow;
 class optionWindow;
 class fieldWindow;
 class pluginWindow;
@@ -31,8 +30,8 @@ class clippingWindow;
 class manipWindow;
 class geometryContextWindow;
 class meshContextWindow;
-class aboutWindow;
-class onelabWindow;
+class helpWindow;
+class onelabGroup;
 class Fl_Widget;
 
 class GVertex;
@@ -44,7 +43,8 @@ class MElement;
 class FlGui{
  private:
   static FlGui *_instance;
-  bool _openedThroughMacFinder;
+  static std::string _openedThroughMacFinder;
+  std::string _lastStatus;
  public:
   std::vector<GVertex*> selectedVertices;
   std::vector<GEdge*> selectedEdges;
@@ -53,7 +53,6 @@ class FlGui{
   std::vector<MElement*> selectedElements;
  public:
   std::vector<graphicWindow*> graph;
-  menuWindow *menu;
   optionWindow *options;
   fieldWindow *fields;
   pluginWindow *plugins;
@@ -64,8 +63,9 @@ class FlGui{
   manipWindow *manip;
   geometryContextWindow *geoContext;
   meshContextWindow *meshContext;
-  aboutWindow *about;
-  onelabWindow *onelab;
+  helpWindow *help;
+  onelabGroup *onelab;
+  openglWindow *fullscreen;
  public:
   FlGui(int argc, char **argv);
   ~FlGui(){}
@@ -82,8 +82,11 @@ class FlGui{
   // wait (at most time seconds) for any events, then process them
   static void wait(double time);
   // is a file opened through the Mac Finder?
-  void setOpenedThroughMacFinder(bool val){ _openedThroughMacFinder = val; }
-  bool getOpenedThroughMacFinder(){ return _openedThroughMacFinder; }
+  static void setOpenedThroughMacFinder(const std::string &name)
+  {
+    _openedThroughMacFinder = name;
+  }
+  static std::string getOpenedThroughMacFinder(){ return _openedThroughMacFinder; }
   // test application-level keyboard shortcuts
   int testGlobalShortcuts(int event);
   // test the arrow shortcuts (not in the application-level shortcuts)
@@ -107,15 +110,23 @@ class FlGui{
   // select an entity in the most recent graphic window
   char selectEntity(int type);
   // display status message
-  void setStatus(const char *msg, int num);
+  void setStatus(const std::string &msg, bool opengl=false);
+  // redisplay last status message
+  void setLastStatus(int col=-1);
+  // display status message and update progress bar
+  void setProgress(const std::string &msg, double val, double min, double max);
+  // set color of progress message
+  void setProgressColor(int col);
   // create the window for physical context dependant definitions
   void callForSolverPlugin(int dim);
-  // add line in message console(s)
+  // add line in message console
   void addMessage(const char *msg);
-  // show the message console
-  void showMessages();
-  // add line in message console(s)
+  // save messages to file
   void saveMessages(const char *fileName);
+  // rebuild the tree
+  void rebuildTree();
+  // open module in tree
+  void openModule(const std::string &name);
 };
 
 void redraw_cb(Fl_Widget *w, void *data);
