@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #ifndef _FILE_DIALOGS_H_
 #define _FILE_DIALOGS_H_
@@ -9,10 +9,10 @@
 #include <string>
 
 typedef enum {
-  FILE_CHOOSER_SINGLE, 
-  FILE_CHOOSER_MULTI, 
-  FILE_CHOOSER_CREATE, 
-  FILE_CHOOSER_DIRECTORY 
+  FILE_CHOOSER_SINGLE,
+  FILE_CHOOSER_MULTI,
+  FILE_CHOOSER_CREATE,
+  FILE_CHOOSER_DIRECTORY
 } FILE_CHOOSER_TYPE;
 
 int fileChooser(FILE_CHOOSER_TYPE type, const char *message,
@@ -21,7 +21,6 @@ std::string fileChooserGetName(int num);
 int fileChooserGetFilter();
 void fileChooserGetPosition(int *x, int *y);
 
-int jpegFileDialog(const char *filename);
 int mpegFileDialog(const char *filename);
 int gifFileDialog(const char *filename);
 int geoFileDialog(const char *filename);
