@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 //
 // Contributed by Matti Pellikka <matti.pellikka@tut.fi>.
 
@@ -39,6 +39,34 @@ bool equalVertices(const std::vector<MVertex*>& v1,
 
 int Cell::_globalNum = 0;
 
+std::pair<Cell*, bool> Cell::createCell(MElement* element, int domain)
+{
+  Cell* cell = new Cell();
+  cell->_dim = element->getDim();
+  cell->_domain = domain;
+  cell->_combined = false;
+  cell->_immune = false;
+  cell->_num = 0;
+
+  for(int i = 0; i < element->getNumPrimaryVertices(); i++)
+    cell->_v.push_back(element->getVertex(i));
+
+  return std::make_pair(cell, cell->_sortVertexIndices());
+}
+
+std::pair<Cell*, bool> Cell::createCell(Cell* parent, int i)
+{
+  Cell* cell = new Cell();
+  cell->_dim = parent->getDim()-1;
+  cell->_domain = parent->getDomain();
+  cell->_combined = false;
+  cell->_immune = false;
+  cell->_num = 0;
+
+  parent->findBdElement(i, cell->_v);
+  return std::make_pair(cell, cell->_sortVertexIndices());
+}
+
 Cell::Cell(MElement* element, int domain)
 {
   _dim = element->getDim();
@@ -65,16 +93,24 @@ Cell::Cell(Cell* parent, int i)
   _sortVertexIndices();
 }
 
-void Cell::_sortVertexIndices()
+bool Cell::_sortVertexIndices()
 {
   std::map<MVertex*, int, MVertexLessThanNum> si;
 
+  bool noinsert = false;
   for(unsigned int i = 0; i < _v.size(); i++)
-    si[_v[i]] = i;
+    noinsert = (!si.insert(std::make_pair(_v[i], i)).second || noinsert);
+
+  if(noinsert == true) {
+    Msg::Warning("The input mesh has degenerate elements, ignored");
+    return false;
+  }
 
   std::map<MVertex*, int, MVertexLessThanNum>::iterator it;
   for(it = si.begin(); it != si.end(); it++)
     _si.push_back(it->second);
+
+  return true;
 }
 
 inline int Cell::getSortedVertex(int vertex) const
@@ -330,15 +366,26 @@ bool CombinedCell::hasVertex(int vertex) const
 void Cell::printCell()
 {
   printf("%d-cell %d: \n" , getDim(), getNum());
-  printf("Vertices: ");
+  printf("  Vertices:");
   for(int i = 0; i < this->getNumVertices(); i++){
-    printf("%d ", this->getMeshVertex(i)->getNum());
+    printf(" %d", this->getMeshVertex(i)->getNum());
   }
   printf(", in subdomain: %d, ", inSubdomain());
   printf("combined: %d. \n" , isCombined() );
 };
 
-void Cell::restoreCell(){
+void Cell::saveCellBoundary()
+{
+  for(biter it = firstCoboundary(); it != lastCoboundary(); it++){
+    it->second.init();
+  }
+  for(biter it = firstBoundary(); it != lastBoundary(); it++){
+    it->second.init();
+  }
+}
+
+void Cell::restoreCellBoundary()
+{
   std::vector<Cell*> toRemove;
   for(biter it = firstCoboundary(true); it != lastCoboundary(); it++){
     it->second.reset();
@@ -436,30 +483,98 @@ bool Cell::hasCoboundary(Cell* cell, bool orig)
     return false;
   }
 }
-/*
-void Cell::printBoundary(bool orig)
+
+Cell::biter Cell::firstBoundary(bool orig)
 {
-  for(biter it = firstBoundary(orig); it != lastBoundary(orig); it++){
-    printf("Boundary cell orientation: %d ", (*it).second);
-    Cell* cell2 = (*it).first;
+  biter it = _bd.begin();
+  if(!orig) while(it->second.get() == 0 && it != _bd.end()) it++;
+  else while(it->second.geto() == 0 && it != _bd.end()) it++;
+  return it;
+}
+
+Cell::biter Cell::lastBoundary()
+{
+  return _bd.end();
+}
+
+Cell::biter Cell::firstCoboundary(bool orig)
+{
+  biter it = _cbd.begin();
+  if(!orig) while(it->second.get() == 0 && it != _cbd.end()) it++;
+  else while(it->second.geto() == 0 && it != _cbd.end()) it++;
+  return it;
+}
+
+Cell::biter Cell::lastCoboundary()
+{
+  return _cbd.end();
+}
+
+int Cell::getBoundarySize(bool orig)
+{
+  int size = 0;
+  for(biter bit = _bd.begin(); bit != _bd.end(); bit++){
+    if(!orig && bit->second.get() != 0) size++;
+    else if(orig && bit->second.geto() != 0) size++;
+  }
+  return size;
+}
+
+int Cell::getCoboundarySize(bool orig)
+{
+  int size = 0;
+  for(biter bit = _cbd.begin(); bit != _cbd.end(); bit++){
+    if(!orig && bit->second.get() != 0) size++;
+    else if(orig && bit->second.geto() != 0) size++;
+  }
+  return size;
+}
+
+void Cell::getBoundary(std::map<Cell*, short int, Less_Cell>& boundary,
+                       bool orig)
+{
+  boundary.clear();
+  for(biter it = firstBoundary(); it != lastBoundary(); it++){
+    Cell* cell = it->first;
+    if(!orig && it->second.get() != 0) boundary[cell] = it->second.get();
+    if(orig && it->second.geto() != 0) boundary[cell] = it->second.geto();
+  }
+}
+
+void Cell::getCoboundary(std::map<Cell*, short int, Less_Cell>& coboundary,
+                         bool orig)
+{
+  coboundary.clear();
+  for(biter it = firstCoboundary(); it != lastCoboundary(); it++){
+    Cell* cell = it->first;
+    if(!orig && it->second.get() != 0) coboundary[cell] = it->second.get();
+    if(orig && it->second.geto() != 0) coboundary[cell] = it->second.geto();
+  }
+}
+
+void Cell::printBoundary()
+{
+  for(biter it = firstBoundary(); it != lastBoundary(); it++){
+    printf("Boundary cell orientation: %d ", it->second.get());
+    Cell* cell2 = it->first;
     cell2->printCell();
   }
-  if(firstBoundary(orig) == lastBoundary(orig)){
+  if(firstBoundary() == lastBoundary()){
     printf("Cell boundary is empty. \n");
   }
 }
 
-void Cell::printCoboundary(bool orig)
+void Cell::printCoboundary()
 {
-  for(biter it = firstCoboundary(orig); it != lastCoboundary(orig); it++){
-    printf("Coboundary cell orientation: %d, ", (*it).second);
-    Cell* cell2 = (*it).first;
+  for(biter it = firstCoboundary(); it != lastCoboundary(); it++){
+    printf("Coboundary cell orientation: %d, ", it->second.get());
+    Cell* cell2 = it->first;
     cell2->printCell();
-    if(firstCoboundary(orig) == lastCoboundary(orig)){
+    if(firstCoboundary() == lastCoboundary()){
       printf("Cell coboundary is empty. \n");
     }
   }
-  }*/
+}
 
 CombinedCell::CombinedCell(Cell* c1, Cell* c2, bool orMatch, bool co)
 {
