@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include <sstream>
 #include "GmshConfig.h"
@@ -17,8 +17,12 @@
 #include "Numeric.h"
 #include "GaussLegendre1D.h"
 #include "Context.h"
+
+#if defined(HAVE_MESH)
 #include "meshGFaceOptimize.h"
 #include "meshGFaceLloyd.h"
+#include "BackgroundMesh.h"
+#endif
 
 #if defined(HAVE_BFGS)
 #include "stdafx.h"
@@ -27,20 +31,6 @@
 
 #define SQU(a)      ((a)*(a))
 
-class data_wrapper{
- private:
-  const GFace* gf;
-  SPoint3 point;
- public:
-  data_wrapper() {gf = NULL; point = SPoint3();}
-  ~data_wrapper() {}
-  const GFace* get_face() {return gf;}
-  void set_face(const GFace* face) {gf = face;}
-  SPoint3 get_point() {return point;}
-  void set_point(SPoint3 _point) {point = SPoint3(_point);}
-};
-
-
 GFace::GFace(GModel *model, int tag)
   : GEntity(model, tag), r1(0), r2(0), compound(0), va_geom_triangles(0)
 {
@@ -64,7 +54,8 @@ GFace::~GFace()
 
 int GFace::getCurvatureControlParameter () const
 {
-  std::map<int,int>::iterator it = CTX::instance()->mesh.curvature_control_per_face.find(tag());
+  std::map<int,int>::iterator it =
+    CTX::instance()->mesh.curvature_control_per_face.find(tag());
   return it == CTX::instance()->mesh.curvature_control_per_face.end() ?
     CTX::instance()->mesh.minCircPoints : it->second ;
 }
@@ -74,7 +65,6 @@ void GFace::setCurvatureControlParameter (int n)
   CTX::instance()->mesh.curvature_control_per_face[tag()] = n;
 }
 
-
 int GFace::getMeshingAlgo () const
 {
   std::map<int,int>::iterator it = CTX::instance()->mesh.algo2d_per_face.find(tag());
@@ -301,11 +291,19 @@ std::string GFace::getAdditionalInfoString()
   else if(l_edges.size()){
     sstream << "{";
     for(std::list<GEdge*>::iterator it = l_edges.begin(); it != l_edges.end(); ++it){
-      if(it != l_edges.begin()) sstream << ",";
+      if(it != l_edges.begin()) sstream << " ";
       sstream << (*it)->tag();
     }
     sstream << "}";
   }
+
+  if(meshAttributes.recombine)
+    sstream << " recombined";
+  if(meshAttributes.Method == MESH_TRANSFINITE)
+    sstream << " transfinite";
+  if(meshAttributes.extrude)
+    sstream << " extruded";
+
   return sstream.str();
 }
 
@@ -558,9 +556,9 @@ end:
         meanPlane.c * v->z() - meanPlane.d;
       if(fabs(d) > lc * 1.e-3) {
         Msg::Debug("Plane surface %d (%gx+%gy+%gz=%g) is not plane!",
-                     tag(), meanPlane.a, meanPlane.b, meanPlane.c, meanPlane.d);
+                   tag(), meanPlane.a, meanPlane.b, meanPlane.c, meanPlane.d);
         Msg::Debug("Control point %d = (%g,%g,%g), val=%g",
-                     v->tag(), v->x(), v->y(), v->z(), d);
+                   v->tag(), v->x(), v->y(), v->z(), d);
         break;
       }
     }
@@ -588,6 +586,44 @@ void GFace::getMeanPlaneData(double plan[3][3]) const
       plan[i][j] = meanPlane.plan[i][j];
 }
 
+void GFace::computeMeshSizeFieldAccuracy(double &avg,double &max_e, double &min_e,
+					 int &nE, int &GS)
+{
+#if defined(HAVE_MESH)
+  std::set<MEdge, Less_Edge> es;
+  for(unsigned int i = 0; i < getNumMeshElements(); i++){
+    MElement *e = getMeshElement(i);
+    for(int j = 0; j < e->getNumEdges(); j++) es.insert(e->getEdge(j));
+  }
+
+  avg = 0;
+  min_e = 1.e22;
+  max_e = 0;
+  nE = es.size();
+  GS = 0;
+  double oneoversqr2 = 1. / sqrt(2.);
+  double sqr2 = sqrt(2.);
+  for (std::set<MEdge, Less_Edge>::const_iterator it = es.begin();
+       it != es.end();++it){
+    double u1,v1,u2,v2;
+    MVertex *vert1 =  it->getVertex(0);
+    vert1->getParameter(0, u1);
+    vert1->getParameter(1, v1);
+    MVertex *vert2 =  it->getVertex(1);
+    vert2->getParameter(0, u2);
+    vert2->getParameter(1, v2);
+    double l1 = BGM_MeshSize(this, u1, v1, vert1->x(), vert1->y(), vert1->z());
+    double l2 = BGM_MeshSize(this, u2, v2, vert2->x(), vert2->y(), vert2->z());
+    double correctLC = 0.5*(l1+l2);
+    double lone = it->length()/correctLC;
+    if (lone > oneoversqr2 && lone < sqr2) GS++;
+    avg += lone >1 ? (1. / lone) - 1. : lone - 1.;
+    max_e = std::max(max_e, lone);
+    min_e = std::min(min_e, lone);
+  }
+#endif
+}
+
 double GFace::curvatureDiv(const SPoint2 &param) const
 {
 
@@ -769,8 +805,8 @@ void GFace::XYZtoUV(double X, double Y, double Z, double &U, double &V,
   int iter;
   double mat[3][3], jac[3][3];
   double umin, umax, vmin, vmax;
-  double initu[NumInitGuess] = {0.487, 0.6, 0.4, 0.7, 0.3, 0.8, 0.2, 1., 0., 0., 1.};
-  double initv[NumInitGuess] = {0.487, 0.6, 0.4, 0.7, 0.3, 0.8, 0.2, 0., 1., 0., 1.};
+  double initu[NumInitGuess] = {0.5, 0.6, 0.4, 0.7, 0.3, 0.8, 0.2, 0.9, 0.1, 1., 0.};
+  double initv[NumInitGuess] = {0.5, 0.6, 0.4, 0.7, 0.3, 0.8, 0.2, 0.9, 0.1, 1., 0.};
 
   Range<double> ru = parBounds(0);
   Range<double> rv = parBounds(1);
@@ -779,6 +815,12 @@ void GFace::XYZtoUV(double X, double Y, double Z, double &U, double &V,
   vmin = rv.low();
   vmax = rv.high();
 
+  const double tol = Precision*(SQU(umax-umin)+SQU(vmax-vmin));
+  for(int i = 0; i < NumInitGuess; i++) {
+    initu[i] = umin + initu[i]*(umax-umin);
+    initv[i] = vmin + initv[i]*(vmax-vmin);
+  }
+
   for(int i = 0; i < NumInitGuess; i++){
     for(int j = 0; j < NumInitGuess; j++){
 
@@ -791,7 +833,7 @@ void GFace::XYZtoUV(double X, double Y, double Z, double &U, double &V,
       err2 = sqrt(SQU(X - P.x()) + SQU(Y - P.y()) + SQU(Z - P.z()));
       if (err2 < 1.e-8 * CTX::instance()->lc) return;
 
-      while(err > Precision && iter < MaxIter) {
+      while(err > tol && iter < MaxIter) {
         P = point(U, V);
         Pair<SVector3, SVector3> der = firstDer(SPoint2(U, V));
         mat[0][0] = der.left().x();
@@ -821,7 +863,7 @@ void GFace::XYZtoUV(double X, double Y, double Z, double &U, double &V,
         V = Vnew;
       }
 
-      if(iter < MaxIter && err <= Precision &&
+      if(iter < MaxIter && err <= tol &&
          Unew <= umax && Vnew <= vmax &&
          Unew >= umin && Vnew >= vmin){
         if (onSurface && err2 > 1.e-4 * CTX::instance()->lc)
@@ -851,8 +893,24 @@ SPoint2 GFace::parFromPoint(const SPoint3 &p, bool onSurface) const
 }
 
 #if defined(HAVE_BFGS)
+
+class data_wrapper{
+ private:
+  const GFace* gf;
+  SPoint3 point;
+ public:
+  data_wrapper() { gf = NULL; point = SPoint3(); }
+  ~data_wrapper() {}
+  const GFace* get_face() { return gf; }
+  void set_face(const GFace* face) { gf = face; }
+  SPoint3 get_point() { return point; }
+  void set_point(SPoint3 _point) { point = SPoint3(_point); }
+};
+
 // Callback function for BFGS
-void bfgs_callback(const alglib::real_1d_array& x, double& func, alglib::real_1d_array& grad, void* ptr) {
+void bfgs_callback(const alglib::real_1d_array& x, double& func,
+                   alglib::real_1d_array& grad, void* ptr)
+{
   data_wrapper* w = static_cast<data_wrapper*>(ptr);
   SPoint3 p = w->get_point();
   const GFace* gf = w->get_face();
@@ -867,20 +925,20 @@ void bfgs_callback(const alglib::real_1d_array& x, double& func, alglib::real_1d
 
   // Value of the gradient
   Pair<SVector3, SVector3> der = gf->firstDer(SPoint2(x[0], x[1]));
-  grad[0] = -(p.x() - pnt.x()) * der.left().x()  - (p.y() - pnt.y()) * der.left().y()  - (p.z() - pnt.z()) * der.left().z();
-  grad[1] = -(p.x() - pnt.x()) * der.right().x() - (p.y() - pnt.y()) * der.right().y() - (p.z() - pnt.z()) * der.right().z();
-  //  printf("func %22.15E Gradients %22.15E %22.15E der %g %g %g\n",func, grad[0], grad[1],der.left().x(),der.left().y(),der.left().z());
+  grad[0] = -(p.x() - pnt.x()) * der.left().x()  - (p.y() - pnt.y()) * der.left().y()
+    - (p.z() - pnt.z()) * der.left().z();
+  grad[1] = -(p.x() - pnt.x()) * der.right().x() - (p.y() - pnt.y()) * der.right().y()
+    - (p.z() - pnt.z()) * der.right().z();
+  //  printf("func %22.15E Gradients %22.15E %22.15E der %g %g %g\n",
+  //         func, grad[0], grad[1],der.left().x(),der.left().y(),der.left().z());
 }
 #endif
 
 GPoint GFace::closestPoint(const SPoint3 &queryPoint, const double initialGuess[2]) const
 {
 #if defined(HAVE_BFGS)
-  //
   // Creating the optimisation problem
-  //
-
-  //  printf("STARTING OPTIMIZATION\n");
+  // printf("STARTING OPTIMIZATION\n");
 
   alglib::ae_int_t dim = 2;
   alglib::ae_int_t corr = 2; // Num of corrections in the scheme in [3,7]
@@ -894,90 +952,94 @@ GPoint GFace::closestPoint(const SPoint3 &queryPoint, const double initialGuess[
 
   Range<double> uu = parBounds(0);
   Range<double> vv = parBounds(1);
+  {
+    GPoint pnt = point(initialGuess[0], initialGuess[1]);
+    SPoint3 spnt(pnt.x(), pnt.y(), pnt.z());
+    double dist = queryPoint.distance(spnt);
+    printf("dist = %12.5E\n",dist);
+  }
 
-  //  FILE *F = fopen ("hop.pos","w");
-  //  fprintf(F,"View \" \" {\n");
-  //  fprintf(F,"SP(%g,%g,%g) {%g};\n",queryPoint.x(),queryPoint.y(),queryPoint.z(),0.0);
+  // FILE *F = fopen ("hop.pos","w");
+  // fprintf(F,"View \" \" {\n");
+  // fprintf(F,"SP(%g,%g,%g) {%g};\n",queryPoint.x(),queryPoint.y(),queryPoint.z(),0.0);
   double initial_guesses = 10.0;
-  for(double u = uu.low(); u<=uu.high()+1.e-5; u+=(uu.high()-uu.low())/initial_guesses) {
+  for(double u = uu.low(); u <= uu.high() + 1.e-5;
+      u += (uu.high() - uu.low()) / initial_guesses) {
     //    printf("%f\n", u);
-    for(double v = vv.low(); v<=vv.high()+1.e-5; v+=(vv.high()-vv.low())/initial_guesses) {
+    for(double v = vv.low(); v <= vv.high() + 1.e-5;
+        v += (vv.high() - vv.low()) / initial_guesses) {
       GPoint pnt = point(u, v);
       SPoint3 spnt(pnt.x(), pnt.y(), pnt.z());
       double dist = queryPoint.distance(spnt);
-      //      fprintf(F,"SP(%g,%g,%g) {%g};\n",pnt.x(), pnt.y(), pnt.z(),dist);
-      //      printf("lmocal (%12.5E %12.5E) (point) : %12.5E %12.5E %12.5E (query) : %12.5E %12.5E %12.5E DSIT %12.5E\n",u,v, pnt.x(), pnt.y(), pnt.z(),queryPoint.x(),queryPoint.y(),queryPoint.z(),
-      //	     dist);
+      // fprintf(F,"SP(%g,%g,%g) {%g};\n",pnt.x(), pnt.y(), pnt.z(),dist);
+      // printf("lmocal (%12.5E %12.5E) (point) : %12.5E %12.5E %12.5E (query) : "
+      //        "%12.5E %12.5E %12.5E DSIT %12.5E\n",u,v, pnt.x(), pnt.y(), pnt.z(),
+      //         queryPoint.x(),queryPoint.y(),queryPoint.z(), dist);
       if (dist < min_dist) {
-	//	printf("min_dist %f\n", dist);
+	// printf("min_dist %f\n", dist);
 	min_dist = dist;
 	min_u = u;
 	min_v = v;
-	GPoint pnt = point(min_u, min_v);
+	// GPoint pnt = point(min_u, min_v);
       }
     }
   }
-  //  fprintf(F,"};\n");
-  //  fclose(F);
-  //  getchar();
+  // fprintf(F,"};\n");
+  // fclose(F);
+  // getchar();
   initial_conditions[0] = min_u;
   initial_conditions[1] = min_v;
 
-  //  printf("Initial conditions : %f %f %12.5E\n", min_u, min_v,min_dist);
-  GPoint pnt = point(min_u, min_v);
-  //    printf("Initial conditions (point) : %f %f %f local (%g %g) Looking for %f %f %f DIST = %12.5E\n",
-  //  	 pnt.x(), pnt.y(), pnt.z(),min_u,min_v,
-  //  	 queryPoint.x(),queryPoint.y(),queryPoint.z(),
-  //  	 min_dist);
+  // printf("Initial conditions : %f %f %12.5E\n", min_u, min_v,min_dist);
+  // GPoint pnt = point(min_u, min_v);
+  // printf("Initial conditions (point) : %f %f %f local (%g %g) "
+  //        "Looking for %f %f %f DIST = %12.5E\n",
+  //         pnt.x(), pnt.y(), pnt.z(),min_u,min_v,
+  //         queryPoint.x(),queryPoint.y(),queryPoint.z(), min_dist);
 
   x.setcontent(dim, &initial_conditions[0]);
 
   minlbfgscreate(2, corr, x, state);
 
-  //
   // Defining the stopping criterion
-  //
   double epsg = 1.e-12;
   double epsf = 0.0;
   double epsx = 0.0;
   alglib::ae_int_t maxits = 500;
 
-  minlbfgssetcond(state,epsg,epsf,epsx,maxits);
+  minlbfgssetcond(state, epsg, epsf, epsx, maxits);
 
-  //
   // Solving the problem
-  //
-
   data_wrapper w;
   w.set_point(queryPoint);
   w.set_face(this);
 
-  minlbfgsoptimize(state,bfgs_callback,NULL,&w);
+  minlbfgsoptimize(state, bfgs_callback, NULL, &w);
 
-  //
   // Getting the results
-  //
   alglib::minlbfgsreport rep;
 
-  minlbfgsresults(state,x,rep);
+  minlbfgsresults(state, x, rep);
 
   GPoint pntF = point(x[0], x[1]);
   if (rep.terminationtype != 4){
-    //    printf("Initial conditions (point) : %f %f %f local (%g %g) Looking for %f %f %f DIST = %12.5E at the end (%f %f) %f %f %f\n",
-    //  	 pnt.x(), pnt.y(), pnt.z(),min_u,min_v,
-    //  	 queryPoint.x(),queryPoint.y(),queryPoint.z(),
-    //  	 min_dist,x[0],x[1],pntF.x(), pntF.y(), pntF.z());
-    //    double DDD =
-    //      ( queryPoint.x() - pntF.x()) * ( queryPoint.x() - pntF.x()) +
-    //      ( queryPoint.y() - pntF.y()) * ( queryPoint.y() - pntF.y()) +
-    //      ( queryPoint.z() - pntF.z()) * ( queryPoint.z() - pntF.z());
-    //    if (sqrt(DDD) > 1.e-4)
-      /*
-      printf("Initial conditions (point) : %f %f %f local (%g %g) Looking for %f %f %f DIST = %12.5E at the end (%f %f) %f %f %f termination %d\n",
-	     pnt.x(), pnt.y(), pnt.z(),min_u,min_v,
-	     queryPoint.x(),queryPoint.y(),queryPoint.z(),
-	     min_dist,x[0],x[1],pntF.x(), pntF.y(), pntF.z(),rep.terminationtype);
-      */
+    // printf("Initial conditions (point) : %f %f %f local (%g %g) "
+    //        "Looking for %f %f %f DIST = %12.5E at the end (%f %f) %f %f %f\n",
+    //         pnt.x(), pnt.y(), pnt.z(),min_u,min_v,
+    //         queryPoint.x(),queryPoint.y(),queryPoint.z(),
+    //         min_dist,x[0],x[1],pntF.x(), pntF.y(), pntF.z());
+    // double DDD =
+    //   ( queryPoint.x() - pntF.x()) * ( queryPoint.x() - pntF.x()) +
+    //   ( queryPoint.y() - pntF.y()) * ( queryPoint.y() - pntF.y()) +
+    //   ( queryPoint.z() - pntF.z()) * ( queryPoint.z() - pntF.z());
+    // if (sqrt(DDD) > 1.e-4)
+    /*
+    printf("Initial conditions (point) : %f %f %f local (%g %g) Looking for %f %f %f "
+           "DIST = %12.5E at the end (%f %f) %f %f %f termination %d\n",
+           pnt.x(), pnt.y(), pnt.z(),min_u,min_v,
+	   queryPoint.x(),queryPoint.y(),queryPoint.z(),
+	   min_dist,x[0],x[1],pntF.x(), pntF.y(), pntF.z(),rep.terminationtype);
+    */
   }
   return pntF;
 #else
@@ -1217,11 +1279,11 @@ int GFace::poincareMesh()
   return vs.size() - es.size() + getNumMeshElements();
 }
 
-int GFace::genusGeom()
+int GFace::genusGeom() const
 {
   int nSeams = 0;
   std::set<GEdge*> single_seams;
-  for (std::list<GEdge*>::iterator it = l_edges.begin();
+  for (std::list<GEdge*>::const_iterator it = l_edges.begin();
        it!=l_edges.end();++it){
     if ((*it)->isSeam(this)){
       nSeams++;
@@ -1233,12 +1295,11 @@ int GFace::genusGeom()
   return nSeams - single_seams.size();
 }
 
-bool GFace::fillPointCloud(double maxDist, 
+bool GFace::fillPointCloud(double maxDist,
 			   std::vector<SPoint3> *points,
 			   std::vector<SPoint2> *uvpoints,
                            std::vector<SVector3> *normals)
 {
-
   if(!points) return false;
 
   if (buildSTLTriangulation()){
@@ -1246,7 +1307,7 @@ bool GFace::fillPointCloud(double maxDist,
       SPoint2 &p0(stl_vertices[stl_triangles[i]]);
       SPoint2 &p1(stl_vertices[stl_triangles[i + 1]]);
       SPoint2 &p2(stl_vertices[stl_triangles[i + 2]]);
-      
+
       GPoint gp0 = point(p0);
       GPoint gp1 = point(p1);
       GPoint gp2 = point(p2);
@@ -1278,7 +1339,7 @@ bool GFace::fillPointCloud(double maxDist,
 	points->push_back(SPoint3(gp.x(), gp.y(), gp.z()));
 	if (uvpoints)uvpoints->push_back(SPoint2(t1,t2));
 	if(normals) normals->push_back(normal(SPoint2(t1,t2)));
-      }	
+      }
     }
   }
   return true;
