@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #ifndef _GFACE_H_
 #define _GFACE_H_
@@ -93,6 +93,11 @@ class GFace : public GEntity
   // edges that bound the face
   virtual std::list<GEdge*> edges() const { return l_edges; }
   virtual std::list<int> edgeOrientations() const { return l_dirs; }
+  inline bool containsEdge (int iEdge) const {
+    for (std::list<GEdge*>::const_iterator it = l_edges.begin() ; it !=l_edges.end() ; ++it)
+      if ((*it)->tag() == iEdge) return true;
+    return false;
+  }
 
   // edges that are embedded in the face
   virtual std::list<GEdge*> embeddedEdges() const { return embedded_edges; }
@@ -133,7 +138,7 @@ class GFace : public GEntity
   // by default, genus is ZERO
   int poincareMesh();
   int genusMesh() { return (poincareMesh() + edgeLoops.size() - 2) / 2; }
-  virtual int genusGeom();
+  virtual int genusGeom() const;
   virtual bool checkTopology() const { return true; }
 
   // return the point on the face corresponding to the given parameter
@@ -244,6 +249,10 @@ class GFace : public GEntity
   // for that face
   void moveToValidRange(SPoint2 &pt) const;
 
+  //compute mesh statistics
+  void computeMeshSizeFieldAccuracy(double &avg,double &max_e, double &min_e,
+				    int &nE, int &GS);
+
   // compound
   void setCompound(GFaceCompound *gfc) { compound = gfc; }
   GFaceCompound *getCompound() const { return compound; }
