@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 //
 // Contributor(s):
 //   Emilie Marchandise
@@ -74,6 +74,33 @@ static void printBound(std::vector<MVertex*> &l, int tag)
 }
 */
 
+static std::vector<MVertex*> getBlob(unsigned int minNbPt, v2t_cont::iterator it, v2t_cont &adj)
+{
+
+  std::vector<MVertex*> vv(1,it->first), bvv = vv;                                                      // Vector of vertices in blob and in boundary of blob
+  do {
+    std::set<MVertex*> nbvv;                                                                            // Set of vertices in new boundary
+    for (std::vector<MVertex*>::iterator itBV = bvv.begin(); itBV != bvv.end(); itBV++) {               // For each boundary vertex...
+      std::vector<MElement*> &adjBV = adj[*itBV];
+      for (std::vector<MElement*>::iterator itBVEl = adjBV.begin(); itBVEl != adjBV.end(); itBVEl++) {
+        for (int iV=0; iV<(*itBVEl)->getNumVertices(); iV++){                                           // ... look for adjacent vertices...
+          MVertex *v = (*itBVEl)->getVertex(iV);
+          if (find(vv.begin(),vv.end(),v) == vv.end()) nbvv.insert(v);                                  // ... and add them in the new boundary if they are not already in the blob
+        }
+      }
+    }
+    if (nbvv.empty()) bvv.clear();
+    else {
+      bvv.assign(nbvv.begin(),nbvv.end());
+      vv.insert(vv.end(),nbvv.begin(),nbvv.end());
+    }
+//  } while (!bvv.empty());
+  } while (vv.size() < minNbPt);                                                                        // Repeat until min. number of points is reached
+
+  return vv;
+
+}
+
 static bool orderVertices(const std::list<GEdge*> &e, std::vector<MVertex*> &l,
                           std::vector<double> &coord)
 {
@@ -148,13 +175,18 @@ static bool computeCGKernelPolygon(std::map<MVertex*,SPoint3> &coordinates,
 
   int nbPts = cavV.size();
   fullMatrix<double> u(100,2);
-  int i = 0;
+  int ipt = 0;
   for(std::vector<MVertex*>::iterator it = cavV.begin(); it != cavV.end(); it++){
     SPoint3 vsp = coordinates[*it];
-    u(i,0) = vsp[0];
-    u(i,1) = vsp[1];
-    i++;
+    u(ipt,0) = vsp[0];
+    u(ipt,1) = vsp[1];
+    ucg += u(ipt,0);
+    vcg += u(ipt,1);
+    ipt++;
   }
+  ucg /= ipt;
+  vcg /= ipt;
+
   double eps = -5.e-7;
   int N = nbPts;
 
@@ -211,6 +243,8 @@ static bool computeCGKernelPolygon(std::map<MVertex*,SPoint3> &coordinates,
 
   int nbFinal = setP.size();
   if(nbFinal > 0){
+    ucg = 0.0;
+    vcg = 0.0;
     for(std::set<int>::iterator it =setP.begin(); it != setP.end(); it++){
       ucg += u(*it,0);
       vcg += u(*it,1);
@@ -420,7 +454,7 @@ void GFaceCompound::printFillTris() const
 
   if (fillTris.size() > 0){
     char name[256];
-    std::list<GFace*>::const_iterator itf = _compound.begin();
+    //std::list<GFace*>::const_iterator itf = _compound.begin();
     sprintf(name, "fillTris-%d.pos", tag());
     FILE * ftri = fopen(name,"w");
     fprintf(ftri,"View \"\"{\n");
@@ -575,6 +609,7 @@ bool GFaceCompound::trivial() const
 // no overlapping of triangles
 bool GFaceCompound::checkOverlap(std::vector<MVertex *> &vert) const
 {
+
   vert.clear();
   bool has_overlap = false;
   double EPS = 1.e-2;
@@ -602,10 +637,11 @@ bool GFaceCompound::checkOverlap(std::vector<MVertex *> &vert) const
 	  has_overlap = true;
 	  MVertex *v1 = orderedLoop[i];
 	  MVertex *v2 = orderedLoop[k];
-	  std::set<MVertex *>::iterator it1 = ov.find(v1);
-	  std::set<MVertex *>::iterator it2 = ov.find(v2);
+	  //std::set<MVertex *>::iterator it1 = ov.find(v1);
+	  //std::set<MVertex *>::iterator it2 = ov.find(v2);
 	  vert.push_back(v1);
 	  vert.push_back(v2);
+	  Msg::Info("=== Overlap");
 	  return has_overlap;
 	}
       }
@@ -613,10 +649,6 @@ bool GFaceCompound::checkOverlap(std::vector<MVertex *> &vert) const
 
   }
 
-  if (has_overlap ) {
-    Msg::Debug("Overlap for compound face %d", this->tag());
-  }
-
   return has_overlap;
 }
 
@@ -654,11 +686,14 @@ bool GFaceCompound::checkOrientation(int iter, bool moveBoundaries) const
     else if (!moveBoundaries){
       if (iter ==0) Msg::Info("--- Flipping : applying cavity checks.");
       Msg::Debug("--- Cavity Check - iter %d -",iter);
-      bool success = one2OneMap();
-      if (success) return checkOrientation(iter+1);
+      oriented = one2OneMap();
+      printStuff(iter);
+      iter++;
+      if (!oriented) return checkOrientation(iter);
     }
   }
-  else if (iter > 0 && iter < iterMax){
+
+  if (iter > 0 && iter < iterMax){
     Msg::Info("--- Flipping : no more flips (%d iter)", iter);
   }
 
@@ -831,12 +866,13 @@ bool GFaceCompound::one2OneMap() const
       double u_cg, v_cg;
       std::vector<MVertex*> cavV;
       myPolygon(vTri, cavV);
-      bool success = computeCGKernelPolygon(coordinates, cavV, u_cg, v_cg);
-      if (success){
+      //bool success =
+         computeCGKernelPolygon(coordinates, cavV, u_cg, v_cg);
+      //if (success){ //if not succes compute with CG polygon
 	nbRepair++;
 	SPoint3 p_cg(u_cg,v_cg,0.0);
 	coordinates[v] = p_cg;
-      }
+      //}
     }
   }
   if (nbRepair == 0) return false;
@@ -893,25 +929,24 @@ bool GFaceCompound::parametrize() const
   // Conformal map parametrization
   else if (_mapping == CONFORMAL){
     std::vector<MVertex *> vert;
-    bool oriented;
+    bool oriented, overlap;
     if (_type == SPECTRAL){
       Msg::Info("Parametrizing surface %d with 'spectral conformal map'", tag());
-      parametrize_conformal_spectral();
+      overlap = parametrize_conformal_spectral();
     }
-    else if (_type == FE){
+    else {
       Msg::Info("Parametrizing surface %d with 'FE conformal map'", tag());
-      parametrize_conformal(0, NULL, NULL);
+      overlap = parametrize_conformal(0, NULL, NULL);
     }
-    printStuff(55);
-    oriented = false; //checkOrientation(0);
-    if (!oriented)  oriented = checkOrientation(0, true);
-    printStuff(77);
-    if (_type==SPECTRAL &&  (!oriented  || checkOverlap(vert)) ){
+    //printStuff(55);
+    oriented = checkOrientation(0);
+    //printStuff(77);
+    if (_type==SPECTRAL &&  (!oriented || overlap) ){
       Msg::Warning("!!! parametrization switched to 'FE conformal' map");
-      parametrize_conformal(0, NULL, NULL);
-      oriented = checkOrientation(0, true);
+      overlap = parametrize_conformal(0, NULL, NULL);
+      oriented = checkOrientation(0);
     }
-    if (!oriented || checkOverlap(vert)){
+    if (!oriented || overlap){
       Msg::Warning("$$$ parametrization switched to 'convex' map");
       _type  = UNITCIRCLE;
       parametrize(ITERU,CONVEX);
@@ -927,7 +962,7 @@ bool GFaceCompound::parametrize() const
     /*
     fullMatrix<double> Oper(3*allNodes.size(),3*allNodes.size());
     _rbf = new GRbf(sizeBox, variableEps, radFunInd, _normals, allNodes, _ordered);
-    _rbf->RbfLapSurface_global_CPM_high_2(_rbf->getXYZ(), _rbf->getN(), Oper);
+    _rbf->RbfLapSurface_local_CPM(false,_rbf->getXYZ(), _rbf->getN(), Oper);
     _rbf->solveHarmonicMap(Oper, _ordered, _coords, coordinates);
     */
     fullMatrix<double> Oper(3*allNodes.size(),3*allNodes.size());
@@ -935,6 +970,7 @@ bool GFaceCompound::parametrize() const
     _rbf = new GRbf(sizeBox, variableEps, radFunInd, _normals, allNodes, _ordered);
 
     linearSystemPETSc<double> sys;
+	  printf("system 0 = %p\n", &sys);
     #if 1
     _rbf->RbfLapSurface_local_CPM_sparse(_ordered, false, _rbf->getXYZ(), _rbf->getN(), sys);
     _rbf->solveHarmonicMap_sparse(sys, _rbf->getXYZ().size1()* 3,_ordered, _coords, coordinates);
@@ -948,6 +984,7 @@ bool GFaceCompound::parametrize() const
 
   if (_mapping != RBF){
     if (!checkOrientation(0)){
+      printStuff(22);
       Msg::Info("### parametrization switched to 'convex map' onto circle");
       printStuff(33);
       _type = UNITCIRCLE;
@@ -1013,6 +1050,7 @@ void GFaceCompound::getBoundingEdges()
       l_edges.push_back(*itf);
       (*itf)->addFace(this);
     }
+
     std::list<GEdge*> loop;
     computeALoop(_unique,loop);
     while(!_unique.empty())  computeALoop(_unique, loop);
@@ -1307,7 +1345,7 @@ SPoint2 GFaceCompound::getCoordinates(MVertex *v) const
 
 	// for the Edge, find the left and right vertices of the initial
 	// 1D mesh and interpolate to find (u,v)
-	MVertex *vL = v0;
+	//MVertex *vL = v0;
 	MVertex *vR = v1;
 	double tB = ge->parBounds(0).low();
 	double tE = ge->parBounds(0).high();
@@ -1333,7 +1371,7 @@ SPoint2 GFaceCompound::getCoordinates(MVertex *v) const
 	  }
 	  else{
 	    itL = coordinates.find(vR);
-	    vL = vR;
+	    //vL = vR;
 	    tL = tR;
 	  }
 	  j++;
@@ -1546,11 +1584,11 @@ bool GFaceCompound::parametrize_conformal_spectral() const
     myAssembler.numberVertex(v, 0, 2);
   }
 
-  // for(std::set<MVertex *>::iterator itv = fillNodes.begin(); itv !=fillNodes.end() ; ++itv){
-  //   MVertex *v = *itv;
-  //   myAssembler.numberVertex(v, 0, 1);
-  //   myAssembler.numberVertex(v, 0, 2);
-  // }
+  for(std::set<MVertex *>::iterator itv = fillNodes.begin(); itv !=fillNodes.end() ; ++itv){
+    MVertex *v = *itv;
+    myAssembler.numberVertex(v, 0, 1);
+    myAssembler.numberVertex(v, 0, 2);
+  }
 
   laplaceTerm laplace1(model(), 1, ONE);
   laplaceTerm laplace2(model(), 2, ONE);
@@ -1567,13 +1605,13 @@ bool GFaceCompound::parametrize_conformal_spectral() const
     }
   }
 
-  // for (std::list<MTriangle*>::iterator it2 = fillTris.begin(); it2 != fillTris.end(); it2++){
-  //   SElement se((*it2));
-  //   laplace1.addToMatrix(myAssembler, &se);
-  //   laplace2.addToMatrix(myAssembler, &se);
-  //   cross12.addToMatrix(myAssembler, &se);
-  //   cross21.addToMatrix(myAssembler, &se);
-  // }
+  for (std::list<MTriangle*>::iterator it2 = fillTris.begin(); it2 != fillTris.end(); it2++){
+    SElement se((*it2));
+    laplace1.addToMatrix(myAssembler, &se);
+    laplace2.addToMatrix(myAssembler, &se);
+    cross12.addToMatrix(myAssembler, &se);
+    cross21.addToMatrix(myAssembler, &se);
+  }
 
   double epsilon = 1.e-6;
   for(std::set<MVertex *>::iterator itv = allNodes.begin(); itv !=allNodes.end() ; ++itv){
@@ -2056,6 +2094,7 @@ GPoint GFaceCompound::point(double par1, double par2) const
   if(!lt && _mapping != RBF){
     //printf("POINT no success %d tris %d quad \n", triangles.size(), quadrangles.size());
     GPoint gp = pointInRemeshedOctree(par1,par2);
+    gp.setNoSuccess();
     return gp;
   }
   else if (!lt && _mapping == RBF){
@@ -2172,11 +2211,81 @@ Pair<SVector3,SVector3> GFaceCompound::firstDer(const SPoint2 &param) const
 void GFaceCompound::secondDer(const SPoint2 &param,
                               SVector3 *dudu, SVector3 *dvdv, SVector3 *dudv) const
 {
+#if defined(HAVE_MESH)
+
   if(!oct) parametrize();
-  //leave debug here (since outputScalarField calls curvatureDiv)
-  Msg::Debug("Computation of the second derivatives is not implemented for compound faces");
+
+  if(adjv.size() == 0){
+    std::vector<MTriangle*> allTri;
+    std::list<GFace*>::const_iterator it = _compound.begin();
+    for( ; it != _compound.end(); ++it){
+      allTri.insert(allTri.end(), (*it)->triangles.begin(), (*it)->triangles.end() );
+    }
+    buildVertexToTriangle(allTri, adjv);
+  }
+
+  if (xuu.size() == 0) computeHessianMapping();
+
+  double U, V;
+  GFaceCompoundTriangle *lt;
+  getTriangle(param.x(), param.y(), &lt, U,V);
+  if(!lt) return;
+
+  MVertex* v0 = lt->tri->getVertex(0);
+  MVertex* v1 = lt->tri->getVertex(1);
+  MVertex* v2 = lt->tri->getVertex(2);
+
+  *dudu = (1-U-V)*xuu[v0] + U*xuu[v1] + V*xuu[v2];
+  *dvdv = (1-U-V)*xvv[v0] + U*xvv[v1] + V*xvv[v2];
+  *dudv = (1-U-V)*xuv[v0] + U*xuv[v1] + V*xuv[v2];
+
+#endif
+  //Msg::Debug("Computation of the second derivatives is not implemented for compound faces");
 }
 
+void GFaceCompound::computeHessianMapping() const
+{
+
+#if defined(HAVE_MESH)
+  unsigned int sysDim = 6; //for 2D
+  unsigned int minNbPtBlob = 3*sysDim;
+  for (v2t_cont::iterator it = adjv.begin(); it != adjv.end(); it++) {
+    MVertex *ver = it->first;
+    std::vector<MVertex*> vv = getBlob(minNbPtBlob, it, adjv);
+    fullMatrix<double> A(vv.size(),sysDim), ATAx(sysDim,sysDim), ATAy(sysDim,sysDim), ATAz(sysDim,sysDim);
+    fullVector<double> bx(vv.size()), ATbx(sysDim), coeffsx(sysDim);
+    fullVector<double> by(vv.size()), ATby(sysDim), coeffsy(sysDim);
+    fullVector<double> bz(vv.size()), ATbz(sysDim), coeffsz(sysDim);
+    for(unsigned int i=0; i<vv.size(); i++) {
+      SPoint3 uv = coordinates[vv[i]];
+      A(i,0) = uv.x()*uv.x(); A(i,1) = uv.x()*uv.y(); A(i,2) = uv.y()*uv.y();
+      A(i,3) = uv.x(); A(i,4) = uv.y(); A(i,5) = 1.;
+      bx(i) = vv[i]->x();
+      by(i) = vv[i]->y();
+      bz(i) = vv[i]->z();
+    }
+    ATAx.gemmWithAtranspose(A,A,1.,0.);
+    ATAy = ATAx; ATAz = ATAx;
+    A.multWithATranspose(bx,1.,0.,ATbx);
+    A.multWithATranspose(by,1.,0.,ATby);
+    A.multWithATranspose(bz,1.,0.,ATbz);
+    ATAx.luSolve(ATbx,coeffsx);
+    ATAy.luSolve(ATby,coeffsy);
+    ATAz.luSolve(ATbz,coeffsz);
+    SPoint3 uv = coordinates[ver];
+    xuu[ver] = SVector3(2.*coeffsx(0),2.*coeffsy(0),2.*coeffsz(0)) ;
+    xvv[ver] = SVector3(2.*coeffsx(2),2.*coeffsy(2),2.*coeffsz(2)) ;
+    xuv[ver] = SVector3(coeffsx(1), coeffsy(1),coeffsz(1));
+    xu[ver]  = SVector3(2.*coeffsx(0)*uv.x()+coeffsx(1)*uv.y()+coeffsx(3),
+			2.*coeffsy(0)*uv.x()+coeffsy(1)*uv.y()+coeffsy(3),
+			2.*coeffsz(0)*uv.x()+coeffsz(1)*uv.y()+coeffsz(3));
+    xv[ver] = SVector3(coeffsx(1)*uv.x()+2.*coeffsx(2)*uv.y()+coeffsx(4),
+		       coeffsy(1)*uv.x()+2.*coeffsy(2)*uv.y()+coeffsy(4),
+		       coeffsz(1)*uv.x()+2.*coeffsz(2)*uv.y()+coeffsz(4));
+  }
+
+#endif
+}
 static void GFaceCompoundBB(void *a, double*mmin, double*mmax)
 {
   GFaceCompoundTriangle *t = (GFaceCompoundTriangle *)a;
