@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #ifndef _GFACE_COMPOUND_H_
 #define _GFACE_COMPOUND_H_
@@ -86,6 +86,11 @@ class GFaceCompound : public GFace {
   mutable v2t_cont adjv;
   mutable bool mapv2Tri;
   mutable std::map<MVertex*, SPoint3> coordinates;
+  mutable std::map<MVertex*, SVector3> xuu;
+  mutable std::map<MVertex*, SVector3> xvv;
+  mutable std::map<MVertex*, SVector3> xuv;
+  mutable std::map<MVertex*, SVector3> xu;
+  mutable std::map<MVertex*, SVector3> xv;
   mutable std::map<SPoint3,SPoint3 > _coordPoints;
   mutable std::map<MVertex*, SVector3> _normals;
   mutable std::list<MTriangle*> fillTris;
@@ -133,6 +138,7 @@ class GFaceCompound : public GFace {
   double getSizeBB(const std::list<GEdge* > &elist) const;
   bool trivial() const;
   void printStuff(int iNewton=0) const;
+  void computeHessianMapping() const;
 
  public:
   GFaceCompound(GModel *m, int tag, std::list<GFace*> &compound,
@@ -229,6 +235,9 @@ class GFaceCompound : public GFace {
                          SVector3 *dudu, SVector3 *dvdv, SVector3 *dudv) const{}
   virtual SPoint2 getCoordinates(MVertex *v) const { return SPoint2(); }
   void parametrize() const {}
+  virtual std::list<GFace*> getCompounds() const { return std::list<GFace*>(); }
+  GPoint intersectionWithCircle (const SVector3 &n1, const SVector3 &n2, const SVector3 &p,
+				 const double &d, double uv[2]) const { return GPoint(); }
 };
 
 #endif
