@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #ifndef _GMODEL_H_
 #define _GMODEL_H_
@@ -37,18 +37,30 @@ class GModel
 {
  private:
   friend class OCCFactory;
-  std::multimap<std::pair<std::vector<int>, std::vector<int> >, std::string> _homologyRequests;
+  std::multimap<std::pair<std::vector<int>, std::vector<int> >,
+                std::pair<std::string, std::vector<int> > > _homologyRequests;
   std::set<GRegion*, GEntityLessThan> _chainRegions;
   std::set<GFace*, GEntityLessThan> _chainFaces;
   std::set<GEdge*, GEntityLessThan> _chainEdges;
   std::set<GVertex*, GEntityLessThan> _chainVertices;
 
+  int _readMSH2(const std::string &name);
+  int _writeMSH2(const std::string &name, double version=2.2, bool binary=false,
+                 bool saveAll=false, bool saveParametric=false,
+                 double scalingFactor=1.0, int elementStartNum=0,
+                 int saveSinglePartition=0,
+                 bool multipleView=false);
+
+  // the maximum vertex and element id number in the mesh
+  int _maxVertexNum, _maxElementNum;
+  int _checkPointedMaxVertexNum, _checkPointedMaxElementNum;
  protected:
   // the name of the model
   std::string _name;
 
   // the name of the file the model was read from
   std::string _fileName;
+  std::set<std::string> _fileNames;
 
   // the visibility flag
   char _visible;
@@ -154,12 +166,29 @@ class GModel
   static int setCurrent(GModel *m);
   int setAsCurrent(){ return setCurrent(this); }
 
-  // find a model by name
-  static GModel *findByName(std::string name);
+  // find a model by name; if fileName is given, return model only if it does
+  // *not* have a link to the fileName
+  static GModel *findByName(const std::string &name, const std::string &fileName="");
 
   // delete everything in a GModel
   void destroy();
 
+  // get/set global vertex/element num
+  int getMaxVertexNumber(){ return _maxVertexNum; }
+  int getMaxElementNumber(){ return _maxElementNum; }
+  void setMaxVertexNumber(int num){ _maxVertexNum = num; }
+  void setMaxElementNumber(int num){ _maxElementNum = num; }
+  void checkPointMaxNumbers()
+  {
+    _checkPointedMaxVertexNum = _maxVertexNum;
+    _checkPointedMaxVertexNum = _maxVertexNum;
+  }
+  void getCheckPointedMaxNumbers(int &maxv, int &maxe)
+  {
+    maxv = _checkPointedMaxVertexNum;
+    maxe = _checkPointedMaxElementNum;
+  }
+
   // delete all the mesh-related caches (this must be called when the
   // mesh is changed)
   void destroyMeshCaches();
@@ -180,8 +209,16 @@ class GModel
   std::string getName(){ return _name; }
 
   // get/set the model file name
-  void setFileName(std::string fileName){ _fileName = fileName; }
+  void setFileName(std::string fileName)
+  {
+    _fileName = fileName;
+    _fileNames.insert(fileName);
+  }
   std::string getFileName(){ return _fileName; }
+  bool hasFileName(const std::string &name)
+  {
+    return _fileNames.find(name) != _fileNames.end();
+  }
 
   // get/set the visibility flag
   char getVisibility(){ return _visible; }
@@ -292,7 +329,7 @@ class GModel
   std::string getElementaryName(int dim, int num);
 
   //get the highest dimension of the GModel
-  int getDim();
+  int getDim() const;
 
   // set the selection flag on all entities
   void setSelection(int val);
@@ -334,7 +371,7 @@ class GModel
 
   // index all the (used) mesh vertices in a continuous sequence,
   // starting at 1
-  int indexMeshVertices(bool all, int singlePartition=0);
+  int indexMeshVertices(bool all, int singlePartition=0, bool renumber=true);
 
   // scale the mesh by the given factor
   void scaleMesh(double factor);
@@ -406,7 +443,10 @@ class GModel
   // In this first attempt, only the highest dimensional mesh is adapted, which is ok if
   // we assume that boundaries are already adapted.
   // This should be fixed.
-  int adaptMesh(std::vector<int> technique, std::vector<simpleFunction<double>*> f, std::vector<std::vector<double> > parameters, int niter, bool meshAll=false);
+  int adaptMesh(std::vector<int> technique,
+		std::vector<simpleFunction<double>*> f,
+		std::vector<std::vector<double> > parameters,
+		int niter, bool meshAll=false);
 
   // make the mesh a high order mesh at order N
   // linear is 1 if the high order points are not placed on the geometry of the model
@@ -450,7 +490,8 @@ class GModel
   GEntity *revolve(GEntity *e, std::vector<double> p1, std::vector<double> p2,
                    double angle);
   GEntity *extrude(GEntity *e, std::vector<double> p1, std::vector<double> p2);
-  std::vector<GEntity*> extrudeBoundaryLayer(GEntity *e, int nbLayers, double hLayers, int dir=1, int view=-1);
+  std::vector<GEntity*> extrudeBoundaryLayer(GEntity *e, int nbLayers, double hLayers,
+                                             int dir=1, int view=-1);
   GEntity *addPipe(GEntity *e, std::vector<GEdge *>  edges);
 
   void addRuledFaces(std::vector<std::vector<GEdge *> > edges);
@@ -478,9 +519,9 @@ class GModel
   // if cutElem is set to false, split the model without cutting the elements
   GModel *buildCutGModel(gLevelset *ls, bool cutElem=true, bool saveTri=false);
 
-  // create a GModel by importing a mesh (vertexMap has a dim equal to
-  // the number of vertices and all the other vectors have a dim equal
-  // to the number of elements)
+  // create a GModel by importing a mesh (vertexMap has a dim equal to the
+  // number of vertices and all the other vectors have a dim equal to the number
+  // of elements)
   static GModel *createGModel(std::map<int, MVertex*> &vertexMap,
                               std::vector<int> &numElement,
                               std::vector<std::vector<int> > &vertexIndices,
@@ -489,9 +530,9 @@ class GModel
                               std::vector<int> &elementary,
                               std::vector<int> &partition);
 
-  // create a GModel from newly created mesh elements
-  // (with their own newly created mesh vertices),
-  // and let element entities have given physical group tags
+  // create a GModel from newly created mesh elements (with their own newly
+  // created mesh vertices), and let element entities have given physical group
+  // tags
   static GModel *createGModel
     (std::map<int, std::vector<MElement*> > &entityToElementsMap,
      std::map<int, std::vector<int> > &entityToPhysicalsMap);
@@ -508,7 +549,7 @@ class GModel
 
   // request homology computation
   void addHomologyRequest(const std::string &type, std::vector<int> &domain,
-                          std::vector<int> &subdomain);
+                          std::vector<int> &subdomain, std::vector<int> &dim);
   void computeHomology();
 
   // "automatic" IO based on Gmsh global functions
@@ -544,7 +585,7 @@ class GModel
   int writeMSH(const std::string &name, double version=2.2, bool binary=false,
                bool saveAll=false, bool saveParametric=false,
                double scalingFactor=1.0, int elementStartNum=0,
-               int saveSinglePartition=0);
+               int saveSinglePartition=0, bool multipleView=false);
   int writePartitionedMSH(const std::string &baseName, bool binary=false,
                           bool saveAll=false, bool saveParametric=false,
                           double scalingFactor=1.0);
@@ -625,6 +666,7 @@ class GModel
 
   // CEA triangulation
   int writeMAIL(const std::string &name, bool saveAll, double scalingFactor);
+
 };
 
 #endif
