@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include <stdlib.h>
 #include "GModelFactory.h"
@@ -198,6 +198,7 @@ GRegion* GeoFactory::addVolume (GModel *gm, std::vector<std::vector<GFace *> > f
   Tree_Add(gm->getGEOInternals()->Volumes, &v);
   v->Typ = MSH_VOLUME;
   v->Num = numv;
+  v->Recombine3D = 0;
 
   //gmsh volume
   GRegion *gr = new gmshRegion(gm,v);
@@ -725,7 +726,7 @@ GEntity *OCCFactory::addSphere(GModel *gm, double xc, double yc, double zc, doub
   gm->destroy();
   gm->_occ_internals->buildLists();
   gm->_occ_internals->buildGModel(gm);
-  return getOCCRegionByNativePtr(gm, TopoDS::Solid(shape));
+  return gm->_occ_internals->getOCCRegionByNativePtr(gm, TopoDS::Solid(shape));
 }
 
 GRegion* OCCFactory::addVolume (GModel *gm, std::vector<std::vector<GFace *> > faces)
@@ -763,7 +764,7 @@ GEntity *OCCFactory::addCylinder(GModel *gm, std::vector<double> p1,
   gm->destroy();
   gm->_occ_internals->buildLists();
   gm->_occ_internals->buildGModel(gm);
-  return getOCCRegionByNativePtr(gm, TopoDS::Solid(shape));
+  return gm->_occ_internals->getOCCRegionByNativePtr(gm, TopoDS::Solid(shape));
 }
 
 GEntity *OCCFactory::addTorus(GModel *gm, std::vector<double> p1,
@@ -795,7 +796,7 @@ GEntity *OCCFactory::addTorus(GModel *gm, std::vector<double> p1,
   gm->destroy();
   gm->_occ_internals->buildLists();
   gm->_occ_internals->buildGModel(gm);
-  return getOCCRegionByNativePtr(gm, TopoDS::Solid(shape));
+  return gm->_occ_internals->getOCCRegionByNativePtr(gm, TopoDS::Solid(shape));
 }
 
 GEntity *OCCFactory::addCone(GModel *gm,  std::vector<double> p1,
@@ -828,7 +829,7 @@ GEntity *OCCFactory::addCone(GModel *gm,  std::vector<double> p1,
   gm->destroy();
   gm->_occ_internals->buildLists();
   gm->_occ_internals->buildGModel(gm);
-  return getOCCRegionByNativePtr(gm,TopoDS::Solid(shape));
+  return gm->_occ_internals->getOCCRegionByNativePtr(gm,TopoDS::Solid(shape));
 }
 
 GEntity *OCCFactory::addBlock(GModel *gm, std::vector<double> p1,
@@ -850,7 +851,7 @@ GEntity *OCCFactory::addBlock(GModel *gm, std::vector<double> p1,
   gm->destroy();
   gm->_occ_internals->buildLists();
   gm->_occ_internals->buildGModel(gm);
-  return getOCCRegionByNativePtr(gm, TopoDS::Solid(shape));
+  return gm->_occ_internals->getOCCRegionByNativePtr(gm, TopoDS::Solid(shape));
 }
 
 GModel *OCCFactory::computeBooleanUnion(GModel* obj, GModel* tool,
