@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #ifndef _GREGION_H_
 #define _GREGION_H_
@@ -26,8 +26,8 @@ class GRegion : public GEntity {
  protected:
   std::list<GFace*> l_faces;
   std::list<int> l_dirs;
-  GRegionCompound *compound; // this model ede belongs to a compound 
-  
+  GRegionCompound *compound; // this model ede belongs to a compound
+
   // replace faces (for gluing) for specific modelers, we have to
   // re-create internal data ...
   virtual void replaceFacesInternal (std::list<GFace*> &) {}
@@ -91,12 +91,14 @@ class GRegion : public GEntity {
 
   // reset the mesh attributes to default values
   virtual void resetMeshAttributes();
-  
+
   // compound
   void setCompound(GRegionCompound *grc) { compound = grc; }
   GRegionCompound *getCompound() const { return compound; }
 
   struct {
+    // do we recombine the tetrahedra of the mesh into hex?
+    int recombine3D;
     // is this surface meshed using a transfinite interpolation
     char Method;
     // the extrusion parameters (if any)
