@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #ifndef _GEO_H_
 #define _GEO_H_
@@ -152,6 +152,7 @@ class Surface{
   double RecombineAngle;
   int TransfiniteSmoothing;
   List_T *Generatrices;
+  List_T *GeneratricesByTag;
   List_T *EmbeddedCurves;
   List_T *EmbeddedPoints;
   List_T *TrsfPoints;
@@ -165,7 +166,6 @@ class Surface{
   // disappear from the class Surface.
   gmshSurface *geometry;
   // the mesh master surface
-  int meshMaster;
   std::map<int,int> edgeCounterparts;
   std::vector<int> compound, compoundBoundary[4];
 };
@@ -182,6 +182,7 @@ class Volume {
   int Typ;
   char Visible;
   int Method;
+  int Recombine3D;
   int QuadTri;
   ExtrudeParams *Extrude;
   List_T *TrsfPoints;
@@ -219,6 +220,8 @@ class GEO_Internals{
   ~GEO_Internals(){ free_all(); }
   void destroy(){ free_all(); alloc_all(); }
   void reset_physicals();
+  std::map<int,int> periodicFaces;
+  std::map<int,int> periodicEdges;
 };
 
 class Shape{
@@ -274,7 +277,8 @@ PhysicalGroup *FindPhysicalGroup(int inum, int type);
 List_T *GetAllEntityNumbers(int dim);
 
 void TranslateShapes(double X,double Y,double Z, List_T *shapes);
-void DilatShapes(double X,double Y,double Z, double A, List_T *shapes);
+void DilatShapes(double X,double Y,double Z, double A, double B, double C,
+                 List_T *shapes);
 void RotateShapes(double Ax,double Ay,double Az,
                   double Px,double Py, double Pz, double alpha, List_T *shapes);
 void SymmetryShapes(double A,double B,double C, double D, List_T *shapes);
