@@ -1,12 +1,13 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #ifndef _MLINE_H_
 #define _MLINE_H_
 
 #include "MElement.h"
+#include "nodalBasis.h"
 
 /*
  * MLine
@@ -37,6 +38,7 @@ class MLine : public MElement {
   virtual int getDim() const { return 1; }
   virtual int getNumVertices() const { return 2; }
   virtual MVertex *getVertex(int num){ return _v[num]; }
+  virtual void setVertex(int num,  MVertex *v){ _v[num] = v; }
   virtual double getInnerRadius(); // half-length of segment line
   virtual double getLength(); // length of segment line
   virtual double getVolume();
@@ -71,12 +73,12 @@ class MLine : public MElement {
   {
     MVertex *tmp = _v[0]; _v[0] = _v[1]; _v[1] = tmp;
   }
-  virtual const polynomialBasis* getFunctionSpace(int o=-1) const;
+  virtual const nodalBasis* getFunctionSpace(int o=-1) const;
   virtual const JacobianBasis* getJacobianFuncSpace(int o=-1) const;
   virtual bool isInside(double u, double v, double w)
   {
     double tol = _isInsideTolerance;
-    if(u < -(1. + tol) || u > (1. + tol))
+    if(u < -(1. + tol) || u > (1. + tol) || fabs(v) > tol || fabs(w) > tol)
       return false;
     return true;
   }
@@ -198,6 +200,8 @@ class MLineN : public MLine {
   }
   virtual int getTypeForMSH() const
   {
+    if(_vs.size() == 0) return MSH_LIN_2;
+    if(_vs.size() == 1) return MSH_LIN_3;
     if(_vs.size() == 2) return MSH_LIN_4;
     if(_vs.size() == 3) return MSH_LIN_5;
     if(_vs.size() == 4) return MSH_LIN_6;
