@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include <sstream>
 #include "GModel.h"
@@ -26,7 +26,6 @@ void gmshEdge::resetMeshAttributes()
   meshAttributes.typeTransfinite = c->typeTransfinite;
   meshAttributes.coeffTransfinite = c->coeffTransfinite;
   meshAttributes.extrude = c->Extrude;
-  setMeshMaster(c->meshMaster);
 }
 
 Range<double> gmshEdge::parBounds(int i) const
@@ -78,7 +77,7 @@ std::string gmshEdge::getAdditionalInfoString()
     std::ostringstream sstream;
     sstream << "{";
     for(int i = 0; i < List_Nbr(c->Control_Points); i++){
-      if(i) sstream << ",";
+      if(i) sstream << " ";
       Vertex *v;
       List_Read(c->Control_Points, i, &v);
       sstream << v->Num;
@@ -246,50 +245,82 @@ SPoint2 gmshEdge::reparamOnFace(const GFace *face, double epar,int dir) const
     }
     return SPoint2(U, V);
   }
-  else if (s->Typ ==  MSH_SURF_TRIC){
+  else if (s->Typ == MSH_SURF_TRIC){
     Curve *C[3];
     for(int i = 0; i < 3; i++)
       List_Read(s->Generatrices, i, &C[i]);
-
-    // FIXME: workaround for exact extrutions
-    bool hack = false;
-    if(CTX::instance()->geom.exactExtrusion && s->Extrude &&
-       s->Extrude->geo.Mode == EXTRUDED_ENTITY && s->Typ != MSH_SURF_PLAN)
-      hack = true;
-
     double U, V;
-    if (C[0]->Num == c->Num) {
-      U = (epar - C[0]->ubeg) / (C[0]->uend - C[0]->ubeg) ;
-      V = 0;
-    }
-    else if (C[0]->Num == -c->Num) {
-      U = (C[0]->uend - epar - C[0]->ubeg) / (C[0]->uend - C[0]->ubeg) ;
-      V = 0;
-    }
-    else if (C[1]->Num == c->Num) {
-      V = (epar - C[1]->ubeg) / (C[1]->uend - C[1]->ubeg) ;
-      U = 1;
-    }
-    else if (C[1]->Num == -c->Num) {
-      V = (C[1]->uend - epar - C[1]->ubeg) / (C[1]->uend - C[1]->ubeg) ;
-      U = 1;
-    }
-    else if (C[2]->Num == c->Num) {
-      U = 1-(epar - C[2]->ubeg) / (C[2]->uend - C[2]->ubeg) ;
-      V = hack ? 1 : U;
-    }
-    else if (C[2]->Num == -c->Num) {
-      U = 1-(C[2]->uend - epar - C[2]->ubeg) / (C[2]->uend - C[2]->ubeg) ;
-      V = hack ? 1 : U;
+    if(CTX::instance()->geom.oldRuledSurface){
+      if (C[0]->Num == c->Num) {
+        U = (epar - C[0]->ubeg) / (C[0]->uend - C[0]->ubeg) ;
+        V = 0;
+      }
+      else if (C[0]->Num == -c->Num) {
+        U = (C[0]->uend - epar - C[0]->ubeg) / (C[0]->uend - C[0]->ubeg) ;
+        V = 0;
+      }
+      else if (C[1]->Num == c->Num) {
+        V = (epar - C[1]->ubeg) / (C[1]->uend - C[1]->ubeg) ;
+        U = 1;
+      }
+      else if (C[1]->Num == -c->Num) {
+        V = (C[1]->uend - epar - C[1]->ubeg) / (C[1]->uend - C[1]->ubeg) ;
+        U = 1;
+      }
+      else if (C[2]->Num == c->Num) {
+        U = 1-(epar - C[2]->ubeg) / (C[2]->uend - C[2]->ubeg) ;
+        V = 1;
+      }
+      else if (C[2]->Num == -c->Num) {
+        U = 1-(C[2]->uend - epar - C[2]->ubeg) / (C[2]->uend - C[2]->ubeg) ;
+        V = 1;
+      }
+      else{
+        Msg::Info("Reparameterizing edge %d on face %d", c->Num, s->Num);
+        return GEdge::reparamOnFace(face, epar, dir);
+      }
+
     }
     else{
-      Msg::Info("Reparameterizing edge %d on face %d", c->Num, s->Num);
-      return GEdge::reparamOnFace(face, epar, dir);
+      // FIXME: workaround for exact extrusions
+      bool hack = false;
+      if(CTX::instance()->geom.exactExtrusion && s->Extrude &&
+         s->Extrude->geo.Mode == EXTRUDED_ENTITY && s->Typ != MSH_SURF_PLAN)
+        hack = true;
+      if (C[0]->Num == c->Num) {
+        U = (epar - C[0]->ubeg) / (C[0]->uend - C[0]->ubeg) ;
+        V = 0;
+      }
+      else if (C[0]->Num == -c->Num) {
+        U = (C[0]->uend - epar - C[0]->ubeg) / (C[0]->uend - C[0]->ubeg) ;
+        V = 0;
+      }
+      else if (C[1]->Num == c->Num) {
+        V = (epar - C[1]->ubeg) / (C[1]->uend - C[1]->ubeg) ;
+        U = 1;
+      }
+      else if (C[1]->Num == -c->Num) {
+        V = (C[1]->uend - epar - C[1]->ubeg) / (C[1]->uend - C[1]->ubeg) ;
+        U = 1;
+      }
+      else if (C[2]->Num == c->Num) {
+        U = 1-(epar - C[2]->ubeg) / (C[2]->uend - C[2]->ubeg) ;
+        V = hack ? 1 : U;
+      }
+      else if (C[2]->Num == -c->Num) {
+        U = 1-(C[2]->uend - epar - C[2]->ubeg) / (C[2]->uend - C[2]->ubeg) ;
+        V = hack ? 1 : U;
+      }
+      else{
+        Msg::Info("Reparameterizing edge %d on face %d", c->Num, s->Num);
+        return GEdge::reparamOnFace(face, epar, dir);
+      }
     }
     return SPoint2(U, V);
   }
-  else
+  else{
     return GEdge::reparamOnFace(face, epar, dir);
+  }
 }
 
 void gmshEdge::writeGEO(FILE *fp)
