@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 //
 // Contributor(s):
 //   Gaetan Bricteux
@@ -1157,3 +1157,45 @@ gLevelsetConrod::gLevelsetConrod(const double *pt, const double *dir1,
 
 gLevelsetConrod::gLevelsetConrod(const gLevelsetConrod &lv)
   : gLevelsetImproved(lv){}
+
+
+// Level-set for NACA0012 airfoil, last coeff. modified for zero-thickness trailing edge
+// cf. http://en.wikipedia.org/wiki/NACA_airfoil
+double gLevelsetNACA00::operator() (const double x, const double y, const double z) const
+{
+
+  static const int maxIter = 100;
+  static const double tol = 1.e-8;
+
+  const double tolr = tol/_c;                 // Tolerance (scaled bu chord)
+  double distSq;                              // Square of dist. between point and boundary
+  bool in = false;                            // Whether the point is inside the airfoil
+
+  const double xt = x-_x0, yt = fabs(y-_y0);  // Point translated according to airfoil origin and symmetry
+
+  if (xt-_c > 1.21125*_t*yt)                  // Behind line normal to airfoil at trailing edge, closest
+    distSq = (xt-_c)*(xt-_c)+yt*yt;           // boundary point is trailing edge
+  else {                                      // Otherwise Newton-Raphson to find closest boundary point
+    const double fact = 5.*_t*_c;
+    double xb = std::max(xt,tolr);
+    for (int it=0; it<maxIter; it++) {
+      const double xbr = xb/_c, sxbr = sqrt(xbr), xbr32 = xbr*sxbr,
+                    xbr2 = xbr*xbr, xbr3 = xbr2*xbr, xbr4 = xbr2*xbr2;
+      double yb = fact*(0.2969*sxbr-0.1260*xbr-0.3516*xbr2+0.2843*xbr3-0.1036*xbr4);
+      const double dyb = fact*(0.14845/sxbr-0.4144*xbr3+0.8529*xbr2-0.7032*xbr-0.126)/_c;
+      const double ddyb = fact*(-0.074225/xbr32-1.2432*xbr2+1.7058*xbr-0.7032)/(_c*_c);
+      const double xx = xt-xb, yy = yt-yb;
+      in = (xt > 0) && (yy < 0);
+      distSq = xx*xx+yy*yy;
+      const double dDistSq = -2.*(xx+dyb*yy);
+      const double ddDistSq = 2.*(1.-ddyb*yy+dyb*dyb);
+      const double mIncr = dDistSq/ddDistSq;
+      if (fabs(mIncr) < tolr) break;
+      else xb -= mIncr;
+      if (xb < tolr) xb = tolr;
+    }
+  }
+
+  return in ? -sqrt(distSq) : sqrt(distSq);
+
+}
