@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 //
 // Contributor(s):
 //   Gaetan Bricteux
@@ -22,6 +22,7 @@
 #include "cartesian.h"
 #include "simpleFunction.h"
 
+
 #if defined(HAVE_POST)
 #include "PView.h"
 #include "OctreePost.h"
@@ -209,7 +210,6 @@ protected:
   void init ();
 
 public:
-  gLevelsetQuadric() : gLevelsetPrimitive() {init(); }
   gLevelsetQuadric(int tag=1) : gLevelsetPrimitive(tag) {init(); }
   gLevelsetQuadric(const gLevelsetQuadric &);
   virtual ~gLevelsetQuadric() {}
@@ -293,6 +293,21 @@ public:
   int type() const { return UNKNOWN; }
 };
 
+class gLevelsetSimpleFunction: public gLevelsetPrimitive
+{
+  simpleFunction<double> *_f;
+public:
+  gLevelsetSimpleFunction(simpleFunction<double> *f, int tag=1){
+    _f = f; 
+  }
+  ~gLevelsetSimpleFunction(){}
+  double operator () (const double x, const double y, const double z) const
+  {
+    return (*_f)(x,y,z);
+  }
+  int type() const { return UNKNOWN; }
+};
+
 class gLevelsetDistGeom: public gLevelsetPrimitive
 {
   cartesianBox<double> *_box;
@@ -585,4 +600,14 @@ public:
 /*   int type2() const {return DISK;} */
 /* }; */
 
+class gLevelsetNACA00: public gLevelsetPrimitive
+{
+  double _x0, _y0, _c, _t;
+public:
+  gLevelsetNACA00(double x0, double y0, double c, double t) : _x0(x0), _y0(y0), _c(c), _t(t) {}
+  ~gLevelsetNACA00() {}
+  double operator () (const double x, const double y, const double z) const;
+  int type() const { return UNKNOWN; }
+};
+
 #endif
