@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include <math.h>
 #include <algorithm>
@@ -23,7 +23,7 @@
 #include <FL/gl.h>
 #endif
 
-static void drawArrays(drawContext *ctx, PView *p, VertexArray *va, GLint type, 
+static void drawArrays(drawContext *ctx, PView *p, VertexArray *va, GLint type,
                        bool useNormalArray)
 {
   if(!va || !va->getNumVertices()) return;
@@ -68,8 +68,19 @@ static void drawArrays(drawContext *ctx, PView *p, VertexArray *va, GLint type,
         double v0 = char2float(*n0), v1 = char2float(*n1);
         ctx->drawTaperedCylinder(opt->lineWidth, v0, v1, 0., 1., x, y, z, opt->light);
       }
-      else
+      else if (opt->lineType == 1)
         ctx->drawCylinder(opt->lineWidth, x, y, z, opt->light);
+      else { /// 2D (for now) MNT DIAGRAMS FOR FRAMES
+	float l = sqrt ((p0[0]-p1[0])*(p0[0]-p1[0]) + (p0[1]-p1[1])*(p0[1]-p1[1]) + (p0[2]-p1[2])*(p0[2]-p1[2]) );
+        char *n0 = va->getNormalArray(3 * i);
+        char *n1 = va->getNormalArray(3 * (i + 1));
+        double v0 = char2float(*n0), v1 = char2float(*n1);
+	float dir [3] = {(p1[0]-p0[0])/l , (p1[1]-p0[1])/l , (p1[2]-p0[2])/l };
+	printf("%g %g %g %g %g %g\n",v0,v1,p0[0],p0[1],p1[0],p1[1]);
+	ctx->drawVector(1, 0, 
+			p0[0] - dir[1] * v0 , p0[1] + dir[0] * v0 , 0.0,
+			p1[0] - dir[1] * v1 , p1[1] + dir[0] * v1 , 0.0,opt->light);
+      }
     }
   }
   else{
@@ -99,7 +110,7 @@ static void drawEllipseArray(drawContext *ctx, PView *p, VertexArray *va)
   if(!va || va->getNumVerticesPerElement() != 4) return;
 
   PViewOptions *opt = p->getOptions();
-  
+
   for(int i = 0; i < va->getNumVertices(); i += 4) {
     float *s = va->getVertexArray(3 * i);
     float vv[3][3];
@@ -127,7 +138,7 @@ static void drawVectorArray(drawContext *ctx, PView *p, VertexArray *va)
   if(!va || va->getNumVerticesPerElement() != 2) return;
 
   PViewOptions *opt = p->getOptions();
-  
+
   for(int i = 0; i < va->getNumVertices(); i += 2){
     float *s = va->getVertexArray(3 * i);
     float *v = va->getVertexArray(3 * (i + 1));
@@ -137,9 +148,9 @@ static void drawVectorArray(drawContext *ctx, PView *p, VertexArray *va)
     if((l || opt->vectorType == 6) && lmax){
       double scale = (opt->arrowSizeMax - opt->arrowSizeMin) / lmax;
       // log scaling
-      if(opt->scaleType == PViewOptions::Logarithmic && 
+      if(opt->scaleType == PViewOptions::Logarithmic &&
          opt->tmpMin > 0 && opt->tmpMax > opt->tmpMin && l != opt->tmpMin){
-        scale = (opt->arrowSizeMax - opt->arrowSizeMin) / l * 
+        scale = (opt->arrowSizeMax - opt->arrowSizeMin) / l *
           log10(l / opt->tmpMin) / log10(opt->tmpMax / opt->tmpMin);
       }
       if(opt->arrowSizeMin && l) scale += opt->arrowSizeMin / l;
@@ -165,7 +176,7 @@ static void drawVectorArray(drawContext *ctx, PView *p, VertexArray *va)
   }
 }
 
-static std::string stringValue(int numComp, double d[9], double norm, 
+static std::string stringValue(int numComp, double d[9], double norm,
                                const char *format)
 {
   char label[100];
@@ -183,7 +194,7 @@ static std::string stringValue(int numComp, double d[9], double norm,
   return std::string(label);
 }
 
-static void drawNumberGlyphs(drawContext *ctx, PView *p, int numNodes, int numComp, 
+static void drawNumberGlyphs(drawContext *ctx, PView *p, int numNodes, int numComp,
                              double **xyz, double **val)
 {
   PViewOptions *opt = p->getOptions();
@@ -230,7 +241,7 @@ static void drawNumberGlyphs(drawContext *ctx, PView *p, int numNodes, int numCo
   }
 }
 
-static void drawNormalVectorGlyphs(drawContext *ctx, PView *p, int numNodes, 
+static void drawNormalVectorGlyphs(drawContext *ctx, PView *p, int numNodes,
                                    double **xyz, double **val)
 {
   PViewOptions *opt = p->getOptions();
@@ -265,7 +276,7 @@ static void drawTangentVectorGlyphs(drawContext *ctx, PView *p, int numNodes,
   for(int i = 0; i < 3; i++)
     t[i] *= opt->tangents * ctx->pixel_equiv_x / ctx->s[i];
   glColor4ubv((GLubyte *) & opt->color.tangents);
-  ctx->drawVector(CTX::instance()->vectorType, 0, pc[0], pc[1], pc[2], t[0], t[1], t[2], 
+  ctx->drawVector(CTX::instance()->vectorType, 0, pc[0], pc[1], pc[2], t[0], t[1], t[2],
                   opt->light);
 }
 
@@ -284,7 +295,6 @@ static void drawGlyphs(drawContext *ctx, PView *p)
 
   // speedup drawing of textured fonts on cocoa mac version
 #if defined(HAVE_FLTK) && defined(__APPLE__) && defined(HAVE_64BIT_SIZE_T)
-#if (FL_MAJOR_VERSION == 1) && (FL_MINOR_VERSION == 3)
   if(opt->intervalsType == PViewOptions::Numeric){
     int numStrings = 0;
     for(int ent = 0; ent < data->getNumEntities(opt->timeStep); ent++)
@@ -292,7 +302,6 @@ static void drawGlyphs(drawContext *ctx, PView *p)
     if(gl_texture_pile_height() < numStrings)
       gl_texture_pile_height(numStrings);
   }
-#endif
 #endif
 
   //double xyz[PVIEW_NMAX][3], val[PVIEW_NMAX][9];
@@ -361,7 +370,7 @@ static void drawGlyphs(drawContext *ctx, PView *p)
       if(dim == 2 && opt->normals)
         drawNormalVectorGlyphs(ctx, p, numNodes, xyz, val);
       else if(dim == 1 && opt->tangents)
-        drawTangentVectorGlyphs(ctx, p, numNodes, xyz, val);  
+        drawTangentVectorGlyphs(ctx, p, numNodes, xyz, val);
     }
   }
   for(int j = 0; j < NMAX; j++){
@@ -397,15 +406,15 @@ class drawPView {
     if(data->getDirty() || !data->getNumTimeSteps()) return;
     if(!opt->visible || opt->type != PViewOptions::Plot3D) return;
     if(!_ctx->isVisible(p)) return;
-   
+
     glPointSize((float)opt->pointSize);
-    gl2psPointSize((float)(opt->pointSize * 
+    gl2psPointSize((float)(opt->pointSize *
                            CTX::instance()->print.epsPointSizeFactor));
-    
+
     glLineWidth((float)opt->lineWidth);
     gl2psLineWidth((float)(opt->lineWidth *
                            CTX::instance()->print.epsLineWidthFactor));
-    
+
     if(opt->lightTwoSide)
       glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
     else
@@ -414,16 +423,16 @@ class drawPView {
     if(opt->axes && opt->type == PViewOptions::Plot3D){
       glColor4ubv((GLubyte *) & opt->color.axes);
       glLineWidth((float)CTX::instance()->lineWidth);
-      gl2psLineWidth((float)(CTX::instance()->lineWidth * 
+      gl2psLineWidth((float)(CTX::instance()->lineWidth *
                              CTX::instance()->print.epsLineWidthFactor));
       if(!opt->axesAutoPosition)
         _ctx->drawAxes(opt->axes, opt->axesTics, opt->axesFormat, opt->axesLabel,
-                       opt->axesPosition, opt->axesMikado);
+                       opt->axesPosition, opt->axesMikado, opt->axesPosition);
       else if(!opt->tmpBBox.empty())
         _ctx->drawAxes(opt->axes, opt->axesTics, opt->axesFormat, opt->axesLabel,
-                       opt->tmpBBox, opt->axesMikado);
+                       opt->tmpBBox, opt->axesMikado, opt->tmpBBox);
     }
-    
+
     if(!CTX::instance()->clipWholeElements){
       for(int i = 0; i < 6; i++)
         if(opt->clip & (1 << i))
@@ -446,7 +455,7 @@ class drawPView {
         glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
         // glBlendEquation(GL_FUNC_ADD);
         glEnable(GL_BLEND);
-        if(p->va_triangles && p->va_triangles->getNumVertices() && 
+        if(p->va_triangles && p->va_triangles->getNumVertices() &&
            eyeChanged(_ctx, p)){
           Msg::Debug("Sorting View[%d] for transparency", p->getIndex());
           p->va_triangles->sort(p->getEye().x(), p->getEye().y(), p->getEye().z());
@@ -494,7 +503,7 @@ class drawPView {
         _ctx->drawString(str, style);
       }
     }
-    
+
     if(CTX::instance()->alpha){
       glDisable(GL_BLEND);
       glEnable(GL_DEPTH_TEST);
@@ -522,7 +531,7 @@ class drawPViewBoundingBox {
 
     glColor4ubv((GLubyte *) & CTX::instance()->color.fg);
     glLineWidth((float)CTX::instance()->lineWidth);
-    gl2psLineWidth((float)(CTX::instance()->lineWidth * 
+    gl2psLineWidth((float)(CTX::instance()->lineWidth *
                            CTX::instance()->print.epsLineWidthFactor));
 
     _ctx->drawBox(bb.min().x(), bb.min().y(), bb.min().z(),
@@ -532,9 +541,9 @@ class drawPViewBoundingBox {
       if(opt->clip & (1 << i))
         _ctx->drawPlaneInBoundingBox(bb.min().x(), bb.min().y(), bb.min().z(),
                                      bb.max().x(), bb.max().y(), bb.max().z(),
-                                     CTX::instance()->clipPlane[i][0], 
-                                     CTX::instance()->clipPlane[i][1], 
-                                     CTX::instance()->clipPlane[i][2], 
+                                     CTX::instance()->clipPlane[i][0],
+                                     CTX::instance()->clipPlane[i][1],
+                                     CTX::instance()->clipPlane[i][2],
                                      CTX::instance()->clipPlane[i][3]);
   }
 };
