@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 //
 // Contributor(s):
 //   Emilie Marchandise
@@ -18,6 +18,7 @@
 #include <fstream>
 #include <sstream>
 #include <iostream>
+#include <math.h>
 #include "OS.h"
 #include "GModel.h"
 #include "MElement.h"
@@ -301,7 +302,9 @@ Centerline::Centerline(std::string fileName): kdtree(0), kdtreeR(0), nodes(0), n
   buildKdTree();
   nbPoints = 25;
   hLayer = 0.3;
+  hSecondLayer = 0.3;
   nbElemLayer = 3;
+  nbElemSecondLayer = 0;
 
   update_needed = false;
   is_cut = 0;
@@ -319,7 +322,9 @@ Centerline::Centerline(): kdtree(0), kdtreeR(0), nodes(0), nodesR(0)
   fileName = "centerlines.vtk";//default
   nbPoints = 25;
   hLayer = 0.3;
+  hSecondLayer = 0.3;
   nbElemLayer = 3;
+  nbElemSecondLayer = 0;
   is_cut = 0;
   is_closed = 0;
   is_extruded = 0;
@@ -342,6 +347,10 @@ Centerline::Centerline(): kdtree(0), kdtreeR(0), nodes(0), nodesR(0)
     (nbElemLayer, "Number of mesh elements the extruded layer");
   options["hLayer"] = new FieldOptionDouble
     (hLayer, "Thickness (% of radius) of the extruded layer");
+  options["nbElemSecondLayer"] = new FieldOptionInt
+    (nbElemSecondLayer, "Number of mesh elements the second extruded layer");
+  options["hSecondLayer"] = new FieldOptionDouble
+    (hSecondLayer, "Thickness (% of radius) of the second extruded layer");
 }
 
 Centerline::~Centerline()
@@ -381,6 +390,7 @@ void Centerline::importFile(std::string fileName)
   mod->load(fileName);
   mod->removeDuplicateMeshVertices(1.e-8);
   current->setAsCurrent();
+  current->setVisibility(1);
 
   int maxN = 0.0;
   std::vector<GEdge*> modEdges = mod->bindingsGetEdges();
@@ -615,7 +625,7 @@ void Centerline::createSplitCompounds()
   NR = current->getMaxElementaryNumber(3);
 
   // Remesh new faces (Compound Lines and Compound Surfaces)
-  Msg::Info("Centerline: creates split compounds");
+  Msg::Info("Centerline: creating split compounds ...");
 
   //Parametrize Compound Lines
   for (int i=0; i < NE; i++){
@@ -625,7 +635,9 @@ void Centerline::createSplitCompounds()
     int num_gec = NE+i+1;
     Msg::Info("Create Compound Line (%d) = %d discrete edge",
               num_gec, pe->tag());
-    current->addCompoundEdge(e_compound,num_gec);
+    /* GEdge *gec = */ current->addCompoundEdge(e_compound,num_gec);
+    //gec->meshAttributes.Method = MESH_TRANSFINITE;
+    //gec->meshAttributes.nbPointsTransfinite = nbPoints;
   }
 
   // Parametrize Compound surfaces
@@ -648,82 +660,6 @@ void Centerline::createSplitCompounds()
   }
 }
 
-void Centerline::cleanMesh()
-{
-  return;  // does not work yet !
-  ////////////////////////////////
-
-  if (!is_cut || !is_closed) return;
-
-  for (int i=0; i < NF; i++){
-    std::ostringstream oss;
-    oss << "new" << NF+i+1 ;
-    std::string name = oss.str();
-    current->setPhysicalName(name, 2, i+1);
-  }
-  Msg::Info("Writing new splitted mesh mySPLITMESH.msh");
-  current->writeMSH("mySPLITMESH.msh", 2.2, false, false);
-
-  std::set<MVertex*> allNod;
-  discreteFace * mySplitMesh;
-  std::vector<std::set<MVertex*> > inOutNod;
-  std::vector<discreteFace* > inOutMesh;
-
-  mySplitMesh= new discreteFace(current, 2*NF+1);
-  mySplitMesh->addPhysicalEntity(2*NF+1);
-  current->setPhysicalName("surface mesh", 2, 2*NF+1);
-  current->add(mySplitMesh);
-  for (unsigned int i=0; i < discFaces.size(); i++){
-    GFace *gfc =  current->getFaceByTag(NF+i+1);
-    for(unsigned int j = 0; j < gfc->triangles.size(); ++j){
-      MTriangle *t = gfc->triangles[j];
-      std::vector<MVertex *> v(3);
-      for(int k = 0; k < 3; k++){
-	v[k] = t->getVertex(k);
-	allNod.insert(v[k]);
-      }
-      mySplitMesh->triangles.push_back(new MTriangle(v[0], v[1], v[2]));
-    }
-    for(unsigned int j = 0; j < gfc->quadrangles.size(); ++j){
-      MQuadrangle *q = gfc->quadrangles[j];
-      std::vector<MVertex *> v(4);
-      for(int k = 0; k < 4; k++){
-	v[k] = q->getVertex(k);
-	allNod.insert(v[k]);
-      }
-      mySplitMesh->quadrangles.push_back(new MQuadrangle(v[0], v[1], v[2], v[3]));
-    }
-  }
-
-  //Removing discrete Vertices - Edges - Faces
-  for (int i=0; i < NV; i++){
-    GVertex *gv = current->getVertexByTag(i+1);
-    current->remove(gv);
-  }
-  for (int i=0; i < NE; i++){
-    GEdge *ge = current->getEdgeByTag(i+1);
-    GEdge *gec = current->getEdgeByTag(NE+i+1);
-    current->remove(ge);
-    current->remove(gec);
-  }
-  for (int i=0; i < NF; i++){
-    GFace *gf  = current->getFaceByTag(i+1);
-    current->remove(gf);
-  }
-  for (unsigned int i=0; i < discFaces.size(); i++){
-    GFace *gfc = current->getFaceByTag(NF+i+1);
-    current->remove(gfc);
-  }
-
-  //Put new mesh in a new discreteFace
- for(std::set<MVertex*>::iterator it = allNod.begin(); it != allNod.end(); ++it)
-   mySplitMesh->mesh_vertices.push_back(*it);
- mySplitMesh->meshStatistics.status = GFace::DONE;
-
- current->createTopologyFromMesh();
-
-}
-
 void Centerline::createFaces()
 {
   std::vector<std::vector<MTriangle*> > faces;
@@ -835,8 +771,8 @@ void Centerline::extrudeBoundaryLayerWall(GEdge* gin, std::vector<GEdge*> boundE
   if (dot(ne,nc) < 0) dir = 1;
   if (dir ==1 && hLayer > 0 ) hLayer *= -1.0;
 
-  int shift = 0;
-  if(is_cut) shift = NE;
+  //int shift = 0;
+  //if(is_cut) shift = NE;
   for (int i= 0; i< NF; i++){
     GFace *gfc ;
     if (is_cut) gfc = current->getFaceByTag(NF+i+1);
@@ -847,10 +783,23 @@ void Centerline::extrudeBoundaryLayerWall(GEdge* gin, std::vector<GEdge*> boundE
       (gfc, nbElemLayer,  hLayer, dir, -5);
     GFace *eFace = (GFace*) extrudedE[0];
     eFace->addPhysicalEntity(5);
-    current->setPhysicalName("outerWall", 2, 5);//tag 5
+    current->setPhysicalName("outerWall", 2, 5);//dim 2 tag 5
     GRegion *eRegion = (GRegion*) extrudedE[1];
     eRegion->addPhysicalEntity(6);
-    current->setPhysicalName("wallVolume", 3, 6);//tag 6
+    current->setPhysicalName("wallVolume", 3, 6);//dim 3 tag 6
+
+    //if double extruded layer
+    if (nbElemSecondLayer > 0){
+      std::vector<GEntity*> extrudedESec = current->extrudeBoundaryLayer
+      	(gfc, nbElemSecondLayer,  hSecondLayer, dir ? 0 : 1, -5);
+      GFace *eFaceSec = (GFace*) extrudedESec[0];
+      eFaceSec->addPhysicalEntity(9);                    //tag 9
+      current->setPhysicalName("outerSecondWall", 2, 9);//dim 2 tag 9
+      GRegion *eRegionSec = (GRegion*) extrudedESec[1];
+      eRegionSec->addPhysicalEntity(10);             //tag 10
+      current->setPhysicalName("wallVolume", 3, 10);//dim 3 tag 10
+    }
+
     for (unsigned int j = 2; j < extrudedE.size(); j++){
       GFace *elFace = (GFace*) extrudedE[j];
       std::list<GEdge*> l_edges = elFace->edges();
@@ -936,16 +885,13 @@ void Centerline::cutMesh()
   for(unsigned int i = 0; i < edges.size(); i++){
     std::vector<MLine*> lines = edges[i].lines;
     double L = edges[i].length;
-    double D = (edges[i].minRad+edges[i].maxRad);
+    double D = 2.*edges[i].minRad;  //(edges[i].minRad+edges[i].maxRad);
     double AR = L/D;
-    printf("*** Centerline branch %d (AR=%.1f) \n", edges[i].tag, AR);
-    //if ( edges[i].children.size()) printf("children (%d) = ", edges[i].children.size());
-    //for (int k= 0; k< edges[i].children.size() ; k++) printf("%d ", edges[i].children[k].tag);
-    //printf("\n");
+    // printf("*** Centerline branch %d (AR=%.1f) \n", edges[i].tag, AR);
 
-    int nbSplit = (int)floor(AR/2 + 0.9);
+    int nbSplit = (int)floor(AR/2 + 0.9); //AR/2 + 0.9
     if( nbSplit > 1 ){
-      printf("->> cut branch in %d parts \n",  nbSplit);
+      //printf("->> cut branch in %d parts \n",  nbSplit);
       double li  = L/nbSplit;
       double lc = 0.0;
       for (unsigned int j= 0; j < lines.size(); j++){
@@ -970,7 +916,7 @@ void Centerline::cutMesh()
       else v2 = lines[lines.size()-1]->getVertex(0);
       SVector3 pt(v1->x(), v1->y(), v1->z());
       SVector3 dir(v2->x()-v1->x(),v2->y()-v1->y(),v2->z()-v1->z());
-      printf("-->> cut branch at bifurcation \n");
+      //printf("-->> cut branch at bifurcation \n");
       std::map<MLine*,double>::iterator itr = radiusl.find(lines[lines.size()-1]);
       //bool cutted =
       cutByDisk(pt, dir, itr->second);
@@ -993,13 +939,13 @@ void Centerline::cutMesh()
   current->exportDiscreteGEOInternals();
 
   //write
-  Msg::Info("Writing splitted mesh 'myPARTS.msh'");
+  Msg::Info("Centerline: writing splitted mesh 'myPARTS.msh'");
   current->writeMSH("myPARTS.msh", 2.2, false, false);
 
   //create compounds
   createSplitCompounds();
 
-  Msg::Info("Splitting mesh by centerlines done ");
+  Msg::Info("Done splitting mesh by centerlines");
 }
 
 bool Centerline::cutByDisk(SVector3 &PT, SVector3 &NORM, double &maxRad)
@@ -1008,7 +954,6 @@ bool Centerline::cutByDisk(SVector3 &PT, SVector3 &NORM, double &maxRad)
   double b = NORM.y();
   double c = NORM.z();
   double d = -a * PT.x() - b * PT.y() - c * PT.z();
-  //printf("cut disk (R=%g)= %g %g %g %g \n", maxRad, a, b, c, d);
 
   int maxStep = 20;
   const double EPS = 0.007;
@@ -1103,6 +1048,7 @@ bool Centerline::cutByDisk(SVector3 &PT, SVector3 &NORM, double &maxRad)
 
 double Centerline::operator() (double x, double y, double z, GEntity *ge)
 {
+
   if (update_needed){
      std::ifstream input;
      input.open(fileName.c_str());
@@ -1114,7 +1060,6 @@ double Centerline::operator() (double x, double y, double z, GEntity *ge)
    }
 
    double xyz[3] = {x,y,z};
-
    //take xyz = closest point on boundary in case we are on the planar in/out faces
    //or in the volume
    bool isCompound = false;
@@ -1151,7 +1096,8 @@ double Centerline::operator() (double x, double y, double z, GEntity *ge)
 
 void  Centerline::operator() (double x, double y, double z, SMetric3 &metr, GEntity *ge)
 {
-  if (update_needed){
+
+   if (update_needed){
      std::ifstream input;
      input.open(fileName.c_str());
      if(StatFile(fileName))
@@ -1161,10 +1107,10 @@ void  Centerline::operator() (double x, double y, double z, SMetric3 &metr, GEnt
      update_needed = false;
    }
 
-   double xyz[3] = {x,y,z};
 
-   //take xyz = closest point on boundary in case we are on the planar IN/OUT
-   //FACES or in VOLUME
+   //take xyz = closest point on boundary in case we are on
+   //the planar IN/OUT FACES or in VOLUME
+   double xyz[3] = {x,y,z};
    bool onTubularSurface = true;
    double ds = 0.0;
    bool isCompound = (ge->dim() == 2 && ge->geomType() == GEntity::CompoundSurface) ?
@@ -1195,6 +1141,13 @@ void  Centerline::operator() (double x, double y, double z, SMetric3 &metr, GEnt
    delete[]index2;
    delete[]dist2;
 
+   //dir_a = direction along the centerline
+   //dir_n = normal direction of the disk
+   //dir_t = tangential direction of the disk
+   SVector3 dir_a = p1-p0; dir_a.normalize();
+   SVector3 dir_n(xyz[0]-p0.x(), xyz[1]-p0.y(), xyz[2]-p0.z()); dir_n.normalize();
+   SVector3 dir_cross  = crossprod(dir_a,dir_n); dir_cross.normalize();
+
    //find discrete curvature at closest vertex
    Curvature& curvature = Curvature::getInstance();
    if( !Curvature::valueAlreadyComputed() ) {
@@ -1204,98 +1157,84 @@ void  Centerline::operator() (double x, double y, double z, SMetric3 &metr, GEnt
    }
    double curv, cMin, cMax;
    SVector3 dMin, dMax;
-   curvature.vertexNodalValuesAndDirections(vertices[index[0]],&dMax, &dMin, &cMax, &cMin, 1);
-   curvature.vertexNodalValues(vertices[index[0]], curv, 0);
-
-   double sign = (curv > 0.0) ? -1.0: 1.0;
-   double beta = CTX::instance()->mesh.smoothRatio;
-   double ratio = 1.1;
+   int isAbs = 1.0;
+   curvature.vertexNodalValuesAndDirections(vertices[index[0]],&dMax, &dMin, &cMax, &cMin, isAbs);
+   curvature.vertexNodalValues(vertices[index[0]], curv, 1);
+   if (cMin == 0) cMin =1.e-12;
+   if (cMax == 0) cMax =1.e-12;
+   double rhoMin = 1./cMin;
+   double rhoMax = 1./cMax;
+   //double signMin = (rhoMin > 0.0) ? -1.0: 1.0;
+   //double signMax = (rhoMax > 0.0) ? -1.0: 1.0;
 
+   //user-defined parameters
+   //define h_n, h_t1, and h_t2
    double thickness = radMax/3.;
-   double rho = radMax;
-   double hwall_n = thickness/20.;
-   double hwall_t = 2*M_PI*rho/nbPoints;
-   double hfar =  radMax/5.;
-   double lc_a = 3.*hwall_t;
+   double h_far = radMax/5.;
+   double beta = (ds <= thickness) ? 1.2 : 2.1; //CTX::instance()->mesh.smoothRatio;
+   double dist = (ds <= thickness) ? ds: thickness;
 
-   //dir_a = direction along the centerline
-   //dir_n = normal direction of the disk
-   //dir_t = tangential direction of the disk
-   SVector3 dir_a = p1-p0; dir_a.normalize();
-   SVector3 dir_n(xyz[0]-p0.x(), xyz[1]-p0.y(), xyz[2]-p0.z()); dir_n.normalize();
-   SVector3 dir_cross  = crossprod(dir_a,dir_n); dir_cross.normalize();
-   SVector3 dir_t1, dir_t2, dir_a1, dir_a2;
-   buildOrthoBasis2(dir_n, dir_t1, dir_t2);
-   buildOrthoBasis2(dir_a, dir_a1, dir_a2);
+   double h_n_0 = thickness/20.;
+   double h_n   = std::min( (h_n_0+ds*log(beta)), h_far);
 
-   double ll1   = ds*(ratio-1) + hwall_n;
-   double lc_n  = std::min(ll1,hfar);
-   double ll2 =  hwall_t *(rho+sign*ds)/rho ; //sign * ds*(ratio-1) + hwall_t;
-   if (ds > thickness) ll2  =  hwall_t *(rho+sign*thickness)/rho ; //lc_a; //hfar
-   double lc_t  = std::min(lc_n*CTX::instance()->mesh.anisoMax, ll2); //std::min(ll2,hfar));
+   double betaMin = 10.0;
+   double betaMax = 3.1;
+   double oneOverD2_min = 1./(2.*rhoMin*rhoMin*(betaMin*betaMin-1)) *
+     (sqrt(1+ (4.*rhoMin*rhoMin*(betaMin*betaMin-1))/(h_n*h_n))-1.);
+   double oneOverD2_max = 1./(2.*rhoMax*rhoMax*(betaMax*betaMax-1)) *
+    (sqrt(1+ (4.*rhoMax*rhoMax*(betaMax*betaMax-1))/(h_n*h_n))-1.);
+   double h_t1_0 = sqrt(1./oneOverD2_min);
+   double h_t2_0 = sqrt(1./oneOverD2_max);
+   //double h_t1 =  h_t1_0*(rhoMin+signMin*dist)/rhoMin ;
+   //double h_t2 =  h_t2_0*(rhoMax+signMax*dist)/rhoMax ;
+   double h_t1  = std::min( (h_t1_0+(dist*log(beta))), radMax);
+   double h_t2  = std::min( (h_t2_0+(dist*log(beta))), h_far);
 
-   lc_n = std::max(lc_n, CTX::instance()->mesh.lcMin);
-   lc_n = std::min(lc_n, CTX::instance()->mesh.lcMax);
-   lc_t = std::max(lc_t, CTX::instance()->mesh.lcMin);
-   lc_t = std::min(lc_t, CTX::instance()->mesh.lcMax);
+   double dCenter = radMax-ds;
+   double h_a_0 = 0.5*radMax;
+   double h_a = h_a_0 - (h_a_0-h_t1_0)/(radMax)*dCenter;
+
+   //length between min and max
+   double lcMin = ((2 * M_PI *radMax) /( 50*nbPoints )); //CTX::instance()->mesh.lcMin;
+   double lcMax =  lcMin*2000.; //CTX::instance()->mesh.lcMax;
+   h_n = std::max(h_n, lcMin);    h_n = std::min(h_n, lcMax);
+   h_t1 = std::max(h_t1, lcMin);  h_t1 = std::min(h_t1, lcMax);
+   h_t2 = std::max(h_t2, lcMin);  h_t2 = std::min(h_t2, lcMax);
 
    //curvature metric
-   SMetric3 curvMetric;
-   if (ds <= thickness){
-     metr = metricBasedOnSurfaceCurvature(dMin, dMax, cMin, cMax, radMax,
-                                          beta, lc_n, lc_t, hwall_t);
+   SMetric3 curvMetric, curvMetric1, curvMetric2;
+   SMetric3 centMetric1, centMetric2, centMetric;
+   if (onInOutlets){
+     metr = buildMetricTangentToCurve(dir_n,h_n,h_t2);
    }
-   else if (ds > thickness && onInOutlets){
-     curvMetric = buildMetricTangentToCurve(dir_n,lc_n,lc_t);
-   }
-   else if (ds > thickness && !onInOutlets){
-     curvMetric = buildMetricTangentToCurve(dir_n,lc_n,lc_a);
-     metr = SMetric3(1./(lc_a*lc_a), 1./(lc_n*lc_n), 1./(lc_t*lc_t), dir_a, dir_n, dir_cross);
-     metr = intersection_conserveM1(metr,curvMetric);
+   else {
+     //on surface and in volume boundary layer
+     if ( ds < thickness ){
+       metr = metricBasedOnSurfaceCurvature(dMin, dMax, cMin, cMax, h_n, h_t1, h_t2);
+     }
+     //in volume
+     else {
+       //curvMetric = metricBasedOnSurfaceCurvature(dMin, dMax, cMin, cMax, h_n, h_t1, h_t2);
+       metr = SMetric3( 1./(h_a*h_a), 1./(h_n*h_n), 1./(h_n*h_n), dir_a, dir_n, dir_cross);
+
+       //metr = intersection_conserveM1_bis(metr, curvMetric);
+       //metr = intersection_conserveM1(metr,curvMetric);
+       //metr = intersection_conserve_mostaniso(metr, curvMetric);
+       //metr = intersection(metr,curvMetric);
+     }
    }
 
    return;
 
-   // double lc_a_curv = sqrt(1./dot(dir_a,curvMetric,dir_a));
-   // if (ds < thickness) lc_a = std::min(lc_a, lc_a_curv);
-   // return;
-
 }
 
 SMetric3 Centerline::metricBasedOnSurfaceCurvature(SVector3 dirMin, SVector3 dirMax,
-                                                   double cmin, double cmax, double radMax,
-						   double beta, double lc_n, double lc_t,
-                                                   double hwall_t)
+                                                   double cmin, double cmax,
+						   double h_n, double h_t1, double h_t2)
 {
-  double lcMin = ((2 * M_PI *radMax) /( 20*nbPoints )); //CTX::instance()->mesh.lcMin;
-  double lcMax = ((2 * M_PI *radMax) /( 0.05*nbPoints)); //CTX::instance()->mesh.lcMax;
-  if (cmin == 0) cmin =1.e-12;
-  if (cmax == 0) cmax =1.e-12;
-  double lambda1 =  ((2 * M_PI) /( fabs(cmin) *  nbPoints ) );
-  double lambda2 =  ((2 * M_PI) /( fabs(cmax) *  nbPoints ) );
-
-  double betaM = 1*beta;
-  double rhoMin = 1./cmin;
-  double rhoMax = 1./cmax;
-  double oneOverD2_min = 1./(2.*rhoMin*rhoMin*(betaM*betaM-1)) *
-    (sqrt(1+ (4.*rhoMin*rhoMin*(betaM*betaM-1))/(lc_n*lc_n))-1.);
-  double oneOverD2_max = 1./(2.*rhoMax*rhoMax*(beta*beta-1)) *
-    (sqrt(1+ (4.*rhoMax*rhoMax*(beta*beta-1))/(lc_n*lc_n))-1.);
-
-  lambda1 = sqrt(1./oneOverD2_min);
-  lambda2 = sqrt(1./oneOverD2_max);
 
   SVector3 dirNorm = crossprod(dirMax,dirMin);
-  dirMin.normalize();
-  dirMax.normalize();
-  dirNorm.normalize();
-  lambda1 = std::max(lambda1, lcMin);
-  lambda2 = std::max(lambda2, lcMin);
-  lambda1 = std::min(lambda1, lcMax);
-  lambda2 = std::min(lambda2, lcMax);
-
-  SMetric3 curvMetric (1./(lambda1*lambda1),1./(lambda2*lambda2),
-                       1./(lc_n*lc_n), dirMin, dirMax, dirNorm); // 1.e-6
+  SMetric3 curvMetric (1./(h_t1*h_t1),1./(h_t2*h_t2),1./(h_n*h_n), dirMin, dirMax, dirNorm);
 
   return curvMetric;
 }
