@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include <stdlib.h>
 #include "GmshConfig.h"
@@ -33,6 +33,7 @@
 #include "Field.h"
 #include "Options.h"
 #include "simple3D.h"
+#include "yamakawa.h"
 
 #if defined(HAVE_POST)
 #include "PView.h"
@@ -188,7 +189,7 @@ static void GetQualityMeasure(std::vector<T*> &ele,
     rhoMin = std::min(rhoMin, r);
     rhoMax = std::max(rhoMax, r);
     double jmin,jmax; ele[i]->scaledJacRange(jmin,jmax);
-    double d = std::min(jmin,1./jmax);
+    double d = jmin;
     disto += d;
     distoMin = std::min(distoMin, d);
     distoMax = std::max(distoMax, d);
@@ -285,7 +286,6 @@ void GetStatistics(double stat[50], double quality[4][100])
       stat[17] = gamma / N ;  stat[18] = gammaMin;  stat[19] = gammaMax;
       stat[20] = eta / N ;    stat[21] = etaMin;    stat[22] = etaMax;
       stat[23] = rho / N ;    stat[24] = rhoMin;    stat[25] = rhoMax;
-      stat[45] = jmin;        stat[46] = jmax;
       stat[46] = disto / N ;  stat[47] = distoMin;  stat[48] = distoMax;
     }
   }
@@ -353,38 +353,52 @@ static void Mesh0D(GModel *m)
     if(gv->points.empty())
       gv->points.push_back(new MPoint(gv->mesh_vertices.back()));
   }
+  for(GModel::viter it = m->firstVertex(); it != m->lastVertex(); ++it){
+    GVertex *gv = *it;
+    if (gv->meshMaster() != gv->tag()){
+      if (gv->correspondingVertices.empty()){
+	GVertex *master = m->getVertexByTag(abs(gv->meshMaster()));
+	if(master)gv->correspondingVertices[gv->mesh_vertices[0]] = (master->mesh_vertices[0]);
+      }
+    }
+  }
+
 }
 
 static void Mesh1D(GModel *m)
 {
   if(TooManyElements(m, 1)) return;
-  Msg::StatusBar(2, true, "Meshing 1D...");
+  Msg::StatusBar(true, "Meshing 1D...");
   double t1 = Cpu();
 
   for(GModel::eiter it = m->firstEdge(); it != m->lastEdge(); ++it)
     (*it)->meshStatistics.status = GEdge::PENDING;
 
-  int nIter = 0;
+  Msg::ResetProgressMeter();
+
+  int nIter = 0, nTot = m->getNumEdges();
   while(1){
     meshGEdge mesher;
-    int nbPending = 0;
+    int nPending = 0;
     for(GModel::eiter it = m->firstEdge(); it != m->lastEdge(); ++it){
       if ((*it)->meshStatistics.status == GEdge::PENDING){
 	mesher(*it);
-	nbPending++;
+	nPending++;
       }
+      if(!nIter) Msg::ProgressMeter(nPending, nTot, false, "Meshing 1D...");
     }
-    if(!nbPending) break;
+    if(!nPending) break;
     if(nIter++ > 10) break;
   }
 
   double t2 = Cpu();
   CTX::instance()->meshTimer[0] = t2 - t1;
-  Msg::StatusBar(2, true, "Done meshing 1D (%g s)", CTX::instance()->meshTimer[0]);
+  Msg::StatusBar(true, "Done meshing 1D (%g s)", CTX::instance()->meshTimer[0]);
 }
 
 static void PrintMesh2dStatistics(GModel *m)
 {
+
   FILE *statreport = 0;
   if(CTX::instance()->createAppendMeshStatReport == 1)
     statreport = fopen(CTX::instance()->meshStatReportFileName.c_str(), "w");
@@ -405,21 +419,25 @@ static void PrintMesh2dStatistics(GModel *m)
   }
 
   for(GModel::fiter it = m->firstFace() ; it != m->lastFace(); ++it){
-    worst = std::min((*it)->meshStatistics.worst_element_shape, worst);
-    best = std::max((*it)->meshStatistics.best_element_shape, best);
-    avg += (*it)->meshStatistics.average_element_shape * (*it)->meshStatistics.nbTriangle;
-    e_avg += (*it)->meshStatistics.efficiency_index;//* (*it)->meshStatistics.nbEdge;
-    e_long = std::max((*it)->meshStatistics.longest_edge_length, e_long);
-    e_short = std::min((*it)->meshStatistics.smallest_edge_length, e_short);
-    if ((*it)->meshStatistics.status == GFace::FAILED ||
-        (*it)->meshStatistics.status == GFace::PENDING) nUnmeshed++;
-    nTotT += (*it)->meshStatistics.nbTriangle;
-    nTotE += (*it)->meshStatistics.nbEdge;
-    nTotGoodLength += (*it)->meshStatistics.nbGoodLength;
-    nTotGoodQuality += (*it)->meshStatistics.nbGoodQuality;
-    numFaces++;
+      if((*it)->geomType() != GEntity::DiscreteSurface){
+      worst = std::min((*it)->meshStatistics.worst_element_shape, worst);
+      best = std::max((*it)->meshStatistics.best_element_shape, best);
+      avg += (*it)->meshStatistics.average_element_shape * (*it)->meshStatistics.nbTriangle;
+      e_avg += (*it)->meshStatistics.efficiency_index;
+      e_long = std::max((*it)->meshStatistics.longest_edge_length, e_long);
+      e_short = std::min((*it)->meshStatistics.smallest_edge_length, e_short);
+      if ((*it)->meshStatistics.status == GFace::FAILED ||
+	  (*it)->meshStatistics.status == GFace::PENDING) nUnmeshed++;
+      nTotT += (*it)->meshStatistics.nbTriangle;
+      nTotE += (*it)->meshStatistics.nbEdge;
+      nTotGoodLength += (*it)->meshStatistics.nbGoodLength;
+      nTotGoodQuality += (*it)->meshStatistics.nbGoodQuality;
+      numFaces++;
+    }
   }
 
+  Msg::Info("*** Efficiency index for surface mesh tau=%g ", 100*exp(e_avg/(double)nTotE));
+
   fprintf(statreport,"\t%16s\t%d\t\t%d\t\t", m->getName().c_str(), numFaces, nUnmeshed);
   fprintf(statreport,"%d\t\t%8.7f\t%8.7f\t%8.7f\t%d\t\t%8.7f\t",
           nTotT, avg / (double)nTotT, best, worst, nTotGoodQuality,
@@ -433,8 +451,8 @@ static void PrintMesh2dStatistics(GModel *m)
 static void Mesh2D(GModel *m)
 {
   if(TooManyElements(m, 2)) return;
-  Msg::StatusBar(2, true, "Meshing 2D...");
-  double t1 = Cpu();
+  Msg::StatusBar(true, "Meshing 2D...");
+  double t1 = GetTimeInSeconds();
 
   for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it)
     (*it)->meshStatistics.status = GFace::PENDING;
@@ -453,24 +471,34 @@ static void Mesh2D(GModel *m)
       else
         f.insert(*it);
 
-    int nIter = 0;
+    Msg::ResetProgressMeter();
+
+    int nIter = 0, nTot = m->getNumFaces();
     while(1){
-      int nbPending = 0;
-      for(std::set<GFace*>::iterator it = f.begin(); it != f.end(); ++it){
-        if ((*it)->meshStatistics.status == GFace::PENDING){
+      int nPending = 0;
+      std::vector<GFace*> _temp; _temp.insert(_temp.begin(), f.begin(), f.end());
+#pragma omp parallel for schedule (dynamic)
+      for(size_t K = 0 ; K < _temp.size() ; K++){
+	if (_temp[K]->meshStatistics.status == GFace::PENDING){
 	  meshGFace mesher (true, CTX::instance()->mesh.multiplePasses);
-          mesher(*it);
-          nbPending++;
-        }
+	  mesher(_temp[K]);
+#pragma omp critical
+	  {
+	    nPending++;
+	  }
+	}
+        if(!nIter) Msg::ProgressMeter(nPending, nTot, false, "Meshing 2D...");
       }
+#pragma omp master
       for(std::set<GFace*>::iterator it = cf.begin(); it != cf.end(); ++it){
         if ((*it)->meshStatistics.status == GFace::PENDING){
 	  meshGFace mesher (true, CTX::instance()->mesh.multiplePasses);
           mesher(*it);
-          nbPending++;
+          nPending++;
         }
+        if(!nIter) Msg::ProgressMeter(nPending, nTot, false, "Meshing 2D...");
       }
-      if(!nbPending) break;
+      if(!nPending) break;
       if(nIter++ > 10) break;
     }
   }
@@ -483,10 +511,10 @@ static void Mesh2D(GModel *m)
 	smoothing smm(CTX::instance()->mesh.optimizeLloyd,6);
 	m->writeMSH("beforeLLoyd.msh");
 	smm.optimize_face(*it);
-	int rec = 1;//(CTX::instance()->mesh.recombineAll || (*it)->meshAttributes.recombine);
+	//int rec = 1;//(CTX::instance()->mesh.recombineAll || (*it)->meshAttributes.recombine);
 	m->writeMSH("afterLLoyd.msh");
-	if(rec) recombineIntoQuads(*it);
-	m->writeMSH("afterRecombine.msh");
+	//if(rec) recombineIntoQuads(*it);
+	//m->writeMSH("afterRecombine.msh");
       }
     }
     /*
@@ -506,20 +534,9 @@ static void Mesh2D(GModel *m)
 
   // collapseSmallEdges(*m);
 
-#if defined(HAVE_ANN)
-  //For centerline field, clean the cut parts
-  Centerline *center = 0;
-  FieldManager *fields = GModel::current()->getFields();
-  if (fields->getBackgroundField() > 0 ){
-    Field *myField = fields->get(fields->getBackgroundField());
-    center = dynamic_cast<Centerline*> (myField);
-  }
-  if (center) center->cleanMesh();
-#endif
-
-  double t2 = Cpu();
+  double t2 = GetTimeInSeconds();
   CTX::instance()->meshTimer[1] = t2 - t1;
-  Msg::StatusBar(2, true, "Done meshing 2D (%g s)", CTX::instance()->meshTimer[1]);
+  Msg::StatusBar(true, "Done meshing 2D (%g s)", CTX::instance()->meshTimer[1]);
 
   PrintMesh2dStatistics(m);
 }
@@ -534,7 +551,7 @@ static void FindConnectedRegions(std::vector<GRegion*> &delaunay,
 static void Mesh3D(GModel *m)
 {
   if(TooManyElements(m, 3)) return;
-  Msg::StatusBar(2, true, "Meshing 3D...");
+  Msg::StatusBar(true, "Meshing 3D...");
   double t1 = Cpu();
 
   // mesh the extruded volumes first
@@ -556,57 +573,84 @@ static void Mesh3D(GModel *m)
   // quality reasons)
   std::vector<std::vector<GRegion*> > connected;
   FindConnectedRegions(delaunay, connected);
+
+  // remove quads elements for volumes that are recombined
+  for(unsigned int i = 0; i < connected.size(); i++){
+    for(unsigned j=0;j<connected[i].size();j++){
+      GRegion *gr = connected[i][j];
+      if(CTX::instance()->mesh.recombine3DAll || gr->meshAttributes.recombine3D){
+	std::list<GFace*> f = gr->faces();
+	for (std::list<GFace*>::iterator it = f.begin();
+	     it != f.end() ; ++it) quadsToTriangles (*it,1000000);
+      }
+    }
+  }
+
   for(unsigned int i = 0; i < connected.size(); i++){
     MeshDelaunayVolume(connected[i]);
-    if(CTX::instance()->mesh.algo3d == ALGO_3D_RTREE){
-      Filler f;
-      f.treat_region(connected[i][0]);
+
+    //Additional code for hex mesh begin
+    for(unsigned j=0;j<connected[i].size();j++){
+      GRegion *gr = connected[i][j];
+      //R-tree
+      if(CTX::instance()->mesh.algo3d == ALGO_3D_RTREE){
+	Filler f;
+	f.treat_region(gr);
+      }
+      //Recombine3D into hex
+      if(CTX::instance()->mesh.recombine3DAll || gr->meshAttributes.recombine3D){
+	Recombinator rec;
+	rec.execute();
+	Supplementary sup;
+	sup.execute();
+	PostOp post;
+	post.execute(0);
+      }
     }
   }
 
   double t2 = Cpu();
   CTX::instance()->meshTimer[2] = t2 - t1;
-  Msg::StatusBar(2, true, "Done meshing 3D (%g s)", CTX::instance()->meshTimer[2]);
-
+  Msg::StatusBar(true, "Done meshing 3D (%g s)", CTX::instance()->meshTimer[2]);
 }
 
 void OptimizeMeshNetgen(GModel *m)
 {
-  Msg::StatusBar(2, true, "Optimizing 3D mesh with Netgen...");
+  Msg::StatusBar(true, "Optimizing 3D mesh with Netgen...");
   double t1 = Cpu();
 
   std::for_each(m->firstRegion(), m->lastRegion(), optimizeMeshGRegionNetgen());
 
   double t2 = Cpu();
-  Msg::StatusBar(2, true, "Done optimizing 3D mesh with Netgen (%g s)", t2 - t1);
+  Msg::StatusBar(true, "Done optimizing 3D mesh with Netgen (%g s)", t2 - t1);
 }
 
 void OptimizeMesh(GModel *m)
 {
-  Msg::StatusBar(2, true, "Optimizing 3D mesh...");
+  Msg::StatusBar(true, "Optimizing 3D mesh...");
   double t1 = Cpu();
 
   std::for_each(m->firstRegion(), m->lastRegion(), optimizeMeshGRegionGmsh());
 
   double t2 = Cpu();
-  Msg::StatusBar(2, true, "Done optimizing 3D mesh (%g s)", t2 - t1);
+  Msg::StatusBar(true, "Done optimizing 3D mesh (%g s)", t2 - t1);
 }
 
 void AdaptMesh(GModel *m)
 {
-  Msg::StatusBar(2, true, "Adapting 3D mesh...");
+  Msg::StatusBar(true, "Adapting 3D mesh...");
   double t1 = Cpu();
 
   for(int i = 0; i < 10; i++)
     std::for_each(m->firstRegion(), m->lastRegion(), adaptMeshGRegion());
 
   double t2 = Cpu();
-  Msg::StatusBar(2, true, "Done adaptating 3D mesh (%g s)", t2 - t1);
+  Msg::StatusBar(true, "Done adaptating 3D mesh (%g s)", t2 - t1);
 }
 
 void RecombineMesh(GModel *m)
 {
-  Msg::StatusBar(2, true, "Recombining 2D mesh...");
+  Msg::StatusBar(true, "Recombining 2D mesh...");
   double t1 = Cpu();
 
   for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it){
@@ -617,14 +661,14 @@ void RecombineMesh(GModel *m)
   CTX::instance()->mesh.changed = ENT_ALL;
 
   double t2 = Cpu();
-  Msg::StatusBar(2, true, "Done recombining 2D mesh (%g s)", t2 - t1);
+  Msg::StatusBar(true, "Done recombining 2D mesh (%g s)", t2 - t1);
 }
 
 //#include <google/profiler.h>
 
 void GenerateMesh(GModel *m, int ask)
 {
-  //  ProfilerStart("gmsh.prof");
+  // ProfilerStart("gmsh.prof");
   if(CTX::instance()->lock) {
     Msg::Info("I'm busy! Ask me that later...");
     return;
@@ -694,5 +738,5 @@ void GenerateMesh(GModel *m, int ask)
 
   CTX::instance()->lock = 0;
   CTX::instance()->mesh.changed = ENT_ALL;
-  //  ProfilerStop();
+  // ProfilerStop();
 }
