@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 //
 // Contributor(s):
 //   Koen Hillewaert
@@ -21,7 +21,7 @@
 #include "Numeric.h"
 #include "Context.h"
 #include "fullMatrix.h"
-#include "polynomialBasis.h"
+#include "BasisFactory.h"
 
 #define SQU(a)      ((a)*(a))
 
@@ -40,28 +40,7 @@ static void myresid(int N, GEdge *ge, double *u, fullVector<double> &r)
   for (int i = 0; i < N - 2; i++) r(i) = L[i + 1] - L[i];
 }
 
-static bool computeEquidistantParameters1(GEdge *ge, double u0, double uN, int N,
-                                         double *u, double underRelax)
-{
-  GPoint p0 = ge->point(u0);
-  GPoint p1 = ge->point(uN);
-  double du = 1. / (N - 1);
-  u[0] = u0;
-  // printf("starting with %g %g %g\n",p0.x(),p0.y(),u0);
-  // printf("ending   with %g %g %g\n",p1.x(),p1.y(),uN);
-  for (int i = 1; i < N; i++){
-    SPoint3 pi (p0.x() + i * du * (p1.x()-p0.x()),
-		p0.y() + i * du * (p1.y()-p0.y()),
-		p0.z() + i * du * (p1.z()-p0.z()));
-    double t;
-    GPoint gp = ge->closestPoint(pi, t);
-    u[i] = gp.u();
-    // printf("going to %g %g u %g\n",pi.x(),pi.y(),gp.u());
-  }
-  return true;
-}
-
-static bool computeEquidistantParameters0(GEdge *ge, double u0, double uN, int N,
+static bool computeEquidistantParameters(GEdge *ge, double u0, double uN, int N,
                                          double *u, double underRelax)
 {
   const double PRECISION = 1.e-6;
@@ -121,59 +100,11 @@ static bool computeEquidistantParameters0(GEdge *ge, double u0, double uN, int N
   return false;
 }
 
-// 1 = geodesics
-static int method_for_computing_intermediary_points = 0;
-static bool computeEquidistantParameters(GEdge *ge, double u0, double uN, int N,
-                                         double *u, double underRelax){
-  if (method_for_computing_intermediary_points == 0) // use linear abscissa
-    return computeEquidistantParameters0(ge,u0,uN,N,u,underRelax);
-  else if (method_for_computing_intermediary_points == 1) // use projection
-    return computeEquidistantParameters1(ge,u0,uN,N,u,underRelax);
-  return false;
-}
 
-static double mylength(GFace *gf, int i, double *u, double *v)
-{
-  return gf->length(SPoint2(u[i], v[i]), SPoint2(u[i + 1], v[i + 1]), 10);
-}
-
-static void myresid(int N, GFace *gf, double *u, double *v, fullVector<double> &r)
-{
-  double L[100];
-  for (int i = 0; i < N - 1; i++) L[i] = mylength(gf, i, u, v);
-  for (int i = 0; i < N - 2; i++) r(i) = L[i + 1] - L[i];
-}
-
-static bool computeEquidistantParameters1(GFace *gf, double u0, double uN,
-					  double v0, double vN, int N,
-					  double *u, double *v)
-{
-  GPoint p0 = gf->point(u0,v0);
-  GPoint p1 = gf->point(uN,vN);
-  double du = 1. / (N - 1);
-  u[0] = u0;
-  u[0] = u0;
-  v[0] = v0;
-  for (int i = 1; i < N; i++){
-    SPoint3 pi(p0.x() + i * du * (p1.x()-p0.x()),
-               p0.y() + i * du * (p1.y()-p0.y()),
-               p0.z() + i * du * (p1.z()-p0.z()));
-    SPoint2 t;
-    GPoint gp = gf->closestPoint(pi, t);
-    u[i] = gp.u();
-    v[i] = gp.v();
-  }
-  return true;
-}
-
-static bool computeEquidistantParameters0(GFace *gf, double u0, double uN,
+static bool computeEquidistantParameters(GFace *gf, double u0, double uN,
                                          double v0, double vN, int N,
                                          double *u, double *v)
 {
-  const double PRECISION = 1.e-6;
-  const int MAX_ITER = 50;
-  const double eps = 1.e-4;
-
   double t[100];
   // initialize the points by equal subdivision of geodesics
   u[0] = u0;
@@ -191,67 +122,6 @@ static bool computeEquidistantParameters0(GFace *gf, double u0, double uN,
 
   return true;
 
-  // create the tangent matrix
-  const int M = N - 2;
-  fullMatrix<double> J(M, M);
-  fullVector<double> DU(M);
-  fullVector<double> R(M);
-  fullVector<double> Rp(M);
-
-  int iter = 1;
-
-  while (iter < MAX_ITER){
-    iter++;
-    myresid(N, gf, u, v, R);
-
-    for (int i = 0; i < M; i++){
-      t[i + 1] += eps;
-      double tempu = u[i + 1];
-      double tempv = v[i + 1];
-      SPoint2 p = gf->geodesic(SPoint2(u0, v0), SPoint2(uN, vN), t[i + 1]);
-      u[i + 1] = p.x();
-      v[i + 1] = p.y();
-      myresid(N, gf, u, v, Rp);
-      for (int j = 0; j < M; j++){
-        J(i, j) = (Rp(j) - R(j)) / eps;
-      }
-      t[i + 1] -= eps;
-      u[i + 1] = tempu;
-      v[i + 1] = tempv;
-    }
-
-    if (M == 1)
-      DU(0) = R(0) / J(0, 0);
-    else
-      J.luSolve(R, DU);
-
-    for (int i = 0; i < M; i++){
-      t[i + 1] -= DU(i);
-      SPoint2 p = gf->geodesic(SPoint2(u0, v0), SPoint2(uN, vN), t[i + 1]);
-      u[i + 1] = p.x();
-      v[i + 1] = p.y();
-    }
-    double newt_norm = DU.norm();
-    if (newt_norm < PRECISION) return true;
-  }
-  // FAILED, use equidistant in param space
-  for (int i = 1; i < N; i++){
-    t[i] = (double)i / (N - 1);
-    SPoint2 p = gf->geodesic(SPoint2(u0, v0), SPoint2(uN, vN), t[i]);
-    u[i] = p.x();
-    v[i] = p.y();
-  }
-  return false;
-}
-
-static bool computeEquidistantParameters(GFace *gf, double u0, double uN,
-                                         double v0, double vN, int N,
-                                         double *u, double *v){
-  if (method_for_computing_intermediary_points == 0) // use linear abscissa
-    return computeEquidistantParameters0(gf,u0,uN,v0,vN,N,u,v);
-  else if (method_for_computing_intermediary_points == 1) // use projection
-    return computeEquidistantParameters1(gf,u0,uN,v0,vN,N,u,v);
-  return false;
 }
 
 static void getEdgeVertices(GEdge *ge, MElement *ele, std::vector<MVertex*> &ve,
@@ -278,7 +148,8 @@ static void getEdgeVertices(GEdge *ge, MElement *ele, std::vector<MVertex*> &ve,
       bool reparamOK = true;
       if(!linear) {
         reparamOK &= reparamMeshVertexOnEdge(v0, ge, u0);
-        if(ge->periodic(0) && ge->getEndVertex()->getNumMeshVertices() > 0 && v1 == ge->getEndVertex()->mesh_vertices[0])
+        if(ge->periodic(0) && ge->getEndVertex()->getNumMeshVertices() > 0 &&
+           v1 == ge->getEndVertex()->mesh_vertices[0])
           u1 = ge->parBounds(0).high();
         else
           reparamOK &= reparamMeshVertexOnEdge(v1, ge, u1);
@@ -465,7 +336,11 @@ static void getFaceVertices(GFace *gf, MElement *incomplete, MElement *ele,
             }
           }
           if(reparamOK){
-            GPoint gp = gf->closestPoint(SPoint3(X, Y, Z), GUESS);
+	    GPoint gp = gf->point(SPoint2(GUESS[0], GUESS[1]));
+            // closest point is not necessary (slow and for high quality HO
+	    // meshes it should be optimized anyway afterwards + closest point
+	    // is still buggy (e.g. BFGS for a plane Ruled Surface)
+            // GPoint gp = gf->closestPoint(SPoint3(X, Y, Z), GUESS);
             if (gp.g()){
               v = new MFaceVertex(gp.x(), gp.y(), gp.z(), gf, gp.u(), gp.v());
             }
@@ -597,14 +472,14 @@ static int getNewFacePoints(int numPrimaryFacePoints, int nPts, fullMatrix<doubl
     switch (nPts){
     case 0 : break;
     case 1 : break;
-    case 2 : points = polynomialBases::find(MSH_TRI_10)->points; start = 9; break;
-    case 3 : points = polynomialBases::find(MSH_TRI_15)->points; start = 12; break;
-    case 4 : points = polynomialBases::find(MSH_TRI_21)->points; start = 15; break;
-    case 5 : points = polynomialBases::find(MSH_TRI_28)->points; start = 18; break;
-    case 6 : points = polynomialBases::find(MSH_TRI_36)->points; start = 21; break;
-    case 7 : points = polynomialBases::find(MSH_TRI_45)->points; start = 24; break;
-    case 8 : points = polynomialBases::find(MSH_TRI_55)->points; start = 27; break;
-    case 9 : points = polynomialBases::find(MSH_TRI_66)->points; start = 30; break;
+    case 2 : points = BasisFactory::create(MSH_TRI_10)->points; start = 9; break;
+    case 3 : points = BasisFactory::create(MSH_TRI_15)->points; start = 12; break;
+    case 4 : points = BasisFactory::create(MSH_TRI_21)->points; start = 15; break;
+    case 5 : points = BasisFactory::create(MSH_TRI_28)->points; start = 18; break;
+    case 6 : points = BasisFactory::create(MSH_TRI_36)->points; start = 21; break;
+    case 7 : points = BasisFactory::create(MSH_TRI_45)->points; start = 24; break;
+    case 8 : points = BasisFactory::create(MSH_TRI_55)->points; start = 27; break;
+    case 9 : points = BasisFactory::create(MSH_TRI_66)->points; start = 30; break;
     default :
       Msg::Error("getFaceVertices not implemented for order %i", nPts +1);
       break;
@@ -613,14 +488,14 @@ static int getNewFacePoints(int numPrimaryFacePoints, int nPts, fullMatrix<doubl
   case 4:
     switch (nPts){
     case 0 : break;
-    case 1 : points = polynomialBases::find(MSH_QUA_9)->points; break;
-    case 2 : points = polynomialBases::find(MSH_QUA_16)->points; break;
-    case 3 : points = polynomialBases::find(MSH_QUA_25)->points; break;
-    case 4 : points = polynomialBases::find(MSH_QUA_36)->points; break;
-    case 5 : points = polynomialBases::find(MSH_QUA_49)->points; break;
-    case 6 : points = polynomialBases::find(MSH_QUA_64)->points; break;
-    case 7 : points = polynomialBases::find(MSH_QUA_81)->points; break;
-    case 8 : points = polynomialBases::find(MSH_QUA_100)->points; break;
+    case 1 : points = BasisFactory::create(MSH_QUA_9)->points; break;
+    case 2 : points = BasisFactory::create(MSH_QUA_16)->points; break;
+    case 3 : points = BasisFactory::create(MSH_QUA_25)->points; break;
+    case 4 : points = BasisFactory::create(MSH_QUA_36)->points; break;
+    case 5 : points = BasisFactory::create(MSH_QUA_49)->points; break;
+    case 6 : points = BasisFactory::create(MSH_QUA_64)->points; break;
+    case 7 : points = BasisFactory::create(MSH_QUA_81)->points; break;
+    case 8 : points = BasisFactory::create(MSH_QUA_100)->points; break;
     default :
       Msg::Error("getFaceVertices not implemented for order %i", nPts +1);
       break;
@@ -723,14 +598,16 @@ static void getRegionVertices(GRegion *gr, MElement *incomplete, MElement *ele,
     switch (nPts){
     case 0: return;
     case 1: return;
-    case 2: points = polynomialBases::find(MSH_TET_20)->points; break;
-    case 3: points = polynomialBases::find(MSH_TET_35)->points; break;
-    case 4: points = polynomialBases::find(MSH_TET_56)->points; break;
-    case 5: points = polynomialBases::find(MSH_TET_84)->points; break;
-    case 6: points = polynomialBases::find(MSH_TET_120)->points; break;
-    case 7: points = polynomialBases::find(MSH_TET_165)->points; break;
-    case 8: points = polynomialBases::find(MSH_TET_220)->points; break;
-    case 9: points = polynomialBases::find(MSH_TET_286)->points; break;
+    case 2:
+      BasisFactory::create(MSH_TET_20)->points.print();
+      points = BasisFactory::create(MSH_TET_20)->points; break;
+    case 3: points = BasisFactory::create(MSH_TET_35)->points; break;
+    case 4: points = BasisFactory::create(MSH_TET_56)->points; break;
+    case 5: points = BasisFactory::create(MSH_TET_84)->points; break;
+    case 6: points = BasisFactory::create(MSH_TET_120)->points; break;
+    case 7: points = BasisFactory::create(MSH_TET_165)->points; break;
+    case 8: points = BasisFactory::create(MSH_TET_220)->points; break;
+    case 9: points = BasisFactory::create(MSH_TET_286)->points; break;
     default:
       Msg::Error("getRegionVertices not implemented for order %i", nPts+1);
       break;
@@ -740,14 +617,14 @@ static void getRegionVertices(GRegion *gr, MElement *incomplete, MElement *ele,
   case TYPE_HEX :
     switch (nPts){
     case 0: return;
-    case 1: points = polynomialBases::find(MSH_HEX_27)->points; break;
-    case 2: points = polynomialBases::find(MSH_HEX_64)->points; break;
-    case 3: points = polynomialBases::find(MSH_HEX_125)->points; break;
-    case 4: points = polynomialBases::find(MSH_HEX_216)->points; break;
-    case 5: points = polynomialBases::find(MSH_HEX_343)->points; break;
-    case 6: points = polynomialBases::find(MSH_HEX_512)->points; break;
-    case 7: points = polynomialBases::find(MSH_HEX_729)->points; break;
-    case 8: points = polynomialBases::find(MSH_HEX_1000)->points; break;
+    case 1: points = BasisFactory::create(MSH_HEX_27)->points; break;
+    case 2: points = BasisFactory::create(MSH_HEX_64)->points; break;
+    case 3: points = BasisFactory::create(MSH_HEX_125)->points; break;
+    case 4: points = BasisFactory::create(MSH_HEX_216)->points; break;
+    case 5: points = BasisFactory::create(MSH_HEX_343)->points; break;
+    case 6: points = BasisFactory::create(MSH_HEX_512)->points; break;
+    case 7: points = BasisFactory::create(MSH_HEX_729)->points; break;
+    case 8: points = BasisFactory::create(MSH_HEX_1000)->points; break;
     default :
       Msg::Error("getRegionVertices not implemented for order %i", nPts+1);
       break;
@@ -755,17 +632,16 @@ static void getRegionVertices(GRegion *gr, MElement *incomplete, MElement *ele,
     start = (nPts+2)*(nPts+2)*(nPts+2) - (nPts)*(nPts)*(nPts) ;
     break;
   case TYPE_PYR:
-    printf("aaaaaand...\n");
     switch (nPts){
     case 0:
     case 1: return;
-    case 2: points = polynomialBases::find(MSH_PYR_30)->points; break;
-    case 3: points = polynomialBases::find(MSH_PYR_55)->points; break;
-    case 4: points = polynomialBases::find(MSH_PYR_91)->points; break;
-    case 5: points = polynomialBases::find(MSH_PYR_140)->points; break;
-    case 6: points = polynomialBases::find(MSH_PYR_204)->points; break;
-    case 7: points = polynomialBases::find(MSH_PYR_285)->points; break;
-    case 8: points = polynomialBases::find(MSH_PYR_385)->points; break;
+    case 2: points = BasisFactory::create(MSH_PYR_30)->points; break;
+    case 3: points = BasisFactory::create(MSH_PYR_55)->points; break;
+    case 4: points = BasisFactory::create(MSH_PYR_91)->points; break;
+    case 5: points = BasisFactory::create(MSH_PYR_140)->points; break;
+    case 6: points = BasisFactory::create(MSH_PYR_204)->points; break;
+    case 7: points = BasisFactory::create(MSH_PYR_285)->points; break;
+    case 8: points = BasisFactory::create(MSH_PYR_385)->points; break;
     default :
       Msg::Error("getRegionVertices not implemented for order %i", nPts+1);
       break;
@@ -781,9 +657,11 @@ static void getRegionVertices(GRegion *gr, MElement *incomplete, MElement *ele,
     const double t1 = points(k, 0);
     const double t2 = points(k, 1);
     const double t3 = points(k, 2);
+    //printf("inside getRegionVertices, point is %g %g %g\n", t1, t2, t3);
     SPoint3 pos;
     incomplete->pnt(t1, t2, t3, pos);
     v = new MVertex(pos.x(), pos.y(), pos.z(), gr);
+    //printf("inside getRegionVertices, vertex is %g %g %g\n", v->x(), v->y(), v->z());
     gr->mesh_vertices.push_back(v);
     vr.push_back(v);
   }
@@ -798,9 +676,9 @@ static void setHighOrder(GEdge *ge, edgeContainer &edgeVertices, bool linear,
     std::vector<MVertex*> ve;
     getEdgeVertices(ge, l, ve, edgeVertices, linear, nbPts);
     if(nbPts == 1)
-      lines2.push_back(new MLine3(l->getVertex(0), l->getVertex(1), ve[0]));
+      lines2.push_back(new MLine3(l->getVertex(0), l->getVertex(1), ve[0], l->getPartition()));
     else
-      lines2.push_back(new MLineN(l->getVertex(0), l->getVertex(1), ve));
+      lines2.push_back(new MLineN(l->getVertex(0), l->getVertex(1), ve, l->getPartition()));
     delete l;
   }
   ge->lines = lines2;
@@ -816,20 +694,20 @@ MTriangle *setHighOrder(MTriangle *t, GFace *gf,
   getEdgeVertices(gf, t, ve, edgeVertices, linear, nPts);
   if(nPts == 1){
     return new MTriangle6(t->getVertex(0), t->getVertex(1), t->getVertex(2),
-                          ve[0], ve[1], ve[2]);
+                          ve[0], ve[1], ve[2], 0, t->getPartition());
   }
   else{
     if(incomplete){
       return new MTriangleN(t->getVertex(0), t->getVertex(1), t->getVertex(2),
-                            ve, nPts + 1);
+                            ve, nPts + 1, 0, t->getPartition());
     }
     else{
       MTriangleN incpl(t->getVertex(0), t->getVertex(1), t->getVertex(2),
-		       ve, nPts + 1);
+		       ve, nPts + 1, 0, t->getPartition());
       getFaceVertices(gf, &incpl, t, vf, faceVertices, linear, nPts);
       ve.insert(ve.end(), vf.begin(), vf.end());
       return new MTriangleN(t->getVertex(0), t->getVertex(1), t->getVertex(2),
-                            ve, nPts + 1);
+                            ve, nPts + 1, 0, t->getPartition());
     }
   }
 }
@@ -844,25 +722,29 @@ static MQuadrangle *setHighOrder(MQuadrangle *q, GFace *gf,
   if(incomplete){
     if(nPts == 1){
       return new MQuadrangle8(q->getVertex(0), q->getVertex(1), q->getVertex(2),
-                              q->getVertex(3), ve[0], ve[1], ve[2], ve[3]);
+                              q->getVertex(3), ve[0], ve[1], ve[2], ve[3],
+			      0, q->getPartition());
     }
     else{
       return new MQuadrangleN(q->getVertex(0), q->getVertex(1), q->getVertex(2),
-                              q->getVertex(3), ve, nPts + 1);
+                              q->getVertex(3), ve, nPts + 1,
+			      0, q->getPartition());
     }
   }
   else {
     MQuadrangleN incpl(q->getVertex(0), q->getVertex(1), q->getVertex(2),
-		       q->getVertex(3), ve, nPts + 1);
+		       q->getVertex(3), ve, nPts + 1, 0, q->getPartition());
     getFaceVertices(gf, &incpl, q, vf, faceVertices, linear, nPts);
     ve.insert(ve.end(), vf.begin(), vf.end());
     if(nPts == 1){
       return new MQuadrangle9(q->getVertex(0), q->getVertex(1), q->getVertex(2),
-                              q->getVertex(3), ve[0], ve[1], ve[2], ve[3], vf[0]);
+                              q->getVertex(3), ve[0], ve[1], ve[2], ve[3], vf[0],
+			      0, q->getPartition());
     }
     else{
       return new MQuadrangleN(q->getVertex(0), q->getVertex(1), q->getVertex(2),
-                              q->getVertex(3), ve, nPts + 1);
+                              q->getVertex(3), ve, nPts + 1,
+			      0, q->getPartition());
     }
   }
 }
@@ -904,17 +786,19 @@ static void setHighOrder(GRegion *gr, edgeContainer &edgeVertices,
     if(nPts == 1){
       tetrahedra2.push_back
         (new MTetrahedron10(t->getVertex(0), t->getVertex(1), t->getVertex(2),
-                            t->getVertex(3), ve[0], ve[1], ve[2], ve[3], ve[4], ve[5]));
+                            t->getVertex(3), ve[0], ve[1], ve[2], ve[3], ve[4], ve[5],
+			    0, t->getPartition()));
     }
     else{
       getFaceVertices(gr, t, vf, faceVertices, edgeVertices, linear, nPts);
       ve.insert(ve.end(), vf.begin(), vf.end());
       MTetrahedronN incpl(t->getVertex(0), t->getVertex(1), t->getVertex(2),
-                          t->getVertex(3), ve, nPts + 1);
+                          t->getVertex(3), ve, nPts + 1, 0, t->getPartition());
       getRegionVertices(gr, &incpl, t, vr, linear, nPts);
       ve.insert(ve.end(), vr.begin(), vr.end());
       MTetrahedron *n = new MTetrahedronN(t->getVertex(0), t->getVertex(1),
-                                          t->getVertex(2), t->getVertex(3), ve, nPts + 1);
+                                          t->getVertex(2), t->getVertex(3), ve, nPts + 1,
+					  0, t->getPartition());
       tetrahedra2.push_back(n);
     }
     delete t;
@@ -933,7 +817,7 @@ static void setHighOrder(GRegion *gr, edgeContainer &edgeVertices,
 			     h->getVertex(3), h->getVertex(4), h->getVertex(5),
 			     h->getVertex(6), h->getVertex(7), ve[0], ve[1], ve[2],
 			     ve[3], ve[4], ve[5], ve[6], ve[7], ve[8], ve[9], ve[10],
-			     ve[11]));
+			     ve[11], 0, h->getPartition()));
       }
       else{
 	getFaceVertices(gr, h, vf, faceVertices, edgeVertices, linear, nPts);
@@ -945,7 +829,8 @@ static void setHighOrder(GRegion *gr, edgeContainer &edgeVertices,
 			     h->getVertex(3), h->getVertex(4), h->getVertex(5),
 			     h->getVertex(6), h->getVertex(7), ve[0], ve[1], ve[2],
 			     ve[3], ve[4], ve[5], ve[6], ve[7], ve[8], ve[9], ve[10],
-			     ve[11], vf[0], vf[1], vf[2], vf[3], vf[4], vf[5], v));
+			     ve[11], vf[0], vf[1], vf[2], vf[3], vf[4], vf[5], v,
+			     0, h->getPartition()));
       }
     }
     else {
@@ -953,12 +838,13 @@ static void setHighOrder(GRegion *gr, edgeContainer &edgeVertices,
       ve.insert(ve.end(), vf.begin(), vf.end());
       MHexahedronN incpl(h->getVertex(0), h->getVertex(1), h->getVertex(2),
                          h->getVertex(3), h->getVertex(4), h->getVertex(5),
-                         h->getVertex(6), h->getVertex(7), ve, nPts + 1);
+                         h->getVertex(6), h->getVertex(7), ve, nPts + 1, 0, h->getPartition());
       getRegionVertices(gr, &incpl, h, vr, linear, nPts);
       ve.insert(ve.end(), vr.begin(), vr.end());
       MHexahedron *n = new MHexahedronN(h->getVertex(0), h->getVertex(1), h->getVertex(2),
                                         h->getVertex(3), h->getVertex(4), h->getVertex(5),
-                                        h->getVertex(6), h->getVertex(7), ve, nPts + 1);
+                                        h->getVertex(6), h->getVertex(7), ve, nPts + 1,
+					0, h->getPartition());
       hexahedra2.push_back(n);
     }
     delete h;
@@ -974,7 +860,8 @@ static void setHighOrder(GRegion *gr, edgeContainer &edgeVertices,
       prisms2.push_back
         (new MPrism15(p->getVertex(0), p->getVertex(1), p->getVertex(2),
                       p->getVertex(3), p->getVertex(4), p->getVertex(5),
-                      ve[0], ve[1], ve[2], ve[3], ve[4], ve[5], ve[6], ve[7], ve[8]));
+                      ve[0], ve[1], ve[2], ve[3], ve[4], ve[5], ve[6], ve[7], ve[8],
+		      0, p->getPartition()));
     }
     else{
       if (nPts == 1) {
@@ -983,7 +870,8 @@ static void setHighOrder(GRegion *gr, edgeContainer &edgeVertices,
 	  (new MPrism18(p->getVertex(0), p->getVertex(1), p->getVertex(2),
 			p->getVertex(3), p->getVertex(4), p->getVertex(5),
 			ve[0], ve[1], ve[2], ve[3], ve[4], ve[5], ve[6], ve[7], ve[8],
-			vf[0], vf[1], vf[2]));
+			vf[0], vf[1], vf[2],
+			0, p->getPartition()));
       }
       else {
         Msg::Error("PrismN generation not implemented");
@@ -1010,47 +898,151 @@ static void setHighOrder(GRegion *gr, edgeContainer &edgeVertices,
   }
   gr->prisms = prisms2;
 
+/* * * * * * * * * * * * * * * * * PYRAMIDS * * * * * * * * * * * * * * * * * */
+
   std::vector<MPyramid*> pyramids2;
-  for(unsigned int i = 0; i < gr->pyramids.size(); i++){
+
+  for(unsigned int i = 0; i < gr->pyramids.size(); i++) {
     MPyramid *p = gr->pyramids[i];
     std::vector<MVertex*> ve, vf, vr;
     getEdgeVertices(gr, p, ve, edgeVertices, linear, nPts);
-    if (nPts == 1) {
-      if(incomplete){
-	pyramids2.push_back
-	  (new MPyramid13(p->getVertex(0), p->getVertex(1), p->getVertex(2),
-			  p->getVertex(3), p->getVertex(4), ve[0], ve[1], ve[2],
-			  ve[3], ve[4], ve[5], ve[6], ve[7]));
-      }
-      else{
-	getFaceVertices(gr, p, vf, faceVertices, edgeVertices, linear, nPts);
-	pyramids2.push_back
-	  (new MPyramid14(p->getVertex(0), p->getVertex(1), p->getVertex(2),
-			  p->getVertex(3), p->getVertex(4), ve[0], ve[1], ve[2],
-			  ve[3], ve[4], ve[5], ve[6], ve[7], vf[0]));
-      }
+
+    getFaceVertices(gr, p, vf, faceVertices, edgeVertices, linear, nPts);
+
+    ve.insert(ve.end(), vf.begin(), vf.end());
+
+    vr.reserve((nPts-1)*(nPts)*(2*(nPts-1)+1)/6);
+
+    int verts_lvl3[12] = {37,40,38,43,46,44,49,52,50,55,58,56};
+
+    int verts_lvl2[8];
+    if (nPts == 4) {
+      verts_lvl2[0] = 42; verts_lvl2[1] = 41;
+      verts_lvl2[2] = 48; verts_lvl2[3] = 47;
+      verts_lvl2[4] = 54; verts_lvl2[5] = 53;
+      verts_lvl2[6] = 60; verts_lvl2[7] = 59;
+    } else {
+      verts_lvl2[0] = 29; verts_lvl2[1] = 30;
+      verts_lvl2[2] = 35; verts_lvl2[3] = 36;
+      verts_lvl2[4] = 38; verts_lvl2[5] = 39;
+      verts_lvl2[6] = 32; verts_lvl2[7] = 33;
     }
-    else {
-      Msg::Error("PyramidN generation not implemented");
-      /*
-      getFaceVertices(gr, p, vf, faceVertices, edgeVertices, linear, nPts);
-      ve.insert(ve.end(), vf.begin(), vf.end());
-      MPyramidN incpl(p->getVertex(0), p->getVertex(1), p->getVertex(2),
-                         p->getVertex(3), p->getVertex(4), ve, nPts + 1);
-      getRegionVertices(gr, &incpl, p, vr, linear, nPts);
-      ve.insert(ve.end(), vr.begin(), vr.end());
-      printf("Hmmm, let's see : %d\n", nPts);
-      MPyramid *n = new MPyramidN(p->getVertex(0), p->getVertex(1), p->getVertex(2),
-				  p->getVertex(3), p->getVertex(4), ve, nPts + 1);
-      pyramids2.push_back(n);
-      */
+
+    int verts_lvl1[4];
+    switch(nPts) {
+      case(4):
+        verts_lvl1[0] = 39;
+        verts_lvl1[1] = 45;
+        verts_lvl1[2] = 51;
+        verts_lvl1[3] = 57;
+        break;
+      case(3):
+        verts_lvl1[0] = 31;
+        verts_lvl1[1] = 37;
+        verts_lvl1[2] = 40;
+        verts_lvl1[3] = 34;
+        break;
+      case(2):
+        verts_lvl1[0] = 21;
+        verts_lvl1[1] = 23;
+        verts_lvl1[2] = 24;
+        verts_lvl1[3] = 22;
+        break;
     }
+
+    for (int q = 0; q < nPts - 1; q++) {
+      std::vector<MVertex*> vq, veq;
+      vq.push_back(ve[2*nPts + q]);
+      vq.push_back(ve[4*nPts + q]);
+      vq.push_back(ve[6*nPts + q]);
+      vq.push_back(ve[7*nPts + q]);
+
+      if (nPts-q == 4)
+        for (int f = 0; f < 12; f++)
+          veq.push_back(ve[verts_lvl3[f]-5]);
+      else if (nPts-q == 3)
+        for (int f = 0; f < 8; f++)
+          veq.push_back(ve[verts_lvl2[f]-5]);
+      else if (nPts-q == 2)
+        for (int f = 0; f < 4; f++)
+          veq.push_back(ve[verts_lvl1[f]-5]);
+
+      if (nPts-q == 2) {
+        MQuadrangle8 incpl2(vq[0], vq[1], vq[2], vq[3],
+                     veq[0], veq[1], veq[2], veq[3]);
+        SPoint3 pointz;
+        incpl2.pnt(0,0,0,pointz);
+        MVertex *v = new MVertex(pointz.x(), pointz.y(), pointz.z(), gr);
+
+        gr->mesh_vertices.push_back(v);
+        std::vector<MVertex*>::iterator cursor = vr.begin();
+        cursor += nPts == 2 ? 0 : 4;
+        vr.insert(cursor, v);
+      }
+      else if (nPts-q == 3) {
+
+        MQuadrangleN incpl2(vq[0], vq[1], vq[2], vq[3], veq, 3);
+        int offsets[4] = {nPts == 4 ? 7 : 0,
+                          nPts == 4 ? 9 : 1,
+                          nPts == 4 ? 11 : 2,
+                          nPts == 4 ? 12 : 3};
+        double quad_v [4][2] = {{-1.0/3.0, -1.0/3.0},
+                             { 1.0/3.0, -1.0/3.0},
+                             { 1.0/3.0,  1.0/3.0},
+                             {-1.0/3.0,  1.0/3.0}};
+        SPoint3 pointz;
+        for (int k = 0; k<4; k++) {
+          incpl2.pnt(quad_v[k][0], quad_v[k][1], 0, pointz);
+          MVertex *v = new MVertex(pointz.x(), pointz.y(), pointz.z(), gr);
+          gr->mesh_vertices.push_back(v);
+          std::vector<MVertex*>::iterator cursor = vr.begin();
+          cursor += offsets[k];
+          vr.insert(cursor, v);
+        }
+      }
+      else if (nPts-q == 4) {
+        MQuadrangleN incpl2(vq[0], vq[1], vq[2], vq[3], veq, 4);
+        int offsets[9] = {0, 1, 2, 3, 5, 8, 10, 6, 13};
+        double quad_v [9][2] = {
+                             { -0.5, -0.5},
+                             {  0.5, -0.5},
+                             {  0.5,  0.5},
+                             { -0.5,  0.5},
+                             {  0.0, -0.5},
+                             {  0.5,  0.0},
+                             {  0.0,  0.5},
+                             { -0.5,  0.0},
+                             {  0.0,  0.0}
+                               };
+        SPoint3 pointz;
+        for (int k = 0; k<9; k++) {
+          incpl2.pnt(quad_v[k][0], quad_v[k][1], 0, pointz);
+          MVertex *v = new MVertex(pointz.x(), pointz.y(), pointz.z(), gr);
+          gr->mesh_vertices.push_back(v);
+          std::vector<MVertex*>::iterator cursor = vr.begin();
+          cursor += offsets[k];
+          vr.insert(cursor, v);
+        }
+      }
+
+    }
+    ve.insert(ve.end(), vr.begin(), vr.end());
+    MPyramid *n = new MPyramidN(p->getVertex(0), p->getVertex(1),
+                                p->getVertex(2), p->getVertex(3),
+                                p->getVertex(4), ve, nPts + 1,
+			                          0, p->getPartition());
+    pyramids2.push_back(n);
+    SPoint3 test_pnt;
+    n->pnt(-1,-1,0, test_pnt);
+
     delete p;
   }
   gr->pyramids = pyramids2;
   gr->deleteVertexArrays();
 }
 
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+
 template<class T>
 static void setFirstOrder(GEntity *e, std::vector<T*> &elements, bool onlyVisible)
 {
@@ -1062,7 +1054,7 @@ static void setFirstOrder(GEntity *e, std::vector<T*> &elements, bool onlyVisibl
     std::vector<MVertex*> v1;
     for(int j = 0; j < n; j++)
       v1.push_back(ele->getVertex(j));
-    elements1.push_back(new T(v1));
+    elements1.push_back(new T(v1, 0, ele->getPartition()));
     delete ele;
   }
   elements = elements1;
@@ -1157,33 +1149,30 @@ static void checkHighOrderTetrahedron(const char* cc, GModel *m,
 {
   bad.clear();
   minJGlob = 1.0;
-  double minGGlob = 1.0;
   double avg = 0.0;
   int count = 0, nbfair=0;
   for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); ++it){
     for(unsigned int i = 0; i < (*it)->tetrahedra.size(); i++){
       MTetrahedron *t = (*it)->tetrahedra[i];
       double disto_ = t->distoShapeMeasure();
-      double gamma_ = t->gammaShapeMeasure();
-      double disto = disto_;
-      minJGlob = std::min(minJGlob, disto);
-      minGGlob = std::min(minGGlob, gamma_);
-      avg += disto; count++;
-      if (disto < 0) bad.push_back(t);
-      else if (disto < 0.2) nbfair++;
+      minJGlob = std::min(minJGlob, disto_);
+      avg += disto_; count++;
+      if (disto_ < 0) bad.push_back(t);
+      else if (disto_ < 0.2) nbfair++;
     }
   }
   if(!count) return;
   if (minJGlob < 0)
-    Msg::Warning("%s : Worst Tetrahedron Smoothness %g Gamma %g NbFair = %d NbBad = %d",
-              cc, minJGlob, minGGlob, nbfair, bad.size());
+    Msg::Warning("%s : Worst Tetrahedron Smoothness %g (%d negative jacobians) "
+                 "Avg Smoothness %g", cc, minJGlob, bad.size(),
+                 avg / (count ? count : 1));
   else
     Msg::Info("%s : Worst Tetrahedron Smoothness %g (%d negative jacobians) "
-                 "Worst Gamma %g Avg Smoothness %g", cc, minJGlob, bad.size(),
-                 minGGlob, avg / (count ? count : 1));
+                 "Avg Smoothness %g", cc, minJGlob, bad.size(),
+                 avg / (count ? count : 1));
 }
 
-extern double mesh_functional_distorsion(MElement *t, double u, double v);
+extern double mesh_functional_distorsion_2D(MElement *t, double u, double v);
 
 void printJacobians(GModel *m, const char *nm)
 {
@@ -1201,7 +1190,7 @@ void printJacobians(GModel *m, const char *nm)
           double u = (double)i / (n - 1);
           double v = (double)k / (n - 1);
           t->pnt(u, v, 0, pt);
-          D[i][k] = mesh_functional_distorsion(t, u, v);
+          D[i][k] = 0.; //mesh_functional_distorsion_2D(t, u, v);
           //X[i][k] = u;
           //Y[i][k] = v;
           //Z[i][k] = 0.0;
@@ -1321,10 +1310,10 @@ void SetOrderN(GModel *m, int order, bool linear, bool incomplete, bool onlyVisi
 
   int nPts = order - 1;
 
-  if (!linear)
-    Msg::StatusBar(2, true, "Meshing order %d, curvilinear ON...", order);
-  else
-    Msg::StatusBar(2, true, "Meshing order %d, curvilinear OFF...", order);
+  char msg[256];
+  sprintf(msg, "Meshing order %d (curvilinear %s)...", order, linear ? "off" : "on");
+
+  Msg::StatusBar(true, msg);
 
   double t1 = Cpu();
 
@@ -1335,23 +1324,25 @@ void SetOrderN(GModel *m, int order, bool linear, bool incomplete, bool onlyVisi
   edgeContainer edgeVertices;
   faceContainer faceVertices;
 
-  int counter = 1;
+  Msg::ResetProgressMeter();
+
+  int counter = 0, nTot = m->getNumEdges() + m->getNumFaces() + m->getNumRegions();
+
   for(GModel::eiter it = m->firstEdge(); it != m->lastEdge(); ++it) {
-    Msg::Info("Meshing curves order %d (%i/%i)...", order, counter, m->getNumEdges());
-    counter++;
+    Msg::Info("Meshing curve %d order %d", (*it)->tag(), order);
+    Msg::ProgressMeter(++counter, nTot, false, msg);
     if (onlyVisible && !(*it)->getVisibility()) continue;
     setHighOrder(*it, edgeVertices, linear, nPts);
   }
 
-  counter = 1;
   for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it) {
-    Msg::Info("Meshing surfaces order %d (%i/%i)...", order, counter, m->getNumFaces());
-    counter++;
+    Msg::Info("Meshing surface %d order %d", (*it)->tag(), order);
+    Msg::ProgressMeter(++counter, nTot, false, msg);
     if (onlyVisible && !(*it)->getVisibility()) continue;
     setHighOrder(*it, edgeVertices, faceVertices, linear, incomplete, nPts);
   }
 
-  highOrderTools hot(m);
+  //  highOrderTools hot(m);
 
   // now we smooth mesh the internal vertices of the faces
   // we do that model face by model face
@@ -1361,10 +1352,9 @@ void SetOrderN(GModel *m, int order, bool linear, bool incomplete, bool onlyVisi
   // printJacobians(m, "smoothness_b.pos");
   // m->writeMSH("RAW.msh");
 
-  counter = 1;
   for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); ++it) {
-    Msg::Info("Meshing volumes order %d (%i/%i)...", order, counter, m->getNumRegions());
-    counter++;
+    Msg::Info("Meshing volume %d order %d", (*it)->tag(), order);
+    Msg::ProgressMeter(++counter, nTot, false, msg);
     if (onlyVisible && !(*it)->getVisibility())continue;
     setHighOrder(*it, edgeVertices, faceVertices, linear, incomplete, nPts);
   }
@@ -1380,16 +1370,16 @@ void SetOrderN(GModel *m, int order, bool linear, bool incomplete, bool onlyVisi
       std::vector<MElement*> v;
       v.insert(v.begin(), (*it)->triangles.begin(), (*it)->triangles.end());
       v.insert(v.end(), (*it)->quadrangles.begin(), (*it)->quadrangles.end());
-      hot.applySmoothingTo(v, (*it));
-      //hot.applySmoothingTo(v, .1,0);
+      //hot.applySmoothingTo(v, (*it));
+      // hot.applySmoothingTo(v, .1,0);
     }
-    //    hot.ensureMinimumDistorsion(0.1);
+    // hot.ensureMinimumDistorsion(0.1);
     checkHighOrderTriangles("Final surface mesh", m, bad, worst);
   }
-
+  checkHighOrderTetrahedron("Volume Mesh", m, bad, worst);
   // m->writeMSH("CORRECTED.msh");
 
-  Msg::StatusBar(2, true, "Done meshing order %d (%g s)", order, t2 - t1);
+  Msg::StatusBar(true, "Done meshing order %d (%g s)", order, t2 - t1);
 }
 
 void computeDistanceFromMeshToGeometry (GModel *m, distanceFromMeshToGeometry_t &dist)
@@ -1417,12 +1407,12 @@ void SetHighOrderComplete(GModel *m, bool onlyVisible)
       MTriangle *t = (*it)->triangles[i];
       std::vector<MVertex*> vf, vt;
       int nPts = t->getPolynomialOrder() - 1;
-      MTriangle TEMP (t->getVertex(0), t->getVertex(1), t->getVertex(2));
+      MTriangle TEMP (t->getVertex(0), t->getVertex(1), t->getVertex(2), 0, t->getPartition());
       getFaceVertices (*it, t, t, vf, faceVertices, false, nPts);
       for (int j=3;j<t->getNumVertices();j++)vt.push_back(t->getVertex(j));
       vt.insert(vt.end(), vf.begin(), vf.end());
       MTriangleN *newTr = new MTriangleN(t->getVertex(0), t->getVertex(1), t->getVertex(2),
-					 vt, nPts + 1);
+					 vt, nPts + 1, 0, t->getPartition());
       newT.push_back(newTr);
 
       delete t;
@@ -1433,13 +1423,14 @@ void SetHighOrderComplete(GModel *m, bool onlyVisible)
       MQuadrangle *t = (*it)->quadrangles[i];
       std::vector<MVertex*> vf, vt;
       int nPts = t->getPolynomialOrder() - 1;
-      MQuadrangle TEMP (t->getVertex(0), t->getVertex(1), t->getVertex(2), t->getVertex(3));
+      MQuadrangle TEMP (t->getVertex(0), t->getVertex(1), t->getVertex(2),
+                        t->getVertex(3), 0, t->getPartition());
       getFaceVertices (*it, t, &TEMP, vf, faceVertices, false, nPts);
       for (int j=4;j<t->getNumVertices();j++)vt.push_back(t->getVertex(j));
       vt.insert(vt.end(), vf.begin(), vf.end());
       newQ.push_back(new MQuadrangleN(t->getVertex(0), t->getVertex(1),
                                       t->getVertex(2), t->getVertex(3),
-				      vt, nPts + 1));
+				      vt, nPts + 1, 0, t->getPartition()));
       delete t;
     }
     (*it)->quadrangles = newQ;
@@ -1471,7 +1462,7 @@ void SetHighOrderInComplete (GModel *m, bool onlyVisible)
            j < t->getNumVertices(); j++)
         toDelete.insert(t->getVertex(j));
       newT.push_back(new MTriangleN(t->getVertex(0), t->getVertex(1), t->getVertex(2),
-				    vt, order));
+				    vt, order, 0, t->getPartition()));
       delete t;
     }
     (*it)->triangles = newT;
@@ -1485,7 +1476,7 @@ void SetHighOrderInComplete (GModel *m, bool onlyVisible)
         vt.push_back(t->getVertex(j));
       newQ.push_back(new MQuadrangleN(t->getVertex(0), t->getVertex(1),
                                       t->getVertex(2), t->getVertex(3),
-				      vt, nPts + 1));
+				      vt, nPts + 1, 0, t->getPartition()));
       delete t;
     }
     (*it)->quadrangles = newQ;
