@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 //
 // Contributor(s):
 //   Tristan Carrier
@@ -776,7 +776,7 @@ void LpCVT::eval(std::vector<SPoint3>& bank,std::vector<int>& movability,int off
   compute_parameters();
 
   for(i=0;i<clipped.size();i++){
-	if(clipped[i].get_quality()<e) continue;
+	if(clipped[i].get_quality()<e) continue; //not exact
 	init_caches(clipped[i],p);
     energy = energy + F(clipped[i],p);
 	grad1 = dF_dC1(clipped[i],p);
@@ -869,7 +869,7 @@ void LpCVT::compute_parameters(){
 
 double LpCVT::get_size(double x,double y,double z){
   //if outside domain return 1.0 (or other value > 0.0)
-  return 0.25;
+  return Size_field::search(x,y,z);
 }
 
 Tensor LpCVT::get_tensor(double x,double y,double z){
@@ -1057,7 +1057,7 @@ SVector3 LpCVT::simple(VoronoiElement element,int p){
 
 SVector3 LpCVT::dF_dC1(VoronoiElement element,int p){
   int i;
-  double u,v,w;
+  double u;//,v,w;
   double comp_x,comp_y,comp_z;
   double weight;
   double rho;
@@ -1086,8 +1086,8 @@ SVector3 LpCVT::dF_dC1(VoronoiElement element,int p){
 
   for(i=0;i<gauss_num;i++){
     u = gauss_points(i,0);
-	v = gauss_points(i,1);
-	w = gauss_points(i,2);
+	//v = gauss_points(i,1);
+	//w = gauss_points(i,2);
 	weight = gauss_weights(i);
 	rho = rho_cache[i];
 	drho_dx = drho_dx_cache[i];
@@ -1109,7 +1109,7 @@ SVector3 LpCVT::dF_dC1(VoronoiElement element,int p){
 
 SVector3 LpCVT::dF_dC2(VoronoiElement element,int p){
   int i;
-  double u,v,w;
+  double v;//u,v,w;
   double comp_x,comp_y,comp_z;
   double weight;
   double rho;
@@ -1137,9 +1137,9 @@ SVector3 LpCVT::dF_dC2(VoronoiElement element,int p){
   gz = generator.z();
 
   for(i=0;i<gauss_num;i++){
-    u = gauss_points(i,0);
+    //u = gauss_points(i,0);
 	v = gauss_points(i,1);
-	w = gauss_points(i,2);
+	//w = gauss_points(i,2);
 	weight = gauss_weights(i);
 	rho = rho_cache[i];
 	drho_dx = drho_dx_cache[i];
@@ -1161,7 +1161,7 @@ SVector3 LpCVT::dF_dC2(VoronoiElement element,int p){
 
 SVector3 LpCVT::dF_dC3(VoronoiElement element,int p){
   int i;
-  double u,v,w;
+  double w;//u,v,w;
   double comp_x,comp_y,comp_z;
   double weight;
   double rho;
@@ -1189,8 +1189,8 @@ SVector3 LpCVT::dF_dC3(VoronoiElement element,int p){
   gz = generator.z();
 
   for(i=0;i<gauss_num;i++){
-    u = gauss_points(i,0);
-	v = gauss_points(i,1);
+    //u = gauss_points(i,0);
+	//v = gauss_points(i,1);
 	w = gauss_points(i,2);
 	weight = gauss_weights(i);
 	rho = rho_cache[i];
@@ -1336,11 +1336,15 @@ double LpCVT::df_dz(SPoint3 p1,SPoint3 p2,Tensor t,int p){
 }
 
 SVector3 LpCVT::bisectors3(SVector3 dIdC,SPoint3 C,SPoint3 x0,SPoint3 x1,SPoint3 x2,SPoint3 x3){
+  double e;
   fullMatrix<double> A(3,3);
   fullMatrix<double> B(3,3);
   fullMatrix<double> M(3,3);
   fullMatrix<double> _dIdC(1,3);
   fullMatrix<double> _val(1,3);
+  
+  e = 0.000000001;	
+	
   A(0,0) = x1.x() - x0.x();
   A(0,1) = x1.y() - x0.y();
   A(0,2) = x1.z() - x0.z();
@@ -1350,7 +1354,22 @@ SVector3 LpCVT::bisectors3(SVector3 dIdC,SPoint3 C,SPoint3 x0,SPoint3 x1,SPoint3
   A(2,0) = x3.x() - x0.x();
   A(2,1) = x3.y() - x0.y();
   A(2,2) = x3.z() - x0.z();
+  
+  if(fabs(A.determinant())<e){
+    srand(time(NULL));
+    A(0,0) = A(0,0) + e*((double)rand())/((double)RAND_MAX);
+    A(0,1) = A(0,1) + e*((double)rand())/((double)RAND_MAX);
+    A(0,2) = A(0,2) + e*((double)rand())/((double)RAND_MAX);
+    A(1,0) = A(1,0) + e*((double)rand())/((double)RAND_MAX);
+    A(1,1) = A(1,1) + e*((double)rand())/((double)RAND_MAX);
+    A(1,2) = A(1,2) + e*((double)rand())/((double)RAND_MAX);
+    A(2,0) = A(2,0) + e*((double)rand())/((double)RAND_MAX);
+    A(2,1) = A(2,1) + e*((double)rand())/((double)RAND_MAX);
+    A(2,2) = A(2,2) + e*((double)rand())/((double)RAND_MAX);
+  }
+
   A.invertInPlace();
+
   B(0,0) = C.x() - x0.x();
   B(0,1) = C.y() - x0.y();
   B(0,2) = C.z() - x0.z();
@@ -1360,11 +1379,14 @@ SVector3 LpCVT::bisectors3(SVector3 dIdC,SPoint3 C,SPoint3 x0,SPoint3 x1,SPoint3
   B(2,0) = C.x() - x0.x();
   B(2,1) = C.y() - x0.y();
   B(2,2) = C.z() - x0.z();
+  
   A.mult_naive(B,M);
+  
   _dIdC(0,0) = dIdC.x();
   _dIdC(0,1) = dIdC.y();
   _dIdC(0,2) = dIdC.z();
   _dIdC.mult_naive(M,_val);
+  
   return SVector3(_val(0,0),_val(0,1),_val(0,2));
 }
 
@@ -1374,6 +1396,7 @@ SVector3 LpCVT::bisectors2(SVector3 dIdC,SPoint3 C,SPoint3 x0,SPoint3 x1,SPoint3
   fullMatrix<double> M(3,3);
   fullMatrix<double> _dIdC(1,3);
   fullMatrix<double> _val(1,3);
+  
   A(0,0) = x1.x() - x0.x();
   A(0,1) = x1.y() - x0.y();
   A(0,2) = x1.z() - x0.z();
@@ -1383,7 +1406,9 @@ SVector3 LpCVT::bisectors2(SVector3 dIdC,SPoint3 C,SPoint3 x0,SPoint3 x1,SPoint3
   A(2,0) = normal1.x();
   A(2,1) = normal1.y();
   A(2,2) = normal1.z();
+  
   A.invertInPlace();
+  
   B(0,0) = C.x() - x0.x();
   B(0,1) = C.y() - x0.y();
   B(0,2) = C.z() - x0.z();
@@ -1393,11 +1418,14 @@ SVector3 LpCVT::bisectors2(SVector3 dIdC,SPoint3 C,SPoint3 x0,SPoint3 x1,SPoint3
   B(2,0) = 0.0;
   B(2,1) = 0.0;
   B(2,2) = 0.0;
+  
   A.mult_naive(B,M);
+  
   _dIdC(0,0) = dIdC.x();
   _dIdC(0,1) = dIdC.y();
   _dIdC(0,2) = dIdC.z();
   _dIdC.mult_naive(M,_val);
+  
   return SVector3(_val(0,0),_val(0,1),_val(0,2));
 }
 
@@ -1407,6 +1435,7 @@ SVector3 LpCVT::bisectors1(SVector3 dIdC,SPoint3 C,SPoint3 x0,SPoint3 x1,SVector
   fullMatrix<double> M(3,3);
   fullMatrix<double> _dIdC(1,3);
   fullMatrix<double> _val(1,3);
+  
   A(0,0) = x1.x() - x0.x();
   A(0,1) = x1.y() - x0.y();
   A(0,2) = x1.z() - x0.z();
@@ -1416,7 +1445,9 @@ SVector3 LpCVT::bisectors1(SVector3 dIdC,SPoint3 C,SPoint3 x0,SPoint3 x1,SVector
   A(2,0) = normal2.x();
   A(2,1) = normal2.y();
   A(2,2) = normal2.z();
+  
   A.invertInPlace();
+  
   B(0,0) = C.x() - x0.x();
   B(0,1) = C.y() - x0.y();
   B(0,2) = C.z() - x0.z();
@@ -1426,11 +1457,14 @@ SVector3 LpCVT::bisectors1(SVector3 dIdC,SPoint3 C,SPoint3 x0,SPoint3 x1,SVector
   B(2,0) = 0.0;
   B(2,1) = 0.0;
   B(2,2) = 0.0;
+  
   A.mult_naive(B,M);
+  
   _dIdC(0,0) = dIdC.x();
   _dIdC(0,1) = dIdC.y();
   _dIdC(0,2) = dIdC.z();
   _dIdC.mult_naive(M,_val);
+  
   return SVector3(_val(0,0),_val(0,1),_val(0,2));
 }
 
@@ -1452,8 +1486,6 @@ void LpSmoother::improve_model(){
   GModel* model = GModel::current();
   GModel::riter it;
 
-  Frame_field::init_model();
-
   for(it=model->firstRegion();it!=model->lastRegion();it++)
   {
     gr = *it;
@@ -1461,13 +1493,11 @@ void LpSmoother::improve_model(){
 	  improve_region(gr);
 	}
   }
-
-  Frame_field::clear();
 }
 
 void LpSmoother::improve_region(GRegion* gr)
 {
-#if defined(HAVE_BFGS)
+  #if defined(HAVE_BFGS)
   unsigned int i;
   int offset;
   double epsg;
@@ -1494,6 +1524,9 @@ void LpSmoother::improve_region(GRegion* gr)
   alglib::real_1d_array x;
   alglib::real_1d_array alglib_scales;
 
+  Frame_field::init_region(gr);
+  Size_field::init_region(gr);
+  Size_field::solve(gr);
   octree = new MElementOctree(gr->model());
 
   for(i=0;i<gr->getNumMeshElements();i++){
@@ -1623,7 +1656,9 @@ void LpSmoother::improve_region(GRegion* gr)
   delete octree;
   free(initial_conditions);
   free(scales);
-#endif
+  Size_field::clear();
+  Frame_field::clear();
+  #endif
 }
 
 int LpSmoother::get_nbr_interior_vertices(){
