@@ -13,11 +13,11 @@ Trevor S. Strickler hereby transfers copyright of QuadTri files to
 Christophe Geuzaine and J.-F. Remacle with the understanding that
 his contribution shall be cited appropriately.
 
-All reused or original Gmsh code is Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+All reused or original Gmsh code is Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 Gmsh is available at: www.geuz.org/gmsh
 
 For Gmsh license information, see the LICENSE.txt file for license information. Please report all
-Gmsh bugs and problems to <gmsh@geuz.org>.
+Gmsh bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, Version 2,
@@ -507,7 +507,7 @@ int MeshQuadToTriTopSurface( GFace *from, GFace *to, std::set<MVertex*,
 
       // count vertices that are on a boundary edge
       int edge_verts_count = 0;
-      int skip_index = 0;
+      //int skip_index = 0;
       int bnd_indices[4];
       for( int p = 0; p < element->getNumVertices(); p++ ){
         if( pos_src_edge.find( element->getVertex(p) ) != pos_src_edge.end() ){
@@ -515,7 +515,7 @@ int MeshQuadToTriTopSurface( GFace *from, GFace *to, std::set<MVertex*,
           bnd_indices[p] = 1;
         }
         else{
-          skip_index = p;
+          //skip_index = p;
           bnd_indices[p] = 0;
         }
       }
