@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include <sstream>
 #include <stdlib.h>
@@ -24,6 +24,8 @@
 #include "MLine.h"
 #include "MTriangle.h"
 #include "MQuadrangle.h"
+#include "CenterlineField.h"
+#include "meshGFaceElliptic.h"
 #include "Context.h"
 #include "GPoint.h"
 #include "GmshMessage.h"
@@ -43,6 +45,76 @@
 #include "meshGFaceLloyd.h"
 #include "meshGFaceBoundaryLayers.h"
 
+inline double myAngle (const SVector3 &a, const SVector3 &b, const SVector3 &d)
+{
+  double cosTheta = dot(a,b);
+  double sinTheta = dot(crossprod(a,b),d);
+  return atan2 (sinTheta,cosTheta);
+}
+
+struct myPlane {
+  SPoint3 p;
+  SVector3 n;
+  double a;
+  // nx x + ny y + nz z + a = 0
+  myPlane(SPoint3 _p, SVector3 _n) : p(_p),n(_n)
+  {
+    n.normalize();
+    a = -(n.x()*p.x()+n.y()*p.y()+n.z()*p.z());
+  }
+  double eval (double x, double y, double z)
+  {
+    return n.x() * x + n.y() * y + n.z() * z + a;
+  }
+};
+
+struct myLine {
+  SPoint3 p;
+  SVector3 t;
+  myLine() : p(0,0,0) , t (0,0,1) {}
+  myLine(myPlane &p1, myPlane &p2)
+  {
+    t = crossprod(p1.n,p2.n);
+    if (t.norm() == 0.0){
+      Msg::Error("parallel planes do not intersect");
+    }
+    else
+      t.normalize();
+    // find a point, assume z = 0
+    double a[2][2] = {{p1.n.x(), p1.n.y()}, {p2.n.x(), p2.n.y()}};
+    double b[2] = {-p1.a, -p2.a}, x[2];
+    if (!sys2x2(a, b, x)){
+      // assume x = 0
+      double az[2][2] = {{p1.n.y(), p1.n.z()}, {p2.n.y(), p2.n.z()}};
+      double bz[2] = {-p1.a, -p2.a};
+      if (!sys2x2(az, bz, x)){
+	// assume y = 0
+	double ay[2][2] = {{p1.n.x(), p1.n.z()}, {p2.n.x(), p2.n.z()}};
+	double by[2] = {-p1.a, -p2.a};
+	if (!sys2x2(ay,by,x)){
+	  Msg::Error("parallel planes do not intersect");
+	}
+	else {
+	  p = SPoint3(x[0], 0., x[1]);
+	}
+      }
+      else{
+	p = SPoint3(0., x[0], x[1]);
+      }
+    }
+    else{
+      p = SPoint3(x[0], x[1], 0.);
+    }
+  }
+  SPoint3 orthogonalProjection (SPoint3 &a)
+  {
+    // (x - a) * t = 0 -->
+    // x = p + u t --> (p + ut - a) * t = 0 --> u = (a-p) * t
+    const double u = dot(a - p, t);
+    return SPoint3(p.x() + t.x() * u,p.y() + t.y() * u,p.z() + t.z() * u);
+  }
+};
+
 static void copyMesh(GFace *source, GFace *target)
 {
   std::map<MVertex*, MVertex*> vs2vt;
@@ -53,7 +125,7 @@ static void copyMesh(GFace *source, GFace *target)
       std::map<int, int>::iterator adnksd = target->edgeCounterparts.find((*it)->tag());
       int source_e;
       if(adnksd != target->edgeCounterparts.end())
-	 source_e = adnksd->second;
+        source_e = adnksd->second;
       else{
 	sign = -1;
         adnksd = target->edgeCounterparts.find(-(*it)->tag());
@@ -65,69 +137,151 @@ static void copyMesh(GFace *source, GFace *target)
           return;
         }
       }
-
       GEdge *se = source->model()->getEdgeByTag(abs(source_e));
       GEdge *te = *it;
       if (source_e * sign > 0){
-	vs2vt[se->getBeginVertex()->mesh_vertices[0]] = te->getBeginVertex()->mesh_vertices[0];
-	vs2vt[se->getEndVertex()->mesh_vertices[0]] = te->getEndVertex()->mesh_vertices[0];
-	for (unsigned i = 0; i < se->mesh_vertices.size(); i++){
-	  MVertex *vs = se->mesh_vertices[i];
-	  MVertex *vt = te->mesh_vertices[i];
-	  vs2vt[vs] = vt;
-	}
+	vs2vt[se->getBeginVertex()->mesh_vertices[0]] =
+          te->getBeginVertex()->mesh_vertices[0];
+	vs2vt[se->getEndVertex()->mesh_vertices[0]] =
+          te->getEndVertex()->mesh_vertices[0];
       }
       else {
-	vs2vt[se->getBeginVertex()->mesh_vertices[0]] = te->getEndVertex()->mesh_vertices[0];
-	vs2vt[se->getEndVertex()->mesh_vertices[0]] = te->getBeginVertex()->mesh_vertices[0];
-	for (unsigned i = 0; i < se->mesh_vertices.size(); i++){
-	  MVertex *vs = se->mesh_vertices[i];
-	  MVertex *vt = te->mesh_vertices[se->mesh_vertices.size() - i - 1];
-	  vs2vt[vs] = vt;
-	}
+	vs2vt[se->getBeginVertex()->mesh_vertices[0]] =
+          te->getEndVertex()->mesh_vertices[0];
+	vs2vt[se->getEndVertex()->mesh_vertices[0]] =
+          te->getBeginVertex()->mesh_vertices[0];
+      }
+      // iterate on source vertices
+      for (unsigned i = 0; i < te->mesh_vertices.size(); i++){
+	MVertex *vt = te->mesh_vertices[i];
+	MVertex *vs = se->mesh_vertices[source_e * sign > 0 ? i :
+                                        te->mesh_vertices.size() - i - 1];
+	vs2vt[vs] = vt;
       }
     }
   }
 
-  SPoint2 param_source[2], param_target[2];
+  bool translation = true;
+  bool rotation = false;
+
+  SVector3 DX;
   int count = 0;
-  for (std::map<MVertex*, MVertex*>::iterator it = vs2vt.begin(); it != vs2vt.end() ; ++it){
+  for (std::map<MVertex*, MVertex*>::iterator it = vs2vt.begin();
+       it != vs2vt.end() ; ++it){
     MVertex *vs = it->first;
     MVertex *vt = it->second;
-    if (vs->onWhat()->dim() == 1){
-      reparamMeshVertexOnFace(vs, source, param_source[count]);
-      reparamMeshVertexOnFace(vt, target, param_target[count++]);
-      if (count == 2) break;
+    if (count == 0)
+      DX = SVector3(vt->x() - vs->x(), vt->y() - vs->y(), vt->z() - vs->z());
+    else {
+      SVector3 DX2 = DX - SVector3 (vt->x() - vs->x(), vt->y() - vs->y(),
+                                    vt->z() - vs->z());
+      if (DX2.norm() > DX.norm() * 1.e-8) translation = false;
     }
+    count ++;
   }
 
-  if (count < 2) return;
+  double rot[3][3] ;
+  myLine LINE;
+  double ANGLE=0;
+  if (!translation){
+    count = 0;
+    rotation = true;
+    std::vector<SPoint3> mps,mpt;
+    for (std::map<MVertex*, MVertex*>::iterator it = vs2vt.begin();
+         it != vs2vt.end() ; ++it){
+      MVertex *vs = it->first;
+      MVertex *vt = it->second;
+      mps.push_back(SPoint3(vs->x(),vs->y(),vs->z()));
+      mpt.push_back(SPoint3(vt->x(), vt->y(), vt->z()));
+    }
+    mean_plane mean_source, mean_target;
+    computeMeanPlaneSimple(mps, mean_source);
+    computeMeanPlaneSimple(mpt, mean_target);
+    myPlane PLANE_SOURCE(SPoint3(mean_source.x,mean_source.y,mean_source.z),
+                         SVector3(mean_source.a,mean_source.b,mean_source.c));
+    myPlane PLANE_TARGET(SPoint3(mean_target.x,mean_target.y,mean_target.z),
+                         SVector3(mean_target.a,mean_target.b,mean_target.c));
+    LINE = myLine(PLANE_SOURCE, PLANE_TARGET);
 
-  const double t1u = param_target[0].x(), t1v = param_target[0].y();
-  const double t2u = param_target[1].x(), t2v = param_target[1].y();
-  const double s1u = param_source[0].x(), s1v = param_source[0].y();
-  const double s2u = param_source[1].x(), s2v = param_source[1].y();
+    // LINE is the axis of rotation
+    // let us compute the angle of rotation
+    count = 0;
+    for (std::map<MVertex*, MVertex*>::iterator it = vs2vt.begin();
+         it != vs2vt.end(); ++it){
+      MVertex *vs = it->first;
+      MVertex *vt = it->second;
+      // project both points on the axis: that should be the same point !
+      SPoint3 ps = SPoint3(vs->x(), vs->y(), vs->z());
+      SPoint3 pt = SPoint3(vt->x(), vt->y(), vt->z());
+      SPoint3 p_ps = LINE.orthogonalProjection (ps);
+      SPoint3 p_pt = LINE.orthogonalProjection (pt);
+      SVector3 dist1 = ps - pt;
+      SVector3 dist2 = p_ps - p_pt;
+      if (dist2.norm() > 1.e-8 * dist1.norm()){
+	rotation = false;
+      }
+      SVector3 t1 = ps - p_ps;
+      SVector3 t2 = pt - p_pt;
+      if (t1.norm() > 1.e-8 * dist1.norm()){
+	if (count == 0)
+          ANGLE = myAngle (t1, t2, LINE.t);
+	else {
+	  double ANGLE2 = myAngle(t1, t2, LINE.t);
+	  if (fabs (ANGLE2-ANGLE) > 1.e-8) rotation = false;
+	}
+	count++;
+      }
+    }
 
-  SVector3 _a(s2u - s1u, s2v - s1v, 0);
-  SVector3 _b(t2u - t1u, t2v - t1v, 0);
-  SVector3 _c = crossprod(_a, _b);
-  double sinA = _c.z();
-  double cosA = dot(_a, _b);
-  const double theta = atan2(sinA, cosA);
-  const double c = cos(theta);
-  const double s = sin(theta);
+    if (rotation){
+      Msg::Info("Periodic mesh rotation found: axis (%g,%g,%g) point (%g %g %g) angle %g",
+                LINE.t.x(), LINE.t.y(), LINE.t.z(), LINE.p.x(), LINE.p.y(), LINE.p.z(),
+                ANGLE * 180 / M_PI);
+      double ux = LINE.t.x();
+      double uy = LINE.t.y();
+      double uz = LINE.t.z();
+      rot[0][0] = cos (ANGLE) + ux*ux*(1.-cos(ANGLE));
+      rot[0][1] = ux*uy*(1.-cos(ANGLE)) - uz * sin(ANGLE);
+      rot[0][2] = ux*uz*(1.-cos(ANGLE)) + uy * sin(ANGLE);
+      rot[1][0] = ux*uy*(1.-cos(ANGLE)) + uz * sin(ANGLE);
+      rot[1][1] = cos (ANGLE) + uy*uy*(1.-cos(ANGLE));
+      rot[1][2] = uy*uz*(1.-cos(ANGLE)) - ux * sin(ANGLE);
+      rot[2][0] = ux*uz*(1.-cos(ANGLE)) - uy * sin(ANGLE);
+      rot[2][1] = uy*uz*(1.-cos(ANGLE)) + ux * sin(ANGLE);
+      rot[2][2] = cos (ANGLE) + uz*uz*(1.-cos(ANGLE));
+    }
+    else {
+      Msg::Error("Only rotations or translations can be currently taken into account "
+                 "for peridic faces: face %d not meshed", target->tag());
+      return;
+    }
+  }
+  else{
+    Msg::Info("Periodic mesh translation found: dx = (%g,%g,%g)",
+              DX.x(), DX.y(), DX.z());
+  }
 
+  // now transform !!!
   for(unsigned int i = 0; i < source->mesh_vertices.size(); i++){
     MVertex *vs = source->mesh_vertices[i];
-    double u, v;
-    vs->getParameter(0, u);
-    vs->getParameter(1, v);
-    // apply transformation
-    const double U =   c * (u - s1u) + s * (v - s1v) + t1u;
-    const double V =  -s * (u - s1u) + c * (v - s1v) + t1v;
-    GPoint gp = target->point(SPoint2(U, V));
-    MVertex *vt = new MFaceVertex(gp.x(), gp.y(), gp.z(), target, U, V);
+
+    SPoint2 XXX;
+    if (translation) {
+      SPoint3 tp (vs->x() + DX.x(),vs->y() + DX.y(),vs->z() + DX.z());
+      XXX = target->parFromPoint(tp);
+    }
+    else if (rotation){
+      SPoint3 ps = SPoint3(vs->x(),vs->y(),vs->z());
+      SPoint3 p_ps = LINE.orthogonalProjection(ps);
+      SPoint3 P = ps - p_ps, res;
+      matvec(rot, P, res);
+      res += p_ps;
+      XXX = target->parFromPoint(res);
+    }
+    GPoint gp = target->point(XXX);
+    MVertex *vt = new MFaceVertex(gp.x(), gp.y(), gp.z(), target, gp.u(), gp.v());
     target->mesh_vertices.push_back(vt);
+    target->correspondingVertices[vt] = vs;
     vs2vt[vs] = vt;
   }
 
@@ -136,27 +290,9 @@ static void copyMesh(GFace *source, GFace *target)
     for (int j = 0; j < 3; j++){
       MVertex *vs = source->triangles[i]->getVertex(j);
       vt[j] = vs2vt[vs];
-      if (!vt[j]){
-	SPoint2 p;
-	reparamMeshVertexOnFace(vs, source, p);
-	const double U =   c * (p.x()-s1u) + s * (p.y()-s1v) + t1u;
-	const double V =  -s * (p.x()-s1u) + c * (p.y()-s1v) + t1v;
-	for (std::list<GEdge*>::iterator it = edges.begin(); it != edges.end(); ++it){
-	  GEdge *te = *it;
-	  for (unsigned k = 0; k < te->lines.size(); k++){
-	    MVertex *gotcha = te->lines[k]->getVertex(0);
-	    reparamMeshVertexOnFace(gotcha, target, p);
-	    const double D = sqrt((U - p.x()) * (U - p.x()) + (V - p.y()) * (V - p.y()));
-	    if (D < 1.e-9){
-	      vt[j] = gotcha;
-	      break;
-	    }
-	  }
-	}
-      }
     }
     if (!vt[0] || !vt[1] ||!vt[2]){
-      Msg::Fatal("yet another error in the copymesh procedure %p %p %p %d %d %d",
+      Msg::Fatal("Yet another error in the copyMesh procedure %p %p %p %d %d %d",
 		 vt[0], vt[1], vt[2], source->triangles[i]->getVertex(0)->onWhat()->dim(),
 		 source->triangles[i]->getVertex(1)->onWhat()->dim(),
 		 source->triangles[i]->getVertex(2)->onWhat()->dim());
@@ -169,6 +305,14 @@ static void copyMesh(GFace *source, GFace *target)
     MVertex *v2 = vs2vt[source->quadrangles[i]->getVertex(1)];
     MVertex *v3 = vs2vt[source->quadrangles[i]->getVertex(2)];
     MVertex *v4 = vs2vt[source->quadrangles[i]->getVertex(3)];
+    if (!v1 || !v2 || !v3 || !v4){
+      Msg::Fatal("Yet another error in the copymesh procedure %p %p %p %p %d %d %d %d",
+		 v1, v2, v3, v4,
+		 source->quadrangles[i]->getVertex(0)->onWhat()->dim(),
+		 source->quadrangles[i]->getVertex(1)->onWhat()->dim(),
+		 source->quadrangles[i]->getVertex(2)->onWhat()->dim(),
+		 source->quadrangles[i]->getVertex(3)->onWhat()->dim());
+    }
     target->quadrangles.push_back(new MQuadrangle(v1, v2, v3, v4));
   }
 }
@@ -297,14 +441,15 @@ static void remeshUnrecoveredEdges(std::map<MVertex*, BDS_Point*> &recoverMapInv
 
 static bool algoDelaunay2D(GFace *gf)
 {
-
-  if(!noSeam(gf))
-    return false;
+  // FIXME
+  //  if(!noSeam(gf))
+  //    return false;
 
   if(gf->getMeshingAlgo() == ALGO_2D_DELAUNAY ||
      gf->getMeshingAlgo() == ALGO_2D_BAMG ||
      gf->getMeshingAlgo() == ALGO_2D_FRONTAL ||
      gf->getMeshingAlgo() == ALGO_2D_FRONTAL_QUAD ||
+     gf->getMeshingAlgo() == ALGO_2D_PACK_PRLGRMS ||
      gf->getMeshingAlgo() == ALGO_2D_BAMG)
     return true;
 
@@ -480,7 +625,7 @@ void filterOverlappingElements(int dim, std::vector<MElement*> &e,
 
 void modifyInitialMeshForTakingIntoAccountBoundaryLayers(GFace *gf)
 {
-  BoundaryLayerColumns *_columns = buidAdditionalPoints2D (gf);
+  BoundaryLayerColumns *_columns = buildAdditionalPoints2D (gf);
 
   if (!_columns)return;
 
@@ -765,6 +910,7 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
         (double)RAND_MAX;
       double YY = CTX::instance()->mesh.randFactor * LC2D * (double)rand() /
         (double)RAND_MAX;
+      //      printf("%22.15E %22.15E \n",XX,YY);
       doc.points[i].where.h = points[i]->u + XX;
       doc.points[i].where.v = points[i]->v + YY;
       doc.points[i].data = points[i];
@@ -1090,36 +1236,42 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
   Msg::Debug("Starting to add internal points");
   // start mesh generation
   if(!algoDelaunay2D(gf) && !onlyInitialMesh){
-    //    if(CTX::instance()->mesh.recombineAll || gf->meshAttributes.recombine || 1) {
-    //      printf("coucou here !!!\n");
-    //      backgroundMesh::unset();
-    //      buildBackGroundMesh (gf);
-    //    }
+       // if(CTX::instance()->mesh.recombineAll || gf->meshAttributes.recombine || 1) {
+       //   backgroundMesh::unset();
+       //   buildBackGroundMesh (gf);
+       // }
     refineMeshBDS(gf, *m, CTX::instance()->mesh.refineSteps, true,
                   &recoverMapInv);
     optimizeMeshBDS(gf, *m, 2);
     refineMeshBDS(gf, *m, CTX::instance()->mesh.refineSteps, false,
                 &recoverMapInv);
     optimizeMeshBDS(gf, *m, 2);
-    //    if(CTX::instance()->mesh.recombineAll || gf->meshAttributes.recombine || 1) {
-    //      backgroundMesh::unset();
-    //    }
+       // if(CTX::instance()->mesh.recombineAll || gf->meshAttributes.recombine || 1) {
+       //   backgroundMesh::unset();
+       // }
   }
+
   /*
   computeMeshSizeFieldAccuracy(gf, *m, gf->meshStatistics.efficiency_index,
-                               gf->meshStatistics.longest_edge_length,
-                               gf->meshStatistics.smallest_edge_length,
-                               gf->meshStatistics.nbEdge,
-                               gf->meshStatistics.nbGoodLength);
+			       gf->meshStatistics.longest_edge_length,
+			       gf->meshStatistics.smallest_edge_length,
+			       gf->meshStatistics.nbEdge,
+			       gf->meshStatistics.nbGoodLength);
   */
+  //printf("=== Efficiency index is tau=%g\n", gf->meshStatistics.efficiency_index);
+
   gf->meshStatistics.status = GFace::DONE;
 
   // fill the small gmsh structures
   BDS2GMSH(m, gf, recoverMap);
 
-  // BOUNDARY LAYER
+  bool infty = false;
+  if (gf->getMeshingAlgo() == ALGO_2D_FRONTAL_QUAD || gf->getMeshingAlgo() == ALGO_2D_PACK_PRLGRMS)
+    infty = true;
   if (!onlyInitialMesh) {
-    if (gf->getMeshingAlgo() == ALGO_2D_FRONTAL_QUAD) buildBackGroundMesh (gf);
+    if (infty)
+      buildBackGroundMesh (gf);
+    // BOUNDARY LAYER
     modifyInitialMeshForTakingIntoAccountBoundaryLayers(gf);
   }
 
@@ -1132,6 +1284,9 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
     else if(gf->getMeshingAlgo() == ALGO_2D_FRONTAL_QUAD){
       bowyerWatsonFrontalLayers(gf,true);
     }
+    else if(gf->getMeshingAlgo() == ALGO_2D_PACK_PRLGRMS){
+      bowyerWatsonParallelograms(gf);
+    }
     else if(gf->getMeshingAlgo() == ALGO_2D_DELAUNAY ||
             gf->getMeshingAlgo() == ALGO_2D_AUTO)
       bowyerWatson(gf);
@@ -1139,7 +1294,8 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
       bowyerWatson(gf,15000);
       meshGFaceBamg(gf);
     }
-    laplaceSmoothing(gf, CTX::instance()->mesh.nbSmoothing);
+    if (!infty || !(CTX::instance()->mesh.recombineAll || gf->meshAttributes.recombine))
+      laplaceSmoothing(gf, CTX::instance()->mesh.nbSmoothing, infty);
   }
 
   if(debug){
@@ -1152,6 +1308,17 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
   if(CTX::instance()->mesh.remove4triangles)
     removeFourTrianglesNodes(gf,false);
 
+  //Emi print efficiency index
+  /*
+  gf->computeMeshSizeFieldAccuracy(gf->meshStatistics.efficiency_index,
+  				   gf->meshStatistics.longest_edge_length,
+  				   gf->meshStatistics.smallest_edge_length,
+  				   gf->meshStatistics.nbEdge,
+  				   gf->meshStatistics.nbGoodLength);
+  */
+  //printf("----- Efficiency index is tau=%g\n", gf->meshStatistics.efficiency_index);
+
+
   // delete the mesh
   delete m;
 
@@ -1170,7 +1337,7 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
 			      gf->additionalVertices.end());
   gf->additionalVertices.clear();
 
-  return true;
+   return true;
 }
 
 // this function buils a list of vertices (BDS) that are consecutive
@@ -1421,6 +1588,32 @@ static bool buildConsecutiveListOfVertices(GFace *gf, GEdgeLoop &gel,
   return true;
 }
 
+static bool meshGeneratorElliptic(GFace *gf, bool debug = true)
+{
+#if defined(HAVE_ANN)
+  Centerline *center = 0;
+  FieldManager *fields = GModel::current()->getFields();
+  if (fields->getBackgroundField() > 0 ){
+    Field *myField = fields->get(fields->getBackgroundField());
+    center = dynamic_cast<Centerline*> (myField);
+  }
+
+  bool recombine =  (CTX::instance()->mesh.recombineAll);
+  int nbBoundaries = gf->edges().size();
+
+  if (center && recombine && nbBoundaries == 2) {
+    printf("--> regular periodic grid generator (elliptic smooth) \n");
+    //bool success  = createRegularTwoCircleGrid(center, gf);
+    bool success  = createRegularTwoCircleGridPeriodic(center, gf);
+    return success;
+  }
+  else return false;
+
+#else
+  return false;
+#endif
+}
+
 static bool meshGeneratorPeriodic(GFace *gf, bool debug = true)
 {
   std::map<BDS_Point*, MVertex*> recoverMap;
@@ -1680,6 +1873,7 @@ static bool meshGeneratorPeriodic(GFace *gf, bool debug = true)
     outputScalarField(m->triangles, name, 1);
   }
 
+
   // start mesh generation for periodic face
   if(!algoDelaunay2D(gf)){
     // need for a BGM for cross field
@@ -1700,11 +1894,55 @@ static bool meshGeneratorPeriodic(GFace *gf, bool debug = true)
                                  gf->meshStatistics.nbEdge,
                                  gf->meshStatistics.nbGoodLength);*/
     gf->meshStatistics.status = GFace::DONE;
+
     //    if(CTX::instance()->mesh.recombineAll || gf->meshAttributes.recombine || 1) {
     //            backgroundMesh::unset();
     //    }
   }
 
+  // This is a structure that we need only for periodic cases
+  // We will duplicate the vertices (MVertex) that are on seams
+
+  std::map<MVertex*, MVertex*> equivalence;
+  std::map<MVertex*, SPoint2> parametricCoordinates;
+  if(algoDelaunay2D(gf)){
+    std::map<MVertex*, BDS_Point*> invertMap;
+    std::map<BDS_Point*, MVertex*>::iterator it = recoverMap.begin();
+    while(it != recoverMap.end()){
+      // we have twice vertex MVertex with 2 different coordinates
+      MVertex  * mv1  =  it->second;
+      BDS_Point* bds = it->first;
+      std::map<MVertex*, BDS_Point*>::iterator invIt = invertMap.find(mv1);
+      if (invIt != invertMap.end()){
+	// create a new "fake" vertex that will be destroyed afterwards
+	MVertex  * mv2 ;
+	if (mv1->onWhat()->dim() == 1) {
+	  double t;
+	  mv1->getParameter(0,t);
+	  mv2  = new MEdgeVertex (mv1->x(),mv1->y(),mv1->z(),mv1->onWhat(), t,
+                                  ((MEdgeVertex*)mv1)->getLc());
+	}
+	else if (mv1->onWhat()->dim() == 0) {
+	  mv2  = new MVertex (mv1->x(),mv1->y(),mv1->z(),mv1->onWhat());
+	}
+	else
+	  Msg::Error("error in seam reconstruction");
+
+	it->second = mv2;
+	equivalence[mv2] = mv1;
+	parametricCoordinates[mv2] = SPoint2(bds->u,bds->v);
+	invertMap[mv2] = bds;
+      }
+      else {
+	parametricCoordinates[mv1] = SPoint2(bds->u,bds->v);
+	invertMap[mv1] = bds;
+      }
+      ++it;
+    }
+    //    recoverMap.insert(new_relations.begin(), new_relations.end());
+  }
+  Msg::Info("%d points that are duplicated for delaunay meshing",equivalence.size());
+
   // fill the small gmsh structures
   {
     std::set<BDS_Point*, PointLessThan>::iterator itp = m->points.begin();
@@ -1720,6 +1958,7 @@ static bool meshGeneratorPeriodic(GFace *gf, bool debug = true)
     }
   }
 
+  std::map<MTriangle*, BDS_Face*> invert_map;
   {
     std::list<BDS_Face*>::iterator itt = m->triangles.begin();
     while (itt != m->triangles.end()){
@@ -1734,8 +1973,12 @@ static bool meshGeneratorPeriodic(GFace *gf, bool debug = true)
           // when a singular point is present, degenerated triangles
           // may be created, for example on a sphere that contains one
           // pole
-          if(v1 != v2 && v1 != v3 && v2 != v3)
+          if(v1 != v2 && v1 != v3 && v2 != v3){
+	    // we are in the periodic case. if we aim at
+	    // using delaunay mesh generation in thoses cases,
+	    // we should double some of the vertices
             gf->triangles.push_back(new MTriangle(v1, v2, v3));
+	  }
         }
         else{
           MVertex *v4 = recoverMap[n[3]];
@@ -1754,17 +1997,30 @@ static bool meshGeneratorPeriodic(GFace *gf, bool debug = true)
     outputScalarField(m->triangles, name, 1);
   }
 
+  bool infty = false;
+  if (gf->getMeshingAlgo() == ALGO_2D_FRONTAL_QUAD ||
+      gf->getMeshingAlgo() == ALGO_2D_PACK_PRLGRMS)
+    infty = true;
+  if (infty)
+    buildBackGroundMesh (gf, &equivalence, &parametricCoordinates);
+  // BOUNDARY LAYER
+  modifyInitialMeshForTakingIntoAccountBoundaryLayers(gf);
+
+
   if(algoDelaunay2D(gf)){
     if(gf->getMeshingAlgo() == ALGO_2D_FRONTAL)
-      bowyerWatsonFrontal(gf);
+      bowyerWatsonFrontal(gf, &equivalence, &parametricCoordinates);
     else if(gf->getMeshingAlgo() == ALGO_2D_FRONTAL_QUAD)
-      bowyerWatsonFrontalLayers(gf,true);
+      bowyerWatsonFrontalLayers(gf,true, &equivalence, &parametricCoordinates);
+    else if(gf->getMeshingAlgo() == ALGO_2D_PACK_PRLGRMS)
+      bowyerWatsonParallelograms(gf,&equivalence, &parametricCoordinates);
     else if(gf->getMeshingAlgo() == ALGO_2D_DELAUNAY ||
             gf->getMeshingAlgo() == ALGO_2D_AUTO)
-      bowyerWatson(gf);
+      bowyerWatson(gf,1000000000, &equivalence, &parametricCoordinates);
     else
       meshGFaceBamg(gf);
-    laplaceSmoothing(gf,CTX::instance()->mesh.nbSmoothing);
+    if (!infty || !(CTX::instance()->mesh.recombineAll || gf->meshAttributes.recombine))
+      laplaceSmoothing(gf, CTX::instance()->mesh.nbSmoothing, infty);
   }
 
   // delete the mesh
@@ -1779,6 +2035,7 @@ static bool meshGeneratorPeriodic(GFace *gf, bool debug = true)
                        gf->meshStatistics.best_element_shape,
                        gf->meshStatistics.nbTriangle,
                        gf->meshStatistics.nbGoodQuality);
+  gf->meshStatistics.status = GFace::DONE;
   return true;
 }
 
@@ -1788,6 +2045,7 @@ void deMeshGFace::operator() (GFace *gf)
   gf->deleteMesh();
   gf->meshStatistics.status = GFace::PENDING;
   gf->meshStatistics.nbTriangle = gf->meshStatistics.nbEdge = 0;
+  gf->correspondingVertices.clear();
 }
 
 // for debugging, change value from -1 to -100;
@@ -1840,6 +2098,7 @@ void meshGFace::operator() (GFace *gf, bool print)
   case ALGO_2D_DELAUNAY : algo = "Delaunay"; break;
   case ALGO_2D_MESHADAPT_OLD : algo = "MeshAdapt (old)"; break;
   case ALGO_2D_BAMG : algo = "Bamg"; break;
+  case ALGO_2D_PACK_PRLGRMS : algo = "Square Packing"; break;
   case ALGO_2D_AUTO :
     algo = (gf->geomType() == GEntity::Plane) ? "Delaunay" : "MeshAdapt";
     break;
@@ -1857,6 +2116,12 @@ void meshGFace::operator() (GFace *gf, bool print)
   Msg::Debug("Computing edge loops");
 
   Msg::Debug("Generating the mesh");
+
+  if(meshGeneratorElliptic(gf)){
+    gf->meshStatistics.status = GFace::DONE;
+    return;
+  }
+
   if ((gf->getNativeType() != GEntity::AcisModel ||
        (!gf->periodic(0) && !gf->periodic(1))) &&
       (noSeam(gf) || gf->getNativeType() == GEntity::GmshModel ||
@@ -2041,6 +2306,16 @@ void partitionAndRemesh(GFaceCompound *gf)
       gf->quadrangles.push_back(new MQuadrangle(v[0], v[1], v[2], v[3]));
     }
 
+    //update mesh statistics
+    gf->meshStatistics.efficiency_index += gfc->meshStatistics.efficiency_index;
+    gf->meshStatistics.longest_edge_length = std::max(gf->meshStatistics.longest_edge_length,
+						     gfc->meshStatistics.longest_edge_length);
+    gf->meshStatistics.smallest_edge_length= std::min(gf->meshStatistics.smallest_edge_length,
+						       gfc->meshStatistics.smallest_edge_length);
+    gf->meshStatistics.nbGoodLength  += gfc->meshStatistics.nbGoodLength;
+    gf->meshStatistics.nbGoodQuality += gfc->meshStatistics.nbGoodQuality;
+    gf->meshStatistics.nbEdge += gfc->meshStatistics.nbEdge;
+
   }
 
   // Removing discrete Vertices - Edges - Faces
