@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include <stdlib.h>
 #include "GmshMessage.h"
@@ -39,13 +39,12 @@ inline double computeEdgeLinearLength(BDS_Point *p1, BDS_Point *p2, GFace *f,
                                       double SCALINGU, double SCALINGV)
 {
   // FIXME SUPER HACK
-
-  //  if (CTX::instance()->mesh.recombineAll || f->meshAttributes.recombine || 1){
-  //    double quadAngle  = backgroundMesh::current()->getAngle (0.5 * (p1->u + p2->u) * SCALINGU, 0.5 * (p1->v + p2->v) * SCALINGV,0);
-  //    const double a [2] = {p1->u,p1->v};
-  //    const double b [2] = {p2->u,p2->v};
-  //    return lengthInfniteNorm(a,b, quadAngle);
-  //  }
+   // if (CTX::instance()->mesh.recombineAll || f->meshAttributes.recombine || 1){
+   //   double quadAngle  = backgroundMesh::current()->getAngle (0.5 * (p1->u + p2->u) * SCALINGU, 0.5 * (p1->v + p2->v) * SCALINGV,0);
+   //   const double a [2] = {p1->u,p1->v};
+   //   const double b [2] = {p2->u,p2->v};
+   //   return lengthInfniteNorm(a,b, quadAngle);
+   // }
 
   GPoint GP = f->point(SPoint2(0.5 * (p1->u + p2->u) * SCALINGU,
                                0.5 * (p1->v + p2->v) * SCALINGV));
@@ -53,7 +52,6 @@ inline double computeEdgeLinearLength(BDS_Point *p1, BDS_Point *p2, GFace *f,
   if (!GP.succeeded())
     return computeEdgeLinearLength(p1,p2);
 
-
   const double dx1 = p1->X - GP.x();
   const double dy1 = p1->Y - GP.y();
   const double dz1 = p1->Z - GP.z();
@@ -102,33 +100,6 @@ inline double computeEdgeLinearLength_new(BDS_Point *p1, BDS_Point *p2,  GFace *
   return l;
 }
 
-inline double computeEdgeMiddleCoord_new(BDS_Point *p1, BDS_Point *p2, GFace *f,
-                                         double SCALINGU, double SCALINGV)
-{
-  const int nbSb = 3;
-  double L = computeEdgeLinearLength(p1,p2);
-  GPoint GP[nbSb];
-  for (int i = 1; i < nbSb; i++){
-    double xi = (double)i / nbSb;
-    GP[i-1] = f->point(SPoint2(((1-xi) * p1->u + xi * p2->u) * SCALINGU,
-                               ((1-xi) * p1->v + xi * p2->v) * SCALINGV));
-    if (!GP[i-1].succeeded())
-      return 0.5;
-    const double dx1 = p1->X - GP[i-1].x();
-    const double dy1 = p1->Y - GP[i-1].y();
-    const double dz1 = p1->Z - GP[i-1].z();
-    double LPLUS = sqrt(dx1 * dx1 + dy1 * dy1 + dz1 * dz1);
-    if (LPLUS > L*.5){
-      double XIMINUS, LPLUS, LMINUS;
-      if (i==1){
-        XIMINUS=0;
-      }
-      return  XIMINUS +  (LPLUS - L*.5)/(LPLUS-LMINUS)/(nbSb-1);
-    }
-  }
-  return 0.5;
-}
-
 inline double computeEdgeMiddleCoord(BDS_Point *p1, BDS_Point *p2, GFace *f,
                                      double SCALINGU, double SCALINGV)
 {
@@ -165,6 +136,7 @@ inline double computeEdgeLinearLength(BDS_Edge *e, GFace *f,
 
 double NewGetLc(BDS_Point *p)
 {
+
   return Extend1dMeshIn2dSurfaces() ?
     std::min(p->lc(), p->lcBGM()) : p->lcBGM();
 }
@@ -194,6 +166,7 @@ double NewGetLc(BDS_Edge *e, GFace *f, double SCALINGU, double SCALINGV)
 {
   double linearLength = computeEdgeLinearLength(e, f, SCALINGU, SCALINGV);
   double l = correctLC_ (e->p1,e->p2,f, SCALINGU, SCALINGV);
+  //printf("BDS correct lc =%g lreal=%g \n", l,linearLength);
   return linearLength / l;
 }
 
@@ -207,8 +180,9 @@ double NewGetLc(BDS_Point *p1, BDS_Point *p2, GFace *f, double su, double sv)
 void computeMeshSizeFieldAccuracy(GFace *gf, BDS_Mesh &m, double &avg,
                                   double &max_e, double &min_e, int &nE, int &GS)
 {
+
   std::list<BDS_Edge*>::iterator it = m.edges.begin();
-  avg = 0;
+  avg = 0.0;
   min_e = 1.e22;
   max_e = 0;
   nE = 0;
@@ -226,6 +200,7 @@ void computeMeshSizeFieldAccuracy(GFace *gf, BDS_Mesh &m, double &avg,
     }
     ++it;
   }
+  avg = 100*exp(1./nE*avg);
 }
 
 // SWAP TESTS i.e. tell if swap should be done
