@@ -1,11 +1,12 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #ifndef _MESH_METRIC_H_
 #define _MESH_METRIC_H_
 
+
 #include <map>
 #include <algorithm>
 #include "STensor3.h"
@@ -25,7 +26,7 @@ class meshMetric: public Field {
                 ISOTROPIC_LINEARINTERP_H=6, SCALED_HESSIAN=7} MetricComputationTechnique;
  private:
   // intersect all metrics added in "setOfMetrics", preserve eigendirections of the "most anisotropic" metric
-  void intersectMetrics();
+  void updateMetrics();
   int _dim;
   double _epsilon, _E, _E_moins, _Np;
   bool needMetricUpdate;
@@ -41,15 +42,16 @@ class meshMetric: public Field {
   std::map<MVertex*,double> vals;
   std::map<MVertex*,SVector3> grads,dgrads[3];
 
+ public:
   typedef std::map<MVertex*,SMetric3> nodalMetricTensor;
   typedef std::map<MVertex*,double> nodalField;
-
+ private:
   nodalMetricTensor _nodalMetrics,_hessian;
   nodalField _nodalSizes, _detMetric;
 
   std::map<int,nodalMetricTensor> setOfMetrics;
   std::map<int,nodalField> setOfSizes;
-  std::map<int,nodalField> setOfDetMetric;
+  //  std::map<int,nodalField> setOfDetMetric;
 
  public:
   meshMetric(std::vector<MElement*> elements);
@@ -78,9 +80,13 @@ class meshMetric: public Field {
   void addMetric(int technique, simpleFunction<double> *fct, std::vector<double> parameters);
 
   inline SMetric3 metricAtVertex (MVertex* v) {
-    if (needMetricUpdate) intersectMetrics();
+    if (needMetricUpdate) updateMetrics();
     return _nodalMetrics[v];
   }
+  // this function scales the mesh metric in order 
+  // to reach a target number of elements 
+  void scaleMetric( int nbElementsTarget, 
+		    nodalMetricTensor &nmt );
 
   void computeMetric();
   void computeMetricLevelSet();
