@@ -1,7 +1,10 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
+//
+// Contributor(s):
+//   Tristan Carrier François Henrotte
 
 #include "SVector3.h"
 #include <list>
@@ -15,13 +18,13 @@ class Filler{
  private:
   static std::vector<MVertex*> new_vertices;
   Metric get_metric(double,double,double);
+  Metric get_metric(double,double,double,GEntity*);
   double get_size(double,double,double);
-  Metric get_clf_metric(double,double,double,GEntity*);
-  double get_clf_size(double,double,double,GEntity*);
+  double get_size(double,double,double,GEntity*);
   bool inside_domain(MElementOctree*,double,double,double);
   bool far_from_boundary(MElementOctree*,Node*);
   void compute_parameters(Node*,GEntity*);
-  void offsprings(GEntity*,MElementOctree*,Node*,Node*,Node*,Node*,Node*,Node*,Node*);
+  void create_spawns(GEntity*,MElementOctree*,Node*,std::vector<Node*>&);
   double improvement(GEntity*,MElementOctree*,SPoint3,double,SVector3);
   void print_segment(SPoint3,SPoint3,std::ofstream&);
   void print_node(Node*,std::ofstream&);
