@@ -1,14 +1,12 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include "GmshConfig.h"
 #include "GmshMessage.h"
 #include "Numeric.h"
 
-#define SQU(a)      ((a)*(a))
-
 double myatan2(double a, double b)
 {
   if(a == 0.0 && b == 0)
@@ -35,6 +33,19 @@ double myacos(double a)
   else
     return acos(a);
 }
+double norm2(double a[3][3]) {
+  double norm2sq =
+    SQU(a[0][0])+
+    SQU(a[0][1])+
+    SQU(a[0][2])+
+    SQU(a[1][0])+
+    SQU(a[1][1])+
+    SQU(a[1][2])+
+    SQU(a[2][0])+
+    SQU(a[2][1])+
+    SQU(a[2][2]);
+  return sqrt(norm2sq);
+}
 
 void matvec(double mat[3][3], double vec[3], double res[3])
 {
@@ -778,12 +789,12 @@ double minimize_grad_fd(double (*func)(fullVector<double> &, void *),
 
   fullVector<double> grad(N);
   fullVector<double> dir(N);
-  double f, feps, finit;
+  double f, feps;//, finit;
 
   for (int iter = 0; iter < MAXIT; iter++){
     // compute gradient of func
     f = func(x, data);
-    if (iter == 0) finit = f;
+    //if (iter == 0) finit = f;
     // printf("Opti iter %d x = (%g %g) f = %g\n",iter,x(0),x(1),f);
     // printf("grad = (");
     for (int j = 0; j < N; j++){
@@ -886,22 +897,22 @@ void signedDistancesPointsTriangle(std::vector<double> &distances,
       SVector3 pp2 = p - p2;
       const double t23 = dot(pp2, t3) / n2t3;
       d = 1.e10;
-      bool found = false;
+      //bool found = false;
       SPoint3 closePt;
       if (t12 >= 0 && t12 <= 1.){
         d = sign * std::min(fabs(d), p.distance(p1 + (p2 - p1) * t12));
         closePt = p1 + (p2 - p1) * t12;
-        found = true;
+        //found = true;
       }
       if (t13 >= 0 && t13 <= 1.){
         if (p.distance(p1 + (p3 - p1) * t13) < fabs(d)) closePt = p1 + (p3 - p1) * t13;
         d = sign * std::min(fabs(d), p.distance(p1 + (p3 - p1) * t13));
-        found = true;
+        //found = true;
       }
       if (t23 >= 0 && t23 <= 1.){
         if (p.distance(p2 + (p3 - p2) * t23) < fabs(d)) closePt = p2 + (p3 - p2) * t23;
         d = sign * std::min(fabs(d), p.distance(p2 + (p3 - p2) * t23));
-        found = true;
+        //found = true;
       }
       if (p.distance(p1) < fabs(d)){
         closePt = p1;
@@ -1238,7 +1249,6 @@ int intersection_segments(SPoint3 &p1, SPoint3 &p2,
 
 void computeMeanPlaneSimple(const std::vector<SPoint3> &points, mean_plane &meanPlane)
 {
-
   double xm = 0., ym = 0., zm = 0.;
   int ndata = points.size();
   int na = 3;
@@ -1273,8 +1283,9 @@ void computeMeanPlaneSimple(const std::vector<SPoint3> &points, mean_plane &mean
   res[0] = V(0, min);
   res[1] = V(1, min);
   res[2] = V(2, min);
-  norme(res);
 
+  double xxx = norme(res);
+  res[3] /= xxx;
   double ex[3], t1[3], t2[3];
 
   ex[0] = ex[1] = ex[2] = 0.0;
@@ -1302,7 +1313,7 @@ void computeMeanPlaneSimple(const std::vector<SPoint3> &points, mean_plane &mean
   meanPlane.a = res[0];
   meanPlane.b = res[1];
   meanPlane.c = res[2];
-  meanPlane.d = -res[3];//BUG HERE
+  meanPlane.d = res[3];//BUG HERE
 
   meanPlane.x = meanPlane.y = meanPlane.z = 0.;
   if(fabs(meanPlane.a) >= fabs(meanPlane.b) &&
