@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #ifndef JACOBIPOLYNOMIALS_H
 #define JACOBIPOLYNOMIALS_H
@@ -11,6 +11,7 @@
 class JacobiPolynomials {
 
  public:
+  JacobiPolynomials() {};
   JacobiPolynomials(double a, double b, int o);
   ~JacobiPolynomials();
 
