@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 //
 // Contributed by Matti Pellikka <matti.pellikka@tut.fi>.
 
@@ -26,6 +26,7 @@ StringXString HomologyPostProcessingOptions_String[] = {
   {GMSH_FULLRC, "PhysicalGroupsOfOperatedChains", NULL, "1, 2"},
   {GMSH_FULLRC, "PhysicalGroupsOfOperatedChains2", NULL, ""},
   {GMSH_FULLRC, "PhysicalGroupsToTraceResults", NULL, ""},
+  {GMSH_FULLRC, "PhysicalGroupsToProjectResults", NULL, ""},
   {GMSH_FULLRC, "NameForResultChains", NULL, "c"},
 };
 
@@ -40,13 +41,25 @@ extern "C"
 std::string GMSH_HomologyPostProcessingPlugin::getHelp() const
 {
   return "Plugin(HomologyPostProcessing) operates on representative "
-    "basis chains of homology and cohomology spaces.\n\n"
+    "basis chains of homology and cohomology spaces. Functionality:\n\n"
+
+    "1. (co)homology basis transformation:\n "
+    "'TransformationMatrix': Integer matrix of the transformation.\n "
+    "'PhysicalGroupsOfOperatedChains': (Co)chains of a (co)homology space basis to be transformed.\n "
+    "Results a new (co)chain basis that is an integer cobination of the given basis. \n\n"
+
+    "2. Make basis representations of a homology space and a cohomology space "
+    "compatible: \n"
+    "'PhysicalGroupsOfOperatedChains': Chains of a homology space basis.\n"
+    "'PhysicalGroupsOfOperatedChains2': Cochains of a cohomology space basis.\n"
+    "Results a new basis for the homology space such that the incidence matrix of the new basis and the basis of the cohomology space is the identity matrix.\n\n"
+
+    "Options:\n"
+    "'PhysicalGroupsToTraceResults': Trace the resulting (co)chains to the given physical groups.\n"
+    "'PhysicalGroupsToProjectResults': Project the resulting (co)chains to the complement of the given physical groups.\n"
+    "'NameForResultChains': Post-processing view name prefix for the results.\n"
+    "'ApplyBoundaryOperatorToResults': Apply boundary operator to the resulting chains.\n";
 
-    "Define the physical groups of chains and their dimension "
-    "to be transformed by the "
-    "given integer transformation matrix. Number of matrix columns must "
-    "equal to the number of chains given. The resulting "
-    "chains are thus integer combinations of the given chains. ";
 }
 
 int GMSH_HomologyPostProcessingPlugin::getNbOptions() const
@@ -128,8 +141,9 @@ PView *GMSH_HomologyPostProcessingPlugin::execute(PView *v)
   std::string matrixString = HomologyPostProcessingOptions_String[0].def;
   std::string opString1 = HomologyPostProcessingOptions_String[1].def;
   std::string opString2 = HomologyPostProcessingOptions_String[2].def;
-  std::string cname = HomologyPostProcessingOptions_String[4].def;
+  std::string cname = HomologyPostProcessingOptions_String[5].def;
   std::string traceString = HomologyPostProcessingOptions_String[3].def;
+  std::string projectString = HomologyPostProcessingOptions_String[4].def;
   int bd = (int)HomologyPostProcessingOptions_Number[0].def;
 
   GModel* m = GModel::current();
@@ -205,6 +219,8 @@ PView *GMSH_HomologyPostProcessingPlugin::execute(PView *v)
 
   std::vector<int> tracePhysicals;
   if(!parseStringOpt(3, tracePhysicals)) return 0;
+  std::vector<int> projectPhysicals;
+  if(!parseStringOpt(4, projectPhysicals)) return 0;
 
   std::vector<Chain<int> > curBasis;
   for(unsigned int i = 0; i < basisPhysicals.size(); i++){
@@ -274,14 +290,19 @@ PView *GMSH_HomologyPostProcessingPlugin::execute(PView *v)
               dim, traceString.c_str());
     for(unsigned int i = 0; i < newBasis.size(); i++)
       newBasis.at(i) = newBasis.at(i).getTrace(m, tracePhysicals);
-    ElemChain::clearVertexCache();
   }
+  if(!projectPhysicals.empty()) {
+    Msg::Info("Taking projection of result %d-chains to the complement of the domain %s", dim, projectString.c_str());
+    for(unsigned int i = 0; i < newBasis.size(); i++)
+      newBasis.at(i) = newBasis.at(i).getProject(m, projectPhysicals);
+  }
+  if(!tracePhysicals.empty() || !projectPhysicals.empty())
+    ElemChain::clearVertexCache();
+
 
-  std::string dims = "";
-  std::string nums = "";
   for(unsigned int i = 0; i < newBasis.size(); i++) {
-    convert(newBasis.at(i).getDim(), dims);
-    convert(i+1, nums);
+    std::string dims = convertInt(newBasis.at(i).getDim());
+    std::string nums = convertInt(i+1);
     newBasis.at(i).setName("C" + dims + " " + cname + nums);
     newBasis.at(i).addToModel(m);
   }
