@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include <string.h>
 #include <algorithm>
@@ -14,42 +14,55 @@
 #include "adaptiveData.h"
 #include "GmshMessage.h"
 
-int PView::_globalNum = 0;
+int PView::_globalTag = 0;
 std::vector<PView*> PView::list;
 
-void PView::_init(int num)
+void PView::_init(int tag)
 {
-  if(num){
-    _num = num;
-    _globalNum = std::max(_globalNum, _num);
+  if(tag >= 0){
+    _tag = tag;
+    _globalTag = std::max(_globalTag, _tag) + 1;
   }
   else{
-    _num = ++_globalNum;
+    _tag = _globalTag++;
   }
+
   _changed = true;
-  _aliasOf = 0;
+  _aliasOf = -1;
   _eye = SPoint3(0., 0., 0.);
   va_points = va_lines = va_triangles = va_vectors = va_ellipses = 0;
   normals = 0;
+
+  for(unsigned int i = 0; i < list.size(); i++){
+    if(list[i]->getTag() == _tag){
+      // in normal operation this should not happen, but we allow it when
+      // programmatically forcing view tags (e.g. when using the views from
+      // within getdp's post-processing operations); this is dangerous, as it
+      // breaks aliases
+      Msg::Info("Removing existing View[%d] (tag = %d)", i, _tag);
+      delete list[i]; // warning: this changes the list
+    }
+  }
+
   list.push_back(this);
   for(unsigned int i = 0; i < list.size(); i++) list[i]->setIndex(i);
 }
 
-PView::PView(int num)
+PView::PView(int tag)
 {
-  _init(num);
+  _init(tag);
   _data = new PViewDataList();
-  _options = new PViewOptions(PViewOptions::reference);
+  _options = new PViewOptions(*PViewOptions::reference());
   if(_options->adaptVisualizationGrid)
     _data->initAdaptiveData(_options->timeStep, _options->maxRecursionLevel,
                             _options->targetError);
 }
 
-PView::PView(PViewData *data, int num)
+PView::PView(PViewData *data, int tag)
 {
-  _init(num);
+  _init(tag);
   _data = data;
-  _options = new PViewOptions(PViewOptions::reference);
+  _options = new PViewOptions(*PViewOptions::reference());
   if(_options->adaptVisualizationGrid)
     _data->initAdaptiveData(_options->timeStep, _options->maxRecursionLevel,
                             _options->targetError);
@@ -59,22 +72,22 @@ PView::PView(PView *ref, bool copyOptions)
 {
   _init();
 
-  if(ref->getAliasOf()){ // alias of an alias
-    PView *orig = getViewByNum(ref->getAliasOf());
-    if(orig) _aliasOf = orig->getNum();
+  if(ref->getAliasOf() >= 0){ // alias of an alias
+    PView *orig = getViewByTag(ref->getAliasOf());
+    if(orig) _aliasOf = orig->getTag();
     else{
       Msg::Warning("Original view for alias does not exist anymore");
-      _aliasOf = ref->getNum();
+      _aliasOf = ref->getTag();
     }
   }
   else
-    _aliasOf = ref->getNum();
+    _aliasOf = ref->getTag();
 
   _data = ref->getData();
   if(copyOptions)
     _options = new PViewOptions(*ref->getOptions());
   else
-    _options = new PViewOptions(PViewOptions::reference);
+    _options = new PViewOptions(*PViewOptions::reference());
   if(_options->adaptVisualizationGrid)
     _data->initAdaptiveData(_options->timeStep, _options->maxRecursionLevel,
                             _options->targetError);
@@ -88,7 +101,7 @@ PView::PView(const std::string &xname, const std::string &yname,
   _data->setXY(x, y);
   _data->setName(yname);
   _data->setFileName(yname + ".pos");
-  _options = new PViewOptions(PViewOptions::reference);
+  _options = new PViewOptions(*PViewOptions::reference());
   _options->type = PViewOptions::Plot2D;
   _options->axes = 3;
   _options->lineWidth = 2.;
@@ -96,7 +109,7 @@ PView::PView(const std::string &xname, const std::string &yname,
   _options->axesLabel[0] = xname;
 }
 
-PView::PView(std::string name, std::string type,
+PView::PView(const std::string &name, const std::string &type,
              GModel *model, std::map<int, std::vector<double> > &data,
              double time, int numComp)
 {
@@ -108,6 +121,8 @@ PView::PView(std::string name, std::string type,
     t = PViewDataGModel::ElementData;
   else if(type == "ElementNodeData")
     t = PViewDataGModel::ElementNodeData;
+  else if(type == "Beam")
+    t = PViewDataGModel::BeamData;
   else{
     Msg::Error("Unknown type of view to create '%s'", type.c_str());
     return;
@@ -117,7 +132,7 @@ PView::PView(std::string name, std::string type,
   d->setName(name);
   d->setFileName(name + ".msh");
   _data = d;
-  _options = new PViewOptions(PViewOptions::reference);
+  _options = new PViewOptions(*PViewOptions::reference());
   if(_options->adaptVisualizationGrid)
     _data->initAdaptiveData(_options->timeStep, _options->maxRecursionLevel,
                             _options->targetError);
@@ -145,17 +160,17 @@ PView::~PView()
 
   // do not delete if another view is an alias of this one
   for(unsigned int i = 0; i < list.size(); i++)
-    if(list[i]->getAliasOf() == _num)
+    if(list[i]->getAliasOf() == _tag)
       return;
 
   // do not delete if this view is an alias and 1) if the original
   // still exists, or 2) if there are other aliases to the same view
-  if(_aliasOf)
+  if(_aliasOf >= 0)
     for(unsigned int i = 0; i < list.size(); i++)
-      if(list[i]->getNum() == _aliasOf || list[i]->getAliasOf() == _aliasOf)
+      if(list[i]->getTag() == _aliasOf || list[i]->getAliasOf() == _aliasOf)
         return;
 
-  Msg::Debug("Deleting data in View[%d] (unique num = %d)", _index, _num);
+  Msg::Debug("Deleting data in View[%d] (tag = %d)", _index, _tag);
   delete _data;
 }
 
@@ -173,7 +188,7 @@ void PView::setOptions(PViewOptions *val)
   if(val)
     _options = val;
   else if(_options) // deep copy options from reference view
-    *_options = PViewOptions::reference;
+    *_options = *PViewOptions::reference();
 }
 
 PViewData *PView::getData(bool useAdaptiveIfAvailable)
@@ -276,22 +291,25 @@ void PView::combine(bool time, int how, bool remove)
       delete *it;
 }
 
-PView *PView::getViewByName(std::string name, int timeStep, int partition)
+PView *PView::getViewByName(const std::string &name, int timeStep, int partition,
+                            const std::string &fileName)
 {
   // search views from most recently to least recently added
   for(int i = list.size() - 1; i >= 0; i--){
     if(list[i]->getData()->getName() == name &&
        ((timeStep < 0 || !list[i]->getData()->hasTimeStep(timeStep)) ||
-        (partition < 0 || !list[i]->getData()->hasPartition(timeStep, partition))))
+        (partition < 0 || !list[i]->getData()->hasPartition(timeStep, partition))) &&
+       (fileName.empty() || !list[i]->getData()->hasFileName(fileName)))
       return list[i];
   }
   return 0;
 }
 
-PView *PView::getViewByNum(int num, int timeStep, int partition)
+PView *PView::getViewByFileName(const std::string &fileName, int timeStep, int partition)
 {
-  for(unsigned int i = 0; i < list.size(); i++){
-    if(list[i]->getNum() == num &&
+  // search views from most recently to least recently added
+  for(int i = list.size() - 1; i >= 0; i--){
+    if(list[i]->getData()->getFileName() == fileName &&
        ((timeStep < 0 || !list[i]->getData()->hasTimeStep(timeStep)) ||
         (partition < 0 || !list[i]->getData()->hasPartition(timeStep, partition))))
       return list[i];
@@ -299,3 +317,13 @@ PView *PView::getViewByNum(int num, int timeStep, int partition)
   return 0;
 }
 
+PView *PView::getViewByTag(int tag, int timeStep, int partition)
+{
+  for(unsigned int i = 0; i < list.size(); i++){
+    if(list[i]->getTag() == tag &&
+       ((timeStep < 0 || !list[i]->getData()->hasTimeStep(timeStep)) ||
+        (partition < 0 || !list[i]->getData()->hasPartition(timeStep, partition))))
+      return list[i];
+  }
+  return 0;
+}
