@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #ifndef _PVIEW_DATA_H_
 #define _PVIEW_DATA_H_
@@ -9,6 +9,7 @@
 #include <string>
 #include <vector>
 #include <map>
+#include <set>
 #include "SBoundingBox3d.h"
 #include "fullMatrix.h"
 
@@ -32,6 +33,7 @@ class PViewData {
   std::string _name;
   // name of the file the data was loaded from
   std::string _fileName;
+  std::set<std::string> _fileNames;
   // index of the view in the file
   int _fileIndex;
   // octree for rapid search
@@ -59,11 +61,19 @@ class PViewData {
 
   // get/set name
   virtual std::string getName(){ return _name; }
-  virtual void setName(std::string val){ _name = val; }
+  virtual void setName(const std::string &val){ _name = val; }
 
   // get/set (the main) filename containing the data
   virtual std::string getFileName(int step=-1){ return _fileName; }
-  virtual void setFileName(std::string val){ _fileName = val; }
+  virtual void setFileName(const std::string &val)
+  {
+    _fileName = val;
+    _fileNames.insert(val);
+  }
+  virtual bool hasFileName(const std::string &val)
+  {
+    return _fileNames.find(val) != _fileNames.end();
+  }
 
   // get/set index of view data in file
   virtual int getFileIndex(){ return _fileIndex; }
@@ -106,47 +116,43 @@ class PViewData {
   // return the number of geometrical entities in the view
   virtual int getNumEntities(int step=-1){ return 0; }
 
-  // return the number of elements in the ent-th entity, or the total
-  // number of elements if ent < 0
+  // return the number of elements in the ent-th entity, or the total number of
+  // elements if ent < 0
   virtual int getNumElements(int step=-1, int ent=-1){ return 0; }
 
-  // return the geometrical dimension of the ele-th element in the
-  // ent-th entity
+  // return the geometrical dimension of the ele-th element in the ent-th entity
   virtual int getDimension(int step, int ent, int ele){ return 0; }
 
-  // return the number of nodes of the ele-th element in the ent-th
-  // entity
+  // return the number of nodes of the ele-th element in the ent-th entity
   virtual int getNumNodes(int step, int ent, int ele){ return 0; }
 
-  // get/set the coordinates and tag of the nod-th node from the
-  // ele-th element in the ent-th entity (if the node has a tag,
-  // getNode returns it)
+  // get/set the coordinates and tag of the nod-th node from the ele-th element
+  // in the ent-th entity (if the node has a tag, getNode returns it)
   virtual int getNode(int step, int ent, int ele, int nod,
                       double &x, double &y, double &z){ return 0; }
   virtual void setNode(int step, int ent, int ele, int nod,
                        double x, double y, double z);
   virtual void tagNode(int step, int ent, int ele, int nod, int tag){}
 
-  // return the number of components available for the ele-th element in
-  // the ent-th entity
+  // return the number of components available for the ele-th element in the
+  // ent-th entity
   virtual int getNumComponents(int step, int ent, int ele){ return 0; }
 
-  // return the number of values available for the ele-th element in
-  // the ent-th entity
+  // return the number of values available for the ele-th element in the ent-th
+  // entity
   virtual int getNumValues(int step, int ent, int ele){ return 0; }
 
   // get the idx'th value for the ele-th element in the ent-th entity
   virtual void getValue(int step, int ent, int ele, int idx, double &val){}
 
-  // gets/set the comp-th component (at the step-th time step)
-  // associated with the node-th node from the ele-th element in the
-  // ent-th entity
+  // gets/set the comp-th component (at the step-th time step) associated with
+  // the node-th node from the ele-th element in the ent-th entity
   virtual void getValue(int step, int ent, int ele, int nod, int comp, double &val){}
   virtual void setValue(int step, int ent, int ele, int nod, int comp, double val);
 
-  // return a scalar value (same as value for scalars, norm for
-  // vectors, etc.) associated with the node-th node from the ele-th
-  // element in the ent-th entity
+  // return a scalar value (same as value for scalars, norm for vectors, etc.)
+  // associated with the node-th node from the ele-th element in the ent-th
+  // entity
   void getScalarValue(int step, int ent, int ele, int nod, double &val,
                       int forceNumComponents=0, int componentMap[9]=0);
 
@@ -225,6 +231,9 @@ class PViewData {
   virtual bool isRemote(){ return false; }
   virtual int fillRemoteVertexArrays(std::string &options){ return 0; }
 
+  // is the view a list-based dataset
+  virtual bool isListBased(){ return false; }
+
   // get GModel (if view supports it)
   virtual GModel *getModel(int step);
 
@@ -234,29 +243,35 @@ class PViewData {
   // get MElement (if view supports it)
   virtual MElement *getElement(int step, int entity, int element);
 
-  // search for the value of the View at point x, y, z. Values are
-  // interpolated using standard first order shape functions in the
-  // post element. If several time steps are present, they are all
-  // interpolated unless time step is set to a different value than
-  // -1.
+  // search for the value of the View at point x, y, z. Values are interpolated
+  // using standard first order shape functions in the post element. If several
+  // time steps are present, they are all interpolated unless time step is set
+  // to a different value than -1.
   bool searchScalar(double x, double y, double z, double *values,
                     int step=-1, double *size=0);
   bool searchScalarWithTol(double x, double y, double z, double *values,
                            int step=-1, double *size=0, double tol=1.e-2);
   bool searchVector(double x, double y, double z, double *values,
                     int step=-1, double *size=0);
+  bool searchVectorWithTol(double x, double y, double z, double *values,
+                           int step=-1, double *size=0, double tol=1.e-2);
   bool searchTensor(double x, double y, double z, double *values,
                     int step=-1, double *size=0);
+  bool searchTensorWithTol(double x, double y, double z, double *values,
+                           int step=-1, double *size=0, double tol=1.e-2);
 
   // I/O routines
-  virtual bool writeSTL(std::string fileName);
-  virtual bool writeTXT(std::string fileName);
-  virtual bool writePOS(std::string fileName, bool binary=false, bool parsed=true,
-                        bool append=false);
-  virtual bool writeMSH(std::string fileName, bool binary=false, bool savemesh=true);
-  virtual bool writeMED(std::string fileName);
+  virtual bool writeSTL(const std::string &fileName);
+  virtual bool writeTXT(const std::string &fileName);
+  virtual bool writePOS(const std::string &fileName, bool binary=false,
+                        bool parsed=true, bool append=false);
+  virtual bool writeMSH(const std::string &fileName, double version=2.2, bool binary=false,
+                        bool savemesh=true, bool multipleView=false);
+  virtual bool writeMED(const std::string &fileName);
   virtual bool toVector(std::vector<std::vector<double> > &vec);
   virtual bool fromVector(const std::vector<std::vector<double> > &vec);
+  virtual void importLists(int N[24], std::vector<double> *V[24]);
+  virtual void getListPointers(int N[24], std::vector<double> *V[24]);
 };
 
 class nameData{
