@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include "PViewDataGModel.h"
 #include "MPoint.h"
@@ -143,7 +143,7 @@ bool PViewDataGModel::finalize(bool computeMinMax, const std::string &interpolat
           // the mesh is curved
           MElement *e = _getOneElementOfGivenType(model, it->first);
           if(e && e->getPolynomialOrder() > 1 && e->getFunctionSpace()){
-            const polynomialBasis *fs = e->getFunctionSpace();
+            const polynomialBasis *fs = (polynomialBasis*) e->getFunctionSpace();
             setInterpolationMatrices(it->first, *(it->second[0]), *(it->second[1]),
                                      fs->coefficients, fs->monomials);
           }
@@ -173,7 +173,7 @@ bool PViewDataGModel::finalize(bool computeMinMax, const std::string &interpolat
       if(!haveInterpolationMatrices(types[i])){
         MElement *e = _getOneElementOfGivenType(model, types[i]);
         if(e){
-          const polynomialBasis *fs = e->getFunctionSpace();
+          const polynomialBasis *fs = (polynomialBasis*) e->getFunctionSpace();
           if(fs){
             if(e->getPolynomialOrder() > 1){
               if(_type == ElementData){
@@ -292,8 +292,10 @@ SBoundingBox3d PViewDataGModel::getBoundingBox(int step)
 {
   if(step < 0 || _steps.empty()){
     SBoundingBox3d tmp;
-    for(unsigned int i = 0; i < _steps.size(); i++)
-      tmp += _steps[i]->getBoundingBox();
+    for(unsigned int i = 0; i < _steps.size(); i++){
+      if(!_steps[i]->getBoundingBox().empty())
+        tmp += _steps[i]->getBoundingBox();
+    }
     return tmp;
   }
   return _steps[step]->getBoundingBox();
