@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include "GmshConfig.h"
 #include "GmshMessage.h"
@@ -25,7 +25,8 @@ bool PViewDataGModel::addData(GModel *model, std::map<int, std::vector<double> >
 
   while(step >= (int)_steps.size())
     _steps.push_back(new stepData<double>(model, numComp));
-
+  _steps[step]->fillEntities();
+  _steps[step]->computeBoundingBox();
   _steps[step]->setTime(time);
 
   int numEnt = (_type == NodeData) ? model->getNumMeshVertices() :
@@ -45,21 +46,24 @@ bool PViewDataGModel::addData(GModel *model, std::map<int, std::vector<double> >
   return true;
 }
 
-bool PViewDataGModel::readMSH(std::string fileName, int fileIndex, FILE *fp,
-                              bool binary, bool swap, int step, double time,
-                              int partition, int numComp, int numEnt,
-                              const std::string &interpolationScheme)
+void PViewDataGModel::destroyData()
 {
-  Msg::Info("Reading step %d (time %g) partition %d: %d records",
-            step, time, partition, numEnt);
+  for(unsigned int i = 0; i < _steps.size(); i++)
+    _steps[i]->destroyData();
+}
 
-  while(step >= (int)_steps.size()){
-    if(_steps.empty() || _steps.back()->getNumData())
-      _steps.push_back(new stepData<double>(GModel::current(), numComp));
-    else // faster since we avoid computing model bounds
-      _steps.push_back(new stepData<double>(*_steps.back()));
-  }
+bool PViewDataGModel::readMSH(const std::string &viewName, const std::string &fileName,
+                              int fileIndex, FILE *fp, bool binary, bool swap,
+                              int step, double time, int partition, int numComp,
+                              int numEnt, const std::string &interpolationScheme)
+{
+  Msg::Info("Reading view `%s' step %d (time %g) partition %d: %d records",
+            viewName.c_str(), step, time, partition, numEnt);
 
+  while(step >= (int)_steps.size())
+    _steps.push_back(new stepData<double>(GModel::current(), numComp));
+  _steps[step]->fillEntities();
+  _steps[step]->computeBoundingBox();
   _steps[step]->setFileName(fileName);
   _steps[step]->setFileIndex(fileIndex);
   _steps[step]->setTime(time);
@@ -117,7 +121,7 @@ bool PViewDataGModel::readMSH(std::string fileName, int fileIndex, FILE *fp,
       _max = std::max(_max, val);
     }
     if(numEnt > 100000)
-      Msg::ProgressMeter(i + 1, numEnt, "Reading data");
+      Msg::ProgressMeter(i + 1, numEnt, true, "Reading data");
   }
 
   if(partition >= 0)
@@ -127,7 +131,8 @@ bool PViewDataGModel::readMSH(std::string fileName, int fileIndex, FILE *fp,
   return true;
 }
 
-bool PViewDataGModel::writeMSH(std::string fileName, bool binary, bool savemesh)
+bool PViewDataGModel::writeMSH(const std::string &fileName, double version, bool binary,
+                               bool savemesh, bool multipleView)
 {
   if(_steps.empty()) return true;
 
@@ -141,7 +146,8 @@ bool PViewDataGModel::writeMSH(std::string fileName, bool binary, bool savemesh)
   bool writeNodesAndElements = savemesh;
   FILE *fp;
   if(writeNodesAndElements){
-    if(!model->writeMSH(fileName, 2.0, binary)) return false;
+    if(!model->writeMSH(fileName, version, binary, false, false, 1.0, 0,
+                        0, multipleView)) return false;
     // append data
     fp = fopen(fileName.c_str(), binary ? "ab" : "a");
     if(!fp){
@@ -150,19 +156,28 @@ bool PViewDataGModel::writeMSH(std::string fileName, bool binary, bool savemesh)
     }
   }
   else{
-    fp = fopen(fileName.c_str(), binary ? "wb" : "w");
-    if(!fp){
-      Msg::Error("Unable to open file '%s'", fileName.c_str());
-      return false;
+    if(multipleView){
+      fp = fopen(fileName.c_str(), binary ? "ab" : "a");
+      if(!fp){
+        Msg::Error("Unable to open file '%s'", fileName.c_str());
+        return false;
+      }
     }
-    fprintf(fp, "$MeshFormat\n");
-    fprintf(fp, "%g %d %d\n", 2.2, binary ? 1 : 0, (int)sizeof(double));
-    if(binary){
-      int one = 1;
-      fwrite(&one, sizeof(int), 1, fp);
-      fprintf(fp, "\n");
+    else{
+      fp = fopen(fileName.c_str(), binary ? "wb" : "w");
+      if(!fp){
+        Msg::Error("Unable to open file '%s'", fileName.c_str());
+        return false;
+      }
+      fprintf(fp, "$MeshFormat\n");
+      fprintf(fp, "%g %d %d\n", 2.2, binary ? 1 : 0, (int)sizeof(double));
+      if(binary){
+        int one = 1;
+        fwrite(&one, sizeof(int), 1, fp);
+        fprintf(fp, "\n");
+      }
+      fprintf(fp, "$EndMeshFormat\n");
     }
-    fprintf(fp, "$EndMeshFormat\n");
   }
 
   if(haveInterpolationMatrices()){
@@ -301,7 +316,65 @@ extern "C" {
 extern int med2mshElementType(med_geometrie_element med);
 extern int med2mshNodeIndex(med_geometrie_element med, int k);
 
-bool PViewDataGModel::readMED(std::string fileName, int fileIndex)
+std::vector<std::string> medGetFieldNames(const std::string &fileName)
+{
+  std::vector<std::string> fieldNames;
+
+#if (MED_MAJOR_NUM == 3)
+  med_idt fid = MEDfileOpen(fileName.c_str(), MED_ACC_RDONLY);
+#else
+  med_idt fid = MEDouvrir((char*)fileName.c_str(), MED_LECTURE);
+#endif
+  if(fid < 0) {
+    Msg::Error("Unable to open file '%s'", fileName.c_str());
+    return fieldNames;
+  }
+
+#if (MED_MAJOR_NUM == 3)
+  med_int numFields = MEDnField(fid);
+#else
+  med_int numFields = MEDnChamp(fid, 0);
+#endif
+
+  for(int index = 0; index < numFields; index++){
+
+    med_int numComp = MEDnChamp(fid, index + 1);
+    if(numComp <= 0){
+      Msg::Error("Could not get number of components for MED field");
+      return fieldNames;
+    }
+
+    char name[MED_TAILLE_NOM + 1], meshName[MED_TAILLE_NOM + 1];
+    char dtUnit[MED_TAILLE_PNOM + 1];
+    std::vector<char> compName(numComp * MED_TAILLE_PNOM + 1);
+    std::vector<char> compUnit(numComp * MED_TAILLE_PNOM + 1);
+    med_int numSteps = 0;
+    med_type_champ type;
+#if (MED_MAJOR_NUM == 3)
+    med_bool localMesh;
+    if(MEDfieldInfo(fid, index + 1, name, meshName, &localMesh, &type,
+                    &compName[0], &compUnit[0], dtUnit, &numSteps) < 0){
+#else
+    if(MEDchampInfo(fid, index + 1, name, &type, &compName[0], &compUnit[0],
+                    numComp) < 0){
+#endif
+      Msg::Error("Could not get MED field info");
+      return fieldNames;
+    }
+    fieldNames.push_back(name);
+  }
+
+#if (MED_MAJOR_NUM == 3)
+  if(MEDfileClose(fid) < 0){
+#else
+  if(MEDfermer(fid) < 0){
+#endif
+    Msg::Error("Unable to close file '%s'", fileName.c_str());
+  }
+  return fieldNames;
+}
+
+bool PViewDataGModel::readMED(const std::string &fileName, int fileIndex)
 {
   med_idt fid = MEDouvrir((char*)fileName.c_str(), MED_LECTURE);
   if(fid < 0){
@@ -335,6 +408,8 @@ bool PViewDataGModel::readMED(std::string fileName, int fileIndex)
 
   Msg::Info("Reading %d-component field <<%s>>", numComp, name);
   setName(name);
+  setFileName(fileName);
+  setFileIndex(fileIndex);
 
   int numCompMsh =
     (numComp <= 1) ? 1 : (numComp <= 3) ? 3 : (numComp <= 9) ? 9 : numComp;
@@ -423,6 +498,8 @@ bool PViewDataGModel::readMED(std::string fileName, int fileIndex)
         }
         while(step >= (int)_steps.size())
           _steps.push_back(new stepData<double>(m, numCompMsh));
+        _steps[step]->fillEntities();
+        _steps[step]->computeBoundingBox();
         _steps[step]->setFileName(fileName);
         _steps[step]->setFileIndex(fileIndex);
         _steps[step]->setTime(dt);
@@ -514,6 +591,7 @@ bool PViewDataGModel::readMED(std::string fileName, int fileIndex)
       if(std::string(profileName) != MED_NOPFL){
         med_int n = MEDnValProfil(fid, profileName);
         if(n > 0){
+          Msg::Debug("MED has full profile");
           profile.resize(n);
 #if (MED_MAJOR_NUM == 3)
           if(MEDprofileRd(fid, profileName, &profile[0]) < 0){
@@ -526,6 +604,7 @@ bool PViewDataGModel::readMED(std::string fileName, int fileIndex)
         }
       }
       if(profile.empty()){
+        Msg::Debug("MED profile is empty -- using continuous sequence");
         profile.resize(numVal / mult);
         for(unsigned int i = 0; i < profile.size(); i++)
           profile[i] = i + 1;
@@ -559,22 +638,30 @@ bool PViewDataGModel::readMED(std::string fileName, int fileIndex)
 #endif
         tags.clear();
 
-      // if we don't have tags, compute the starting index (i.e., how
-      // many elements of different type are in the mesh before these
-      // ones)
+      // if we don't have tags, compute the starting index (i.e., how many
+      // elements of different type are in the mesh before these ones)
       int startIndex = 0;
-      if(!nodal && tags.empty()){
-        for(int i = 1; i < pairs[pair].second; i++){
-#if (MED_MAJOR_NUM == 3)
-          med_int n = MEDmeshnEntity(fid, meshName, MED_NO_DT, MED_NO_IT,
-                                     MED_CELL, eleType[i], MED_CONNECTIVITY,
-                                     MED_NODAL, &changeOfCoord, &geoTransform);
-#else
-          med_int n = MEDnEntMaa(fid, meshName, MED_CONN, MED_MAILLE,
-                                 eleType[i], MED_NOD);
-#endif
-          if(n > 0) startIndex += n;
+      if(tags.empty()){
+        int maxv, maxe;
+        _steps[step]->getModel()->getCheckPointedMaxNumbers(maxv, maxe);
+        if(nodal){
+          startIndex += maxv;
         }
+        else{
+          for(int i = 1; i < pairs[pair].second; i++){
+#if (MED_MAJOR_NUM == 3)
+            med_int n = MEDmeshnEntity(fid, meshName, MED_NO_DT, MED_NO_IT,
+                                       MED_CELL, eleType[i], MED_CONNECTIVITY,
+                                       MED_NODAL, &changeOfCoord, &geoTransform);
+#else
+            med_int n = MEDnEntMaa(fid, meshName, MED_CONN, MED_MAILLE,
+                                   eleType[i], MED_NOD);
+#endif
+            if(n > 0) startIndex += n;
+          }
+          startIndex += maxe;
+        }
+        Msg::Debug("MED has no tags -- assuming starting index %d", startIndex);
       }
 
       // compute entity numbers using profile, then fill step data
@@ -590,6 +677,7 @@ bool PViewDataGModel::readMED(std::string fileName, int fileIndex)
           }
           num = tags[profile[i] - 1];
         }
+
         double *d = _steps[step]->getData(num, true, mult);
         for(int j = 0; j < mult; j++){
           // reorder nodes if we have ElementNode data
@@ -610,7 +698,7 @@ bool PViewDataGModel::readMED(std::string fileName, int fileIndex)
   return true;
 }
 
-bool PViewDataGModel::writeMED(std::string fileName)
+bool PViewDataGModel::writeMED(const std::string &fileName)
 {
   if(_steps.empty()) return true;
 
@@ -731,14 +819,14 @@ bool PViewDataGModel::writeMED(std::string fileName)
 
 #else
 
-bool PViewDataGModel::readMED(std::string fileName, int fileIndex)
+bool PViewDataGModel::readMED(const std::string &fileName, int fileIndex)
 {
   Msg::Error("Gmsh must be compiled with MED support to read '%s'",
              fileName.c_str());
   return false;
 }
 
-bool PViewDataGModel::writeMED(std::string fileName)
+bool PViewDataGModel::writeMED(const std::string &fileName)
 {
   Msg::Error("Gmsh must be compiled with MED support to write '%s'",
              fileName.c_str());
