@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include <string.h>
 #include <set>
@@ -341,7 +341,8 @@ static void writeTextPOS(FILE *fp, int nbc, int nb, std::vector<double> &TD,
   }
 }
 
-bool PViewDataList::writePOS(std::string fileName, bool binary, bool parsed, bool append)
+bool PViewDataList::writePOS(const std::string &fileName, bool binary, bool parsed,
+                             bool append)
 {
   if(_adaptive){
     Msg::Warning("Writing adapted dataset (will only export current time step)");
@@ -434,7 +435,7 @@ bool PViewDataList::writePOS(std::string fileName, bool binary, bool parsed, boo
   return true;
 }
 
-static void createVertices(std::vector<double> &list, int nbelm, int nbnod, 
+static void createVertices(std::vector<double> &list, int nbelm, int nbnod,
                            std::vector<MVertex*> &nodes)
 {
   if(!nbelm) return;
@@ -445,10 +446,10 @@ static void createVertices(std::vector<double> &list, int nbelm, int nbnod,
     double *z = &list[i + 2 * nbnod];
     for(int j = 0; j < nbnod; j++)
       nodes.push_back(new MVertex(x[j], y[j], z[j]));
-  }    
+  }
 }
 
-static void createElements(std::vector<double> &list, int nbelm, int nbnod, 
+static void createElements(std::vector<double> &list, int nbelm, int nbnod,
                            MVertexPositionSet &pos, std::vector<MElement*> &elements,
                            double eps, int type)
 {
@@ -473,7 +474,7 @@ static void createElements(std::vector<double> &list, int nbelm, int nbnod,
     break;
   case TYPE_QUA :
     switch(nbnod){
-    case 4: t = MSH_QUA_4; break; 
+    case 4: t = MSH_QUA_4; break;
     case 8: t = MSH_QUA_8; break;
     case 9: t = MSH_QUA_9; break;
     }
@@ -524,11 +525,12 @@ static void createElements(std::vector<double> &list, int nbelm, int nbnod,
   }
 }
 
-bool PViewDataList::writeMSH(std::string fileName, bool binary, bool savemesh)
+bool PViewDataList::writeMSH(const std::string &fileName, double version, bool binary,
+                             bool savemesh, bool multipleView)
 {
   if(_adaptive){
     Msg::Warning("Writing adapted dataset (will only export current time step)");
-    return _adaptive->getData()->writeMSH(fileName, binary);
+    return _adaptive->getData()->writeMSH(fileName, version, binary);
   }
 
   FILE *fp = fopen(fileName.c_str(), "w");
@@ -562,7 +564,7 @@ bool PViewDataList::writeMSH(std::string fileName, bool binary, bool savemesh)
 
   int num = 0;
   for(unsigned int i = 0; i < vertices.size(); i++)
-    if(vertices[i]->getIndex() < 0) 
+    if(vertices[i]->getIndex() < 0)
       vertices[i]->setIndex(++num);
 
   fprintf(fp, "$MeshFormat\n2.2 0 8\n$EndMeshFormat\n");
@@ -577,8 +579,12 @@ bool PViewDataList::writeMSH(std::string fileName, bool binary, bool savemesh)
 
   fprintf(fp, "$Elements\n");
   fprintf(fp, "%d\n", (int)elements.size());
-  for(unsigned int i = 0; i < elements.size(); i++)
-    elements[i]->writeMSH(fp, 2.2, false, i + 1);
+  for(unsigned int i = 0; i < elements.size(); i++){
+    if(version > 2.2)
+      Msg::Warning("Unable to write file in version '%g': using version 2.2",
+                   version);
+    elements[i]->writeMSH2(fp, 2.2, false, i + 1);
+  }
   fprintf(fp, "$EndElements\n");
 
   if(haveInterpolationMatrices()){
@@ -636,3 +642,26 @@ bool PViewDataList::writeMSH(std::string fileName, bool binary, bool savemesh)
   fclose(fp);
   return true;
 }
+
+void PViewDataList::importLists(int N[24], std::vector<double> *V[24])
+{
+  for(int i = 0; i < 24; i++){
+    std::vector<double> *list = 0;
+    int *nbe = 0, nbc, nbn;
+    _getRawData(i, &list, &nbe, &nbc, &nbn);
+    *nbe = N[i];
+    *list = *V[i]; // deep copy
+  }
+  finalize();
+}
+
+void PViewDataList::getListPointers(int N[24], std::vector<double> *V[24])
+{
+  for(int i = 0; i < 24; i++){
+    std::vector<double> *list = 0;
+    int *nbe = 0, nbc, nbn;
+    _getRawData(i, &list, &nbe, &nbc, &nbn);
+    N[i] = *nbe; 
+    V[i] = list; // copy pointer only
+  }
+}
