@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #ifndef _PVIEW_DATA_REMOTE_H_
 #define _PVIEW_DATA_REMOTE_H_
@@ -17,16 +17,16 @@
 // The container for a remote dataset (does not contain any actual
 // data)
 class PViewDataRemote : public PViewData {
- private: 
+ private:
   onelab::localNetworkClient *_remote;
   double _min, _max;
   int _numTimeSteps;
   double _time;
   SBoundingBox3d _bbox;
  public:
-  PViewDataRemote(onelab::localNetworkClient *remote, double min, double max, int numsteps, 
-                  double time, SBoundingBox3d &bbox)
-    : _remote(remote), _min(min), _max(max), _numTimeSteps(numsteps), 
+  PViewDataRemote(onelab::localNetworkClient *remote, double min, double max,
+                  int numsteps, double time, SBoundingBox3d &bbox)
+    : _remote(remote), _min(min), _max(max), _numTimeSteps(numsteps),
       _time(time), _bbox(bbox) {}
   ~PViewDataRemote(){}
   bool finalize(bool computeMinMax=true, const std::string &interpolationScheme="")
@@ -34,14 +34,14 @@ class PViewDataRemote : public PViewData {
     return true;
   }
   int getNumTimeSteps(){ return _numTimeSteps; }
-  double getMin(int step=-1, bool onlyVisible=false, int forceNumComponents=0, 
+  double getMin(int step=-1, bool onlyVisible=false, int forceNumComponents=0,
                 int componentMap[9]=0){ return _min; }
   double getMax(int step=-1, bool onlyVisible=false, int forceNumComponents=0,
                 int componentMap[9]=0){ return _max; }
   SBoundingBox3d getBoundingBox(int step=-1){ return _bbox; }
   double getTime(int step){ return _time; }
   // need to return != 0 for "empty" tests
-  int getNumElements(int step=-1, int ent=-1){ return -1; }  
+  int getNumElements(int step=-1, int ent=-1){ return -1; }
   void setMin(double min){ _min = min; }
   void setMax(double max){ _max = max; }
   void setBoundingBox(SBoundingBox3d &bbox){ _bbox = bbox; }
