@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2012 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include "GmshConfig.h"
 #include "GmshMessage.h"
@@ -11,7 +11,7 @@
 #include "StringUtils.h"
 #include "Context.h"
 
-bool PView::readPOS(std::string fileName, int fileIndex)
+bool PView::readPOS(const std::string &fileName, int fileIndex)
 {
   FILE *fp = fopen(fileName.c_str(), "rb");
   if(!fp){
@@ -81,7 +81,7 @@ bool PView::readPOS(std::string fileName, int fileIndex)
   return true;
 }
 
-bool PView::readMSH(std::string fileName, int fileIndex)
+bool PView::readMSH(const std::string &fileName, int fileIndex)
 {
   FILE *fp = fopen(fileName.c_str(), "rb");
   if(!fp){
@@ -210,7 +210,7 @@ bool PView::readMSH(std::string fileName, int fileIndex)
         if(p) d = dynamic_cast<PViewDataGModel*>(p->getData());
         bool create = d ? false : true;
         if(create) d = new PViewDataGModel(type);
-        if(!d->readMSH(fileName, fileIndex, fp, binary, swap, timeStep,
+        if(!d->readMSH(viewName, fileName, fileIndex, fp, binary, swap, timeStep,
                        time, partition, numComp, numEnt, interpolationScheme)){
           Msg::Error("Could not read data in msh file");
           if(create) delete d;
@@ -237,49 +237,28 @@ bool PView::readMSH(std::string fileName, int fileIndex)
 
 #if defined(HAVE_MED)
 
-extern "C" {
-#include <med.h>
-}
+extern std::vector<std::string> medGetFieldNames(const std::string &fileName);
 
-bool PView::readMED(std::string fileName, int fileIndex)
+bool PView::readMED(const std::string &fileName, int fileIndex)
 {
-#if (MED_MAJOR_NUM == 3)
-  med_idt fid = MEDfileOpen(fileName.c_str(), MED_ACC_RDONLY);
-#else
-  med_idt fid = MEDouvrir((char*)fileName.c_str(), MED_LECTURE);
-#endif
-  if(fid < 0) {
-    Msg::Error("Unable to open file '%s'", fileName.c_str());
-    return false;
-  }
+  std::vector<std::string> fieldNames = medGetFieldNames(fileName);
 
-#if (MED_MAJOR_NUM == 3)
-  med_int numFields = MEDnField(fid);
-#else
-  med_int numFields = MEDnChamp(fid, 0);
-#endif
-
-#if (MED_MAJOR_NUM == 3)
-  if(MEDfileClose(fid) < 0){
-#else
-  if(MEDfermer(fid) < 0){
-#endif
-    Msg::Error("Unable to close file '%s'", fileName.c_str());
-    return false;
-  }
-
-  for(int index = 0; index < numFields; index++){
-    if(fileIndex < 0 || index == fileIndex){
-      PViewDataGModel *d = new PViewDataGModel();
+  for(unsigned int index = 0; index < fieldNames.size(); index++){
+    if(fileIndex < 0 || (int)index == fileIndex){
+      PViewDataGModel *d = 0;
+      // we use the filename as a kind of "partition" indicator, allowing to
+      // complete datasets provided in separate files (e.g. coming from DDM)
+      PView *p = getViewByName(fieldNames[index], -1, -1, fileName);
+      if(p) d = dynamic_cast<PViewDataGModel*>(p->getData());
+      bool create = d ? false : true;
+      if(create) d = new PViewDataGModel();
       if(!d->readMED(fileName, index)){
         Msg::Error("Could not read data in MED file");
-        delete d;
+        if(create) delete d;
         return false;
       }
       else{
-        d->setFileName(fileName);
-        d->setFileIndex(index);
-        new PView(d);
+        if(create) new PView(d);
       }
     }
   }
@@ -289,7 +268,7 @@ bool PView::readMED(std::string fileName, int fileIndex)
 
 #else
 
-bool PView::readMED(std::string fileName, int fileIndex)
+bool PView::readMED(const std::string &fileName, int fileIndex)
 {
   Msg::Error("Gmsh must be compiled with MED support to read '%s'",
              fileName.c_str());
@@ -298,9 +277,9 @@ bool PView::readMED(std::string fileName, int fileIndex)
 
 #endif
 
-bool PView::write(std::string fileName, int format, bool append)
+bool PView::write(const std::string &fileName, int format, bool append)
 {
-  Msg::StatusBar(2, true, "Writing '%s'...", fileName.c_str());
+  Msg::StatusBar(true, "Writing '%s'...", fileName.c_str());
 
   bool ret;
   switch(format){
@@ -309,7 +288,8 @@ bool PView::write(std::string fileName, int format, bool append)
   case 2: ret = _data->writePOS(fileName, false, true, append); break; // parsed
   case 3: ret = _data->writeSTL(fileName); break;
   case 4: ret = _data->writeTXT(fileName); break;
-  case 5: ret = _data->writeMSH(fileName, CTX::instance()->mesh.binary); break;
+  case 5: ret = _data->writeMSH(fileName, CTX::instance()->mesh.mshFileVersion,
+                                CTX::instance()->mesh.binary); break;
   case 6: ret = _data->writeMED(fileName); break;
   case 10:
     {
@@ -319,7 +299,8 @@ bool PView::write(std::string fileName, int format, bool append)
       else if(ext == ".stl")
         ret = _data->writeSTL(fileName);
       else if(ext == ".msh")
-        ret = _data->writeMSH(fileName, CTX::instance()->mesh.binary);
+        ret = _data->writeMSH(fileName, CTX::instance()->mesh.mshFileVersion,
+                              CTX::instance()->mesh.binary);
       else if(ext == ".med")
         ret = _data->writeMED(fileName);
       else
@@ -329,6 +310,6 @@ bool PView::write(std::string fileName, int format, bool append)
   default: ret = false; Msg::Error("Unknown view format %d", format); break;
   }
 
-  if(ret) Msg::StatusBar(2, true, "Done writing '%s'", fileName.c_str());
+  if(ret) Msg::StatusBar(true, "Done writing '%s'", fileName.c_str());
   return ret;
 }
